﻿/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7691 $
 *   $Date: 2021-09-19 11:36:55 +0200 (So, 19. Sep 2021) $
 *   $Id: Application_Test.kt 7691 2021-09-19 09:36:55Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.Log as UIQ3_Log

@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.SmallTest
class Application_Test
{

   companion object
   {
      private val TAG = UIQ3_Log.createTag(Calculator_Service_Test::class.java)
   } // companion object

   fun i(Text: String) = UIQ3_Log.i(TAG, Text)

   /**
    * Test the zip file name
    */
   @org.junit.Test
   fun Test_00_Configuration()
   {
      UIQ3_Log.d(TAG, "+ Test_00_Configuration")

      val API = android.os.Build.VERSION.SDK_INT

      i("Build Configuration")
      i("> Debug                   = " + BuildConfig.DEBUG)
      i("> Demo                    = " + Constants.DEMO)

      val Context = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application>()
      val Resources = Context.resources
      val Configuration = Resources.configuration

      i("API                       = " + API)
      i(" 1: fontScale             = " + Configuration.fontScale)
      i(" 1: keyboard              = " + Configuration.keyboard)
      i(" 1: keyboardHidden        = " + Configuration.keyboardHidden)
      i(" 1: locale                = " + Configuration.locale)
      i(" 1: mcc                   = " + Configuration.mcc)
      i(" 1: mnc                   = " + Configuration.mnc)
      i(" 1: navigation            = " + Configuration.navigation)
      i(" 1: orientation           = " + Configuration.orientation)
      i(" 1: touchscreen           = " + Configuration.touchscreen)
      i(" 3: hardKeyboardHidden    = " + Configuration.hardKeyboardHidden)
      i(" 4: screenLayout          = " + Configuration.screenLayout)
      i(" 5: navigationHidden      = " + Configuration.navigationHidden)
      i(" 8: uiMode                = " + Configuration.uiMode)
      i("13: screenHeightDp        = " + Configuration.screenHeightDp)
      i("13: screenWidthDp         = " + Configuration.screenWidthDp)
      i("13: smallestScreenWidthDp = " + Configuration.smallestScreenWidthDp)
      i("17: densityDpi            = " + Configuration.densityDpi)
      i("23: screenRound           = " + Configuration.isScreenRound)
      i("24: locales               = " + Configuration.locales)
      i("26: colorMode             = " + Configuration.colorMode)
      i("26: screenHdr             = " + Configuration.isScreenHdr)
      i("26: screenWideColorGamut  = " + Configuration.isScreenWideColorGamut)

      if (API >= android.os.Build.VERSION_CODES.R)
      {
	 i("30: NightModeActive       = " + Configuration.isNightModeActive)
      } // if

      if (API >= android.os.Build.VERSION_CODES.S)
      {
	 i("31: fontWeightAdjustment  = " + Configuration.fontWeightAdjustment)
      } // if

      UIQ3_Log.d(TAG, "- Test_00_Configuration")
   } // Test_00_Configuration

   /**
    * test preferences.
    */
   @org.junit.Test
   fun test_01_Preferences()
   {
      UIQ3_Log.d(TAG, "+ test_01_Preferences")

      val Application = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application_>()

      i("Application               = " + Application)

      Application.Set_Test_Preferences()

      UIQ3_Log.d(TAG, "- test_01_Preferences")
   } // test_01_Preferences
} // Application_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
