/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7749 $
 *   $Date: 2021-11-24 16:28:47 +0100 (Mi, 24. Nov 2021) $
 *   $Id: Contact_Activity_Test.kt 7749 2021-11-24 15:28:47Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.*
import org.jetbrains.annotations.NotNull
import net.sourceforge.uiq3.Log as UIQ3_Log

@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.LargeTest
class Contact_Activity_Test
{
   companion object
   {
      private val TAG = UIQ3_Log.createTag(Contact_Activity_Test::class.java)
   } // companion object

   /**
    * <p>activity under test</p>
    */
   @get:org.junit.Rule(order = 1)
   var activityRule = androidx.test.ext.junit.rules.ActivityScenarioRule(Contact_Activity_::class.java)

   /**
    * <p>make screen shots before and after tests. Activated with log settings.</p>
    */
   @get:org.junit.Rule(order = 2)
   val screenshotRule = net.sourceforge.uiq3.test.ScreenshotWatcher()

   private fun Take_Screen_Shot(
      @NotNull Name: String,
      @NotNull Step: String,
      Wait_Before: Long = Second(0.5f)) =
      Take_Screen_Shot(Contact_Activity_Test.TAG, Name, Step, Wait_Before)

   @org.junit.Before
   fun Before()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Before")

      val Context = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application>()

      Before(Context)

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Before")
      return
   } // Before

   @org.junit.After
   fun After()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ After")

      val Context = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application>()

      After(Context)

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- After")
      return
   } // After

   @org.junit.Test
   fun Test_01_Open()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_01_Open")

      Take_Screen_Shot(Name = "Test_01_Open", Step = "1")

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_01_Open")
      return
   } // Test_01_Open

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_02_Bug_Report()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_02_Bug_Report")

      Take_Screen_Shot(Name = "Test_02_Bug_Report", Step = "1")
      Click_Button(R.id.Contact_Button_Bug_Report)
      Take_Screen_Shot(Name = "Test_02_Bug_Report", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_02_Bug_Report")
   } // Test_02_Bug_Report

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_03_Mail()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_03_Mail")

      Take_Screen_Shot(Name = "Test_03_Mail", Step = "1")
      Click_Button(R.id.Contact_Button_Mail)
      Take_Screen_Shot(Name = "Test_03_Mail", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_03_Mail")
   } // Test_03_Mail

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_04_Market()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_04_Market")

      Take_Screen_Shot(Name = "Test_04_Market", Step = "1")
      Click_Button(R.id.Contact_Button_Market)
      Take_Screen_Shot(Name = "Test_04_Market", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_04_Market")
   } // Test_04_Market

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_05_YouTube_Videos()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_05_YouTube_Videos")

      Take_Screen_Shot(Name = "Test_05_YouTube_Videos", Step = "1")
      Click_Button(R.id.Contact_Button_YouTube_Videos)
      Take_Screen_Shot(Name = "Test_05_YouTube_Videos", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_05_YouTube_Videos")
   } // Test_05_YouTube_Videos

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_06_Manual()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_06_Manual")

      Take_Screen_Shot(Name = "Test_06_Manual", Step = "1")
      Click_Button(R.id.Contact_Button_Manual)
      Take_Screen_Shot(Name = "Test_06_Manual", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_06_Manual")
   } // Test_06_Manual

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_07_SourceForge_Project()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_07_SourceForge_Project")

      Take_Screen_Shot(Name = "Test_07_SourceForge_Project", Step = "1")
      Click_Button(R.id.Contact_Button_SourceForge_Project)
      Take_Screen_Shot(Name = "Test_07_SourceForge_Project", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_07_SourceForge_Project")
   } // Test_07_SourceForge_Project

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_08_Homepage()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_08_Homepage")

      Take_Screen_Shot(Name = "Test_08_Homepage", Step = "1")
      Click_Button(R.id.Contact_Button_Homepage)
      Take_Screen_Shot(Name = "Test_08_Homepage", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_08_Homepage")
   } // Test_08_Homepage

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_09_History()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_09_History")

      Take_Screen_Shot(Name = "Test_09_History", Step = "1")
      Click_Button(R.id.Contact_Button_History)
      Take_Screen_Shot(Name = "Test_09_History", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_09_History")
   } // Test_09_History

   @androidx.test.filters.LargeTest
   @org.junit.Test
   fun Test_10_Privacy()
   {
      UIQ3_Log.d(Contact_Activity_Test.TAG, "+ Test_10_Privacy")

      Take_Screen_Shot(Contact_Activity_Test.TAG, Name = "Test_10_Privacy", Step = "1")
      Click_Button(R.id.Contact_Button_Privacy)
      Take_Screen_Shot(Contact_Activity_Test.TAG, Name = "Test_10_Privacy", Step = "2", Wait_Before = Second(2.0f))
      Click_Device_Back()

      UIQ3_Log.d(Contact_Activity_Test.TAG, "- Test_10_Privacy")
   } // Test_10_Privacy
} // Contact_Activity_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
