/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.Log
import net.sourceforge.uiq3.test.And
import net.sourceforge.uiq3.test.Given
import net.sourceforge.uiq3.test.Info
import net.sourceforge.uiq3.test.On_Service
import net.sourceforge.uiq3.test.Second
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.When
import org.hamcrest.CoreMatchers.notNullValue
import org.hamcrest.MatcherAssert.assertThat

/**
 * <p>test the calculator engine</p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.SmallTest
class Droid_Calculator_Test
{ // Droid_Calculator_Test
   companion object
   {
      private val TAG = Log.createTag(Droid_Calculator_Test::class.java)
   }

   init
   {
      Info("As a user I want to initialize And start the calculator")
   }

   /**
    * <p>service under test</p>
    */
   @get:org.junit.Rule(order = 1)
   var serviceRule = androidx.test.rule.ServiceTestRule()

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_Without_Running()
   {
      Given("A calculator instance")

      On_Service(serviceRule) { Service ->
	 val Test = Droid_Calculator(Service)

	 When("operation codes are initialized")

	 Test.Init_Opcodes()

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then no error is reported")

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)
      } // On_Service
   } // Scenario

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_Read_Listing_File()
   {
      Given("An calculator instance")

      On_Service(serviceRule) { Service ->
	 val Test = Droid_Calculator(Service)

	 When("operation codes are initialized")

	 Test.Init_Opcodes()

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then the listing file can be read")

	 Test.Read_Listing_File("/rpn-45.lst")

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)
      } // On_Service
   } // Scenario

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_Running()
   {
      Given("A calculator instance")

      On_Service(serviceRule) { Service ->
	 val Test = Droid_Calculator(Service)

	 When("calculator is initialized")

	 Test.Init_Opcodes()

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then the calculator can be run")

	 val Calculator_Thread = Thread(Test, "Calculator Thread")

	 Calculator_Thread.start()

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)

	 And("stopped")

	 synchronized(Test)
	 {
	    java.util.concurrent.TimeUnit.MILLISECONDS.sleep(Second(.5f))
	 }
	 Test.stop()
      } // On_Service
   } // Scenario

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_toString()
   {
      Given("an Calculator instance created And started")

      On_Service(serviceRule) { Service ->
	 val Test = Droid_Calculator(Service)

	 Then("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)

	 And("then the calculator can be stopped")

	 Test.stop()
      } // On_Service
   } // Scenario
}

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
