/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7749 $
 *   $Date: 2021-11-24 16:28:47 +0100 (Mi, 24. Nov 2021) $
 *   $Id: Help_Activity_Test.kt 7749 2021-11-24 15:28:47Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.*
import org.jetbrains.annotations.NotNull
import net.sourceforge.uiq3.Log as UIQ3_Log

@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.LargeTest
class Help_Activity_Test
{
   companion object
   {
      private val TAG = UIQ3_Log.createTag(Help_Activity_Test::class.java)
   } // companion object

   /**
    * <p>activity under test</p>
    */
   @get:org.junit.Rule(order = 1)
   var activityRule = androidx.test.ext.junit.rules.ActivityScenarioRule(Help_Activity_::class.java)

   /**
    * <p>make screen shots before and after tests. Activated with log settings.</p>
    */
   @get:org.junit.Rule(order = 2)
   val screenshotRule = net.sourceforge.uiq3.test.ScreenshotWatcher()

   private fun Take_Screen_Shot(
      @NotNull Name: String,
      @NotNull Step: String,
      Wait_Before: Long = Second(0.5f)) =
      Take_Screen_Shot(Help_Activity_Test.TAG, Name, Step, Wait_Before)

   @org.junit.Before
   fun Before()
   {
      UIQ3_Log.d(Help_Activity_Test.TAG, "+ Before")

      this.activityRule.scenario.onActivity() { Activity ->
         Before(Activity)
      }

      UIQ3_Log.d(Help_Activity_Test.TAG, "- Before")
      return
   } // Before

   @org.junit.After
   fun After()
   {
      UIQ3_Log.d(Help_Activity_Test.TAG, "+ After")

      this.activityRule.scenario.onActivity() { Activity ->
         After(Activity)
      }

      UIQ3_Log.d(Help_Activity_Test.TAG, "- After")
      return
   } // After

   @org.junit.Test
   fun Test_00_Open()
   {
      UIQ3_Log.d(Help_Activity_Test.TAG, "+ Test_00_Open")

      Take_Screen_Shot(Name = "Test_00_Open", Step = "1")

      UIQ3_Log.d(Help_Activity_Test.TAG, "- Test_00_Open")
      return
   } // Test_00_Open

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_01_Help ()
   {
      UIQ3_Log.d (TAG, "+ Test_01_Help")

      this.activityRule.scenario.onActivity() { Activity ->
         Activity.Open_Page (Activity.Help, "help.html")
         Take_Screen_Shot (Name = "Test_01_Help", Step = "1")
      }

      UIQ3_Log.d (TAG, "- Test_01_Help")
   } // Test_01_Help
} // Help_Activity_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
