/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ********************************************************** }}}1 **********
 *
 */

@file:Suppress("ImplicitThis")

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.*
import net.sourceforge.uiq3.Log as UIQ3_Log
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.IsNot.not
import org.hamcrest.core.IsNull.nullValue
import org.jetbrains.annotations.NotNull

@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.LargeTest
class Main_Activity_Test
{
   companion object
   {
      private val TAG = UIQ3_Log.createTag(Main_Activity_Test::class.java)
   } // companion object

   /**
    * <p>activity under test</p>
    */
   @get:org.junit.Rule(order = 1)
   var activityRule = androidx.test.ext.junit.rules.ActivityScenarioRule(Main_Activity_::class.java)

   /**
    * <p>make screen shots before and after tests. Activated with log settings.</p>
    */
   @get:org.junit.Rule(order = 2)
   val screenshotRule = net.sourceforge.uiq3.test.ScreenshotWatcher()

   private fun Take_Screen_Shot(
      @NotNull Name: String,
      @NotNull Step: String,
      Wait_Before: Long = Second(0.5f)
   ) =
      Take_Screen_Shot(Main_Activity_Test.TAG, Name, Step, Wait_Before)

   private fun SCI9_Math_Test() = SCI9_Math_Test(activityRule)
   private fun Small_Math_Test() = Small_Math_Test(activityRule)
   private fun Switch_To_Calculator() = Switch_To_Calculator(activityRule)
   private fun Switch_To_Landscape() = Switch_To_Landscape(activityRule)
   private fun Switch_To_Menu() = Switch_To_Menu(activityRule)
   private fun Switch_To_Portrait() = Switch_To_Portrait(activityRule)
   private fun Switch_To_Printer() = Switch_To_Printer(activityRule)
   private fun Wait_For_Calculator_Start() = Wait_For_Calculator_Start(activityRule)

   @org.junit.Before
   fun Before()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Before")

      Wait_For_Calculator_Start()

      this.activityRule.scenario.onActivity() { Activity ->
	 Before(Activity)

	 val Registry = androidx.test.espresso.IdlingRegistry.getInstance()

	 Registry.register(Activity)
      }

      On_Calculator(this.activityRule) { _, _, Calculator ->
	 Calculator.resetAll()
      } // On_Calculator

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Before")
      return
   } // Before

   @org.junit.After
   fun After()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ After")

      this.activityRule.scenario.onActivity() { Activity ->
	 After(Activity)

	 val Registry = androidx.test.espresso.IdlingRegistry.getInstance()

	 Registry.unregister(Activity)
      }

      UIQ3_Log.d(Main_Activity_Test.TAG, "- After")
      return
   } // After

   @org.junit.Test
   @androidx.test.filters.SmallTest
   fun Test_00_Open()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_00_Open")

      Small_Math_Test()
      Take_Screen_Shot(TAG, Name = "Test_00_Open", Step = "1")

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_00_Open")
      return
   } // Test_00_Open

   @org.junit.Test
   @androidx.test.filters.SmallTest
   fun Test_01_Open_Main()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_01_Open_Main")

      When("Calculator is shown")
      Switch_To_Calculator()
      Small_Math_Test()
      Take_Screen_Shot(Name = "Test_01_Open_Main", Step = "1")

      Then("„toString can be called")
      activityRule.scenario.onActivity { Activity ->
	 val text = Activity.Calculator().toString()
	 Info(text)
	 assertThat(text, not(nullValue()))
      }

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_01_Open_Main")
      return
   } // Test_01_Open_Main

   @org.junit.Test
   @androidx.test.filters.SmallTest
   fun Test_01_Open_Menu()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_01_Open_Menu")

      When("Menu is shown")
      Switch_To_Menu()
      Take_Screen_Shot(Name = "Test_01_Open_Menu", Step = "1")

      Then("„toString can be called")
      activityRule.scenario.onActivity { Activity ->
	 val text = Activity.Menu.toString()
	 Info(text)
	 assertThat(text, not(nullValue()))
      }

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_01_Open_Menu")
      return
   } // Test_01_Open_Menu

   @org.junit.Test
   @androidx.test.filters.SmallTest
   fun Test_01_Open_Printer()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_01_Open_Printer")

      When("Printer is shown")

      Switch_To_Printer()
      Take_Screen_Shot(Name = "Test_01_Open_Printer", Step = "1")

      Then("„toString can be called")
      activityRule.scenario.onActivity { Activity ->
	 val text = Activity.Printer.toString()
	 Info(text)
	 assertThat(text, not(nullValue()))
      }

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_01_Open_Printer")
      return
   } // Test_01_Open_Printer

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_02_Portrait()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_02_Portrait")

      Switch_To_Portrait()
      Small_Math_Test()
      Take_Screen_Shot(Name = "Test_02_Portrait", Step = "1")

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_02_Portrait")
   } // Test_02_Portrait

   /**
    * Test print button
    */
   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_03_Landscape()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_03_Landscape")

      Switch_To_Landscape()
      SCI9_Math_Test()
      Take_Screen_Shot(Name = "Test_03_Landscape", Step = "1")

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_03_Landscape")
   } // Test_03_Landscape


   @androidx.test.filters.MediumTest
   @org.junit.Test
   fun Test_07_Rotation()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ test_08_Rotation")

      Switch_To_Calculator()
      Small_Math_Test()
      Switch_To_Portrait()
      Take_Screen_Shot(Name = "Main_08_Rotation", Step = "1")

      Switch_To_Landscape()
      Take_Screen_Shot(Main_Activity_Test.TAG, Name = "Main_08_Rotation", Step = "2")

      UIQ3_Log.d(Main_Activity_Test.TAG, "- test_08_Rotation")
   } // test_08_Rotation

   @androidx.test.filters.MediumTest
   @org.junit.Test
   fun Test_08_SCI9()
   {
      UIQ3_Log.d(Main_Activity_Test.TAG, "+ Test_08_SCI9")

      Switch_To_Calculator()
      SCI9_Math_Test()
      Switch_To_Portrait()
      Take_Screen_Shot(Name = "Test_08_SCI9", Step = "1")

      Switch_To_Landscape()
      Take_Screen_Shot(Name = "Test_08_SCI9", Step = "2")

      UIQ3_Log.d(Main_Activity_Test.TAG, "- Test_08_SCI9")
   } // Test_08_SCI9

} // Main_Activity_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
