/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ********************************************************** }}}1 **********
 *
 */

@file:Suppress("unused")

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.Second
import org.jetbrains.annotations.NotNull

/**
 * <p>Test calculator</p>
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@Suppress("PLATFORM_CLASS_MAPPED_TO_KOTLIN")
@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
class Mock_Calculator :
   Calculator
{
   /**
    * <p>factory object</p>
    * @author "Martin Krischik" «krischik@users.sourceforge.net»
    */
   companion object
   {
      private val TAG = net.sourceforge.uiq3.Log.createTag(Mock_Calculator::class.java)
   } // Calculator_Test

   /**
    * create an mock calculator with a mock canvas
    * @param Form canvas
    * @return calculator
    */
   constructor (Form: Mock_Canvas)
   {
      this.reset()

      val Calculator_Thread = Thread(this, "Calculator Thread")

      Calculator_Thread.start()

      synchronized(this)
      {
	 (this as java.lang.Object).wait(Second(0.5f))
      }

      while (!this.isRunning)
      {
	 Thread.sleep(Second(0.5f))
      }

      this.form = Form
   } // constructor

   /**
    *
    * create an mock calculator with a mock canvas and load file into calculator
    * @param File file to load
    * @param Form canvas
    * @return calculator
    */
   constructor (File: String, Form: Mock_Canvas)
   {
      this.reset()

      val Calculator_Thread = Thread(this, "Calculator Thread")

      Calculator_Thread.start()

      synchronized(lock = this)
      {
	 (this as java.lang.Object).wait(Second(0.5f))
      }

      while (this.isRunning)
      {
	 Thread.sleep(Second(0.5f))
      }

      this.form = Form
   } // constructor

   override fun Save_Energy(enable: Boolean)
   {
      return
   } // Save_Energy

   /**
    * <p>display additional informations on the error. this is device and setup dependent.</p>
    * @param Exception Exception to display
    */
   override fun displayException(@NotNull Exception: Throwable)
   {
      net.sourceforge.uiq3.Log.d(TAG, "displayException", arrayOf(Exception));

      net.sourceforge.uiq3.Log.wtf(TAG, "calculator exception", Exception)

      net.sourceforge.uiq3.Log.d(TAG, "displayException");
      return;
   } // displayException

   override fun Load()
   {
      return
   }

   override fun Load(Filename: String)
   {
      return
   }

   override fun Save()
   {
      return
   }

   override fun Save(s: String)
   {
   }

   /**
    * Clear all alpha register. And just calling "Clear" is not enough.
    */
   override fun displayClear()
   {
      return
   }

   /**
    * Display value as formatted text
    *
    * @param Text
    * to be displayed
    */
   override fun Display_Formatted(Text: ShortArray)
   {
      return
   }

   /**
    * * Display Key merge info.
    *
    * @param Text
    * info to be displayed
    */
   override fun Display_Key(Text: ShortArray)
   {
      return
   }

   /**
    * Reset current value Display
    */
   override fun Display_L()
   {
      return
   }

   /**
    * Display texts as dual line message
    *
    * @param Text
    * to be displayed
    */
   override fun Display_Message(Text: ShortArray)
   {
      return
   }
} // Calculator_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
