﻿/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7751 $
 *   $Date: 2021-11-30 16:23:17 +0100 (Di, 30. Nov 2021) $
 *   $Id: Printer_Fragment_Test.kt 7751 2021-11-30 15:23:17Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.Log as UIQ3_Log
import net.sourceforge.uiq3.hp45.Main_Activity_ as Main
import net.sourceforge.uiq3.hp45.R.id as Widget
import net.sourceforge.uiq3.test.*

@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
@org.junit.runner.RunWith(androidx.test.ext.junit.runners.AndroidJUnit4::class)
@androidx.test.filters.LargeTest
class Printer_Fragment_Test
{
   companion object
   {
      private val TAG = UIQ3_Log.createTag(Printer_Fragment_Test::class.java)

   } // companion object

   /**
    * <p>activity under test</p>
    */
   @get:org.junit.Rule(order = 1)
   var activityRule = androidx.test.ext.junit.rules.ActivityScenarioRule(Main::class.java)

   /**
    * <p>make screen shots before and after tests. Activated with log settings.</p>
    */
   @get:org.junit.Rule(order = 2)
   val screenshotRule = net.sourceforge.uiq3.test.ScreenshotWatcher()

   private fun Wait_For_Calculator_Start() = Wait_For_Calculator_Start(activityRule)

   @org.junit.Before
   fun Before()
   {
      UIQ3_Log.d(Printer_Fragment_Test.TAG, "+ Before")

      Wait_For_Calculator_Start(activityRule)

      this.activityRule.scenario.onActivity() { Activity ->
	 Before(Activity)

	 val Registry = androidx.test.espresso.IdlingRegistry.getInstance()

	 Registry.register(Activity)
      }

      UIQ3_Log.d(Printer_Fragment_Test.TAG, "- Before")
      return
   } // Before

   @org.junit.After
   fun After()
   {
      UIQ3_Log.d(Printer_Fragment_Test.TAG, "+ After")

      this.activityRule.scenario.onActivity() { Activity ->
	 After(Activity)

	 val Registry = androidx.test.espresso.IdlingRegistry.getInstance()

	 Registry.unregister(Activity)
      } // On_Calculator

      UIQ3_Log.d(Printer_Fragment_Test.TAG, "- After")
      return
   } // After

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_00_Open()
   {
      UIQ3_Log.d(TAG, "+ Test_00_Open")

      Switch_To_Printer(activityRule)
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_00_Open", Step = "1")

      UIQ3_Log.d(TAG, "- Test_00_Open")
   } // Test_00_Open

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_01_Button_Print_Display()
   {
      UIQ3_Log.d(TAG, "+ Test_01_Button_Print_Display")

      Small_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Display)
      Async_Assert_Printer(activityRule, Expected_Text = " 163.0000      ", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_01_Button_Print_Display", Step = "1")

      UIQ3_Log.d(TAG, "- Test_01_Button_Print_Display")
   } // Test_01_Button_Print_Display

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_01_Button_Print_Register()
   {
      UIQ3_Log.d(TAG, "+ Test_01_Button_Print_Register")

      Small_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Register)
      Async_Assert_Printer(activityRule, Expected_Text = "c / X :  1.630000002  ⁰²", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_01_Button_Print_Register", Step = "1")

      UIQ3_Log.d(TAG, "- Test_01_Button_Print_Register")
   } // Test_01_Button_Print_Register

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_01_Button_Print_Memory()
   {
      UIQ3_Log.d(TAG, "+ Test_01_Button_Print_Memory")

      Small_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Memory)
      Async_Assert_Printer(activityRule, Expected_Text = "LASTx :  1.620000002  ⁰²", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_01_Button_Print_Memory", Step = "1")

      UIQ3_Log.d(TAG, "- Test_01_Button_Print_Memory")
   } // Test_01_Button_Print_Memory

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_02_Portrait()
   {
      UIQ3_Log.d(TAG, "+ Test_02_Portrait")

      Switch_To_Portrait(activityRule)
      SCI9_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Display)
      Async_Assert_Printer(activityRule, Expected_Text = " 1.630000002  02", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_02_Portrait", Step = "1")

      UIQ3_Log.d(TAG, "- Test_02_Portrait")
   } // Test_02_Portrait

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_03_Landscape()
   {
      UIQ3_Log.d(TAG, "+ Test_03_Landscape")

      Switch_To_Landscape(activityRule)
      SCI9_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Display)
      Async_Assert_Printer(activityRule, Expected_Text = " 1.630000002  02", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_03_Landscape", Step = "1")

      UIQ3_Log.d(TAG, "- Test_03_Landscape")
   } // Test_03_Landscape

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_04_ErrorDisplay()
   {
      UIQ3_Log.d(TAG, "+ Test_04_ErrorDisplay")

      On_Calculator(activityRule) { Activity, _, Calculator ->

	 Given("Error messages shown on printer")

	 Preferences_Activity.Error_Handler(
	    Activity,
	    net.sourceforge.uiq3.calculator.Preferences_Activity.Error_Handle_Type.Printer
	 )

	 When("the calculator handles an error")

	 Calculator.displayException(Exception("Test_04_ErrorDisplay"))
      } // On_Calculator

      Then("The error is displayed on the printer")

      Async_Assert_Printer(activityRule, Expected_Text = "Test_04_ErrorDisplay", Timeout = Second(3.0f))

      Switch_To_Printer(activityRule)
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_04_ErrorDisplay", Step = "1")

      UIQ3_Log.d(TAG, "- Test_04_ErrorDisplay")
   } // Test_04_ErrorDisplay

   @androidx.test.filters.SmallTest
   @org.junit.Test
   fun Test_06_SaveState()
   {
      UIQ3_Log.d(TAG, "+ Test_06_SaveState")

      Given("a printout generated in portrait orientation")

      Switch_To_Portrait(activityRule)
      SCI9_Math_Test(activityRule)
      Switch_To_Printer(activityRule)
      Click_Buttons(Widget.Button_Clear, Widget.Button_Print_Display)
      Async_Assert_Printer(activityRule, Expected_Text = " 1.630000002  02", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_06_SaveState", Step = "1")

      When("rotated to to landscape")

      Switch_To_Landscape(activityRule)

      Then("printout was saved in state")

      Async_Assert_Printer(activityRule, Expected_Text = " 1.630000002  02", Timeout = Second(1.5f))
      Take_Screen_Shot(Printer_Fragment_Test.TAG, Name = "Test_06_SaveState", Step = "2")

      UIQ3_Log.d(TAG, "- Test_06_SaveState")
   } // Test_06_SaveState
} // Printer_Fragment_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
