/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.test

import androidx.test.ext.junit.rules.ActivityScenarioRule as Rule
import net.sourceforge.uiq3.calculator.Preferences_Activity.Error_Handle_Type as Error_Handle
import net.sourceforge.uiq3.hp45.Application_ as Application
import net.sourceforge.uiq3.hp45.Calculator_Service_ as Service
import net.sourceforge.uiq3.hp45.Main_Activity_ as Main
import net.sourceforge.uiq3.hp45.Preferences_Activity as Preferences
import net.sourceforge.uiq3.hp45.R.id
import net.sourceforge.uiq3.Log as UIQ3_Log

typealias Activity_Service_Statements = (
   net.sourceforge.uiq3.hp45.Main_Activity,
   net.sourceforge.uiq3.calculator.Service) -> Unit

typealias Calculator_Statements = (
   net.sourceforge.uiq3.hp45.Main_Activity,
   net.sourceforge.uiq3.calculator.Service,
   net.sourceforge.uiq3.hp45.Calculator) -> Unit

typealias Calculator_Service_Statements = (
   net.sourceforge.uiq3.hp45.Calculator_Service,
   net.sourceforge.uiq3.hp45.Calculator) -> Unit

typealias Service_Statements = (
   net.sourceforge.uiq3.hp45.Calculator_Service,) -> Unit

object HP45_Utilities
{
   /**
    * Logging tag
    */
   val TAG = net.sourceforge.uiq3.Log.createTag(javaClass)
} // HP45_Utilities

internal fun Before(Context: android.content.Context)
{
   UIQ3_Log.d(HP45_Utilities.TAG, "+ Before")

   // since the new test framework doesn't execute alphabetically we can't have a
   // handy reset test any more and all reset work needs to be done in After.
   //
   Preferences.Error_Handler(Context, Error_Handle.All)

   UIQ3_Log.d(HP45_Utilities.TAG, "- Before")
   return
} // Before

internal fun After(Context: android.content.Context)
{
   UIQ3_Log.d(HP45_Utilities.TAG, "+ After")

   // since the new test framework doesn't execute alphabetically we can't have a
   // handy reset test any more and all reset work needs to be done in After.
   //
   Preferences.Error_Handler(Context, Error_Handle.All)

   UIQ3_Log.d(HP45_Utilities.TAG, "- After")
   return
} // After

/**
 *
 * @param activityRule   rule to access the activity to test with
 * @param Operation      statements to execute
 */
internal fun On_Service(activityRule: Rule<Main>, Operation: Activity_Service_Statements)
{
   activityRule.scenario.onActivity { Activity ->
      val Service = Activity.Service()

      if (Service is net.sourceforge.uiq3.hp45.Calculator_Service)
      {
	 Operation(Activity, Service)
      }
      else
      {
	 org.junit.Assert.fail("Main_Activity_Test has no service or service is the wrong type.")
      } // if
   } // onActivity

   return
} // On_Service

/**
 * <p>execute statements with calculator</p>
 *
 * @param serviceRule    rule to access the service test driver to test with
 * @param Operation      statements to execute
 */
internal fun On_Service(
   serviceRule: androidx.test.rule.ServiceTestRule,
   Operation: Service_Statements)
{
   val Application = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application>()
   val Service_Intent = android.content.Intent(Application, Service::class.java)

   serviceRule.startService(Service_Intent)

   val Binder: android.os.IBinder = serviceRule.bindService(Service_Intent)

   if (Binder is net.sourceforge.uiq3.calculator.IBinder<*>)
   {
      val Service = Binder.service

      if (Service is Service)
      {
	 Service.createCalculator()

	 Operation(Service)
      }
      else
      {
	 org.junit.Assert.fail("Binder has no service or service is the wrong type.")
      } // if
   }
   else
   {
      org.junit.Assert.fail("service binder was not set")
   } // if

   return
} // On_Tape


/**
 * <p>execute statements with calculator</p>
 *
 * @param activityRule    rule to access the activity to test with
 * @param Operation      statements to execute
 */
internal fun On_Calculator(activityRule: Rule<Main>, Operation: Calculator_Statements)
{
   activityRule.scenario.onActivity { Activity ->
      val Service = Activity.Service()

      if (Service is net.sourceforge.uiq3.hp45.Calculator_Service)
      {
	 val Calculator = Service.getRPN45()

	 if (Calculator is net.sourceforge.uiq3.hp45.Droid_Calculator)
	 {
	    Operation(Activity, Service, Calculator)
	 }
	 else
	 {
	    org.junit.Assert.fail("Service has no calculator or calculator is the wont type.")
	 } // if
      }
      else
      {
	 org.junit.Assert.fail("Main_Activity_Test has no service or service is the wrong type.")
      } // if
   } // onActivity

   return
} // On_Calculator

/**
 * <p>execute statements with calculator</p>
 *
 * @param serviceRule    rule to access the service test driver to test with
 * @param Operation      statements to execute
 */
internal fun On_Calculator(serviceRule: androidx.test.rule.ServiceTestRule, Operation: Calculator_Service_Statements)
{
   val Application = androidx.test.core.app.ApplicationProvider.getApplicationContext<Application>()
   val Service_Intent = android.content.Intent(Application, Service::class.java)

   serviceRule.startService(Service_Intent)

   val Binder: android.os.IBinder = serviceRule.bindService(Service_Intent)

   if (Binder is net.sourceforge.uiq3.calculator.IBinder<*>)
   {
      val Service = Binder.service

      if (Service is Service)
      {
	 Service.createCalculator()

	 val Calculator = Service.getRPN45()

	 if (Calculator is net.sourceforge.uiq3.hp45.Droid_Calculator)
	 {
	    Operation(Service, Calculator)
	 }
	 else
	 {
	    org.junit.Assert.fail("Service has no calculator or calculator is the wont type.")
	 } // if
      }
      else
      {
	 org.junit.Assert.fail("Binder has no service or service is the wrong type.")
      } // if
   }
   else
   {
      org.junit.Assert.fail("service binder was not set")
   } // if

   return
} // On_Calculator

/**
 * Wait until all keystrokes from form have been read — and then an extra 50ms.
 *
 * @param activityRule    rule to access the activity to test with
 * @param Time_Out        Maximum time to wait for the keystroked to be consumed
 * @param Final_Wait      a little extra time for things to settle down.
 * @return
 */
internal fun Wait_For_Execution_Finished(
   activityRule: Rule<Main>,
   Time_Out: Long = Second(10.0f),
   Final_Wait: Long = Second(0.05f))
{
   UIQ3_Log.d(
      HP45_Utilities.TAG,
      "+ Wait_For_Execution_Finished" +
	 "\n> activityRule                   = %1\$-16s" +
	 "\n> Time_Out                       = %2\$d" +
	 "\n> Final_Wait                     = %3\$d",
      activityRule, Time_Out, Final_Wait)

   activityRule.scenario.onActivity { Activity ->
      Wait_Loop@
      for (i in 0..Time_Out step Time_Out / 10)
      {
	 if (!Activity.isReadKeyAvailable) break@Wait_Loop
      } // for
   } // On_Calculator

   UIQ3_Log.d(HP45_Utilities.TAG, "- Wait_For_Execution_Finished")
} // Wait_For_Execution_Finished

/**
 * <p>Wait for the calculator to start. Called in Before methods.</p>
 *
 * @param activityRule rule to access the main activity.
 */
internal fun Wait_For_Calculator_Start(activityRule: Rule<Main>)
{
   On_Service(activityRule) { _, Service ->
      Async_Assert_True(
	 Message = { "Calculator not created" },
	 Condition = { Service.isCalculatorStarted },
	 Timeout = Second(10.0f),
	 Wait = Second(0.5f))
   } // On_Service

   return
} // Wait_For_Calculator_Start

/**
 * wait for the display to show two specific lines
 *
 * @param activityRule    rule to access the activity to test with
 * @param Expected          expected content of line 0
 */
internal fun Async_Assert_Display(
   activityRule: Rule<Main>,
   Expected: String,
   Timeout: Long = Second(10.0f),
   Wait: Long = Second(0.5f))
{
   UIQ3_Log.d(
      HP45_Utilities.TAG,
      "+ Async_Assert_Display" +
	 "\n> activityRule = %1\$.32s" +
	 "\n> Expected     = „%2\$s“" +
	 "\n> Timeout      = %3\$d" +
	 "\n> Wait         = %4\$d",
      activityRule, Expected, Timeout, Wait)

   Wait_For_Execution_Finished(activityRule)

   Async_Assert_True(
      Message =
      {
	 var Actual = ""

	 activityRule.scenario.onActivity { Activity ->
	    Actual = Activity.getDisplayText()
	 }

	 "Display showed „" + Actual + "” instead of „" + Expected + "“."
      },
      Condition =
      {
	 var Actual = ""

	 activityRule.scenario.onActivity { Activity ->
	    Actual = Activity.getDisplayText()
	 }

	 Actual == Expected
      }, Timeout, Wait)

   UIQ3_Log.d(HP45_Utilities.TAG, "- Async_Assert_Display")
} // Async_Assert_Display

/**
 * wait for the printer to contain the given text
 *
 * @param activityRule    rule to access the activity to test with
 * @param Expected_Text          expected content of line 0
 */
internal fun Async_Assert_Printer(
   activityRule: Rule<Main>,
   Expected_Text: String,
   Timeout: Long = Second(10.0f),
   Wait: Long = Second(0.5f))
{
   UIQ3_Log.d(
      HP45_Utilities.TAG,
      "+ Async_Assert_Display" +
	 "\n> activityRule  = %1\$.32s" +
	 "\n> Expected_Text = „%2\$s“" +
	 "\n> Timeout       = %3\$d" +
	 "\n> Wait          = %4\$d",
      activityRule, Expected_Text, Timeout, Wait)

   Wait_For_Execution_Finished(activityRule)

   Async_Assert_True(
      Message =
      {
	 var Actual = ""

	 activityRule.scenario.onActivity { Activity ->
	    Actual = Activity.Printout()
	 }

	 "Printout „" + Actual + "” does not contain „" + Expected_Text + "”."
      },
      Condition =
      {
	 var Actual = ""

	 activityRule.scenario.onActivity { Activity ->
	    Actual = Activity.Printout()
	 }

	 Actual.contains(Expected_Text)
      }, Timeout, Wait)

   UIQ3_Log.d(HP45_Utilities.TAG, "- Async_Assert_Display")
} // Async_Assert_Display

/**
 * Switch to Calculator page to perform Calculator tests.
 */
internal fun Switch_To_Calculator(activityRule: Rule<Main>) =
   activityRule.scenario.onActivity { Activity ->
      Activity.Switch_To_Page(net.sourceforge.uiq3.hp45.IMain_Activity.Page_Type.Calculator)
   }

/**
 * Switch to Calculator page to perform Calculator tests.
 */
internal fun Switch_To_Menu(activityRule: Rule<Main>) =
   activityRule.scenario.onActivity { Activity ->
      Activity.Switch_To_Page(net.sourceforge.uiq3.hp45.IMain_Activity.Page_Type.Menu)
   }

/**
 * Switch to Calculator page to perform Calculator tests.
 */
internal fun Switch_To_Portrait(activityRule: Rule<*>)
{
   activityRule.scenario.onActivity { Activity ->
      Activity.setRequestedOrientation(android.content.pm.ActivityInfo.SCREEN_ORIENTATION_PORTRAIT)
   }

   Thread.sleep(Second(0.5f))
   return
}

/**
 * Switch to Calculator page to perform Calculator tests.
 */
internal fun Switch_To_Landscape(activityRule: Rule<*>)
{
   activityRule.scenario.onActivity { Activity ->
      Activity.setRequestedOrientation(android.content.pm.ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE)
   }

   Thread.sleep(Second(0.5f))
   return
}

/**
 * Switch to Calculator page to perform Calculator tests.
 */
internal fun Switch_To_Printer(activityRule: Rule<Main>)
{
   UIQ3_Log.d(HP45_Utilities.TAG, "+ Switch_To_Printer")

   activityRule.scenario.onActivity { Activity ->
      Activity.Switch_To_Page(net.sourceforge.uiq3.hp45.IMain_Activity.Page_Type.Printer)
   }

   Async_Assert_True(
      Message =
      {
	 "Printer fragment not available"
      },
      Condition =
      {
	 var Visibel = false

	 activityRule.scenario.onActivity { Activity ->

	    val Printer = Activity.Printer()

	    Visibel = Printer != null && Printer.Test_Button_Clear()
	 }

	 Visibel
      },
      Timeout = 5000)

   UIQ3_Log.d(HP45_Utilities.TAG, "- Switch_To_Printer")
}  // Switch_To_Printer

/**
 * Quick test to see if the calculator is working.
 */
internal fun Small_Math_Test(activityRule: Rule<Main>)
{
   UIQ3_Log.d(HP45_Utilities.TAG, "+ Small_Math_Test" + "\n> activityRule                = %1\$.32s", activityRule)

   Scenario("Small Math Test")
   {
      When("Button to calculate 1+2×3⁴ are clicked")

      Click_Buttons(
	 id.Button_CLx,
	 id.Button_FIX_SCI,
	 id.Button_4,
	 id.Button_CLx,
	 id.Button_1,
	 id.Button_ENTER,
	 id.Button_2,
	 id.Button_ENTER,
	 id.Button_3,
	 id.Button_ENTER,
	 id.Button_4,
	 id.Button_GOLD,
	 id.Button_1x,
	 id.Button_Mult,
	 id.Button_Plus)

      Then("„163“ is displayed")

      Async_Assert_Display(activityRule, Expected = " 163.0000      ")
   } // Scenario

   UIQ3_Log.d(HP45_Utilities.TAG, "- Small_Math_Test")
   return
} // Small_Math_Test

/**
 * Quick test to see if the calculator is working.
 */
internal fun SCI9_Math_Test(activityRule: Rule<Main>)
{
   UIQ3_Log.d(HP45_Utilities.TAG, "+ SCI9_Math_Test" + "\n> activityRule                = %1\$s", activityRule)

   Scenario("Small Math Test")
   {
      When("Button to calculate 1+2×3⁴ are clicked")

      Click_Buttons(
	 id.Button_CLx,
	 id.Button_GOLD,
	 id.Button_FIX_SCI,
	 id.Button_9,
	 id.Button_1,
	 id.Button_ENTER,
	 id.Button_2,
	 id.Button_ENTER,
	 id.Button_3,
	 id.Button_ENTER,
	 id.Button_4,
	 id.Button_GOLD,
	 id.Button_1x,
	 id.Button_Mult,
	 id.Button_Plus)

      Then("„ 1.630000002  02“ is displayed")

      Async_Assert_Display(activityRule, Expected = " 1.630000002  02")
   } // Scenario

   UIQ3_Log.d(HP45_Utilities.TAG, "- SCI9_Math_Test")
   return
} // SCI9_Math_Test

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
