/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7691 $
 *   $Date: 2021-09-19 11:36:55 +0200 (So, 19. Sep 2021) $
 *   $Id: Application.java 7691 2021-09-19 09:36:55Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import android.content.Context;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * <p>
 * Calculator Application to handle application startup und shutdown.
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @version 1.0 $Revision: 7691 $
 * @since 1.0
 */
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EApplication
public class Application
   extends
      net.sourceforge.uiq3.calculator.Application
{
   private static class On_Data_Changed
      implements
         android.content.SharedPreferences.OnSharedPreferenceChangeListener
   {
      @SuppressWarnings ("FieldNotUsedInToString")
      private final java.lang.ref.WeakReference <Application> owner;

      private On_Data_Changed (final Application owner)
      {
         this.owner = new java.lang.ref.WeakReference <> (owner);
      }

      /**
       * @param sharedPreferences
       *           the shared preferences.
       * @param key
       *           key of the element changed
       */
      @Override
      public void onSharedPreferenceChanged (
         final android.content.SharedPreferences sharedPreferences,
         final String key)
      {
         // net.sourceforge.uiq3.Log.d (Application.TAG, "+ onSharedPreferenceChanged");
         // net.sourceforge.uiq3.Log.v (Application.TAG, "> sharedPreferences : " + sharedPreferences);
         // net.sourceforge.uiq3.Log.v (Application.TAG, "> key               : " + key);

         final Application application = this.owner.get ();
         if (application != null)
         {
            Preferences_Activity.Data_Changed (application.getApplicationContext ());
         }

         // net.sourceforge.uiq3.Log.d (Application.TAG, "- onSharedPreferenceChanged");
         return;
      } // onSharedPreferenceChanged

      @org.jetbrains.annotations.TestOnly
      @Override
      public String toString ()
      {
         return Application.TAG + "{super=" + super.toString () + ", owner=" + this.owner + '}';
      }
   } // On_Data_Changed

   /**
    * <p>
    * Logging tag
    * </p>
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Application.class);
   /**
    * <p>
    * listener handling setting changes.
    * </p>
    */
   @Nullable
   private Application.On_Data_Changed changeListener;
   /**
    * <p>
    * shared setting for the application.
    * </p>
    */
   @Nullable
   private android.content.SharedPreferences settings;

   /**
    * <p>
    * application start up
    * </p>
    */
   @Override
   public synchronized void onCreate ()
   {
      super.onCreate ();
      net.sourceforge.uiq3.Log.d (Application.TAG, "+ onCreate");

      final android.app.ActivityManager activityManager = (android.app.ActivityManager) this.getSystemService (
         Context.ACTIVITY_SERVICE);

      if (this.changeListener == null)
      {
         // net.sourceforge.uiq3.Log.v (Application.TAG, "> Create preference change listener");

         this.changeListener = new Application.On_Data_Changed (this);
      } // if

      if (this.settings == null)
      {
         // net.sourceforge.uiq3.Log.v (Application.TAG, "> Create or get preferences");

         this.settings = android.preference.PreferenceManager.getDefaultSharedPreferences (
            this.getApplicationContext ());

         this.settings.registerOnSharedPreferenceChangeListener (this.changeListener);

         android.preference.PreferenceManager.setDefaultValues (
            /* contect   => */this.getApplicationContext (),
            /* resId     => */R.xml.preferences,
            /* readAgain => */false);
      } // if

      net.sourceforge.uiq3.Log.d (Application.TAG, "- onCreate");
      return;
   } // onCreate

   /**
    * <p>
    * application shut down
    * </p>
    */
   @Override
   public synchronized void onTerminate ()
   {
      super.onTerminate ();
      net.sourceforge.uiq3.Log.d (Application.TAG, "+ onTerminate");

      if ((this.settings != null) && (this.changeListener != null))
      {
         this.settings.unregisterOnSharedPreferenceChangeListener (this.changeListener);
      } // if

      this.settings = null;
      this.changeListener = null;

      net.sourceforge.uiq3.Log.d (Application.TAG, "- onTerminate");
      return;
   } // onTerminate

   /**
    * <p>
    * set preferences to values suitable for unit testing
    * </p>
    */
   @SuppressWarnings ({
      "FeatureEnvy" })
   @org.jetbrains.annotations.TestOnly
   void Set_Test_Preferences ()
   {
      net.sourceforge.uiq3.Log.d (Application.TAG, "+ Test_Set_Preferences");

      @NotNull
      final IPreferences_ Preferences = Preferences_Activity.Preferences (this);

      net.sourceforge.uiq3.Log.i (Application.TAG, "Preferences");
      net.sourceforge.uiq3.Log.i (Application.TAG, "  Feedback_Vibrate        = " + Preferences.Feedback_Vibrate ());

      net.sourceforge.uiq3.Log.d (Application.TAG, "- Set_Test_Preferences");
      return;
   } // Set_Test_Preferences

   @org.jetbrains.annotations.TestOnly
   @Override
   @NotNull
   public String toString ()
   {
      return Application.TAG + "{super=" + super.toString () + ", changeListener=" + this.changeListener +
         ", settings=" + this.settings + '}';
   }
} // Application

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
