/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7700 $
 *   $Date: 2021-09-28 11:21:12 +0200 (Di, 28. Sep 2021) $
 *   $Id: Calculator_Button_2.java 7700 2021-09-28 09:21:12Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * <p>
 * A Button to control a NumberPickerDialog
 * </p>
 *
 * <pre>
 * ╔═══════════╗
 * ║┌─────────┐║paddingTop
 * ║│╔═══════╗│║topMargin
 * ║│║ Shift ║│║
 * ║│╚═══════╝│║bottomMargin
 * ╟┼─────────┼╢Split
 * ║│╔═══════╗│║topMargin
 * ║│║Button ║│║
 * ║│╚═══════╝│║bottomMargin
 * ║└─────────┘║paddingBottom
 * ╚═══════════╝
 * </pre>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @version 1.0 $Revision: 7700 $
 * @since 1.0
 */
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EViewGroup (resName = "calculator_button_2")
public class Calculator_Button_2
   extends
   android.view.ViewGroup
   implements
   net.sourceforge.uiq3.calculator.ICalculator_Button
{
   /**
    * <p>
    * default text colour for the 2ndF text
    * </p>
    */
   private static boolean Default_Button_Bold;
   /**
    * <p>
    * default values have been initialized.
    * </p>
    */
   private static final java.util.concurrent.atomic.AtomicBoolean Default_Initialized =
      new java.util.concurrent.atomic.AtomicBoolean (false);
   /**
    * <p>
    * default text colour for the shift text
    * </p>
    */
   private static int Default_Shift_Color;
   /**
    * <p>
    * default button text size
    * </p>
    */
   private static float Default_Textsize;
   /**
    * <p>
    * TAG as class name for logging
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = net.sourceforge.uiq3.Log.createTag (Calculator_Button_2.class);
   } // static

   /**
    * <p>
    * Initialise default values
    * </p>
    */
   private static void Init_Defaults (final android.content.Context context)
   {
      if (Calculator_Button_2.Default_Initialized.getAndSet (true))
      {
	 final android.content.res.Resources Resources = context.getResources ();

	 Calculator_Button_2.Default_Button_Bold = Resources.getBoolean (R.bool.Button_Bold);
	 Calculator_Button_2.Default_Shift_Color = Resources.getColor (R.color.Inv_Text);
	 Calculator_Button_2.Default_Textsize = Resources.getDimension (R.dimen.Button_Type_5_Textsize);

	 // net.sourceforge.uiq3.Log.v (
	 // Calculator_Button_2.TAG, "> Default_Textsize       = " + Default_Textsize);
	 // net.sourceforge.uiq3.Log.v (
	 // Calculator_Button_2.TAG, "> Default_Shift_Color    = " + Default_Shift_Color);
      } // if
      return;
   } // Init_Defaults

   @org.androidannotations.annotations.res.StringRes (resName = "Main_Description")
   protected String Main_Function;
   @org.androidannotations.annotations.res.StringRes (resName = "Gold_Description")
   protected String Gold_Description;
   /**
    * Button. Not null when view has been initialised.
    */
   @org.androidannotations.annotations.ViewById (resName = "Button")
   @NotNull
   android.widget.Button Button;
   /**
    * <p>
    * Background for button
    * </p>
    */
   private final int Button_Background;
   /**
    * <p>
    * Button text should be displayed in bold
    * </p>
    */
   private final boolean Button_Bold;
   /**
    * <p>
    * button height. Will be measured when doMeasure is called for the first time
    * </p>
    */
   private int Button_Height;
   /**
    * <p>
    * text to display on button
    * </p>
    */
   private final String Button_Text;
   /**
    * <p>
    * Text colour for button
    * </p>
    */
   private final int Button_Text_Color;
   /**
    * <p>
    * text size for button
    * </p>
    */
   private final float Button_Text_Size;
   /**
    * <p>
    * button width. Will be measured when doMeasure is called for the first time
    * </p>
    */
   private int Button_Width;
   /**
    * shift text text. Not null when view has been initialised.
    */
   @org.androidannotations.annotations.ViewById (resName = "Shift")
   @NotNull
   android.widget.TextView Shift;
   /**
    * <p>
    * background drawable for shift text
    * </p>
    */
   private final int Shift_Background;
   /**
    * <p>
    * text colour for shift text
    * </p>
    */
   private final int Shift_Color;
   /**
    * <p>
    * shift height. Will be measured when doMeasure is called for the first time
    * </p>
    */
   private int Shift_Height;
   /**
    * <p>
    * string to display for shift text
    * </p>
    */
   private final String Shift_Text;
   /**
    * <p>
    * button width. Will be measured when doMeasure is called for the first time
    * </p>
    */
   private int Shift_Width;

   /**
    * <p>
    * Simple constructor to use when creating a view from code.
    * </p>
    *
    * @param context
    *    The Context the view is running in, through which it can access the current theme, resources, etc.
    */
   public Calculator_Button_2 (@NotNull final android.content.Context context)
   {
      this (context, null);
   } // Calculator_Button_2

   /**
    * <p>
    * Constructor that is called when inflating a view from XML. This is called when a view is being constructed from an
    * XML file, supplying attributes that were specified in the XML file. This version uses a default style of 0, so the
    * only attribute values applied are those in the Context's Theme and the given AttributeSet.
    * </p>
    * <p>
    * The method onFinishInflate() will be called after all children have been added.
    * </p>
    *
    * @param context
    *    The Context the view is running in, through which it can access the current theme, resources, etc.
    * @param attrs
    *    The attributes of the XML tag that is inflating the view.
    */
   public Calculator_Button_2 (
      @NotNull final android.content.Context context,
      @Nullable final android.util.AttributeSet attrs)
   {
      this (context, attrs, 0);
   } // Calculator_Button_2

   /**
    * <p>
    * Perform inflation from XML and apply a class-specific base style. This constructor of View allows subclasses to
    * use their own base style when they are inflating. For example, a Button class's constructor would call this
    * version of the super class constructor and supply R.attr.buttonStyle for defStyle; this allows the theme's button
    * style to modify all of the base view attributes (in particular its background) as well as the Button class's
    * attributes.
    * </p>
    *
    * @param context
    *    The Context the view is running in, through which it can access the current theme, resources, etc.
    * @param attrs
    *    The attributes of the XML tag that is inflating the view.
    * @param defStyle
    *    The default style to apply to this view. If 0, no style will be applied (beyond what is included in the theme).
    *    This may either be an attribute resource, whose value will be retrieved from the current theme, or an explicit
    *    style resource.
    */
   public Calculator_Button_2 (
      @NotNull final android.content.Context context,
      @Nullable final android.util.AttributeSet attrs,
      final int defStyle)
   {
      super (context, attrs, defStyle);
      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "+ Calculator_Button_2");
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> context    : " + context);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> attrs      : " + attrs);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> defStyle   : " + defStyle);

      Calculator_Button_2.Init_Defaults (context);

      final android.content.res.TypedArray Typed_Array = context.obtainStyledAttributes (
	 /* set          => */attrs,
	 /* attrs        => */R.styleable.Calculator_Button,
	 /* defStyleAttr => */defStyle,
	 /* defStyleRes  => */R.style.Calculator_Button);

      this.Button_Background = Typed_Array.getResourceId (R.styleable.Calculator_Button_Button_Background, 0);
      this.Button_Bold = Typed_Array.getBoolean (
	 R.styleable.Calculator_Button_Button_Bold,
	 Calculator_Button_2.Default_Button_Bold);
      this.Button_Text = Typed_Array.getString (R.styleable.Calculator_Button_Button_Text);
      this.Button_Text_Color = Typed_Array.getColor (
	 R.styleable.Calculator_Button_Button_Color,
	 android.graphics.Color.BLACK);
      this.Button_Text_Size = Typed_Array.getDimension (
	 R.styleable.Calculator_Button_Button_Size,
	 Calculator_Button_2.Default_Textsize);
      this.Button_Width = 0;
      this.Button_Height = 0;

      this.Shift_Background = Typed_Array.getResourceId (R.styleable.Calculator_Button_Shift_Background, 0);
      this.Shift_Color = Typed_Array.getColor (
	 R.styleable.Calculator_Button_Shift_Color,
	 Calculator_Button_2.Default_Shift_Color);
      this.Shift_Text = Typed_Array.getString (R.styleable.Calculator_Button_Shift_Text);
      this.Shift_Height = 0;
      this.Shift_Width = 0;

      Typed_Array.recycle ();

      this.setClickable (true);
      // this.setOrientation (android.widget.LinearLayout.VERTICAL);

      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Alpha_Background       = " + this.Alpha_Background);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Alpha_Color            = " + this.Alpha_Color);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Alpha_Text             = “" + this.Alpha_Text + "”");
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Button_Background      = " + this.Button_Background);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Button_Color           = " + this.Button_Color);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Button_Size            = " + this.Button_Size);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Button_Text            = “" + this.Button_Text + "”");
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Hex_Background         = " + this.Hex_Background);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Hex_Color              = " + this.Hex_Color);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Hex_Text               = “" + this.Hex_Text + "”");
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> SecondF_Background     = " + this.SecondF_Background);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> SecondF_Color          = " + this.SecondF_Color);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> SecondF_Text           = “" + this.SecondF_Text + "”");
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Shift_Background       = " + this.Shift_Background);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Shift_Color            = " + this.Shift_Color);
      // net.sourceforge.uiq3.Log.v (
      // Calculator_Button_2.TAG, "> Shift_Text             = “" + this.Shift_Text + "”");

      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "- Calculator_Button_2");
      return;
   } // Calculator_Button_2

   /**
    * <p>
    * update the text displayed on the button.
    * </p>
    */
   @android.annotation.SuppressLint ("NewApi")
   @TestOnly
   @org.androidannotations.annotations.AfterViews
   protected void After_Views ()
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "+ After_Views");
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Alpha       = " + Alpha);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Button      = " + Button);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Button Size = " + Button.getTextSize ());
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Hex         = " + Hex);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> SecondF     = " + SecondF);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Shift       = " + Shift);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Shift Size  = " + Shift.getTextSize ());

      this.Button.setBackgroundResource (this.Button_Background);
      this.Button.setText (this.Button_Text);
      this.Button.setContentDescription (Main_Function + " " + this.Button_Text);
      this.Button.setTextColor (this.Button_Text_Color);
      this.Button.setTextSize (android.util.TypedValue.COMPLEX_UNIT_PX, this.Button_Text_Size);

      this.Shift.setBackgroundResource (this.Shift_Background);
      this.Shift.setText (this.Shift_Text);
      this.Shift.setContentDescription (Gold_Description + " " + this.Shift_Text);
      this.Shift.setTextColor (this.Shift_Color);

      this.setContentDescription (
	 Main_Function + " " + this.Button_Text + ", " + Gold_Description + " " + this.Shift_Text);

      final android.graphics.Typeface typefaceCondensed = net.sourceforge.uiq3.Fonts.dejavusans_condensed (getContext ());

      this.Shift.setTypeface (typefaceCondensed);

      if (this.Button_Bold)
      {
	 final android.graphics.Typeface typefaceNormal = net.sourceforge.uiq3.Fonts.dejavusans_bold (getContext ());
	 this.Button.setTypeface (typefaceNormal, android.graphics.Typeface.BOLD);
      }
      else
      {
	 final android.graphics.Typeface typefaceNormal = net.sourceforge.uiq3.Fonts.dejavusans (getContext ());
	 this.Button.setTypeface (typefaceNormal, android.graphics.Typeface.NORMAL);
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "- After_Views");
   } // After_Views

   /**
    * <p>
    * get the actual button
    * </p>
    *
    * @return the button
    */
   @Override
   @NotNull
   public android.widget.Button getButton ()
   {
      return this.Button;
   }

   /**
    * Returns a set of default layout parameters. These parameters are requested when the View passed to
    * {@link android.view.ViewGroup#addView(android.view.View)} has no layout parameters already set. If null is
    * returned, an exception is thrown from addView.
    *
    * @return a set of default layout parameters or null
    */
   @Override
   protected LayoutParams generateDefaultLayoutParams ()
   {
      return new MarginLayoutParams (LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
   }

   /**
    * Returns a new set of layout parameters based on the supplied attributes set.
    *
    * @param attrs
    *    the attributes to build the layout parameters from
    * @return an instance of {@link android.view.ViewGroup.LayoutParams} or one of its descendants
    */
   @Override
   public LayoutParams generateLayoutParams (final android.util.AttributeSet attrs)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Button_3.TAG, "+ generateLayoutParams");
      // net.sourceforge.uiq3.Log.v (Calculator_Button_3.TAG, "> attrs      : " + attrs);
      // net.sourceforge.uiq3.Log.d (Calculator_Button_3.TAG, "- generateLayoutParams");

      return new MarginLayoutParams (this.getContext (), attrs);
   }

   /**
    * Returns a safe set of layout parameters based on the supplied layout params. When a ViewGroup is passed a View
    * whose layout params do not pass the test of {@link #checkLayoutParams(android.view.ViewGroup.LayoutParams)}, this
    * method is invoked. This method should return a new set of layout params suitable for this ViewGroup, possibly by
    * copying the appropriate attributes from the specified set of layout params.
    *
    * @param p
    *    The layout parameters to convert into a suitable set of layout parameters for this ViewGroup.
    * @return an instance of {@link android.view.ViewGroup.LayoutParams} or one of its descendants
    */
   @Override
   protected LayoutParams generateLayoutParams (final LayoutParams p)
   {
      return new MarginLayoutParams (p);
   }

   /**
    * Called from layout when this view should assign a size and position to each of its children. Derived classes with
    * children should override this method and call layout on each of their children.
    *
    * <pre>
    * ╔═══════════╗
    * ║┌─────────┐║paddingTop
    * ║│╔═══════╗│║topMargin
    * ║│║ Shift ║│║
    * ║│╚═══════╝│║bottomMargin
    * ╟┼─────────┼╢Split
    * ║│╔═══════╗│║topMargin
    * ║│║Button ║│║
    * ║│╚═══════╝│║bottomMargin
    * ║└─────────┘║paddingBottom
    * ╚═══════════╝
    * </pre>
    *
    * @param changed
    *    This is a new size or position for this view
    * @param left
    *    Left position, relative to parent
    * @param top
    *    Top position, relative to parent
    * @param right
    *    Right position, relative to parent
    * @param bottom
    *    Bottom position, relative to parent
    */
   @SuppressWarnings ("FeatureEnvy")
   @Override
   protected void onLayout (final boolean changed, final int left, final int top, final int right, final int bottom)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "+ onLayout");
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> changed        = " + changed);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> left           = " + left);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> top            = " + top);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> right          = " + right);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> bottom         = " + bottom);

      // paddings to be applied to the parent frame
      //
      final int Padding_Left = this.getPaddingLeft ();
      final int Padding_Top = this.getPaddingTop ();
      final int Padding_Right = this.getPaddingRight ();
      final int Padding_Bottom = this.getPaddingBottom ();
      // size of the parent frame
      //
      final int Width = right - left;
      final int Height = bottom - top;
      // Split the two elements.
      //
      final int Split = this.Shift_Height + Padding_Top;

      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Padding_Left   = " + Padding_Left);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Padding_Top    = " + Padding_Top);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Padding_Right  = " + Padding_Right);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Padding_Bottom = " + Padding_Bottom);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Width          = " + Width);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Height         = " + Height);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Split          = " + Split);

      // get the shift layout parameters so that we can honour their margins
      //
      final MarginLayoutParams Shift_Layout = (MarginLayoutParams) this.Shift.getLayoutParams ();

      // Shift is displayed at the top position
      //
      this.Shift.layout (
	 /* left   => */Padding_Left + Shift_Layout.leftMargin,
	 /* top    => */Padding_Top + Shift_Layout.topMargin,
	 /* right  => */Width - Padding_Right - Shift_Layout.rightMargin,
	 /* bottom => */Split - Shift_Layout.bottomMargin);

      // get the button layout parameters so that we can honour their margins
      //
      final MarginLayoutParams Button_Layout = (MarginLayoutParams) this.Button.getLayoutParams ();

      // Button is displayed at the bottom position
      //
      this.Button.layout (
	 /* left   => */Padding_Left + Button_Layout.leftMargin,
	 /* top    => */Split + Button_Layout.topMargin,
	 /* right  => */Width - Padding_Right - Button_Layout.rightMargin,
	 /* bottom => */Height - Padding_Bottom - Button_Layout.bottomMargin);

      // net.sourceforge.uiq3.OS.Print_View (this.Shift.getText (), this.Shift);
      // net.sourceforge.uiq3.OS.Print_View (this.Button.getText (), this.Button);

      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "- onLayout");
      return;
   }

   /**
    * <p>
    * Measure the view and its content to determine the measured width and the measured height. This method is invoked
    * by {@link #measure(int, int)} and should be overriden by subclasses to provide accurate and efficient measurement
    * of their contents.
    * </p>
    * <p>
    * <strong>CONTRACT:</strong> When overriding this method, you <em>must</em> call
    * {@link #setMeasuredDimension(int, int)} to store the measured width and height of this view. Failure to do so will
    * trigger an <code>IllegalStateException</code>, thrown by {@link #measure(int, int)}. Calling the superclass'
    * {@link android.view.View#onMeasure(int, int)} is a valid use.
    * </p>
    * <p>
    * The base class implementation of measure defaults to the background size, unless a larger size is allowed by the
    * MeasureSpec. Subclasses should override {@link android.view.View#onMeasure(int, int)} to provide better
    * measurements of their content.
    * </p>
    * <p>
    * If this method is overridden, it is the subclass's responsibility to make sure the measured height and width are
    * at least the view's minimum height and width ( {@link #getSuggestedMinimumHeight()} and
    * {@link #getSuggestedMinimumWidth()}).
    * </p>
    *
    * <pre>
    * ╔═══════════╗
    * ║┌─────────┐║paddingTop
    * ║│╔═══════╗│║topMargin
    * ║│║ Shift ║│║
    * ║│╚═══════╝│║bottomMargin
    * ╟┼─────────┼╢Split
    * ║│╔═══════╗│║topMargin
    * ║│║Button ║│║
    * ║│╚═══════╝│║bottomMargin
    * ║└─────────┘║paddingBottom
    * ╚═══════════╝
    * </pre>
    *
    * @param widthMeasureSpec
    *    horizontal space requirements as imposed by the parent. The requirements are encoded with
    *    {@link android.view.View.MeasureSpec}.
    * @param heightMeasureSpec
    *    vertical space requirements as imposed by the parent. The requirements are encoded with
    *    {@link android.view.View.MeasureSpec}.
    * @see #getMeasuredWidth()
    * @see #getMeasuredHeight()
    * @see #setMeasuredDimension(int, int)
    * @see #getSuggestedMinimumHeight()
    * @see #getSuggestedMinimumWidth()
    * @see android.view.View.MeasureSpec#getMode(int)
    * @see android.view.View.MeasureSpec#getSize(int)
    */
   @SuppressWarnings ({
      "ConditionalExpression",
      "HardcodedLineSeparator",
      "FeatureEnvy"})
   @Override
   protected void onMeasure (final int widthMeasureSpec, final int heightMeasureSpec)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "+ onMeasure");
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> widthMeasureSpec  = " +
      // MeasureSpec.toString (widthMeasureSpec));
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> heightMeasureSpec = " +
      // MeasureSpec.toString (heightMeasureSpec));

      // compute padding
      //
      final int Width_Padding = this.getPaddingLeft () + this.getPaddingRight ();
      final int Height_Padding = this.getPaddingTop () + this.getPaddingBottom ();

      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Width_Padding     = " + Width_Padding);
      // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> Height_Padding    = " +
      // Height_Padding);

      // breakdown specs
      //
      final int Width_Spec = MeasureSpec.getMode (widthMeasureSpec);
      final int Height_Spec = MeasureSpec.getMode (heightMeasureSpec);
      final int Width_Size = MeasureSpec.getSize (widthMeasureSpec);
      final int Height_Size = MeasureSpec.getSize (heightMeasureSpec);

      // Either both specifications are UNSPECIFIED or both are not. Else the result
      // will be faulty.
      //
      net.sourceforge.uiq3.Asserted.isFalse (
	 (Width_Spec == MeasureSpec.UNSPECIFIED) ^ (Height_Spec == MeasureSpec.UNSPECIFIED));

      // Calculate Height and Width for this view and its children.
      //
      final int Height;
      final int Width;

      switch (Width_Spec)
      {
	 case MeasureSpec.AT_MOST: // -2147483648
	 case MeasureSpec.EXACTLY: // 1073741824
	 {
	    // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> set width  to " + Width_Size);
	    // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> set height to " + Height_Size);

	    Width = Width_Size;
	    Height = Height_Size;

	    this.Shift.measure (
	       /* widthMeasureSpec  => */MeasureSpec.makeMeasureSpec (
		  Width == 0
		     ? 0
		     : Width - Width_Padding,
		  Width_Spec),
	       /* heightMeasureSpec => */MeasureSpec.makeMeasureSpec (
		  Height == 0
		     ? 0
		     : this.Shift_Height,
		  Height_Spec));
	    this.Button.measure (
	       /* widthMeasureSpec  => */MeasureSpec.makeMeasureSpec (
		  Width == 0
		     ? 0
		     : Width - Width_Padding,
		  Width_Spec),
	       /* heightMeasureSpec => */MeasureSpec.makeMeasureSpec (
		  Height == 0
		     ? 0
		     : Height - Height_Padding - this.Shift_Height,
		  Height_Spec));

	    break;
	 }
	 case android.view.View.MeasureSpec.UNSPECIFIED:
	 {
	    // net.sourceforge.uiq3.Log.v (Calculator_Button_2.TAG, "> calculate preferred size");

	    this.Shift.measure (
	       /* widthMeasureSpec  => */MeasureSpec.UNSPECIFIED,
	       /* heightMeasureSpec => */MeasureSpec.UNSPECIFIED);
	    this.Shift_Height = this.Shift.getMeasuredHeight ();
	    this.Shift_Width = this.Shift.getMeasuredWidth ();

	    this.Button.measure (
	       /* widthMeasureSpec  => */MeasureSpec.UNSPECIFIED,
	       /* heightMeasureSpec => */MeasureSpec.UNSPECIFIED);
	    this.Button_Height = this.Button.getMeasuredHeight ();
	    this.Button_Width = this.Button.getMeasuredWidth ();

	    // Width is the width of the largest child
	    //
	    Width = Width_Padding + net.sourceforge.uiq3.Utils.max (this.Button_Width, this.Shift_Width);

	    // Height is the height of all children plus one padding per child
	    //
	    Height = Height_Padding + this.Button_Height + this.Shift_Height;

	    break;
	 }
	 default:
	 {
	    net.sourceforge.uiq3.Log.e (
	       Calculator_Button_2.TAG,
	       "unknown measure spec!" + "\nwidthMeasureSpec  = " + MeasureSpec.toString (widthMeasureSpec) +
		  "\nheightMeasureSpec = " + MeasureSpec.toString (heightMeasureSpec));

	    Width = 0;
	    Height = 0;

	    break;
	 }
      } // switch

      this.setMeasuredDimension (Width, Height);

      // net.sourceforge.uiq3.OS.Print_View (this.Shift.getText (), this.Shift);
      // net.sourceforge.uiq3.OS.Print_View (this.Button.getText (), this.Button);
      // net.sourceforge.uiq3.OS.Print_View ("Parent", this);

      // net.sourceforge.uiq3.Log.d (Calculator_Button_2.TAG, "- onMeasure");
      return;
   }

   /**
    * <p>
    * shift text view — used for unit testing
    * </p>
    *
    * @return the shift text view
    */
   @TestOnly
   public android.widget.TextView Shift ()
   {
      return this.Shift;
   }

   /**
    * <p>
    * switch display for button
    * </p>
    *
    * @param Large_Screen
    *    display for large screens
    */
   @Override
   public void switchDisplay (final boolean Large_Screen)
   {
      // net.sourceforge.uiq3.Log.d (TAG, "+ Switch_Display");
      // net.sourceforge.uiq3.Log.v (TAG, "> Large_Screen   = " + Large_Screen);

      // there is no large screen for the RPN-45

      // net.sourceforge.uiq3.Log.d (TAG, "- Switch_Display");
      return;
   }

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Calculator_Button_2 {\n" +
	 "super = " + super.toString () +
	 ",\nMain_Function = „" + Main_Function + '“' +
	 ",\nGold_Description = „" + Gold_Description + '“' +
	 ",\nButton = " + Button +
	 ",\nButton_Background = " + Button_Background +
	 ",\nButton_Bold = " + Button_Bold +
	 ",\nButton_Height = " + Button_Height +
	 ",\nButton_Text = „" + Button_Text + '“' +
	 ",\nButton_Text_Color = " + Button_Text_Color +
	 ",\nButton_Text_Size = " + Button_Text_Size +
	 ",\nButton_Width = " + Button_Width +
	 ",\nShift = " + Shift +
	 ",\nShift_Background = " + Shift_Background +
	 ",\nShift_Color = " + Shift_Color +
	 ",\nShift_Height = " + Shift_Height +
	 ",\nShift_Text = „" + Shift_Text + '“' +
	 ",\nShift_Width = " + Shift_Width +
	 "\n}";
   }
} // Calculator_Button_2

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
