/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7750 $
 *   $Date: 2021-11-29 11:43:17 +0100 (Mo, 29. Nov 2021) $
 *   $Id: Calculator_Fragment.java 7750 2021-11-29 10:43:17Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import androidx.annotation.NonNull;

/**
 * <p>
 * The activity (GUI) for the calculator. Display and Buttons are managed here.
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@SuppressWarnings ("HardcodedLineSeparator")
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EFragment (resName = "calculator")
public class Calculator_Fragment
   extends
   net.sourceforge.uiq3.calculator.Calculator_Fragment<Calculator_Service, Main_Activity, IPreferences_>
   implements
   ICalculator_Fragment
{
   /**
    * Hander class to update the text field.
    *
    * @author "Martin Krischik" «krischik@users.sourceforge.net»
    */
   private static final class Text_Handler
      extends
      android.os.Handler
   {
      /**
       * <p>
       * Owner of this handler
       * </p>
       */
      @NotNull
      @SuppressWarnings ("FieldNotUsedInToString") final java.lang.ref.WeakReference<Calculator_Fragment> Owner;

      /**
       * <p>
       * create Hander
       * </p>
       *
       * @param Owner
       *    the owner of the handler
       */
      Text_Handler (@NotNull final Calculator_Fragment Owner)
      {
	 super ();
	 // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ Text_Handler");

	 this.Owner = new java.lang.ref.WeakReference<> (Owner);

	 // net.sourceforge.uiq3.Log.d (Activity.TAG, "- Text_Handler");
	 return;
      } // Text_Handler

      /**
       * <p>
       * update text field
       * </p>
       *
       * @see android.os.Handler#handleMessage(android.os.Message)
       */
      @Override
      public void handleMessage (final android.os.Message message)
      {
	 // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ Text_Handler.handleMessage");
	 // net.sourceforge.uiq3.Log.v (Calculator.TAG, "> message.obj = " + message.obj);

	 @Nullable final Calculator_Fragment Owner = this.Owner.get ();

	 if ((Owner != null) && (Owner.Text_Display != null))
	 {
	    @SuppressWarnings ("ConstantConditions") final CharSequence Characters = Owner.Text_Display.getText ();
	    final String Text = Characters.toString ();

	    if (!Text.equals (Owner.Utf16_Text))
	    {
	       // noinspection ConstantConditions
	       Owner.Text_Display.setText (Owner.Utf16_Text);
	    } // if
	 } // if

	 // net.sourceforge.uiq3.Log.d (Calculator_Activity.TAG, "- Text_Handler.handleMessage");
	 return;
      } // handleMessage

      @SuppressWarnings ("HardcodedLineSeparator")
      @Override
      @org.jetbrains.annotations.TestOnly
      public String toString ()
      {
	 return "Text_Handler" + "{\nOwner=" + this.Owner.get () + '}';
      } // toString
   } // Text_Handler

   /**
    * <p>
    * Clear button
    * </p>
    */
   @SuppressWarnings ("OctalInteger")
   private static final int Key_CLx = 070;
   /**
    * logging tag
    */
   static final String TAG = net.sourceforge.uiq3.Log.createTag (Calculator_Fragment.class);
   /**
    * <p>
    * a button for the haptic feedback.
    * </p>
    */
   @org.androidannotations.annotations.ViewById (resName = "Button_0")
   @Nullable
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_0;
   /**
    * The text field to update
    */
   @org.androidannotations.annotations.ViewById (resName = "Text_Display")
   @Nullable
   protected android.widget.TextView Text_Display;

   /**
    * Called when the views are injected.
    *
    * @see android.app.Activity#onCreate(android.os.Bundle)
    */
   @org.androidannotations.annotations.AfterViews
   protected void After_Views ()
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ After_Views");

      if (this.Handler == null)
      {
	 // net.sourceforge.uiq3.Log.v (Calculator.TAG, "> Create Handler.");

	 this.Handler = new Text_Handler (this);

	 this.Send_Redraw_All ();
      } // if

      final Main_Activity Activity = this.Activity ();

      if (Activity != null)
      {
	 Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Calculator);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    net.sourceforge.uiq3.hp45.Calculator_Fragment.TAG,
	    "Fragment not attached to Activity");
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- After_Views");
      return;
   } // After_Views

   /**
    * <p>
    * Returns the instance as a Fragment (saves you an instanceOf check and cast)
    * </p>
    *
    * @return this
    */
   @SuppressWarnings ("ReturnOfThis")
   @Override
   @NotNull
   public androidx.fragment.app.Fragment As_Fragment ()
   {
      return this;
   } // As_Fragment

   /**
    * <p>
    * a button for the haptic feedback.
    * </p>
    */
   @Nullable
   @org.jetbrains.annotations.TestOnly
   public net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_0 ()
   {
      return this.Button_0;
   }

   /**
    * <p>
    * display additional information on the error. this is device and setup dependent.
    * </p>
    *
    * @param Exception
    *    Exception to display
    */
   @Override
   public void displayException (@NotNull final Throwable Exception)
   {
      // net.sourceforge.uiq3.Log.d (
      // Calculator_Fragment.TAG,
      // "+ displayException" + "\n> Exception         =",
      // Exception);

      final Main_Activity Activity = this.Activity ();

      if (Activity != null)
      {
	 Activity.displayException (Exception);
      }

      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "- displayException");
      return;
   }

   /**
    * <p>
    * Text currently displayed
    * </p>
    *
    * @return text currently displayed
    * @see net.sourceforge.uiq3.calculator.ICalculator_Fragment#getDisplayText()
    */
   @Override
   @NotNull
   public String getDisplayText ()
   {
      final CharSequence Retval = this.Text_Display != null
	 ? this.Text_Display.getText ()
	 : "";

      return Retval.toString ();
   } // Get_Text

   /**
    * <p>
    * create buttons
    * </p>
    */
   @SuppressWarnings ("MagicNumber")
   private void Init_Keyboard ()
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "+ Init_Keyboard");

      if (this.Keys == null)
      {
	 // net.sourceforge.uiq3.Log.v (Activity.TAG, "  > Create Keyboard.");

	 // The Calc_Key_Code are all octal:
	 // noinspection OctalInteger
	 this.Keys = new Key[] {
	    /*  0 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_1x,
	    /* Calc_Key_Code  => */(short) 006,
	    /* Java_Key_Code  => */81),
	    /*  1 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_ln,
	    /* Calc_Key_Code  => */(short) 004,
	    /* Java_Key_Code  => */87),
	    /*  2 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_ex,
	    /* Calc_Key_Code  => */(short) 003,
	    /* Java_Key_Code  => */69),
	    /*  3 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_FIX_SCI,
	    /* Calc_Key_Code  => */(short) 002,
	    /* Java_Key_Code  => */82),
	    /*  4 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_GOLD,
	    /* Calc_Key_Code  => */(short) 000,
	    /* Java_Key_Code  => */-4),
	    /*  5 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_x2,
	    /* Calc_Key_Code  => */(short) 056,
	    /* Java_Key_Code  => */65),
	    /*  6 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_toP,
	    /* Calc_Key_Code  => */(short) 054,
	    /* Java_Key_Code  => */83),
	    /*  7 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_sin,
	    /* Calc_Key_Code  => */(short) 053,
	    /* Java_Key_Code  => */68),
	    /*  8 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_cos,
	    /* Calc_Key_Code  => */(short) 052,
	    /* Java_Key_Code  => */70),
	    /*  9 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_tan,
	    /* Calc_Key_Code  => */(short) 050,
	    /* Java_Key_Code  => */71),
	    /* 10 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_xy,
	    /* Calc_Key_Code  => */(short) 016,
	    /* Java_Key_Code  => */44),
	    /* 11 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Rdown,
	    /* Calc_Key_Code  => */(short) 014,
	    /* Java_Key_Code  => */90),
	    /* 12 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_STO,
	    /* Calc_Key_Code  => */(short) 013,
	    /* Java_Key_Code  => */88),
	    /* 13 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_RCL,
	    /* Calc_Key_Code  => */(short) 012,
	    /* Java_Key_Code  => */67),
	    /* 14 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_percent,
	    /* Calc_Key_Code  => */(short) 010,
	    /* Java_Key_Code  => */86),
	    /* 15 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_ENTER,
	    /* Calc_Key_Code  => */(short) 074,
	    /* Java_Key_Code  => */32),
	    /* 16 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_CHS,
	    /* Calc_Key_Code  => */(short) 073,
	    /* Java_Key_Code  => */99),
	    /* 17 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_EEX,
	    /* Calc_Key_Code  => */(short) 072,
	    /* Java_Key_Code  => */-3),
	    /* 18 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_CLx,
	    /* Calc_Key_Code  => */(short) net.sourceforge.uiq3.hp45.Calculator_Fragment.Key_CLx,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_DEL),
	    /* 19 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Minus,
	    /* Calc_Key_Code  => */(short) 066,
	    /* Java_Key_Code  => */98),
	    /* 20 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_7,
	    /* Calc_Key_Code  => */(short) 064,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_7),
	    /* 21 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_8,
	    /* Calc_Key_Code  => */(short) 063,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_8),
	    /* 22 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_9,
	    /* Calc_Key_Code  => */(short) 062,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_9),
	    /* 23 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Plus,
	    /* Calc_Key_Code  => */(short) 026,
	    /* Java_Key_Code  => */104),
	    /* 24 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_4,
	    /* Calc_Key_Code  => */(short) 024,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_4),
	    /* 25 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_5,
	    /* Calc_Key_Code  => */(short) 023,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_5),
	    /* 26 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_6,
	    /* Calc_Key_Code  => */(short) 022,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_6),
	    /* 27 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Mult,
	    /* Calc_Key_Code  => */(short) 036,
	    /* Java_Key_Code  => */120),
	    /* 28 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_1,
	    /* Calc_Key_Code  => */(short) 034,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_1),
	    /* 29 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_2,
	    /* Calc_Key_Code  => */(short) 033,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_2),
	    /* 20 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_3,
	    /* Calc_Key_Code  => */(short) 032,
	    /* Java_Key_Code  => */android.view.KeyEvent.KEYCODE_3),
	    /* 31 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Div,
	    /* Calc_Key_Code  => */(short) 046,
	    /* Java_Key_Code  => */122),
	    /* 32 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_0,
	    /* Calc_Key_Code  => */(short) 044,
	    /* Java_Key_Code  => */112),
	    /* 33 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Dot,
	    /* Calc_Key_Code  => */(short) 043,
	    /* Java_Key_Code  => */46),
	    /* 34 => */new Key (
	    /* owner          => */this,
	    /* id             => */net.sourceforge.uiq3.hp45.R.id.Button_Σplus,
	    /* Calc_Key_Code  => */(short) 042,
	    /* Java_Key_Code  => */115)};
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "- Init_Keyboard");
      return;
   } // Init_Keyboard

   /**
    * Called when a fragment is first attached to its activity. {@link #onCreate(android.os.Bundle)} will be called
    * after this.
    */
   @SuppressWarnings ("MagicNumber")
   @Override
   public void onActivityCreated (@Nullable final android.os.Bundle savedInstanceState)
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ onActivityCreated");
      // net.sourceforge.uiq3.Log.v (Calculator.TAG, "  > savedInstanceState      = " +
      // savedInstanceState);

      super.onActivityCreated (savedInstanceState);

      if ((savedInstanceState != null) && (this.Text_Display != null))
      {
	 this.Text_Display.setText (savedInstanceState.getString ("Text_Display"));
      } // if

      final androidx.fragment.app.FragmentActivity activity = this.getActivity ();
      final android.view.View Content_View = this.getView ();

      if (Content_View != null)
      {
	 final android.view.ViewGroup Root_View = (android.view.ViewGroup) Content_View.getRootView ();
      }

      this.Init_Keyboard ();

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- onActivityCreated");
      return;
   } // onActivityCreated

   /**
    * <p>
    * Called when a fragment is first attached to its activity. {@link #onCreate(android.os.Bundle)} will be called
    * after this.
    * </p>
    *
    * @param activity
    *    the activity
    */
   @Override
   public void onAttach (@NonNull final android.app.Activity activity)
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ onAttach");

      super.onAttach (activity);

      if (activity instanceof Main_Activity)
      {
	 final Main_Activity Main_Activity = (Main_Activity) activity;
	 this.Activity (Main_Activity);
	 Main_Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Calculator);
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- onAttach");
      return;
   } // onAttach

   /**
    * <p>
    * Called when the fragment is no longer attached to its activity. This is called after {@link #onDestroy()}.
    * </p>
    */
   @Override
   public void onDetach ()
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ onDetach");
      super.onDetach ();

      final Main_Activity Activity = this.Activity ();

      if (Activity != null)
      {
	 Activity.Bind_Fragment (null, IMain_Activity.Page_Type.Calculator);
	 this.Activity (null);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    net.sourceforge.uiq3.hp45.Calculator_Fragment.TAG,
	    "Fragment not attached to Activity");
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- onDetach");
      return;
   } // onDetach

   /**
    * Called when the fragment is visible to the user and actively running. This is generally tied to
    * {@link android.app.Activity#onResume() Activity.onResume} of the containing Activity's lifecycle.
    *
    * @see android.app.Activity#onResume()
    */
   @Override
   public void onResume ()
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ onResume");

      super.onResume ();

      if (this.Handler != null)
      {
	 final android.os.Message Keyboard_Message = this.Handler.obtainMessage ();
	 this.Handler.sendMessage (Keyboard_Message);
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- onResume");
      return;
   } // onResume

   /**
    * Called to ask the fragment to save its current dynamic state, so it can later be reconstructed in a new instance
    * of its process is restarted. If a new instance of the fragment later needs to be created, the data you place in
    * the Bundle here will be available in the Bundle given to {@link #onCreate(android.os.Bundle)},
    * {@link #onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)} , and
    * {@link #onActivityCreated(android.os.Bundle)}.
    * <p/>
    * <p>
    * This corresponds to
    * {@link android.app.Activity#onSaveInstanceState(android.os.Bundle) Activity.onSaveInstanceState(Bundle)} and most
    * of the discussion there applies here as well. Note however: <em>this method may be called at any time before
    * {@link #onDestroy()}</em>. There are many situations where a fragment may be mostly torn down (such as when placed
    * on the back stack with no UI showing), but its state will not be saved until its owning activity actually needs to
    * save its state.
    *
    * @param outState
    *    Bundle in which to place your saved state.
    */
   @Override
   public void onSaveInstanceState (@NotNull final android.os.Bundle outState)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "+ onSaveInstanceState");

      super.onSaveInstanceState (outState);

      final CharSequence text = this.Text_Display != null
	 ? this.Text_Display.getText ()
	 : "";
      outState.putString ("Text_Display", text.toString ());

      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "- onSaveInstanceState");
      return;
   } // onSaveInstanceState

   /**
    * <p>
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    * </p>
    *
    * @param Indicators
    *    indicators to be shown
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#Set_Indicators(boolean[])
    */
   @Override
   public final void setIndicators (final boolean @NotNull [] Indicators)
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ setIndicators");

      assert false : "Should never be called!";

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- setIndicators");
      return;
   } // setIndicators

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    *    indicators to be shown
    * @noinspection ZeroLengthArrayAllocation
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setIndicators(boolean[])
    */
   @Override public boolean @NotNull []
   getIndicators ()
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ getIndicators");

      assert false : "Should never be called!";

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- getIndicators");
      return new boolean[0];
   } // getIndicators

   /**
    * <p>
    * set the calculators on of status.
    * </p>
    *
    * @param on
    *    calculator is switched on
    */
   @Override
   public void setOnOff (final boolean on)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "+ Set_On_Off");
      // net.sourceforge.uiq3.Log.v (Calculator_Fragment.TAG, "> on = " + on);

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 activity.setOnOff (on);
      }

      // net.sourceforge.uiq3.Log.d (Calculator_Fragment.TAG, "- Set_On_Off");
      return;
   }

   /**
    * Get calculator key code of the currently pressed key.
    *
    * @param Pressed_Key
    *    calculator key code
    */
   @SuppressWarnings ("FeatureEnvy")
   @Override
   public void setPressedKey (final short Pressed_Key)
   {
      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "+ Set_Pressed_Key");

      super.setPressedKey (Pressed_Key);

      final Main_Activity Activity = this.Activity ();

      if ((Pressed_Key != net.sourceforge.uiq3.calculator.IKey.No_Key) && (Activity != null))
      {
	 if (Pressed_Key == net.sourceforge.uiq3.hp45.Calculator_Fragment.Key_CLx)
	    /*
	     * Key 70 is the CLx key. this key should switch on the calculator when is went
	     * into battery saving.
	     */
	 {
	    Activity.Restart_Calculator ();
	    Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Calculator);
	 } // if

	 if (Preferences_Activity.Feedback_Vibrate (Activity.getApplicationContext ()) && (this.Button_0 != null))
	 {
	    this.Button_0.performHapticFeedback (android.view.HapticFeedbackConstants.VIRTUAL_KEY);
	 } // if
      } // if

      // net.sourceforge.uiq3.Log.d (Calculator.TAG, "- Set_Pressed_Key");
      return;
   } // Set_Pressed_Key

   /**
    * <p>
    * Set text to be displayed (calculator alpha codes). Null implementation for calculators without alpha display.
    * </p>
    *
    * @param Row
    *    Display row to set
    * @param Text
    *    to be displayed
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setDisplayText(int, short[])
    */
   @Override
   public void setDisplayText (final int Row, final short @NotNull [] Text)
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ Set_Indicators");

      assert false : "Should never be called!";

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- Set_Indicators");
      return;
   } // setDisplayText

   @Override
   public short @NotNull []
   getDisplayText (final int i)
   {
      return new short[0];
   }

   @Override
   public @NotNull String getJavaText (final int i)
   {
      return getJavaText ();
   }

   @Override
   public @NotNull String getJavaText ()
   {
      final CharSequence text;
      if (Text_Display != null)
      {
	 text = Text_Display.getText ();
      }
      else
      {
	 text = "";
      }
      return text.toString ();
   }

   /**
    * <p>
    * Generate string for debug purpose
    * </p>
    *
    * @return string with debug informations
    */
   @SuppressWarnings ("HardcodedLineSeparator")
   @NotNull
   @org.jetbrains.annotations.TestOnly
   @Override
   public String toString ()
   {
      return net.sourceforge.uiq3.hp45.Calculator_Fragment.TAG + "{\nsuper=" + super.toString () + ",\nButton_0=" +
	 this.Button_0 + ",\nText_Display=" + this.Text_Display + '}';
   } // toString
} // Calculator_Fragment

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
