/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7685 $
 *   $Date: 2021-09-16 11:41:15 +0200 (Do, 16. Sep 2021) $
 *   $Id: Calculator_Service.java 7685 2021-09-16 09:41:15Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@SuppressWarnings ("HardcodedLineSeparator")
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EService
public class Calculator_Service
   extends
   net.sourceforge.uiq3.calculator.Service
{
   /**
    * Logging tag
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Calculator_Service.class);
   /**
    * <p>
    * Service currently running in foreground
    * </p>
    */
   private boolean Foreground;
   /**
    * The calculator engine.
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   @Nullable
   protected Droid_Calculator HP45;

   /**
    * Set Canvas to display result read keyboard
    *
    * @param Form
    *    Activity where calculator is displayed in
    */
   @Override
   public void setCalculatorActivity (final net.sourceforge.uiq3.ui.ICalculatorCanvas Form)
   {
      net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "+ Calculator_Activity" + "> Form = %1$.32s", Form);

      super.setCalculatorActivity (Form);

      if (this.HP45 != null)
      {
	 this.HP45.setForm (Form);
      } // if

      net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "- Calculator_Activity");
      return;
   } // Calculator_Activity

   /**
    * Create Main engine.
    *
    * @return New calculator engine
    * @throws java.io.IOException
    *    I/O Error
    */
   @SuppressWarnings ({"FeatureEnvy"})
   @Override
   @NotNull
   public net.sourceforge.uiq3.calculator.ICalculator createCalculator ()
      throws
      java.io.IOException
   {
      net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "+ Create_Calculator");

      if (this.HP45 == null)
      {
	 this.HP45 = new Droid_Calculator (this);
	 this.HP45.Save_Energy (false);
	 this.HP45.setForm (this.Calculator_Activity);
	 this.HP45.Read_Listing_File (net.sourceforge.uiq3.hp45.Calculator.HP45_LST);
	 this.HP45.Init_Opcodes ();
      } // if

      net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "- Create_Calculator" + "\n> Retval = %1$.32s", this.HP45);
      return this.HP45;
   } // Create_Calculator

   /**
    * Switch service to foreground and show a notification (which you have to sind API 5)
    *
    * @param Enable
    *    true: show or remove foreground service notification, false: deactivate Foreground and remove notification
    * @param Text
    *    the foreground text as call by name
    * @param Text_1
    *    First Text row
    * @param Text_2
    *    Second Text row (unused for FX-602P)
    */
   @Override public void
   foreground (
      final boolean enable,
      @NotNull final kotlin.jvm.functions.Function2<? super short[], ? super short[], CharSequence[]> convertText,
      final short @NotNull [] textLine1,
      final short @NotNull [] textLine2)
   {
      // net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "+ Foreground");
      // net.sourceforge.uiq3.Log.v (Calculator_Service.TAG, "> Show = " + Enable);
      // net.sourceforge.uiq3.Log.v (Calculator_Service.TAG, "> convertText = " +
      // java.util.Arrays.toString (convertText));

      // do noting in the RPN-45 Emulator.

      this.Foreground = enable;

      // net.sourceforge.uiq3.Log.d (Calculator_Service.TAG, "- Foreground");
      return;
   } // Foreground

   /**
    * Logging tag
    */
   @Nullable
   public Droid_Calculator getRPN45 ()
   {
      return this.HP45;
   }

   /**
    * Restore calculator status
    */
   @Override
   public void restoreStatus ()
   {
      // not needed
      return;
   } // Restore_Status

   /**
    * Save calculator status
    */
   @Override
   public void saveStatus ()
   {
      // not needed
      return;
   } // Save_Status

   @SuppressWarnings ("HardcodedLineSeparator")
   @org.jetbrains.annotations.TestOnly
   @Override
   @NotNull
   public
   String toString ()
   {
      return TAG + "{\nForeground=" + this.Foreground + '}';
   }
} // Service

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
