/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7685 $
 *   $Date: 2021-09-16 11:41:15 +0200 (Do, 16. Sep 2021) $
 *   $Id: Droid_Calculator.java 7685 2021-09-16 09:41:15Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@SuppressWarnings ("HardcodedLineSeparator")
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
public class Droid_Calculator
   extends
   Calculator
   implements
   net.sourceforge.uiq3.calculator.Droid_Calculator
{
   /**
    * Name of the file in which the calculator state is stored.
    */
   private static final String RAM_FILE_NAME = "rpn-45.ram";
   /**
    * Name of the file in which the calculator state is stored.
    */
   private static final String ROM_FILE_NAME = "rpn-45.rom";
   /**
    * Logging tag
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Droid_Calculator.class);
   /**
    * <p>
    * A backup manager to use for save operations.
    * </p>
    */
   @NotNull
   private final android.app.backup.BackupManager Backup_Manager;
   /**
    * <p>
    * Service to which the calculator belongs.
    * </p>
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   @NotNull
   protected final net.sourceforge.uiq3.calculator.Service Service;

   /**
    * <p>
    * create calculator
    * </p>
    *
    * @param service
    *    the service in which the calculator runs<
    */
   Droid_Calculator (@NotNull final net.sourceforge.uiq3.calculator.Service service)
   {
      super ();
      this.Service = service;
      this.Backup_Manager = new android.app.backup.BackupManager (this.Service);
   }

   /**
    * <p>
    * get cache file for cache the rom in
    * </p>
    *
    * @return a file to cache the rom
    */
   @NotNull
   private java.io.File composeCacheFile ()
   {
      return new java.io.File (this.Service.getCacheDir (), Droid_Calculator.ROM_FILE_NAME);
   }

   @Override
   public void displayClear ()
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ displayClear");

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- displayClear");
      return;
   } // displayClear

   /**
    * <p>
    * display additional information on the error. this is device and setup dependent.
    * </p>
    *
    * @param Exception
    *    Exception to display
    */
   @SuppressWarnings ("FeatureEnvy")
   @Override
   public void displayException (@NotNull final Throwable Exception)
   {
      net.sourceforge.uiq3.Log.d (
	 Droid_Calculator.TAG,
	 "+ displayException" + "\n> Exception         = ",
	 Exception);

      final net.sourceforge.uiq3.calculator.Preferences_Activity.Error_Handle_Type
	 Error_Handler
	 = Preferences_Activity.Error_Handler (this.Service);
      final Class<? extends Throwable> Exception_Class = Exception.getClass ();
      final String Error_Message =
	 Exception_Class.getSimpleName () + ": " + Exception.getMessage () + net.sourceforge.uiq3.OS.New_Line;

      // net.sourceforge.uiq3.Log.v (Droid_Calculator.TAG, "> Error_Handler     = %1$s", Error_Handler);
      // net.sourceforge.uiq3.Log.v (Droid_Calculator.TAG, "> Printer_Fragment  = %1$s", Printer_Fragment);
      // net.sourceforge.uiq3.Log.v (Droid_Calculator.TAG, "> Form              = %1$s", this.Form);

      switch (Error_Handler)
      {
	 case Printer:
	 {
	    this.Print_Exception (Error_Message);

	    break;
	 }
	 case All:
	    this.Print_Exception (Error_Message);

	    if (this.form != null)
	    {
	       this.form.displayException (Exception);
	    } // if
	    break;
	 case None:
	 {
	    break;
	 }
	 case Toast:
	 {
	    if (this.form != null)
	    {
	       this.form.displayException (Exception);
	    }
	    break;
	 }
      } // switch

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- displayException");
      return;
   } // displayException

   @Override
   public void Display_Formatted (final short @NotNull [] Text)
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Display_Formatted");

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Display_Formatted");
      return;
   }

   @Override
   public void Display_Key (final short @NotNull [] Text)
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Display_Key");

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Display_Key");
      return;
   }

   @Override
   public void Display_L ()
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Display_L");

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Display_L");
      return;
   }

   @Override
   public void Display_Message (final short @NotNull [] Text)
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Display_Message");

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Display_Message");
      return;
   }

   /**
    * <p>
    * performs a full reset by deleting the cache file.
    * </p>
    */
   @SuppressWarnings ("ResultOfMethodCallIgnored")
   @Override
   public void resetAll ()
   {
      //net.sourceforge.uiq3.Log.d (TAG, "+ resetAll");

      final java.io.File cacheFile = this.composeCacheFile ();

      if (!cacheFile.delete ())
      {
	 net.sourceforge.uiq3.Log.w (Droid_Calculator.TAG, "LOG00141:no cache file «" + cacheFile + "» to delete.");
      }
      super.resetAll ();

      //net.sourceforge.uiq3.Log.d (TAG, "- resetAll");
      return;
   }

   @Override
   public void Load (@NotNull final String Filename)
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Load");

      try (
	 final java.io.FileInputStream File_In = this.Service.openFileInput (Filename);
	 final java.io.DataInputStream Data_In = new java.io.DataInputStream (File_In);
      )
      {
	 this.Load (Data_In);
      }
      catch (@NotNull final Throwable Exception)
      {
	 net.sourceforge.uiq3.Log.e (Droid_Calculator.TAG, "LOG00140:Could not load calculator state.", Exception);
	 this.reset ();
      } // try

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Load");
      return;
   }

   /**
    * @see Calculator#Load()
    */
   @TestOnly
   @Override
   public void Load ()
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Load");

      this.Load (Droid_Calculator.RAM_FILE_NAME);

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Load");
      return;
   } // Load

   @Override public void
   Save (@NotNull final String s)
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Load");

      this.Save (Droid_Calculator.RAM_FILE_NAME);

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Load");
      return;
   }

   /**
    * <p>
    * print an error message
    * </p>
    *
    * @param Error_Message
    *    the error message to print
    */
   private void Print_Exception (final String Error_Message)
   {
      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Print_Exception" + "\n> Error Message  =", Error_Message);

      final net.sourceforge.uiq3.calculator.IPrinter_Fragment Printer_Fragment =
	 (net.sourceforge.uiq3.calculator.IPrinter_Fragment) this.form;

      if (Printer_Fragment != null)
      {
	 final String Printout = Printer_Fragment.Printout ();

	 // net.sourceforge.uiq3.Log.v (Droid_Calculator.TAG, "> Printout          = %1$s", Printout);

	 Printer_Fragment.Printout (Printout + "\n" + Error_Message);
      } // if

      net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Print_Exception");
      return;
   } // Print_Exception

   /**
    * This reads a zip file that is expected to contain one entry which is the RPN-45 firmware listing. The instructions
    * are parsed for later execution.
    *
    * @param File_Name
    *    File name to read from
    * @throws java.io.IOException
    *    file could not be read
    */
   @Override
   public synchronized void
   Read_Listing_File (final String File_Name)
      throws
      java.io.IOException
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Read_Listing_File");

      final java.io.File Cache_File = this.composeCacheFile ();

      // net.sourceforge.uiq3.Log.i (Droid_Calculator.TAG, "Read ROM from cache.");
      this.setDisplayText ("Read ROM cache…");

      try (
	 final java.io.FileInputStream File_In = new java.io.FileInputStream (Cache_File);
	 final java.io.DataInputStream Data_In = new java.io.DataInputStream (File_In))
      {
	 for (int Group = 0; Group < Calculator.Max_Group; Group++)
	 {
	    for (int Rom = 0; Rom < Calculator.Max_Rom; Rom++)
	    {
	       for (int Pointer = 0; Pointer < Calculator.Rom_Size; Pointer++)
	       {
		  final short Opcode = Data_In.readShort ();

		  net.sourceforge.uiq3.Asserted.range (Opcode, 0, Calculator.Opcode_Size, "Opcode");

		  this.ucode[Group][Rom][Pointer] = Opcode;
	       } // for
	    } // for
	 } // for
      }
      catch (@NotNull final java.lang.Exception Exception)
      {
	 net.sourceforge.uiq3.Log.w (Droid_Calculator.TAG, "LOG00100: Read ROM from cache failed", Exception);

	 // If anything at all goes wrong on reading the cache file we read the original
	 // code and create a new cache file.

	 super.Read_Listing_File (File_Name);
	 this.reset ();
	 this.setDisplayText ("Write ROM cache…");

	 try (
	    final java.io.FileOutputStream File_Out = new java.io.FileOutputStream (Cache_File);
	    final java.io.DataOutputStream Data_Out = new java.io.DataOutputStream (File_Out))
	 {
	    for (final short[][] Rom : this.ucode)
	    {
	       for (final short[] Pointer : Rom)
	       {
		  for (final short Opcode : Pointer)
		  {
		     net.sourceforge.uiq3.Asserted.range (Opcode, 0, Calculator.Opcode_Size, "Opcode");

		     Data_Out.writeShort (Opcode);
		  } // for
	       } // for
	    } // for
	 } // try
      } // try

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Read_Listing_File");
      return;
   }

   /**
    * @see Calculator#Save()
    */
   @TestOnly
   @Override
   public void Save ()
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Save");

      try
      {
	 final java.io.FileOutputStream File_Out = this.Service.openFileOutput (
	    Droid_Calculator.RAM_FILE_NAME,
	    android.content.Context.MODE_PRIVATE);
	 final java.io.DataOutputStream Data_Out = new java.io.DataOutputStream (File_Out);

	 this.Save (Data_Out);

	 Data_Out.close ();
	 File_Out.close ();

	 this.Backup_Manager.dataChanged ();
      }
      catch (@NotNull final java.io.IOException Exception)
      {
	 net.sourceforge.uiq3.Log.e (Droid_Calculator.TAG, "LOG00150:Could not save calculator state.", Exception);
      } // try

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Save");
      return;
   } // Save

   /**
    * Enable or disable energy savings mode. for example by reducing thread priority.
    *
    * @param enable
    *    true: start saving energy, false: run with normal speed.
    */
   @Override
   public void Save_Energy (final boolean enable)
   {
      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "+ Save_Energy ()");

      if (enable)
      {
	 try
	 {
	    synchronized (this)
	    {
	       this.wait (net.sourceforge.uiq3.calculator.Droid_Calculator.Half_A_Second);
	    } // synchronized
	 }
	 catch (@NotNull final InterruptedException Exception)
	 {
	    net.sourceforge.uiq3.Log.e (Droid_Calculator.TAG, "LOG00160:could not save energy.", Exception);
	 } // try
	 android.os.Process.setThreadPriority (android.os.Process.THREAD_PRIORITY_BACKGROUND);
      }
      else
      {
	 android.os.Process.setThreadPriority (android.os.Process.THREAD_PRIORITY_DEFAULT);
      } // if

      // net.sourceforge.uiq3.Log.d (Droid_Calculator.TAG, "- Save_Energy ()");
      return;
   } // Save_Energy

   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Droid_Calculator {\n" +
	 "super = " + super.toString () + ",\n" +
	 "\nBackup_Manager = " + Backup_Manager +
	 "\n}";
   }
} // Droid_Calculator

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :

