/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7685 $
 *   $Date: 2021-09-16 11:41:15 +0200 (Do, 16. Sep 2021) $
 *   $Id: IMain_Activity.java 7685 2021-09-16 09:41:15Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * <p>
 * The activity (GUI) for the calculator. Display and Buttons are managed here.
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
public interface IMain_Activity
   extends
      net.sourceforge.uiq3.calculator.IMain_Activity <net.sourceforge.uiq3.hp45.Calculator_Service>,
      net.sourceforge.uiq3.calculator.ICalculator_Fragment,
      net.sourceforge.uiq3.calculator.IMenu_Fragment,
      net.sourceforge.uiq3.calculator.IPrinter_Fragment
{
   /**
    * List of application pages.
    * <dl>
    * <dt>Products</dt>
    * <dd>Product list</dd>
    * <dt>Leasing</dt>
    * <dd>Leasing Calculator.</dd>
    * </dl>
    */
   public enum Page_Type
   {
         /**
          * <p>
          * List of products
          * </p>
          */
         Calculator (1, net.sourceforge.uiq3.hp45.R.string.Activity_Name),
         /**
          * <p>
          * List of products
          * </p>
          */
         Menu (0, net.sourceforge.uiq3.hp45.R.string.Menu_Name),
         /**
          * <p>
          * calculate leasing
          * </p>
          */
         Printer (2, net.sourceforge.uiq3.hp45.R.string.Printer);
      /**
       * <p>
       * Total count of alpha register
       * </p>
       */
      public static final int Count = Page_Type.values ().length;

      /**
       * <p>
       * get voucher type from database value.
       * </p>
       *
       * @param pos
       *           0 … 3
       * @return the voucher type
       * @throws ArrayIndexOutOfBoundsException
       *            pos was not 0 … 1
       */
      @NotNull
      public static Page_Type value (final int pos)
         throws ArrayIndexOutOfBoundsException
      {
         final Page_Type Retval;

         Search_Enum:
         {
            for (final Page_Type i : Page_Type.values ())
            {
               if (i.pos == pos)
               {
                  Retval = i;
                  break Search_Enum;
               } // if
            } // for

            throw new IllegalArgumentException ("No enum const " + Page_Type.class + " for pos " + pos);
         } // Search_Enum

         return Retval;
      } // value

      /**
       * <p>
       * Integer representation of enum value.
       * </p>
       */
      public final int pos;
      /**
       * <p>
       * Resource Id of the title string
       * </p>
       */
      public final int titleResId;

      /**
       * <p>
       * Create enum entry
       * </p>
       *
       * @param pos
       *           numerical pos used for aray index
       * @param titleResId
       *           Resource Id of the title string
       */
      Page_Type (final int pos, final int titleResId)
      {
         this.pos = pos;
         this.titleResId = titleResId;
      } // A_Typ

      /**
       * <p>
       * create debug string
       * </p>
       *
       * @return a string for the debugger
       */
      @SuppressWarnings ("HardcodedLineSeparator")
      @org.jetbrains.annotations.TestOnly
      @Override
      @NotNull
      public String toString ()
      {
         final StringBuilder Retval = new StringBuilder (100);

         Retval.append ("Page_Type");
         Retval.append ("{\nsuper=");
         Retval.append (super.toString ());
         Retval.append (",\npos=");
         Retval.append (this.pos);
         Retval.append (",\ntitleResId=");
         Retval.append (this.titleResId);
         Retval.append ('}');

         return Retval.toString ();
      } // toString
   } // Page_Type

   /**
    * <p>
    * Bind the fragment to its activity. Note thar the fragment manager might create new
    * fragments without the use of the page adapter.
    * </p>
    *
    * @param Fragment
    *           fragment to display (null ⇒ unbind fragment)
    * @param Page
    *           Page to which the fragment belongs.
    */
   public void Bind_Fragment (@Nullable final IFragment Fragment, final Page_Type Page);

   /**
    * <p>
    * switches the page (in phone layout).
    * </p>
    *
   * @param page
    *           page to display
    */
   @SuppressWarnings ("UnusedDeclaration")
   @org.jetbrains.annotations.TestOnly
   public void Switch_To_Page (@NotNull final Page_Type page);
} // IMain

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
