/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7745 $
 *   $Date: 2021-11-21 20:26:52 +0100 (So, 21. Nov 2021) $
 *   $Id: Main_Activity.java 7745 2021-11-21 19:26:52Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

import android.annotation.SuppressLint;

import net.sourceforge.uiq3.Required;

/**
 * <p>
 * The activity (GUI) for the calculator. Display and Buttons are managed here.
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@android.annotation.SuppressLint ("Registered")
@SuppressWarnings ("HardcodedLineSeparator")
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EActivity (resName = "main")
@org.androidannotations.annotations.OptionsMenu (resName = "calculator")
public class Main_Activity
   extends
   net.sourceforge.uiq3.calculator.Main_Activity<Calculator_Service>
   implements
   IMain_Activity
{
   /**
    * <p>
    * </p>
    *
    * @author "Martin Krischik" «krischik@users.sourceforge.net»
    * @version 1.0
    * @since 1.0
    */
   @SuppressWarnings ("PrivateMemberAccessBetweenOuterAndInnerClass")
   public static class Pager_Adapter
      extends
      androidx.fragment.app.FragmentPagerAdapter
   {
      /**
       * <p>
       * The Menu page is smaller
       * </p>
       */
      static final float One_Half = 1.f / 2.f;
      /**
       * <p>
       * The Menu page is smaller
       * </p>
       */
      static final float Two_Third = 2.f / 3.f;
      /**
       * <p>
       * owner of the adapter
       * </p>
       */
      java.lang.ref.WeakReference<Main_Activity> owner;

      /**
       * <p>
       * Create the page adapter
       * </p>
       *
       * @param owner
       *    owner of the adater
       */
      Pager_Adapter (@NotNull final Main_Activity owner)
      {
	 super (owner.getSupportFragmentManager ());
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Pager_Adapter.PagerAdapter");
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> fragmentManager = " + fragmentManager);

	 this.owner = new java.lang.ref.WeakReference<> (owner);

	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Pager_Adapter.PagerAdapter");
	 return;
      } // PagerAdapter

      /**
       * <p>
       * true when then display in landscape View — note that tablets are never in landscape view - they go into Tablet
       * view when in landscape.
       * </p>
       *
       * @return true when in lanscape mode
       */
      private boolean Landscape_View ()
      {
	 final Main_Activity main = this.owner.get ();
	 final boolean Retval;

	 if (main != null)
	 {
	    final android.content.res.Resources resources = main.getResources ();

	    Retval = (
	       resources.getConfiguration ().orientation == android.content.res.Configuration.ORIENTATION_LANDSCAPE) &&
	       !main.Tablet_View ();
	 }
	 else
	 {
	    Retval = false;
	 } // if

	 return Retval;
      } // Landscape_View

      /**
       * <p>
       * page count
       * </p>
       *
       * @return page count
       */
      @Override
      public int getCount ()
      {
	 final Main_Activity main = this.owner.get ();
	 final int Retval;

	 if ((main == null) || (main.Tablet_View == null))
	 {
	    Retval = 0;
	 }
	 else
	 {
	    Retval = main.Tablet_View ()
	       ? Page_Type.Count - 1
	       : Page_Type.Count;
	 }

	 return Retval;
      } // getCount

      /**
       * get fragment for page
       *
       * @param position
       *    page position
       * @return a fragment
       */
      @SuppressWarnings ("FeatureEnvy")
      @NotNull
      @Override
      public androidx.fragment.app.Fragment getItem (final int position)
      {
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Pager_Adapter.getItem");
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> position  : " + position);

	 @Nullable final IFragment Retval;

	 final Main_Activity main = this.owner.get ();
	 if (main != null)
	 {
	    final Page_Type page;

	    if (main.Tablet_View () && (position >= Page_Type.Calculator.pos))
	    {
	       page = Page_Type.value (position + 1);
	    }
	    else
	    {
	       page = Page_Type.value (position);
	    } // if

	    // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> page      : " + page);

	    switch (page)
	    {
	       case Menu:
	       {
		  // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> create menu fragment");

		  main.Menu = new Menu_Fragment_ ();
		  Retval = main.Menu;
		  break;
	       }
	       case Calculator:
	       {
		  // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> create calculator fragment");

		  main.calculatorFragment = new Calculator_Fragment_ ();
		  Retval = main.calculatorFragment;
		  break;
	       }
	       case Printer:
	       {
		  // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> create printer fragment");

		  main.Printer = new Printer_Fragment_ ();
		  Retval = main.Printer;
		  break;
	       }
	       default:
	       {
		  Retval = null;
	       }
	    } // switch
	 }
	 else
	 {
	    Retval = null;
	 } // if

	 if (Retval != null)
	 {
	    Retval.Bind_Service (main.Service ());
	    Retval.Bind_Activity (main);
	    Retval.Bind_Preferences (main.Preferences);
	 } // if
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Retval : " + Retval);
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Pager_Adapter.getItem");
	 return Retval != null
	    ? Retval.As_Fragment ()
	    : new androidx.fragment.app.Fragment ();
      } // getItem

      /**
       * <p>
       * Get Title for page
       * </p>
       *
       * @param position
       *    page index
       * @return title string
       */
      @NotNull
      @Override
      public CharSequence getPageTitle (final int position)
      {
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ PagerAdapter.getPageTitle");
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> position  : " + position);

	 final Page_Type page = Page_Type.value (position);
	 final Main_Activity main = this.owner.get ();
	 final String Retval;
	 if (main != null)
	 {
	    final android.content.res.Resources resources = main.getResources ();
	    Retval = resources.getString (page.titleResId);
	 }
	 else
	 {
	    Retval = "";
	 }

	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> Retval     : " + Retval);
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- PagerAdapter.getPageTitle");
	 return Retval;
      } // getPageTitle

      /**
       * Returns the proportional width of a given page as a percentage of the ViewPager's measured width from
       * (0.f-1.f]
       *
       * @param position
       *    The position of the page requested
       * @return Proportional width for the given page position
       */
      @Override
      public float getPageWidth (final int position)
      {
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ getPageWidth");
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> position = " + position);

	 final Main_Activity main = this.owner.get ();
	 final float Retval;

	 if (main != null)
	 {
	    if ((position == Page_Type.Printer.pos) && this.Landscape_View ())
	    {
	       Retval = Pager_Adapter.Two_Third;
	    }
	    else if ((position == Page_Type.Menu.pos) && this.Landscape_View ())
	    {
	       Retval = Pager_Adapter.One_Half;
	    }
	    else if (position == Page_Type.Menu.pos)
	    {
	       Retval = Pager_Adapter.Two_Third;
	    }
	    else
	    {
	       Retval = 1f;
	    } // if
	 }
	 else
	 {
	    Retval = 1f;
	 } // if

	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Retval = " + Retval);
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- getPageWidth");
	 return Retval;
      } // getPageWidth

      @Override
      @org.jetbrains.annotations.TestOnly
      public String toString ()
      {
	 return Main_Activity.TAG + "{super=" + super.toString () + ", owner=" + this.owner + '}';
      }
   } // Pager_Adapter

   private class Toogle_On_Off
      implements
      android.widget.CompoundButton.OnCheckedChangeListener
   {
      /**
       * Called when the checked state of a compound button has changed.
       *
       * @param buttonView
       *    The compound button view whose state has changed.
       * @param isChecked
       *    The new checked state of buttonView.
       */
      @Override
      public void onCheckedChanged (final android.widget.CompoundButton buttonView, final boolean isChecked)
      {
	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ onCheckedChanged");
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> buttonView = " + buttonView);
	 // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> isChecked  = " + isChecked);

	 if (isChecked)
	 {
	    Main_Activity.this.Restart_Calculator ();
	 }
	 else
	 {
	    Main_Activity.this.Stop_Calculator ();
	 }
	 // Switch_To_Page (null);

	 // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- onCheckedChanged");
	 return;
      }
   }

   /**
    * <p>
    * class logger tag.
    * </p>
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Main_Activity.class);
   /**
    * Thread in which the virtual calculator CPU is running.
    */
   @Nullable
   @SuppressWarnings ({
      "FieldNotUsedInToString",
      "unused",
      "FieldCanBeLocal"})
   private Thread CPU_Thread;
   /**
    * <p>
    * the view of the on of menue etnry.
    * </p>
    */
   @Nullable
   private androidx.appcompat.widget.SwitchCompat On_Off_Switch;
   /**
    * <p>
    * Tablet view active — valid after After_Views
    * </p>
    */
   @Nullable
   private Boolean Tablet_View;
   /**
    * <p>
    * Menu fragment
    * </p>
    */
   @Nullable
   Menu_Fragment Menu;
   /**
    * <p>
    * the on off action bar enty
    * </p>
    */
   @org.androidannotations.annotations.OptionsMenuItem (resName = "Menu_On_Off")
   android.view.MenuItem Menu_On_Off;
   /**
    * <p>
    * Handles the pages in phone view
    * </p>
    */
   @org.androidannotations.annotations.ViewById (resName = "Main_Pager")
   @NotNull
   androidx.viewpager.widget.ViewPager Pager;
   /**
    * <p>
    * Calculator fragment
    * </p>
    *
    * @Nullable
    */
   @org.androidannotations.annotations.FragmentById (resName = "Main_Calculator")
   protected Calculator_Fragment calculatorFragment;
   /**
    * <p>
    * The application preferences
    * </p>
    */
   @org.androidannotations.annotations.sharedpreferences.Pref
   protected IPreferences_ Preferences;
   /**
    * <p>
    * Printer fragment (tablet only)
    * </p>
    */
   @Nullable
   protected Printer_Fragment Printer;

   /**
    * <p>
    * called after the views have been injected. Used to set values.
    * </p>
    */
   @org.androidannotations.annotations.AfterViews
   protected void After_Views ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ After_Views");

      if (this.calculatorFragment != null)
      {
	 this.After_Views_Tablet ();
	 this.Tablet_View = true;
      }
      else
      {
	 this.After_Views_Phone ();
	 this.Tablet_View = false;
      }

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- After_Views");
      return;
   } // afterViews

   /**
    * <p>
    * called after the views have been injected on phones. Used to set values.
    * </p>
    */
   private void After_Views_Phone ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Phone_After_Views");

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Phone_After_Views");
      return;
   } // Phone_After_Views

   /**
    * <p>
    * called after the views have been injected on tablets. Used to set values.
    * </p>
    */
   @SuppressWarnings ("FeatureEnvy")
   private void After_Views_Tablet ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Tablet_After_Views");

      final Calculator_Service Service = this.Service ();

      assert this.calculatorFragment != null : "Function only called on tablets.";

      this.calculatorFragment.Bind_Activity (this);
      this.calculatorFragment.Bind_Preferences (this.Preferences);
      this.calculatorFragment.Bind_Service (Service);

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Tablet_After_Views");
      return;
   } // Tablet_After_Views

   /**
    * Set calculator Thread so the canvas can wake up the calculator after key / touch screen clicks.
    *
    * @param CPU_Thread
    *    CPU Thread.
    */
   @Override
   public void bindCPUThread (@Nullable final Thread CPU_Thread)
   {
      this.CPU_Thread = CPU_Thread;

      return;
   } // Set_CPU_Thread

   /**
    * <p>
    * Bind the fragment to its activity. Note that the fragment manager might create new fragments (via reflection)
    * without the use of the page adapter.
    * </p>
    *
    * @param Fragment
    *    fragment to display (null => unbind fragment)
    * @param Page
    *    Page to which the fragment belongs.
    */
   @SuppressWarnings ("FeatureEnvy")
   @Override
   public void Bind_Fragment (@Nullable final IFragment Fragment, @NotNull final Page_Type Page)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Bind_Fragment");
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Fragment = " + Fragment);
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Page     = " + Page);

      switch (Page)
      {
	 case Menu:
	 {
	    if (Fragment == null)
	    {
	       this.Menu = null;
	    }
	    else if (Fragment instanceof Menu_Fragment)
	    {
	       if (this.Menu != null)
	       {
		  net.sourceforge.uiq3.Log.w (
		     Main_Activity.TAG,
		     "Rebind " + this.Menu.getTag () + " => " + Fragment.getTag ());
	       }
	       // noinspection CastToConcreteClass
	       this.Menu = (Menu_Fragment) Fragment;
	    }
	    else
	    {
	       net.sourceforge.uiq3.Log.e (
		  Main_Activity.TAG,
		  "LOG01880:The bound fragment is not menu page");
	    } // if

	    break;
	 }
	 case Calculator:
	 {
	    // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> bind calculator fragment");

	    if (Fragment == null)
	    {
	       this.calculatorFragment = null;
	    }
	    else if (Fragment instanceof Calculator_Fragment)
	    {
	       if (this.calculatorFragment != null)
	       {
		  net.sourceforge.uiq3.Log.w (
		     Main_Activity.TAG,
		     "Rebind " + this.calculatorFragment.getTag () + " => " + Fragment.getTag ());
	       }
	       // noinspection CastToConcreteClass
	       this.calculatorFragment = (Calculator_Fragment) Fragment;
	    }
	    else
	    {
	       net.sourceforge.uiq3.Log.e (
		  Main_Activity.TAG,
		  "LOG01880:The bound fragment is not calculator page");
	    } // if

	    break;
	 }
	 case Printer:
	 {
	    // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> bind printer fragment");

	    if (Fragment == null)
	    {
	       this.Printer = null;
	    }
	    else if (Fragment instanceof Printer_Fragment)
	    {
	       if (this.Printer != null)
	       {
		  net.sourceforge.uiq3.Log.w (
		     Main_Activity.TAG,
		     "Rebind " + this.Printer.getTag () + " => " + Fragment.getTag ());
	       }
	       // noinspection CastToConcreteClass
	       this.Printer = (Printer_Fragment) Fragment;
	    }
	    else
	    {
	       net.sourceforge.uiq3.Log.e (
		  Main_Activity.TAG,
		  "LOG01890:The bound fragment is not printer page");
	    } // if
	    break;
	 }
      } // switch

      final Calculator_Service Service = this.Service ();

      if ((Service != null) && (Fragment != null))
      {
	 Fragment.Bind_Service (Service);
      }

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Bind_Fragment");
      return;
   } // Bind_Fragment

   /**
    * <p>
    * Bind or unbind the calculator service
    * </p>
    */
   @Override
   public void Bind_Service (@Nullable final Calculator_Service Service)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Bind_Service");
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Service = " + Service);

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.Bind_Service (Service);
      } // if
      if (this.Printer != null)
      {
	 this.Printer.Bind_Service (Service);
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Bind_Service");
      return;
   } // Bind_Service

   /**
    * <p>
    * the clear printout button was clicked
    * </p>
    */
   @Override
   @TestOnly
   public void Button_Clear_Click ()
   {
      if (this.Printer != null)
      {
	 this.Printer.Button_Clear_Click ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01870:Printer fragment missing");
      } // if
   } // Button_Clear_Click

   /**
    * <p>
    * he form feed button was clicked
    * </p>
    */
   @Override
   @TestOnly
   public void Button_Feed_Click ()
   {
      if (this.Printer != null)
      {
	 this.Printer.Button_Feed_Click ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01860:Printer fragment missing");
      } // if
   } // Button_Feed_Click

   /**
    * <p>
    * the print display button was clicked
    * </p>
    */
   @Override
   @TestOnly
   public void Button_Print_Click ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Button_Print_Click");

      if (this.Printer != null)
      {
	 this.Printer.Button_Print_Click ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01850:Printer fragment missing");
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Button_Print_Click");
      return;
   } // Button_Print_Click

   /**
    * <p>
    * Calculator fragment (tablet only)
    * </p>
    */
   @TestOnly
   @Nullable
   public Calculator_Fragment Calculator ()
   {
      return this.calculatorFragment;
   }

   /**
    * <p>
    * Get calculator key code of the currently pressed key.
    * </p>
    *
    * @return calculator key code
    */
   @Override
   public short getPressedKey ()
   {
      final short Retval;

      if (this.calculatorFragment != null)
      {
	 Retval = this.calculatorFragment.getPressedKey ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01840:Calculator fragment missing");

	 Retval = net.sourceforge.uiq3.calculator.IKey.No_Key;
      }

      return Retval;
   } // Get_Pressed_Key

   /**
    * <p>
    * Get calculator key code of the last key (de)pressed.
    * </p>
    *
    * @return calculator key code
    */
   @Override
   public short getReadKey ()
   {
      final short Retval;

      if (this.calculatorFragment != null)
      {
	 Retval = this.calculatorFragment.getReadKey ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01830:Calculator fragment missing");

	 Retval = net.sourceforge.uiq3.calculator.IKey.No_Key;
      }

      return Retval;
   } // Get_Read_Key

   /**
    * <p>
    * Text currently displayed
    * </p>
    *
    * @return text currently displayed
    */
   @NotNull
   @Override
   public String getDisplayText ()
   {
      final String Retval;

      if (this.calculatorFragment != null)
      {
	 Retval = this.calculatorFragment.getDisplayText ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01831:Calculator fragment missing");

	 Retval = "";
      }

      return Retval;
   }

   /**
    * <p>
    * Text currently displayed
    * </p>
    *
    * @return text currently displayed
    */
   @NotNull
   public String getRegisterText ()
   {
      final StringBuilder retval = new StringBuilder (100);

      final Droid_Calculator rpn45 = getRPN45 ();

      if (rpn45 != null)
      {
	 for (int i = 0; i <= 6; i++)
	 {
	    retval.append (rpn45.registerToString (i));
	    retval.append (Printer_Fragment.New_Line);
	 }
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01832:Calculator missing");
      }

      return retval.toString ();
   }

   /**
    * <p>
    * Text currently displayed
    * </p>
    *
    * @return text currently displayed
    */
   @NotNull
   public String getMemoryText ()
   {
      final StringBuilder retval = new StringBuilder (100);

      final Droid_Calculator rpn45 = getRPN45 ();

      if (rpn45 != null)
      {
	 for (int i = 0; i <= 9; i++)
	 {
	    retval.append (rpn45.memoryToString (i));
	    retval.append (Printer_Fragment.New_Line);
	 }
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01832:Calculator missing");
      }

      return retval.toString ();
   }

   /**
    * <p>
    * Vibrate an button click
    * </p>
    *
    * @return true if we should vibrate
    */
   private boolean Is_New_Version ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Is_New_Version");

      final android.content.res.Resources resources = this.getResources ();
      final org.androidannotations.api.sharedpreferences.StringPrefField Version_Name =
	 this.Preferences.Version_Name ();
      final String Preference_Version = Version_Name.get ();
      final String Resource_Version = resources.getString (R.string.Version_Name);

      net.sourceforge.uiq3.Log.i (
	 Main_Activity.TAG,
	 "LOG01750:Version : " + Preference_Version + " -> " + Resource_Version);

      Version_Name.put (Resource_Version);

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Is_New_Version");
      return !Resource_Version.equalsIgnoreCase (Preference_Version);
   } // Is_New_Version

   /**
    * copy text to clip board
    */
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Reset")
   @Override
   public void Menu_Reset ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Reset");

      if (this.Menu != null)
      {
	 this.Menu.Menu_Reset ();
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Reset");
      return;
   } // Menu_Reset

   /**
    * copy text to clip board
    */
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Copy_Display")
   @Override
   public void Menu_Copy_Display ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Copy_Display");

      if (this.Menu != null)
      {
	 this.Menu.Menu_Copy_Display ();
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Copy_Display");
      return;
   } // Menu_Copy_Display

   /**
    * copy text to clip board
    */
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Copy_Printer")
   @Override
   public void Menu_Copy_Printer ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Copy_Printer");

      if (this.Menu != null)
      {
	 this.Menu.Menu_Copy_Display ();
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Copy_Printer");
      return;
   } // Menu_Copy_Printer

   /**
    * show documents activity
    */
   @Override
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Documents")
   public void Menu_Documents ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "≶ Menu_Documents");

      throw new IllegalStateException (Constants.Not_Needed);
   } // Menu_Documents

   /**
    * show feedback activity
    */
   @Override
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Feedback")
   public void Menu_Feedback ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Feedback");

      if (this.Menu != null)
      {
	 this.Menu.Menu_Feedback ();
      } // if

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Feedback");
      return;
   } // Menu_Feedback

   /**
    * show help activity
    */
   @Override
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Help")
   public void Menu_Help ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Help");

      if (this.Menu != null)
      {
	 this.Menu.Menu_Help ();
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Help");
      return;
   } // Menu_Help

   /**
    * show preferences
    */
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Settings")
   @Override
   public void Menu_Settings ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Menu_Settings");

      this.startActivity (new android.content.Intent (this, Preferences_Activity_.class));
      this.Switch_To_Page (null);

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Menu_Settings");
      return;
   } // Menu_Settings

   /**
    * <p>
    * Printer fragment (tablet only)
    * </p>
    */
   @TestOnly
   @Nullable
   public Printer_Fragment Printer ()
   {
      return this.Printer;
   } // Test_Printer

   /**
    * <p>
    * Text currently displayed
    * </p>
    *
    * @return text currently displayed
    */
   @Override
   @NotNull
   @TestOnly
   public String Printout ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Printout");

      final String Retval;

      if (this.Printer != null)
      {
	 Retval = this.Printer.Printout ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01832:Printer fragment missing");

	 Retval = "";
      }

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Printout" + "\n> Retval = „%1$s“", Retval);
      return Retval;
   } // Printout

   /**
    * Set text to be displayed (java string).
    *
    * @param Text
    *    to be displayed
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#Printout(String)
    */
   @Override
   @TestOnly
   public void Printout (@NotNull final String Text)
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Printout" + "\n> Text = „%1$s“", Text);

      if (this.Printer != null)
      {
	 this.Printer.Printout (Text);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01825:Printer fragment missing");
      } // if

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Printout");
      return;
   } // Printout

   /**
    * <p>
    * Check if a calculator key code is available
    * </p>
    *
    * @return true when a key is available.
    */
   @Override
   @TestOnly
   public final boolean isReadKeyAvailable ()
   {
      return (this.calculatorFragment != null) && this.calculatorFragment.isReadKeyAvailable ();
   } // isReadKeyAvailable

   /**
    * <p>
    * Start service if it is not currently running.
    * </p>
    */
   @Override
   public void Restart_Calculator ()
   {
      final net.sourceforge.uiq3.calculator.IService Service = Main_Activity.this.Service ();

      if (((Service == null) || !Service.isCalculatorStarted ()) && (this.Service_Intend != null))
      {
	 this.startService (this.Service_Intend);
      } // if

      return;
   }

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    *    indicators to be shown
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setIndicators(boolean[])
    */
   @Override public void
   setIndicators (final boolean @NotNull [] Indicators)
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ setIndicators");

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setIndicators (Indicators);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01791:Calculator fragment missing");
      } // if

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- setIndicators");
      return;
   } // setIndicators

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    *    indicators to be shown
    * @noinspection ZeroLengthArrayAllocation
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setIndicators(boolean[])
    */
   @Override public boolean @NotNull []
   getIndicators ()
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ getIndicators");

      final boolean @NotNull [] retval;

      if (this.calculatorFragment != null)
      {
	 retval = this.calculatorFragment.getIndicators ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01791:Calculator fragment missing");
	 retval = new boolean[0];
      } // if

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- getIndicators");
      return retval;
   } // getIndicators

   /**
    * <p>
    * set the calculators on of status.
    * </p>
    *
    * @param on
    *    calculator is switched on
    */
   @Override
   public void setOnOff (final boolean on)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Set_On_Off");
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> on = " + on);

      if (this.On_Off_Switch != null)
      {
	 this.runOnUiThread (() -> Main_Activity.this.On_Off_Switch.setChecked (on)); // Runnable
      }

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Set_On_Off");
      return;
   }

   /**
    * Get calculator key code of the currently pressed key.
    *
    * @param Pressed_Key
    *    calculator key code
    */
   @Override
   public void setPressedKey (final short Pressed_Key)
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ Set_Pressed_Key");

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setPressedKey (Pressed_Key);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01790:Calculator fragment missing");
      } // if

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- Set_Pressed_Key");
      return;
   } // Set_Pressed_Key

   /**
    * <p>
    * Get calculator key code of the last key (de)pressed.
    * </p>
    *
    * @param Read_Key
    *    current key read
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setReadKey()
    */
   @Override
   public void setReadKey (final short Read_Key)
   {
      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setReadKey (Read_Key);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01810:Calculator fragment missing");
      } // if

      return;
   } // Set_Read_Key

   /**
    * <p>
    * Get calculator key code of the last key (de)pressed.
    * </p>
    *
    * @param Read_Keys
    *    current key read
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setReadKeys(short..)
    */
   @Override
   public void setReadKeys (final short @NotNull ... Read_Keys)
   {
      net.sourceforge.uiq3.Log.d (
	 TAG,
	 "+ Set_Read_Key" + "\n> Set_Read_Keys  = %1$d",
	 java.util.Arrays.toString (Read_Keys));

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setReadKeys (Read_Keys);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (Main_Activity.TAG, "LOG01810:Calculator fragment missing");
      } // if

      net.sourceforge.uiq3.Log.d (TAG, "- Set_Read_Keys");
      return;
   } // Set_Read_Keys

   /**
    * Set text to be displayed (calculator alpha codes).
    *
    * @param Row
    *    display row
    * @param Alpha_Text
    *    to be displayed
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setDisplayText(int, short[])
    */
   @Override
   public void setDisplayText (final int Row, final short @NotNull [] Alpha_Text)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Set_Text");
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Row = " + Row);
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Alpha_Text = " + java.util.Arrays.toString
      // (Alpha_Text));

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setDisplayText (Row, Alpha_Text);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01791:Calculator fragment missing");
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Set_Text");
      return;
   } // Set_Text

   /**
    * Set text to be displayed (java string).
    *
    * @param Text
    *    to be displayed
    * @see net.sourceforge.uiq3.ui.ICalculatorCanvas#setJavaText(String)
    */
   @Override
   public void setJavaText (@NotNull final String Text)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ Set_Text");
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "> Text = " + Text.length ());

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.setJavaText (Text);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01820:Calculator fragment missing");
      } // if

      // this.Printout (Text);

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- Set_Text");
      return;
   } // Set_Text

   @Override
   public short @NotNull [] getDisplayText (final int row)
   {
      return this.calculatorFragment.getDisplayText (row);
   }

   @Override
   public @NotNull String getJavaText (final int row)
   {
      return this.calculatorFragment.getJavaText (row);
   }

   @Override
   public @NotNull String getJavaText ()
   {
      return this.calculatorFragment.getJavaText ();
   }

   /**
    * <p>
    * show the action bar (a click event handler, declared in the layout xml)
    * </p>
    *
    * @param view
    *    dummy — needed for call via reflection
    */
   @SuppressWarnings ("UnusedParameters")
   public void Show_Menu (@Nullable final android.view.View view)
   {
      this.Switch_To_Page (Page_Type.Menu);

      return;
   } // Show_Menu

   /**
    * get calculator from service
    */
   @Nullable
   public Droid_Calculator getRPN45 ()
   {
      final Calculator_Service Service = this.Service ();
      return (Service != null)
	 ? Service.getRPN45 ()
	 : null;
   }

   /**
    * <p>
    * Stop the calculator
    * </p>
    */
   @Override
   public void Stop_Calculator ()
   {
      final Calculator_Service Service = this.Service ();

      if ((Service != null) && Service.isCalculatorStarted ())
      {
	 final Droid_Calculator calculator = Service.getRPN45 ();

	 if (calculator != null)
	 {
	    calculator.stop ();
	 }
      } // if
   }

   /**
    * <p>
    * switches the page (in phone layout).
    * </p>
    *
    * @param Page
    *    page to display
    */
   @Override
   @TestOnly
   public void Switch_To_Page (@Nullable final Page_Type Page)
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- switchToPage" + "\n> page : %1$s", Page);

      if (Page == null)
      {
	 if (this.Tablet_View ())
	 {
	    this.Pager.setCurrentItem (Page_Type.Printer.pos);
	 }
	 else
	 {
	    this.Pager.setCurrentItem (Page_Type.Calculator.pos);
	 } // if
      }
      else
      {
	 this.Pager.setCurrentItem (Page.pos);
      } // if

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- switchToPage");
   } // Switch_To_Page

   /**
    * <p>
    * check if wa are in tablet mode
    * </p>
    *
    * @return true if we are in tablet mode
    */
   boolean Tablet_View ()
   {
      Required.notNull (this.Tablet_View, "Tablet mode is only known between onCreate and onDestroy");

      return this.Tablet_View;
   }

   /**
    * Returns the name of the resources (used for logging and idempotency of registration).
    */
   @Override
   public String getName ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "± getName");

      return Calculator_Fragment.TAG;
   }

   /**
    * Returns {@code true} if resource is currently idle. Espresso will <b>always</b> call this method from the main
    * thread, therefore it should be non-blocking and return immediately.
    */
   @Override
   public boolean isIdleNow ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ isIdleNow");

      final boolean retval;

      if (this.calculatorFragment != null)
      {
	 retval = this.calculatorFragment.isIdleNow ();
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01840:Calculator fragment missing");

	 retval = true;
      } // if

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- isIdleNow" + "\n> retval = %1$b", retval);
      return retval;
   }

   /**
    * Called when the activity is first created.
    *
    * @see android.app.Activity#onCreate(android.os.Bundle)
    */
   @SuppressLint ("NewApi")
   @Override
   public void onCreate (@Nullable final android.os.Bundle Saved_Instance_State)
   {
      // net.sourceforge.uiq3.Log.d (Activity.TAG, "+ onCreate");
      // net.sourceforge.uiq3.Log.v (Activity.TAG, "  > savedInstanceState      = " +
      // savedInstanceState);

      super.onCreate (Saved_Instance_State);

      if (this.Service_Intend == null)
      {
	 // net.sourceforge.uiq3.Log.v (Activity.TAG, "  > Create Intend .");

	 this.Service_Intend = new android.content.Intent (
	    /* packageContext => */this,
	    /* cls            => */Calculator_Service_.class);
      } // if

      // net.sourceforge.uiq3.Log.d (Activity.TAG, "- onCreate");
      return;
   } // onCreate

   /**
    * Initialize the contents of the Activity's standard options menu. You should place your menu items in to
    * <var>menu</var>.
    * <p/>
    * <p>
    * This is only called once, the first time the options menu is displayed. To update the menu every time it is
    * displayed, see {@link #onPrepareOptionsMenu}.
    * <p/>
    * <p>
    * The default implementation populates the menu with standard system menu items. These are placed in the
    * {@link android.view.Menu#CATEGORY_SYSTEM} group so that they will be correctly ordered with application-defined
    * menu items. Deriving classes should always call through to the base implementation.
    * <p/>
    * <p>
    * You can safely hold on to <var>menu</var> (and any items created from it), making modifications to it as desired,
    * until the next time onCreateOptionsMenu() is called.
    * <p/>
    * <p>
    * When you add items to the menu, you can implement the Activity's {@link #onOptionsItemSelected} method to handle
    * them there.
    *
    * @param menu
    *    The options menu in which you place your items.
    * @return You must return true for the menu to be displayed; if you return false it will not be shown.
    * @see #onPrepareOptionsMenu
    * @see #onOptionsItemSelected
    */
   @Override
   public boolean onCreateOptionsMenu (final android.view.Menu menu)
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ onCreateOptionsMenu");
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> menu = " + menu);

      this.On_Off_Switch = (androidx.appcompat.widget.SwitchCompat) androidx.core.view.MenuItemCompat.getActionView (
	 this.Menu_On_Off);

      assert this.On_Off_Switch != null : "Views should now be set";

      this.On_Off_Switch.setOnCheckedChangeListener (new Main_Activity.Toogle_On_Off ());
      this.On_Off_Switch.setChecked (true);

      final boolean retval = super.onCreateOptionsMenu (menu);

      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> retval = " + retval);
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- onCreateOptionsMenu");

      return retval;
   } // onCreateOptionsMenu

   /**
    * @see android.app.Activity#onDestroy()
    */
   @SuppressWarnings ({
      "AssignmentToNull",
      "ConstantConditions"})
   @Override
   protected void onDestroy ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ onDestroy");
      super.onDestroy ();

      this.Service_Intend = null;
      this.calculatorFragment = null;
      this.Printer = null;
      this.Pager = null;
      this.Tablet_View = null;

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- onDestroy");
      return;
   } // onDestroy

   /**
    * Called after onRestoreInstanceState(Bundle), onRestart(), or onPause(), for your activity to start interacting
    * with the user. This is a good place to begin animations, open exclusive-access devices (such as the camera), etc
    *
    * @see android.app.Activity#onResume()
    */
   @SuppressLint ("NewApi")
   @Override
   public final void onResume ()
   {
      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ onResume");
      super.onResume ();

      if (this.Connection != null)
      {
	 final net.sourceforge.uiq3.calculator.IService Service = this.Service ();

	 if (Service != null)
	 {
	    Service.restoreStatus ();
	 } // if
      } // if

      // net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- onResume");
      return;
   } // onResume

   /**
    * @param Out_State
    *    out state
    * @see android.app.Activity#onSaveInstanceState(android.os.Bundle)
    */
   @Override
   public void onSaveInstanceState (@NotNull final android.os.Bundle Out_State)
   {
      net.sourceforge.uiq3.Log.d (TAG, "+ onSaveInstanceState" + "> Out_State = „%1$.32s“", Out_State);

      super.onSaveInstanceState (Out_State);

      final Calculator_Service Service = this.Service ();

      if (Service != null)
      {
	 Service.saveStatus ();
      } // if

      net.sourceforge.uiq3.Log.d (TAG, "- onSaveInstanceState" + "> Out_State = „%1$.32s“", Out_State);
      return;
   } // onSaveInstanceState

   /**
    * <p>
    * Open calculator and load file (if one was passed).
    * </p>
    * <p>
    * Open calculator
    * </p>
    *
    * @see android.app.Activity#onStart()
    */
   @Override
   public void onStart ()
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "+ onStart");

      super.onStart ();

      if (this.Is_New_Version ())
      {
	 this.startActivity (new android.content.Intent (this, Info_Activity_.class));
      } // if

      final Main_Activity.Pager_Adapter Fragment_Adapter = new Pager_Adapter (this);

      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Fragment_Adapter = " + Fragment_Adapter);
      // net.sourceforge.uiq3.Log.v (Main_Activity.TAG, "> Pager            = " + this.Pager);

      this.Pager.setAdapter (Fragment_Adapter);
      this.Switch_To_Page (null);

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- onStart");
      return;
   } // onStart

   /**
    * Registers the given {@link ResourceCallback} with the resource. Espresso will call this method:
    *
    * <ul>
    *   <li>with its implementation of {@link ResourceCallback} so it can be notified asynchronously
    *       that your resource is idle
    *   <li>from the main thread, but you are free to execute the callback's onTransitionToIdle from
    *       any thread
    *   <li>once (when it is initially given a reference to your IdlingResource)
    * </ul>
    *
    * <p>You only need to call this upon transition from busy to idle - if the resource is already
    * idle when the method is called invoking the call back is optional and has no significant
    * impact.
    *
    * @param callback
    */
   @Override
   public void registerIdleTransitionCallback (final androidx.test.espresso.IdlingResource.ResourceCallback callback)
   {
      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- isIdleNow" + "\n> callback = %1$b", callback);

      if (this.calculatorFragment != null)
      {
	 this.calculatorFragment.registerIdleTransitionCallback (callback);
      }
      else
      {
	 net.sourceforge.uiq3.Log.w (
	    Main_Activity.TAG,
	    "LOG01840:Calculator fragment missing");
      } // if

      net.sourceforge.uiq3.Log.d (Main_Activity.TAG, "- registerIdleTransitionCallback");
      return;
   } // registerIdleTransitionCallback

   /**
    * <p>
    * Generate string for debug purpose
    * </p>
    *
    * @return string with debug informations
    */
   @SuppressWarnings ("HardcodedLineSeparator")
   @NotNull
   @org.jetbrains.annotations.TestOnly
   @Override
   public String toString ()
   {
      return Main_Activity.TAG + "{\nsuper=" + super.toString () + ",\nPreferences=" +
	 this.Preferences + ",\nTablet_View=" + this.Tablet_View + ",\nMenu=" + this.Menu + ",\nPrinter=" +
	 this.Printer + ",\nPager=" + this.Pager + '}';
   }
} // Main_Activity

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
