/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7700 $
 *   $Date: 2021-09-28 11:21:12 +0200 (Di, 28. Sep 2021) $
 *   $Id: Menu_Fragment.java 7700 2021-09-28 09:21:12Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

/**
 * <p>
 * The menu fragment for the calculator. menu are managed here.
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EFragment (resName = "menu")
public class Menu_Fragment
   extends
   net.sourceforge.uiq3.calculator.Menu_Fragment<Calculator_Service, Main_Activity, IPreferences_>
   implements
   IFragment,
   net.sourceforge.uiq3.calculator.IMenu_Fragment
{
   /**
    * <p>
    * class logger tag.
    * </p>
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Main_Activity.class);
   /**
    * <p>
    * for storing the display and the printout on the clipboard
    * </p>
    */
   @org.androidannotations.annotations.SystemService
   android.content.ClipboardManager Manager;
   /**
    * <p>
    * the toolbar for the main activity is only visible in the menu fragment
    * </p>
    */
   @org.androidannotations.annotations.ViewById (resName = "Menu_Toolbar")
   androidx.appcompat.widget.Toolbar Toolbar;

   /**
    * <p>
    * called after the views have been injected. Used to set values.
    * </p>
    */
   @org.androidannotations.annotations.AfterViews
   protected void After_Views ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ After_Views");

      // this.Toolbar.setLogo (R.drawable.ic_menu);

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 activity.setSupportActionBar (this.Toolbar);
      }

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- After_Views");
      return;
   } // After_Views

   /**
    * <p>
    * Perform a full calculator reset
    * </p>
    */
   @org.androidannotations.annotations.Click (resName = "Menu_Reset")
   @Override
   public void Menu_Reset ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Reset");

      final Main_Activity activity = this.Activity ();
      if (activity != null)
      {
	 final Droid_Calculator rpn45 = activity.getRPN45 ();

	 if (rpn45 != null)
	 {
	    rpn45.resetAll ();
	 } // if
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Reset");
      return;
   } // Menu_On

   /**
    * <p>
    * On slide in menu was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Menu_Copy_Display")
   public void Menu_Copy_Display ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Copy_Display");

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 this.Manager.setPrimaryClip (android.content.ClipData.newPlainText (
	    "RPN-45 Display",
	    activity.getDisplayText ()));

	 activity.Switch_To_Page (null);
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Copy_Display");
      return;
   } // Menu_On

   /**
    * <p>
    * On slide in menu was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Menu_Copy_Printer")
   public void Menu_Copy_Printer ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Copy_Printer");

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 this.Manager.setPrimaryClip (android.content.ClipData.newPlainText ("RPN-45 Printout", activity.Printout ()));
	 activity.Switch_To_Page (null);
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Copy_Printer");
      return;
   } // Menu_Copy_Printer

   /**
    * show documents activity
    */
   @Override
   @org.androidannotations.annotations.OptionsItem (resName = "Menu_Documents")
   public void Menu_Documents ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "≶ Menu_Documents");

      throw new IllegalStateException (net.sourceforge.uiq3.hp45.Constants.Not_Needed);
   } // Menu_Documents

   /**
    * <p>
    * Feedback slide in menu was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Menu_Feedback")
   public void Menu_Feedback ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Feedback");

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 activity.startActivity (new android.content.Intent (activity, Contact_Activity_.class));
	 activity.Switch_To_Page (null);
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Feedback");
      return;
   } // Menu_Feedback

   /**
    * <p>
    * Feedback slide in menu was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Menu_Help")
   public void Menu_Help ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Help");

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 activity.startActivity (new android.content.Intent (activity, Help_Activity_.class));
	 activity.Switch_To_Page (null);
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Help");
      return;
   } // Menu_Help

   /**
    * <p>
    * Settings slide in menu was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Menu_Settings")
   public void Menu_Settings ()
   {
      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "+ Menu_Settings");

      final Main_Activity activity = this.Activity ();

      if (activity != null)
      {
	 activity.startActivity (new android.content.Intent (activity, Preferences_Activity_.class));
	 activity.Switch_To_Page (null);
      } // if

      net.sourceforge.uiq3.Log.d (Menu_Fragment.TAG, "- Menu_Settings");
      return;
   } // Menu_Settings

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override
   public @org.jetbrains.annotations.NotNull String toString ()
   {
      return Menu_Fragment.TAG + "{super=" + super.toString () + "\n, Manager=" + this.Manager + "\n, Toolbar=" +
	 this.Toolbar + '}';
   }
} // Menu_Fragment

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
