/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7750 $
 *   $Date: 2021-11-29 11:43:17 +0100 (Mo, 29. Nov 2021) $
 *   $Id: Preferences_Activity.java 7750 2021-11-29 10:43:17Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;

/**
 * @author martin
 */
@SuppressWarnings ({"HardcodedLineSeparator"})
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EActivity (resName = "preferences")
public class Preferences_Activity
   extends
   net.sourceforge.uiq3.calculator.Preferences_Activity
{
   /**
    * logging tag
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Preferences_Activity.class);
   /**
    * cache the preferences.
    */
   private static net.sourceforge.uiq3.hp45.IPreferences_ Preferences;

   /**
    * <p>
    * Mark preferences as changed for later backup
    * </p>
    *
    * @param Context
    *    context in which we operate on
    */
   @SuppressWarnings ("UnusedDeclaration")
   static synchronized void Data_Changed (@NotNull final android.content.Context Context)
   {
      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "+ Data_Changed" + "\n> Context : %1$.16s", Context);

      if (net.sourceforge.uiq3.calculator.Preferences_Activity.Backup_Manager == null)
      {
	 net.sourceforge.uiq3.calculator.Preferences_Activity.Backup_Manager = new android.app.backup.BackupManager (
	    Context);
      } // if

      net.sourceforge.uiq3.calculator.Preferences_Activity.Backup_Manager.dataChanged ();

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "- Data_Changed");
      return;
   } // Data_Changed

   /**
    * Save / load using the FA 2 interface
    *
    * @param Context
    *    we need a Context to read the preferences
    * @return bytes to be kept in RPN-45 printout
    */
   static Error_Handle_Type Error_Handler (final android.content.Context Context)
   {
      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "+ Error_Handler" + "\n> Context : %1$.16s", Context);

      final net.sourceforge.uiq3.hp45.IPreferences_ Preferences = new net.sourceforge.uiq3.hp45.IPreferences_ (Context);
      Error_Handle_Type Retval;

      try
      {
	 final org.androidannotations.api.sharedpreferences.StringPrefField field = Preferences.Error_Handler ();
	 final String Text = field.get ();
	 Retval = Error_Handle_Type.value (Integer.parseInt (Text));
      }
      catch (@NotNull final NumberFormatException Exception)
      {
	 net.sourceforge.uiq3.Log.e (Preferences_Activity.TAG, "LOG01910: Wrong printer size — reset to default");

	 Retval = Error_Handle_Type.None;
      } // if
      catch (@NotNull final IllegalArgumentException Exception)
      {
	 net.sourceforge.uiq3.Log.e (Preferences_Activity.TAG, "LOG01911: Wrong printer size — reset to default");

	 Retval = Error_Handle_Type.None;
      } // if

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "- Error_Handler");
      return Retval;
   } // Error_Handler

   /**
    * set the large screen (used for unit tests only).
    *
    * @param Context
    *    we need a context to read the preferences
    * @param Error_Handler
    *    printout text
    */
   @org.jetbrains.annotations.TestOnly
   public static void Error_Handler (
      @NotNull final android.content.Context Context,
      @NotNull final Error_Handle_Type Error_Handler)
   {
      net.sourceforge.uiq3.Log.d (
	 Preferences_Activity.TAG,
	 "+ Error_Handler" + "\n> Context                 = %1$.16s" + "\n> Error_Handler           = %2$.16s",
	 Context, Error_Handler);

      final net.sourceforge.uiq3.hp45.IPreferences_ Preferences = Preferences_Activity.Preferences (Context);
      final org.androidannotations.api.sharedpreferences.StringPrefField field = Preferences.Error_Handler ();
      field.remove ();
      field.put (String.valueOf (Error_Handler.pos));

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "- Error_Handler");
      return;
   } // Error_Handler

   /**
    * <p>
    * Vibrate an button click
    * </p>
    *
    * @param Context
    *    we need a Context to read the preferences
    * @return true if we should vibrate
    */
   static boolean Feedback_Vibrate (final android.content.Context Context)
   {
      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "+ Feedback_Vibrate" + "\n> Context : %1$.16s", Context);

      final net.sourceforge.uiq3.hp45.IPreferences_ Preferences = new net.sourceforge.uiq3.hp45.IPreferences_ (Context);
      final org.androidannotations.api.sharedpreferences.BooleanPrefField field = Preferences.Feedback_Vibrate ();
      final boolean Retval = field.get ();

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "- Feedback_Vibrate");
      return Retval;
   } // Feedback_Vibrate

   /**
    * set feedback vibrate (used for unit tests only).
    *
    * @param Context
    *    we need a context to read the preferences
    * @param Error_Handler
    *    printout text
    */
   @org.jetbrains.annotations.TestOnly
   public static void Feedback_Vibrate (@NotNull final android.content.Context Context, final boolean Enable)
   {
      net.sourceforge.uiq3.Log.d (
	 Preferences_Activity.TAG,
	 "+ Feedback_Vibrate" + "\n> Context                 = %1$.16s" + "\n> Enable                = %2$.16s",
	 Context, Enable);

      final net.sourceforge.uiq3.hp45.IPreferences_ Preferences = Preferences_Activity.Preferences (Context);
      final org.androidannotations.api.sharedpreferences.BooleanPrefField field = Preferences.Feedback_Vibrate ();
      field.remove ();
      field.put (Enable);

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "- Feedback_Vibrate");
      return;
   } // Feedback_Vibrate

   /**
    * get the RPN-45 printout
    *
    * @param Context
    *    we need a context to read the preferences
    * @return printout text
    */
   static String HP45_Printout (@NotNull final android.content.Context Context)
   {
      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "+ HP45_Printout" + "\n> Context : %1$.16s", Context);

      final IPreferences_ Preferences = Preferences_Activity.Preferences (Context);
      final org.androidannotations.api.sharedpreferences.StringPrefField field = Preferences.HP45_Printout ();
      final String Retval = field.get ();

      net.sourceforge.uiq3.Log.d (
	 Preferences_Activity.TAG, "+ HP45_Printout" + "\n> Retval  : %1$.16s", Retval);
      return Retval;
   } // Get_HP45_Printout

   /**
    * get the RPN-45 printout
    *
    * @param Context
    *    we need a context to read the preferences
    * @param Printout
    *    printout text
    */
   static void HP45_Printout (
      @NotNull final android.content.Context Context,
      @NotNull final String Printout)
   {
      net.sourceforge.uiq3.Log.d (
	 Preferences_Activity.TAG,
	 "+ HP45_Printout" + "  > Context                 = %1$.32s" + "  > Printout                = „%2$.32s“",
	 Context, Printout);

      final IPreferences_ Preferences = Preferences_Activity.Preferences (Context);
      final org.androidannotations.api.sharedpreferences.StringPrefField field = Preferences.HP45_Printout ();
      field.put (Printout);

      net.sourceforge.uiq3.Log.d (Preferences_Activity.TAG, "+ HP45_Printout");
      return;
   } // HP45_Printout

   /**
    * preferences from context
    *
    * @param Context
    *    a context (only needed on the first call — but how will you know you are the first to call)
    * @return the prefrences
    */
   static synchronized net.sourceforge.uiq3.hp45.IPreferences_ Preferences (
      @NotNull final android.content.Context Context)
   {
      if (Preferences_Activity.Preferences == null)
      {
	 Preferences_Activity.Preferences = new net.sourceforge.uiq3.hp45.IPreferences_ (Context);
      } // if

      return Preferences_Activity.Preferences;
   } // Preferences

   /**
    * Called when the views are prepared.
    */
   @Override
   @org.androidannotations.annotations.AfterViews
   public void After_Views ()
   {
      super.After_Views ();
   } // AfterViews

   @Override
   protected void Directory_Locator (final android.net.Uri value)
   {
      throw new IllegalStateException (net.sourceforge.uiq3.hp45.Constants.Not_Needed);
   }

   @Override
   protected android.net.Uri Directory_Locator ()
   {
      throw new IllegalStateException (net.sourceforge.uiq3.hp45.Constants.Not_Needed);
   }
} // Preferences_Activity

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
