/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7750 $
 *   $Date: 2021-11-29 11:43:17 +0100 (Mo, 29. Nov 2021) $
 *   $Id: Printer_Fragment.java 7750 2021-11-29 10:43:17Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * <p>
 * the printer fragment
 * </p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@SuppressWarnings ("HardcodedLineSeparator")
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
@org.androidannotations.annotations.EFragment (resName = "printer")
public class Printer_Fragment
   extends
   net.sourceforge.uiq3.calculator.Printer_Fragment<Calculator_Service, Main_Activity, IPreferences_>
   implements
   IFragment
{
   /**
    * logging tag
    */
   private static final String TAG = net.sourceforge.uiq3.Log.createTag (Printer_Fragment.class);
   private android.app.Activity activity;
   /**
    * <p>
    * the clear printout button
    * </p>
    */
   @org.androidannotations.annotations.ViewById (resName = "Button_Clear")
   protected android.widget.Button Button_Clear;
   /**
    * <p>
    * the form feed button
    * </p>
    */
   @org.jetbrains.annotations.TestOnly
   @org.androidannotations.annotations.ViewById (resName = "Button_Feed")
   protected android.widget.Button Button_Feed;
   /**
    * <p>
    * the print display botton
    * </p>
    */
   @org.jetbrains.annotations.TestOnly
   @org.androidannotations.annotations.ViewById (resName = "Button_Print_Display")
   protected android.widget.Button Button_Print_Display;
   /**
    * <p>
    * the print display botton
    * </p>
    */
   @org.jetbrains.annotations.TestOnly
   @org.androidannotations.annotations.ViewById (resName = "Button_Print_Register")
   protected android.widget.Button Button_Print_Register;
   /**
    * <p>
    * the print display botton
    * </p>
    */
   @org.jetbrains.annotations.TestOnly
   @org.androidannotations.annotations.ViewById (resName = "Button_Print_Memory")
   protected android.widget.Button Button_Print_Memory;
   /**
    * <p>
    * Printout widget.
    * </p>
    */
   @org.androidannotations.annotations.ViewById (resName = "Tablet_Printout")
   @Nullable
   protected android.widget.TextView Printout;

   /**
    * <p>
    * init widgets
    * </p>
    */
   @org.androidannotations.annotations.AfterViews
   protected void After_Views ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ After_Views");

      // super.onCreated reads Utf16_Text in savedInstanceState
      //
      if (this.Printout != null)
      {
	 this.Printout.setText (this.Old_Printout);
      }

      if (this.Activity != null)
      {
	 this.Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Printer);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- After_Views");
      return;
   } // After_Views

   /**
    * <p>
    * the clear printout button was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Button_Clear")
   public final void Button_Clear_Click ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Button_Clear_Click");

      this.Printout ("");

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Button_Clear_Click");
      return;
   } // Button_Clear_Click

   /**
    * <p>
    * the form feed button was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Button_Feed")
   public final void Button_Feed_Click ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Button_Feed_Click");

      final String New_Printout = this.Old_Printout + Printer_Fragment.New_Line;
      this.Printout (New_Printout);

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Button_Feed_Click");
      return;
   } // Button_Feed_Click

   /**
    * <p>
    * the primary print button was clicked
    * </p>
    */
   @Override
   @org.androidannotations.annotations.Click (resName = "Button_Print_Display")
   public final void Button_Print_Click ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Button_Print_Click");

      if (this.Activity != null)
      {
	 final String newPrintout =
	    this.Old_Printout + Printer_Fragment.New_Line + this.Activity.getDisplayText ();
	 this.Printout (newPrintout);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Button_Print_Click");
      return;
   } // Button_Print_Click

   /**
    * <p>
    * the primary print button was clicked
    * </p>
    */
   @org.androidannotations.annotations.Click (resName = "Button_Print_Register")
   public final void Button_Register_Click ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Button_Register_Click");

      if (this.Activity != null)
      {
	 final String newPrintout =
	    this.Old_Printout + Printer_Fragment.New_Line + this.Activity.getRegisterText ();

	 this.Printout (newPrintout);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Button_Register_Click");
      return;
   } // Button_Register_Click

   /**
    * <p>
    * the primary print button was clicked
    * </p>
    */
   @org.androidannotations.annotations.Click (resName = "Button_Print_Memory")
   public final void Button_Memory_Click ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Button_Memory_Click");

      if (this.Activity != null)
      {
	 final String newPrintout =
	    this.Old_Printout + Printer_Fragment.New_Line + this.Activity.getMemoryText ();

	 this.Printout (newPrintout);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Button_Memory_Click");
      return;
   } // Button_Register_Click

   /**
    * Set text to be displayed (java string).
    *
    * @param Text
    *    to be displayed
    */
   @Override
   @org.androidannotations.annotations.UiThread
   public void Printout (@NotNull final String Text)
   {
      if (!Text.equals (this.Old_Printout))
      {
	 net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ Printout" + "\n> Text = „%1$s“", Text);

	 if (this.Printout != null)
	 {
	    this.Printout.setText (Text);
	 } // if

	 super.Printout (Text);

	 net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- Printout");
      } // if
      return;
   } // Printout

   /**
    * <p>
    * Test if the Clear button is set and click able. That happens when the fragment is loaded into memory
    * </p>
    */
   @org.jetbrains.annotations.TestOnly
   public boolean Test_Button_Clear ()
   {
      return this.Button_Clear != null && this.Button_Clear.isClickable ();
   }

   /**
    * Called when the fragment's activity has been created and this fragment's view hierarchy instantiated. It can be
    * used to do final initialization once these pieces are in place, such as retrieving views or restoring state. It is
    * also useful for fragments that use {@link #setRetainInstance(boolean)} to retain their instance, as this callback
    * tells the fragment when it is fully associated with the new activity instance. This is called after
    * {@link #onCreateView} and before {@link #onStart()}.
    *
    * @param savedInstanceState
    *    If the fragment is being re-created from a previous saved state, this is the state.
    */
   @Override
   public void onActivityCreated (@Nullable final android.os.Bundle savedInstanceState)
   {
      net.sourceforge.uiq3.Log.d (
	 Printer_Fragment.TAG, "+ onActivityCreated" + "\n> savedInstanceState = %1$s", savedInstanceState);

      // super.onActivityCreated reads Utf16_Text in savedInstanceState
      //
      super.onActivityCreated (savedInstanceState);

      final android.app.Activity activity = this.getActivity ();

      if (activity != null)
      {
	 Old_Printout = net.sourceforge.uiq3.hp45.Preferences_Activity.HP45_Printout (activity);

	 final android.graphics.Typeface Typeface = net.sourceforge.uiq3.Fonts.dejavusans_mono (activity);

	 if (this.Printout != null)
	 {
	    this.Printout.setTypeface (Typeface, android.graphics.Typeface.BOLD);
	 } // if
      } // if

      if (this.Printout != null)
      {
	 this.Printout.setText (this.Old_Printout);
      }

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- onActivityCreated");
      return;
   } // onActivityCreated

   /**
    * Called when a fragment is first attached to its activity. {@link #onCreate(android.os.Bundle)} will be called
    * after this.
    */
   @Override
   public void onAttach (@androidx.annotation.NonNull final android.app.Activity activity)
   {
      this.activity = activity;
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ onAttach" + "\n> activity = %1$.32s", activity);

      super.onAttach (activity);

      if (activity instanceof Main_Activity)
      {
	 this.Activity = (Main_Activity) activity;
	 this.Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Printer);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- onAttach");
      return;
   }

   /**
    * Called when the fragment is no longer attached to its activity. This is called after {@link #onDestroy()}.
    */
   @Override
   public void onDetach ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ onDetach");

      super.onDetach ();

      if (this.Activity != null)
      {
	 this.Activity.Bind_Fragment (null, IMain_Activity.Page_Type.Printer);
	 this.Activity = null;
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- onDetach");
      return;
   }

   /**
    * Called when the fragment is visible to the user and actively running. This is generally tied to
    * {@link android.app.Activity#onResume() Activity.onResume} of the containing Activity's lifecycle.
    */
   @Override
   public void onResume ()
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ onResume");

      super.onResume ();

      if (this.Activity != null)
      {
	 this.Activity.Bind_Fragment (this, IMain_Activity.Page_Type.Printer);
      } // if

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- onResume");
      return;
   } // onResume

   /**
    * Called to ask the fragment to save its current dynamic state, so it can later be reconstructed in a new instance
    * of its process is restarted. If a new instance of the fragment later needs to be created, the data you place in
    * the Bundle here will be available in the Bundle given to {@link #onCreate(android.os.Bundle)},
    * {@link #onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)} , and
    * {@link #onActivityCreated(android.os.Bundle)}.
    * <p/>
    * <p>
    * This corresponds to
    * {@link android.app.Activity#onSaveInstanceState(android.os.Bundle) Activity.onSaveInstanceState(Bundle)} and most
    * of the discussion there applies here as well. Note however:
    * <em>this method may be called at any time before {@link #onDestroy()}</em>. There are many situations where a
    * fragment may be mostly torn down (such as when placed on the back stack with no UI showing), but its state will
    * not be saved until its owning activity actually needs to save its state.
    *
    * @param outState
    *    Bundle in which to place your saved state.
    */
   @Override
   public void onSaveInstanceState (@NotNull final android.os.Bundle outState)
   {
      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "+ onSaveInstanceState" + "\n> outState = %1$s", outState);

      if (this.Printout != null)
      {
	 final CharSequence Text = this.Printout.getText ();
	 this.Old_Printout = Text.toString ();
      }
      else
      {
	 this.Old_Printout = "";
      } // if

      final android.app.Activity activity = this.getActivity ();

      if (activity != null)
      {
	 net.sourceforge.uiq3.hp45.Preferences_Activity.HP45_Printout (activity, Old_Printout);
      } // if

      // super.onSaveInstanceState stores Utf16_Text in outState
      //
      super.onSaveInstanceState (outState);

      net.sourceforge.uiq3.Log.d (Printer_Fragment.TAG, "- onSaveInstanceState" + "\n> outState = %1$.32s", outState);
      return;
   } // onSaveInstanceState

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override
   @NotNull
   @org.jetbrains.annotations.TestOnly
   public String toString ()
   {
      return Printer_Fragment.TAG +
	 "{\nsuper=" + super.toString () +
	 ",\nButton_Clear=" + this.Button_Clear +
	 ",\nPrintout=" + this.Printout + '}';
   } // toString
} // Printer_Fragment

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
