#!/opt/local/bin/zsh
#
# ********************************************************** {{{1 **********
#   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
# **************************************************************************
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
# **************************************************************************
#   $Author: krischik $
#   $Revision: 8451 $
#   $Date: 2023-01-24 13:54:39 +0100 (Di, 24. Jan 2023) $
#   $Id: Build.command 8451 2023-01-24 12:54:39Z krischik $
# *********************************************************** }}}1 **********

setopt No_XTrace;
setopt Err_Exit;

typeset -l Local_Path="${PROJECT_HOME}/target/surefire-reports"

function Test ()
{
    local in_App="${1}"
    local in_Option="${2}"

    pushd "${in_App}"
	scala					\
	    -classpath  "${UIQ3_SCALASCRIPT}"	\
	    -save "src/test/scripts/Start.cmd"	\
	    "nobuild" "${in_Option}" "noclean"		\
	    1>&1 2>&2 &>>"${Local_Path}/${0:t:r}.out"
    popd
}

pushd "${PROJECT_HOME}/Android"
    if test -f "${Local_Path}/${0:t:r}.out"; then
	rm "${Local_Path}/${0:t:r}.out"
    fi

    {
	pushd "HP45"
	    # Build and assemble.
	    #
	    ../gradlew					\
		"assembleDebug"				\
		"assembleAndroidTest"			\
		1>&1 2>&2 &>>"${Local_Path}/${0:t:r}.out"
	popd

	#
	# Run instrumentation tests.
	#
	# Note: Instrumentation tests are only available for the full
	#       debug version. Release and Demo versions can't be tested
	# Note: Using Gradle to run the tests will mess up the test files
	#	and remove the start icon from the launcher.
	#
	Test "HP45"    "debug"

	# publish to maven
	#
	./gradlew					\
	    "publishFastPublicationToMavenLocal"	\
	    1>&1 2>&2 &>>"${Local_Path}/${0:t:r}.out"
    } always {
	Retval=${?}

	printf "\e[42m%-30.30s : \e[43m %-60.60s \e[m\n" "Result"     "${Retval} / ${TRY_BLOCK_ERROR}"

	Search_Errors "${Local_Path}/${0:t:r}.out"

	echo 'gview "${Local_Path}/${0:t:r}.out" &'
    }
popd;

exit ${Retval}

############################################################ {{{1 ###########
# vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 noexpandtab :
# vim: set textwidth=0 filetype=zsh foldmethod=marker nospell :
