/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

@file:Repository("file:///Users/Shared/Work/Repositories/Local")
@file:DependsOn("net.sourceforge.uiq3:Kotlin-Script:7.2.0")

import com.krischik.script.Android.adb
import com.krischik.script.Android.clean
import com.krischik.script.Android.deleteScreenshots
import com.krischik.script.Android.downloadScreenshots
import com.krischik.script.Android.forAllDevices
import com.krischik.script.Android.openScreenshots
import com.krischik.script.Android.prepareScreenshots
import com.krischik.script.Android.reinstall
import com.krischik.script.Android.test
import com.krischik.script.Android.testBackup
import com.krischik.script.Android.testClass
import com.krischik.script.Android.testSize
import com.krischik.script.Echo
import com.krischik.script.Gradle.gradle
import com.krischik.script.Maven.mvn
import com.krischik.script.Shell.Err_Exit_Call
import com.krischik.script.Shell.Logger

val Build		= args.contains("build")
val Debug		= args.contains("debug")
val Clean		= args.contains("clean")
val Show_Help		= args.contains("show_help")
val Show_Info		= args.contains("show_info")
val Show_Contact	= args.contains("show_contact")
val Show_Preferences	= args.contains("show_preferences")
val Project_Home	= java.io.File(System.getenv ("PROJECT_HOME"))
val Project_Android	= java.io.File(Project_Home, "Android")
val Workspace		= java.io.File(System.getenv ("Workspace"))
val Temp		= java.io.File(Workspace, "Temp")
val Type		= "full"
val Option		= if (Debug) { "debug"	} else { "release" }
val Group_Name		= "hp45"
val Package_Dir		= "build/outputs/apk/" + Type + "/" + Option
val Package_File	= Package_Dir + "/HP45-Droid-" + Type + "-" + Option + ".apk"
val Package_Name	= "net.sourceforge.uiq3." + Group_Name
val Test_Dir		= "build/outputs/apk/androidTest/" + Type + "/debug"
val Test_File		= Test_Dir + "/HP45-Droid-" + Type + "-debug-androidTest.apk"
val Test_Name		= "net.sourceforge.uiq3." + Group_Name + ".test"
val Size		= if (args.contains("test_small"))
{
    "small"
}
else
{
    null
}

if (Build)
{
   Echo(" Build «" + Package_File + "»")

   Err_Exit_Call(
      mvn + listOf(
	 "--activate-profiles", Option,
	 "--projects", "Java/HP45/Library",
	 "--define", "skipTests=true",
	 "install"
      ),
      Project_Home
   )

   Err_Exit_Call(
      gradle + listOf(
	 ":HP45:assemble" + Type.capitalize() + Option.capitalize(),
	 ":HP45:assembleAndroidTest"
      ),
      Project_Android
   )
} // if

forAllDevices { Device ->
   Echo("Test «" + Device + "»")

   if (Clean)
   {
      clean(Device, Package_Name)
   }

   reinstall(Device, Package_File, Package_Name)
   reinstall(Device, Test_File, Test_Name)

   if (Size != null)
   {
      Echo("size «" + Size + "»")

      testSize(Device, Test_Name, Size)
   }
   else
   {
      Echo("Test all classes")

      prepareScreenshots (Device, Package_Name)
      deleteScreenshots	 (Device, Package_Name, Temp)
      test		 (Device, Test_Name)
      downloadScreenshots(Device, Package_Name, Temp)
      openScreenshots	 (Device, Package_Name, Temp)
      deleteScreenshots	 (Device, Package_Name)
      testBackup	 (Device, Package_Name)
   }
} // forAllDevices

//////////////////////////////////////////////////////////// {{{1 ///////////
// vim: set wrap tabstop=8 shiftwidth=4 softtabstop=4 noexpandtab :
// vim: set textwidth=0 filetype=kotlin fileformat=unix foldmethod=marker nospell :
