/*
 * ******************************************************** {{{1 ************
 *   Copyright © 2005 … 2017 "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 *  ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: BalloonOverlayView.java 7681 2021-09-02 14:15:29Z krischik $
 *  ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.maps;

import org.jetbrains.annotations.NotNull;

/**
 * A view representing a MapView marker information balloon.
 * <p>
 * This class has a number of Android resource dependencies:
 * <ul>
 * <li>drawable/balloon_overlay_bg_selector.xml</li>
 * <li>drawable/balloon_overlay_close.png</li>
 * <li>drawable/balloon_overlay_focused.9.png</li>
 * <li>drawable/balloon_overlay_unfocused.9.png</li>
 * <li>layout/balloon_map_overlay.xml</li>
 * </ul>
 * </p>
 *
 * @author Jeff Gilfelt
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @version 1.2 $Revision: 7681 $
 * @since 1.2
 */
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
public class BalloonOverlayView <Item extends com.google.android.maps.OverlayItem>
   extends
      android.widget.FrameLayout
{

   @NotNull
   private final android.widget.LinearLayout layout;
   @NotNull
   private final android.widget.TextView snippet;
   @NotNull
   private final android.widget.TextView title;

   /**
    * Create a new BalloonOverlayView.
    *
    * @param context
    *           - The activity context.
    * @param balloonBottomOffset
    * @param overlayId
    * @param itemTitleId
    * @param itemSnippet
    * @param closeButton
    */
   public BalloonOverlayView (
      @NotNull final android.content.Context context,
      final int balloonBottomOffset,
      final int overlayId,
      final int itemTitleId,
      final int itemSnippet,
      final int closeButton)
   {

      super (context);

      this.setPadding (10, 0, 10, balloonBottomOffset);
      this.layout = new android.widget.LinearLayout (context);
      this.layout.setVisibility (android.view.View.VISIBLE);

      @org.jetbrains.annotations.NotNull
      final android.view.LayoutInflater inflater = (android.view.LayoutInflater) context.getSystemService (
         android.content.Context.LAYOUT_INFLATER_SERVICE);
      final android.view.View v = inflater.inflate (overlayId, this.layout);

      this.title = (android.widget.TextView) v.findViewById (itemTitleId);
      this.snippet = (android.widget.TextView) v.findViewById (itemSnippet);

      @org.jetbrains.annotations.NotNull
      final android.widget.ImageView close = (android.widget.ImageView) v.findViewById (closeButton);

      close.setOnClickListener (new OnClickListener ()
      {
         @Override
         public void onClick (final android.view.View v)
         {
            BalloonOverlayView.this.layout.setVisibility (android.view.View.GONE);
         }
      });

      @org.jetbrains.annotations.NotNull
      final android.widget.FrameLayout.LayoutParams params = new android.widget.FrameLayout.LayoutParams (
         android.view.ViewGroup.LayoutParams.WRAP_CONTENT,
         android.view.ViewGroup.LayoutParams.WRAP_CONTENT);

      params.gravity = android.view.Gravity.NO_GRAVITY;

      this.addView (this.layout, params);

   }

   /**
    * Sets the view data from a given overlay item.
    *
    * @param item
    *           - The overlay item containing the relevant view data (title and snippet).
    */
   public void setData (@org.jetbrains.annotations.NotNull final Item item)
   {

      this.layout.setVisibility (android.view.View.VISIBLE);
      if (item.getTitle () != null)
      {
         this.title.setVisibility (android.view.View.VISIBLE);
         this.title.setText (item.getTitle ());
      }
      else
      {
         this.title.setVisibility (android.view.View.GONE);
      }
      if (item.getSnippet () != null)
      {
         this.snippet.setVisibility (android.view.View.VISIBLE);
         this.snippet.setText (item.getSnippet ());
      }
      else
      {
         this.snippet.setVisibility (android.view.View.GONE);
      }

   }

}
