/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005…2011 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7697 $
 *   $Date: 2021-09-23 18:15:07 +0200 (Do, 23. Sep 2021) $
 *   $Id: OS.java 7697 2021-09-23 16:15:07Z krischik $
 *   $HeadURL: svn+ssh://krischik@svn.code.sf
 * .net/p/uiq3/code/trunk/Java/Calculator/Desktop/src/main/java/net/sourceforge/uiq3/OS.java $
 * ********************************************************** }}}1 ***********
 */
package net.sourceforge.uiq3

import java.util.logging.Level as Log_Level


/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
object OS
{
   /**
    *
    *  class logger instance
    */
   private val logger: java.util.logging.Logger

   /**
    *
    *  class logger tag.
    */
   private val TAG: String

   @JvmField
   val New_Line: String

   init
   {
      TAG = Utils::class.java.name
      logger = java.util.logging.Logger.getLogger(TAG)
      New_Line = System.lineSeparator()
   }

   /**
    *
    * open web page / help page
    *
    * @param Website
    */
   @JvmStatic
   public fun openPage(Website: java.net.URI)
   {
      logger.entering(TAG, "openPage", Website)

      if (java.awt.Desktop.isDesktopSupported())
      {
	 val desktop = java.awt.Desktop.getDesktop()

	 if (desktop.isSupported(java.awt.Desktop.Action.BROWSE))
	 {
	    try
	    {
	       desktop.browse(Website)
	    }
	    catch (exception: java.io.IOException)
	    {
	       logger.log(Log_Level.SEVERE, "LOG00550: ", exception)
	    } // try
	 }
	 else
	 {
	    logger.log(Log_Level.WARNING, "LOG00552: No browser supported")
	 } // if
      }
      else
      {
	 logger.log(Log_Level.WARNING, "LOG00551: No desktop supported")
      } // if


      logger.exiting(TAG, "openPage")
      return
   } // openPage

   @JvmStatic
   public fun setClipboard(component: javax.swing.text.JTextComponent)
   {
      logger.entering(TAG, "setClipboard", component)

      try
      {
	 val text: String = component.getText()
	 setClipboard(text)
      }
      catch (exception: Exception)
      {
	 logger.log(Log_Level.WARNING, "LOG00554: Clipboard could not be set.", exception)
      }

      logger.exiting(TAG, "setClipboard")
   } // setClipboard

   @JvmStatic
   public fun setClipboard(text: String)
   {
      logger.entering(TAG, "setClipboard", text)

      try
      {
	 val toolkit = java.awt.Toolkit.getDefaultToolkit()
	 val clipboard = toolkit.systemClipboard
	 val selection = java.awt.datatransfer.StringSelection(text)

	 clipboard.setContents(selection, selection)
      }
      catch (exception: Exception)
      {
	 logger.log(Log_Level.WARNING, "LOG00553: Clipboard could not be set.", exception)
      }

      logger.exiting(TAG, "setClipboard")
   } // setClipboard

   @JvmStatic
   public fun setWindowIcon(
      frame: javax.swing.JFrame,
      name: String
   )
   {
      logger.entering(TAG, "setApplicationIcon", arrayOf(frame, name))

      try
      {
	 val frameClass = frame.javaClass
	 val resourceStream = frameClass.getResourceAsStream(name)

	 if (resourceStream != null)
	 {
	    val applicationIcon = javax.imageio.ImageIO.read(resourceStream)

	    frame.setIconImage(applicationIcon)
	 }
	 else
	 {
	    logger.log(Log_Level.WARNING, "LOG00555: Icon resource „${name}” not found for class «${frameClass}».")
	 }
      }
      catch (exception: Exception)
      {
	 logger.log(Log_Level.WARNING, "LOG00555:  Icon resource „${name}” not loaded.", exception)
      }

      logger.exiting(TAG, "setApplicationIcon")
   } // setApplicationIcon
} // OS
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :

