/*
 ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 ****************************************************************************
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or ENDIFFTNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see «http://www.gnu.org/licenses/».
 ************************************************************* }}}1 ********
 */
package net.sourceforge.uiq3.calculator

/**
 *
 * @author Martin Krischik
 */
@java.beans.JavaBean(description = "Calculator Button")
abstract class Button<
   Canvas_Type : net.sourceforge.uiq3.ui.ICalculatorCanvas?,
   Key_Type : IKey?> :
   javax.swing.JPanel,
   net.sourceforge.uiq3.ui.IButton,
   IKey
{
   companion object
   {
      /**
       * class logger instance
       */
      private val Logger: java.util.logging.Logger

      /**
       * class logger tag.
       */
      private val TAG: String

      init
      {
	 this.TAG = Button::class.java.name
	 this.Logger = java.util.logging.Logger.getLogger(this.TAG)
      }
   }

   /**
    * Key number.
    */
   override val keyNr: Int
      get() = this.data?.keyNr ?: IKey.No_Key.toInt()

   /**
    * Key data.
    */
   protected val data: Key_Type?

   /**
    * Canvas the button belongs to
    */
   protected val canvas: Canvas_Type?

   /**
    * get standard button for click operations.
    *
    * @return the actual button
    */
   protected abstract val button: javax.swing.JButton

   /**
    * get calculator keycode handled by this button. This key code is used inside the Calculator.
    *
    * @return key code or No_Key if no key pressed
    */
   override val calculatorKeyCode
      get() = this.data?.calculatorKeyCode ?: IKey.No_Key

   override val javaKeyCode
      get() = this.data?.javaKeyCode ?: java.awt.event.KeyEvent.VK_UNDEFINED

   override fun isKeyHit(Java_Key_Code: Int) =
      this.data != null && this.data.isKeyHit(Java_Key_Code)

   /**
    * name of button
    */
   override val buttonText
      get() = this.data?.buttonText ?: calculatorKeyCode.toString()

   /**
    * Creates new form Calculator_Button_5
    */
   protected constructor() : super()
   {
      Button.Logger.entering(Button.TAG, "Calculator_Button_5")

      this.data = null
      this.canvas = null

      Button.Logger.exiting(Button.TAG, "Calculator_Button_5")
   }

   /**
    * Creates new form Button
    */
   protected constructor(
      canvas: Canvas_Type,
      key: Key_Type
   ) : super()
   {
      Button.Logger.entering(Button.TAG, "Calculator_Button", arrayOf(canvas, key))

      this.data = key
      this.canvas = canvas

      Button.Logger.exiting(Button.TAG, "Calculator_Button")
   } // Button

   /**
    * Perform a click on the button
    */
   override fun doClick()
   {
      this.button.doClick();
   }

   override fun toString(): String
   {
      return "Calculator_Button{keyNr=${this.keyNr}, data=${this.data}}"
   }
}
// vim: set textwidth=120 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=kotlin fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
