/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

import org.jetbrains.annotations.TestOnly

/**
 * Calculator frame
 */
abstract class Frame<
   Canvas_Type : net.sourceforge.uiq3.ui.ICalculatorCanvas?> :
   javax.swing.JFrame(),
   net.sourceforge.uiq3.ui.IFrame
{
   companion object
   {
      protected const val Extra_Width: Int = 16
      protected const val Extra_Height: Int = 32

      /**
       * <p>extra border für split view
       */
      @JvmStatic
      protected val border = java.awt.Dimension(Companion.Extra_Width, Companion.Extra_Height)
   }

   abstract val calculatorCanvas: Canvas_Type

   init
   {
   }

   override var pressedKey: Short
      get() = calculatorCanvas?.pressedKey ?: net.sourceforge.uiq3.calculator.IKey.No_Key
      set(value)
      {
	 calculatorCanvas?.pressedKey = value
      }

   override var readKey: Short
      get() = calculatorCanvas?.readKey ?: net.sourceforge.uiq3.calculator.IKey.No_Key
      set(value)
      {
	 calculatorCanvas?.readKey = value
      }

   override val isReadKeyAvailable: Boolean
      get() = calculatorCanvas?.isReadKeyAvailable ?: false

   /**
    * Set text to be displayed (as java string, used by RPN-45 only).
    *
    * @param Text
    * to be displayed
    */
   override var javaText: String
      get() = calculatorCanvas?.javaText ?: ""
      set(value)
      {
	 calculatorCanvas?.javaText = value
      }

   /**
    * Get text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   override fun getDisplayText(Row: Int): ShortArray
   {
      return calculatorCanvas?.getDisplayText(Row) ?: ShortArray(0)
   }

   /**
    * Get text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   override fun getJavaText(Row: Int): String
   {
      return calculatorCanvas?.getJavaText(Row) ?: ""
   }
}

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :
