/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.printer

import net.sourceforge.uiq3.calculator.ICalculator

@java.beans.JavaBean(description = "printer for calculators")
abstract class Printer<Calculator_Type: ICalculator?> (val owner: Calculator_Type) :
   IPrinter
{
   companion object
   {
      /**
       * Class logger tag.
       */
      private var TAG: String

      /**
       * Class logger instance.
       */
      private val Logger: java.util.logging.Logger

      /**
       * New line string
       */
      val New_Line: String

      init
      {
	 TAG = Canvas::class.java.name
	 Logger = java.util.logging.Logger.getLogger(Printer.TAG);
	 New_Line = System.lineSeparator();
      } // static
   }

   /**
    * Printout
    */
   protected var data = StringBuilder(IPrinter.Default_Printout_Size)

   /**
    * Handle property changes.
    */
   private val propertyChangeSupport = java.beans.PropertyChangeSupport(this)

   override fun addPropertyChangeListener(listener: java.beans.PropertyChangeListener?)
   {
      Logger.entering(TAG, "addPropertyChangeListener", listener)

      this.propertyChangeSupport.addPropertyChangeListener(listener)

      Logger.exiting(TAG, "addPropertyChangeListener")
      return
   }

   override fun removePropertyChangeListener(listener: java.beans.PropertyChangeListener?)
   {
      Logger.entering(TAG, "removePropertyChangeListener", listener)

      this.propertyChangeSupport.removePropertyChangeListener(listener)

      Logger.exiting(TAG, "removePropertyChangeListener")
      return
   }

   @java.beans.BeanProperty(bound = true, description = "The text to printed")
   override fun getPrintout(): String = data.toString()

   override fun setPrintout(value: String)
   {
      Logger.entering(TAG, "setPrintout", value)

      val oldData = this.data.toString();

      this.data.delete(0, this.data.length);
      this.data.append(value);
      this.propertyChangeSupport.firePropertyChange(IPrinter.PRINTOUT_PROPERTY, oldData, this.data.toString());

      Logger.exiting(TAG, "setPrintout")
      return
   }

   /**
    * current length of the printed text
    */
   @java.beans.BeanProperty(bound = false, description = "Current length of the printed text")
   override fun getPrintoutLength(): Int = this.data.length

   override fun clearPrintout()
   {
      Logger.entering(TAG, "clearPrintout")

      this.printout = ""

      Logger.exiting(TAG, "clearPrintout")
      return
   }

   override fun printLine()
   {
      Logger.entering(TAG, "printLine")

      this.data.append(New_Line);

      Logger.exiting(TAG, "printLine")
      return
   }

   override fun printLine(line: String)
   {
      Logger.entering(TAG, "printLine", line)

      val oldData = this.data.toString();

      this.data.append(line);
      this.data.append(New_Line);
      this.propertyChangeSupport.firePropertyChange(IPrinter.PRINTOUT_PROPERTY, oldData, this.data.toString());

      Logger.exiting(TAG, "printLine")
      return
   }
}

// vim: set textwidth=0 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=kotlin fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
