/*
 ************************************************************ {{{1 **********
 *   Copyright © 2010 Martin Krischik
 ****************************************************************************
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or ENDIFFTNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ************************************************************* }}}1 ********
 */
package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@java.beans.JavaBean(description = "Calculator Button with three functions")
public class Button_3<
   Canvas_Type extends Canvas,
   Key_Type extends Key>
   extends net.sourceforge.uiq3.calculator.Button<Canvas_Type, Key_Type>
{
   /**
    * class logger instance
    */
   private static final java.util.logging.Logger Logger;
   /**
    * class logger tag.
    */
   private static final String TAG;
   /**
    *
    */
   private static final long serialVersionUID = -2695463726023423746L;

   static
   {
      TAG = Button_3.class.getName ();
      Logger = java.util.logging.Logger.getLogger (TAG);
   }

   /**
    * Creates new form Button_3
    */
   public Button_3 ()
   {
      super ();
      Logger.entering (TAG, "Button_3");

      initComponents ();

      Logger.exiting (TAG, "Button_3");
   }

   /**
    * Creates new form Button_3
    */
   protected Button_3 (
      @NotNull final Canvas_Type canvas,
      @NotNull final Key_Type key,
      final int keyNr)
   {
      super (canvas, key);
      Logger.entering (TAG, "Button_3", new Object[] {canvas, keyNr});

      final Key_Type data = getData ();

      assert data != null;

      data.setCanvas (canvas);

      initComponents ();

      Logger.exiting (TAG, "Button_3");
   } // Button_3

   /**
    * get standard button for click operations.
    *
    * @return the actual button
    */
   @NotNull
   @Override
   public javax.swing.JButton getButton ()
   {
      return Standard_Button;
   }

   //@formatter:off
   @SuppressWarnings({"TooBroadScope", "DanglingJavadoc", "MagicNumber"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {
      java.awt.GridBagConstraints gridBagConstraints;

      setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
      setOpaque(false);
      setLayout(new java.awt.GridBagLayout());

      Shift_Text.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
      Shift_Text.setText("Shift");
      Shift_Text.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
      Shift_Text.setName("Shift_Text"); // NOI18N
      Shift_Text.setRequestFocusEnabled(false);
      Shift_Text.setVerifyInputWhenFocusTarget(false);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.ipadx = 2;
      gridBagConstraints.ipady = 2;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
      add(Shift_Text, gridBagConstraints);

      Standard_Button.setText("Standard");
      Standard_Button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
      Standard_Button.setMargin(new java.awt.Insets(2, 2, 2, 2));
      Standard_Button.setName("Standard_Button"); // NOI18N
      Standard_Button.addActionListener(this::Standard_ButtonActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.ipadx = 2;
      gridBagConstraints.ipady = 2;
      gridBagConstraints.weightx = 3.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
      add(Standard_Button, gridBagConstraints);

      Alpha_Text.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
      Alpha_Text.setText("Alpha");
      Alpha_Text.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
      Alpha_Text.setName("Alpha_Text"); // NOI18N
      Alpha_Text.setRequestFocusEnabled(false);
      Alpha_Text.setVerifyInputWhenFocusTarget(false);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.ipadx = 2;
      gridBagConstraints.ipady = 2;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
      add(Alpha_Text, gridBagConstraints);
   }// </editor-fold>//GEN-END:initComponents

    private void Standard_ButtonActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_Standard_ButtonActionPerformed
       Logger.entering (TAG, "StandardActionPerformed", evt);

       final Canvas_Type canvas = getCanvas ();
       final Key_Type data = getData ();

       if (canvas != null && data != null)
       {
          canvas.buttonReleased (this.getKeyNr ());
       }

       Logger.exiting (TAG, "StandardActionPerformed");
    }//GEN-LAST:event_Standard_ButtonActionPerformed

   // Variables declaration - do not modify//GEN-BEGIN:variables
   protected final javax.swing.JLabel Alpha_Text = new javax.swing.JLabel();
   protected final javax.swing.JLabel Shift_Text = new javax.swing.JLabel();
   protected final javax.swing.JButton Standard_Button = new javax.swing.JButton();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Button_3 {\n" +
         "super = " + super.toString () +
         ",\nAlpha_Text = " + Alpha_Text +
         ",\nShift_Text = " + Shift_Text +
         ",\nStandard_Button = " + Standard_Button +
         "\n}";
   }
}

// vim: set textwidth=120 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=java fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
