/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;

/**
 * Listen for Keyboard and Mouse events.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public class CanvasListener
   implements
      java.awt.event.MouseListener,
      java.awt.event.KeyListener
{
   /**
    * <p>
    * class logger instance
    * </p>
    */
   private static final java.util.logging.Logger logger = java.util.logging.Logger.getLogger (
      CanvasListener.class.getName ());
   /**
    * <p>
    * class logger tag.
    * </p>
    */
   private static final String TAG = CanvasListener.class.getName ();

   /**
    * canvas to which this listener instance belongs.
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   private final Canvas Owner;

   /**
    * create new listener instance.
    *
    * @param Owner
    *           canvas to which this listener instance belongs.
    */
   public CanvasListener (final Canvas Owner)
   {
      this.Owner = Owner;
      return;
   } // Calculator_Listener

   /**
    * A Key has been pressed
    *
    * @param Event
    *           A key event
    * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
    */
   @Override
   public void keyPressed (@NotNull final java.awt.event.KeyEvent Event)
   {
      CanvasListener.logger.entering (CanvasListener.TAG, "keyPressed", Event);

//      this.Owner.keyPressed (Event.getKeyCode ());

      CanvasListener.logger.exiting (CanvasListener.TAG, "keyPressed");
      return;
   } // keyPressed

   /**
    * A Key has been released
    *
    * @param Event
    *           A key event
    * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
    */
   @Override
   public void keyReleased (@NotNull final java.awt.event.KeyEvent Event)
   {
      CanvasListener.logger.entering (CanvasListener.TAG, "keyReleased", Event);

      this.Owner.keyReleased (Event.getKeyCode ());

      CanvasListener.logger.exiting (CanvasListener.TAG, "keyReleased");
      return;
   } // keyReleased

   /**
    * A Key has been typed - this event is ignored
    *
    * @param Event
    *           A key event
    * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
    */
   @Override
   public void keyTyped (final java.awt.event.KeyEvent Event)
   {
      return;
   } // keyTyped

   /**
    * A mouse click has been performed - this event is ignored.
    *
    * @param Event
    *           A mouse event
    * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
    */
   @Override
   public void mouseClicked (final java.awt.event.MouseEvent Event)
   {
      return;
   } // mouseClicked

   /**
    * mouse pointer entered canvas - this event is ignored.
    *
    * @param Event
    *           A mouse event
    * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
    */
   @Override
   public void mouseEntered (final java.awt.event.MouseEvent Event)
   {
      return;
   } // mouseEntered

   /**
    * mouse pointer exited canvas - this event is ignored.
    *
    * @param Event
    *           A mouse event
    * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
    */
   @Override
   public void mouseExited (final java.awt.event.MouseEvent Event)
   {
      return;
   } // mouseExited

   /**
    * mouse button has been pressed
    *
    * @param Event
    *           A mouse event
    * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
    */
   @Override
   public void mousePressed (@NotNull final java.awt.event.MouseEvent Event)
   {
      final java.awt.Point Location = Event.getPoint ();

//      this.Owner.pointerPressed (/* x => */Location.x, /* y => */Location.y);

      return;
   } // mousePressed

   /**
    * mouse button has been released
    *
    * @param Event
    *           A mouse event
    * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
    */
   @Override
   public void mouseReleased (@NotNull final java.awt.event.MouseEvent Event)
   {
      final java.awt.Point Location = Event.getPoint ();

//      this.Owner.pointerReleased (/* x => */Location.x, /* y => */Location.y);

      return;
   } // mouseReleased
} // Calculator_Listener

// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :
