/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.calculator;

import javax.swing.event.HyperlinkEvent.EventType;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 *
 * @author martin.krischik
 */
@SuppressWarnings ("HardcodedFileSeparator")
public class Help_Dialog
   extends javax.swing.JDialog
{
   private class Follow_Hyperlink
      implements javax.swing.event.HyperlinkListener
   {
      private Follow_Hyperlink ()
      {
	 super ();
      } // Follow_Hyperlink

      @Override
      public void hyperlinkUpdate (final javax.swing.event.HyperlinkEvent Event)
      {
	 logger.entering (Help_Dialog.TAG, "Follow_Hyperlink.hyperlinkUpdate", Event);

	 final EventType Event_Type = Event.getEventType ();

	 if (Event_Type == javax.swing.event.HyperlinkEvent.EventType.ENTERED)
	 {
	    Help_Pane.setCursor (
	       java.awt.Cursor.getPredefinedCursor (java.awt.Cursor.HAND_CURSOR));
	 }
	 else if (Event_Type == javax.swing.event.HyperlinkEvent.EventType.EXITED)
	 {
	    Help_Pane.setCursor (
	       java.awt.Cursor.getPredefinedCursor (java.awt.Cursor.DEFAULT_CURSOR));
	 }
	 else if (Event_Type == javax.swing.event.HyperlinkEvent.EventType.ACTIVATED)
	 // Hier wird auf ein Klick reagiert
	 {
	    if (Event instanceof javax.swing.text.html.HTMLFrameHyperlinkEvent)
	    {
	       final javax.swing.text.html.HTMLFrameHyperlinkEvent evt =
		  (javax.swing.text.html.HTMLFrameHyperlinkEvent) Event;
	       Editor_Document.processHTMLFrameHyperlinkEvent (evt);
	    }
	    else
	    {
	       try
	       {
		  // Normaler Link
		  Help_Pane.setPage (Event.getURL ());
	       }
	       catch (final Throwable throwable)
	       {
		  logger.log (
		     java.util.logging.Level.SEVERE,
		     "LOG00010:Could not open page.",
		     throwable);
	       } // try
	    } // if
	 } // if

	 logger.exiting (Help_Dialog.TAG, "Follow_Hyperlink.hyperlinkUpdate");
	 return;
      } // hyperlinkUpdate
   } // Follow_Hyperlink

   private static final long serialVersionUID = 8025033125795181000L;

   /**
    * <p> class logger instance </p>
    */
   private static final java.util.logging.Logger logger;
   /**
    * <p> class logger tag. </p>
    */
   private static final String TAG;

   /**
    * Resource Bundle used for I18N
    */
   private static final java.util.ResourceBundle Calculator_Desktop; // NOI18N

   static
   {
      TAG = Help_Dialog.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
      Calculator_Desktop = java.util.ResourceBundle.getBundle ("net/sourceforge/uiq3/calculator/Bundle"); // NOI18N
   }

   @Nullable
   private net.sourceforge.uiq3.calculator.IProperties properties;

   /**
    * Creates new form Help
    * @param parent parent window
    * @param modal open modal
    */
   public Help_Dialog (
      final java.awt.Frame parent,
      final boolean modal)
   {
      super (parent, modal);
      Help_Dialog.logger.entering (Help_Dialog.TAG, "Help", new Object[] {parent, modal});

      Editor_Kit = new javax.swing.text.html.HTMLEditorKit ();
      Editor_Document = (javax.swing.text.html.HTMLDocument) Editor_Kit.createDefaultDocument ();

      initComponents ();

      Help_Pane.addHyperlinkListener (new Follow_Hyperlink ());

      Help_Dialog.logger.exiting (Help_Dialog.TAG, "Help");
      return;
   } // Help_Dialog
   
   /**
    * Creates new form Help
    * @param parent parent window
    * @param properties open modal
    */
   public Help_Dialog (
      @NotNull final java.awt.Frame parent,
      @NotNull final net.sourceforge.uiq3.calculator.IProperties properties)
   {
      this (parent, false);
      Help_Dialog.logger.entering (Help_Dialog.TAG, "Help", new Object[] {parent, properties});

      this.properties = properties;

      Help_Pane.setFont (properties.getText_Large_Font ());
      Help_Pane.setForeground (java.awt.Color.black);
      Help_Pane.setBackground (java.awt.Color.white);

      Help_Dialog.logger.exiting (Help_Dialog.TAG, "Help");
      return;
   } // Help_Dialog

   /**
    * <p>The HTML Editor</p>
    */
   private final javax.swing.text.html.HTMLEditorKit Editor_Kit;
   /**
    * <p>The HTML Document</p>
    */
   private final javax.swing.text.html.HTMLDocument Editor_Document;

   /**
    * Initialize Values
    */
   public void Initialize ()
   {
      Help_Dialog.logger.entering (Help_Dialog.TAG, "Initialize");

      if (this.properties != null)
      {
	 this.properties.restoreWindowStatus (this);
      }

      try
      {
	 final java.net.URL Help_Resource = Help_Dialog.class.getResource ("/assets/help.html");
	 Help_Dialog.logger.log (
	    java.util.logging.Level.INFO,
	    "LOG00020: Open Help : {0}",
	    Help_Resource);
	 Help_Pane.setPage (Help_Resource);
      }
      catch (final java.io.IOException exception)
      {
	 Help_Dialog.logger.log (
	    java.util.logging.Level.SEVERE,
	    "LOG00000: Could not load help file.",
	    exception);
      } //  try

      Help_Dialog.logger.exiting (Help_Dialog.TAG, "Initialize");
      return;
   } // Initialize

   //@formatter:off
   @SuppressWarnings({"AnonymousInnerClass", "DanglingJavadoc", "ChainedMethodCall", "LocalCanBeFinal", "MagicNumber", "override"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {

      setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
      setTitle(Calculator_Desktop.getString("Help_Title")); // NOI18N
      setName("Help_Dialog"); // NOI18N
      addWindowListener(new java.awt.event.WindowAdapter()
      {
         public void windowClosing(java.awt.event.WindowEvent evt)
         {
            formWindowClosing(evt);
         }
      });

      Help_Pane.setEditable(false);
      Help_Pane.setBackground(javax.swing.UIManager.getDefaults().getColor("EditorPane.background"));
      Help_Pane.setContentType("text/html"); // NOI18N
      Help_Pane.setDocument(Editor_Document);
      Help_Pane.setEditorKit(Editor_Kit);
      Help_Scroll.setViewportView(Help_Pane);

      OK_Button.setText(Calculator_Desktop.getString("Help_Button_OK")); // NOI18N
      OK_Button.setName("OK_Button"); // NOI18N
      OK_Button.addActionListener(this::OK_ButtonActionPerformed);

      javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
      getContentPane().setLayout(layout);
      layout.setHorizontalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
            .addContainerGap()
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
               .addComponent(Help_Scroll, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 421, Short.MAX_VALUE)
               .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                  .addGap(0, 0, Short.MAX_VALUE)
                  .addComponent(OK_Button)))
            .addContainerGap())
      );
      layout.setVerticalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
            .addContainerGap()
            .addComponent(Help_Scroll, javax.swing.GroupLayout.DEFAULT_SIZE, 243, Short.MAX_VALUE)
            .addGap(18, 18, 18)
            .addComponent(OK_Button)
            .addContainerGap())
      );

      pack();
   }// </editor-fold>//GEN-END:initComponents

   private void OK_ButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OK_ButtonActionPerformed
   {//GEN-HEADEREND:event_OK_ButtonActionPerformed
      Help_Dialog.logger.entering (Help_Dialog.TAG, "OK_ButtonActionPerformed", evt);

      if (this.properties != null)
      {
	 this.properties.saveWindowStatus (this);
      }

      this.setVisible (false);
      this.dispose ();

      Help_Dialog.logger.exiting (Help_Dialog.TAG, "OK_ButtonActionPerformed", evt);
   }//GEN-LAST:event_OK_ButtonActionPerformed

   private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
   {//GEN-HEADEREND:event_formWindowClosing
      Help_Dialog.logger.entering (Help_Dialog.TAG, "formWindowClosing", evt);

      if (this.properties != null)
      {
	 this.properties.saveWindowStatus (this);
      }

      this.setVisible (false);
      this.dispose ();

      Help_Dialog.logger.exiting (Help_Dialog.TAG, "formWindowClosing", evt);
   }//GEN-LAST:event_formWindowClosing
   
   // Variables declaration - do not modify//GEN-BEGIN:variables
   private final javax.swing.JEditorPane Help_Pane = new javax.swing.JEditorPane();
   private final javax.swing.JScrollPane Help_Scroll = new javax.swing.JScrollPane();
   private final javax.swing.JButton OK_Button = new javax.swing.JButton();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Help_Dialog {\n" +
	 "super = " + super.toString () +
	 ",\nproperties = " + properties +
	 ",\nEditor_Kit = " + Editor_Kit +
	 ",\nEditor_Document = " + Editor_Document +
	 ",\nHelp_Pane = " + Help_Pane +
	 ",\nHelp_Scroll = " + Help_Scroll +
	 ",\nOK_Button = " + OK_Button +
	 "\n}";
   }
} // Help_Dialog
