/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

/**
 * @author krma1
 */
@SuppressWarnings ("HardcodedFileSeparator")
@java.beans.JavaBean (description = "Theme chooser panel")
public class Theme_Canvas
   extends javax.swing.JPanel
{
   private static class Look_And_Feel_Model
      extends javax.swing.AbstractListModel<String>
   {
      private static final long serialVersionUID = -642326951339359668L;
      final String[] names;

      private Look_And_Feel_Model (final java.util.Collection<String> nameList)
      {
	 super ();
	 names = nameList.toArray (String[]::new);
      }

      @Override public int
      getSize ()
      {
	 return names.length;
      }

      @Override public String
      getElementAt (final int index)
      {
	 return names[index];
      }

      @SuppressWarnings ("HardcodedLineSeparator")
      @Override @TestOnly @NotNull
      public String
      toString ()
      {
	 return "Look_And_Feel_Model {\n" +
	    "super = " + super.toString () +
	    ",\nnames = " + java.util.Arrays.toString (names) +
	    "\n}";
      }
   }

   /**
    *
    */
   private static final long serialVersionUID = 6871009617435159865L;
   /**
    * <p> class logger instance </p>
    */
   private static final java.util.logging.Logger logger;
   /**
    * <p> class logger tag. </p>
    */
   private static final String TAG;
   /**
    * Resource Bundle used for I18N
    */
   private static final java.util.ResourceBundle bundle; // NOI18N

   static
   {
      TAG = Theme_Canvas.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
      bundle = java.util.ResourceBundle.getBundle ("net/sourceforge/uiq3/calculator/Bundle"); // NOI18N
   }

   /**
    * Creates new form Theme_Canvas
    */
   public Theme_Canvas ()
   {
      super ();
      logger.entering (TAG, "Theme_Canvas");

      this.initComponents ();

      logger.exiting (TAG, "Theme_Canvas");
   }

   /**
    * Creates new form Theme_Canvas
    */
   public Theme_Canvas (final IProperties properties)
   {
      this ();
      logger.entering (TAG, "Theme_Canvas");

      Theme_Info.setFont (properties.getText_Small_Font ());

      properties.addPropertyChangeListener (evt -> {
	 if (java.util.Objects.equals (evt.getPropertyName (), IProperties.TEXT_SMALL_FONT_ID))
	 {
	    Theme_Info.setFont (properties.getText_Small_Font ());
	 }
      });

      logger.exiting (TAG, "Theme_Canvas");
   }

   /**
    * set look and feel size
    *
    * @param lookAndFeelSize
    *    look and feel to show
    */
   public void
   setLookAndFeelSize (final IProperties.Look_And_Feel_Size_Type lookAndFeelSize)
   {
      Theme_Canvas.logger.entering (Theme_Canvas.TAG, "setLookAndFeelSize", lookAndFeelSize);

      switch (lookAndFeelSize)
      {
	 case Small:
	 {
	    Small_Size_Button.setSelected (true);
	    break;
	 }
	 case Medium:
	 {
	    Medium_Size_Button.setSelected (true);
	    break;
	 }
	 case Large:
	 {
	    Large_Size_Button.setSelected (true);
	    break;
	 }
      } // switch

      Theme_Canvas.logger.exiting (Theme_Canvas.TAG, "setLookAndFeelSize");
      return;
   } // setLookAndFeelSize

   /**
    * set look and feel size
    *
    * @return lookAndFeelSize look and feel to show
    */
   public IProperties.Look_And_Feel_Size_Type
   getLookAndFeelSize ()
   {
      Theme_Canvas.logger.entering (Theme_Canvas.TAG, "getLookAndFeelSize");

      final IProperties.Look_And_Feel_Size_Type retval;

      if (Small_Size_Button.isSelected ())
      {
	 retval = IProperties.Look_And_Feel_Size_Type.Small;
      }
      else if (Large_Size_Button.isSelected ())
      {
	 retval = IProperties.Look_And_Feel_Size_Type.Large;
      }
      else
      {
	 retval = IProperties.Look_And_Feel_Size_Type.Medium;
      } // if

      Theme_Canvas.logger.exiting (Theme_Canvas.TAG, "getLookAndFeelSize", retval);
      return retval;
   } // setLookAndFeelSize

   /**
    * set look and feel size
    *
    * @param lookAndFeelName
    *    look and feel to select
    */
   public void
   setLookAndFeelName (final String lookAndFeelName)
   {
      Theme_Canvas.logger.entering (Theme_Canvas.TAG, "setLookAndFeelName", lookAndFeelName);

      Look_And_Feel_List.setSelectedValue (lookAndFeelName, true);

      Theme_Canvas.logger.exiting (Theme_Canvas.TAG, "setLookAndFeelName");
      return;
   } // setLookAndFeelName

   /**
    * set look and feel size
    *
    * @return lookAndFeelSize look and feel to show
    */
   public String
   getLookAndFeelName ()
   {
      Theme_Canvas.logger.entering (Theme_Canvas.TAG, "getLookAndFeelName");

      final String selectedValue = Look_And_Feel_List.getSelectedValue ();
      final String retval = selectedValue != null
	 ? selectedValue
	 : IProperties.Defaults.Look_And_Feel_Name;

      Theme_Canvas.logger.exiting (Theme_Canvas.TAG, "getLookAndFeelName", retval);
      return retval;
   } // setLookAndFeelName

   /**
    * get list of preinstalled look and feels
    *
    * @return list of  look and feel to display
    */
   public javax.swing.ListModel<String>
   getLookAndFeelModel ()
   {
      Theme_Canvas.logger.entering (Theme_Canvas.TAG, "getLookAndFeelList");

      final javax.swing.UIManager.LookAndFeelInfo[] available = javax.swing.UIManager.getInstalledLookAndFeels ();
      final java.util.stream.Stream<String> nameStream = java.util.Arrays.stream (available).map (
	 javax.swing.UIManager.LookAndFeelInfo::getClassName);
      final java.util.List<String> nameList = nameStream.collect (java.util.stream.Collectors.toList ());

      // Add look and feel frm FlatLaf which are included in the final jar file.
      nameList.add (0, "net.sourceforge.uiq3.calculator.fx602p");
      nameList.add (1, "net.sourceforge.uiq3.calculator.fx603p");
      nameList.add (2, "net.sourceforge.uiq3.calculator.hp45");
      nameList.add ("com.formdev.flatlaf.FlatLightLaf");
      nameList.add ("com.formdev.flatlaf.FlatDarkLaf");
      nameList.add ("com.formdev.flatlaf.FlatIntelliJLaf");
      nameList.add ("com.formdev.flatlaf.FlatDarculaLaf");
      nameList.add ("com.formdev.flatlaf.themes.FlatMacLightLaf");
      nameList.add ("com.formdev.flatlaf.themes.FlatMacDarkLaf");

      final javax.swing.ListModel<String> retval = new Look_And_Feel_Model (nameList);

      Theme_Canvas.logger.exiting (Theme_Canvas.TAG, "getLookAndFeelList", retval);
      return retval;
   } // getLookAndFeelList

   //@formatter:off
   @SuppressWarnings({"LocalCanBeFinal", "DanglingJavadoc", "ChainedMethodCall", "MagicNumber"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {

      setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createTitledBorder(""), bundle.getString("Theme_Label"))); // NOI18N
      setName(bundle.getString("Theme_Canvas_Title")); // NOI18N

      Size_Group.add(Small_Size_Button);
      Small_Size_Button.setText(bundle.getString("Small_Size_Button")); // NOI18N
      Small_Size_Button.setName("Small_Size_Button"); // NOI18N

      Size_Group.add(Medium_Size_Button);
      Medium_Size_Button.setText(bundle.getString("Medium_Size_Button")); // NOI18N
      Medium_Size_Button.setName("Medium_Size_Button"); // NOI18N

      Size_Group.add(Large_Size_Button);
      Large_Size_Button.setText(bundle.getString("Large_Size_Button")); // NOI18N
      Large_Size_Button.setName("Large_Size_Button"); // NOI18N

      Look_And_Feel_Scroll.setName("Look_And_Feel_Scroll"); // NOI18N

      Look_And_Feel_List.setModel(getLookAndFeelModel());
      Look_And_Feel_List.setName("Look_And_Feel_List"); // NOI18N
      Look_And_Feel_Scroll.setViewportView(Look_And_Feel_List);

      Theme_Info.setText(bundle.getString("Theme_Info")); // NOI18N
      Theme_Info.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
      Theme_Info.setName("Theme_Info"); // NOI18N

      javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
      this.setLayout(layout);
      layout.setHorizontalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
            .addContainerGap()
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
               .addGroup(layout.createSequentialGroup()
                  .addComponent(Small_Size_Button)
                  .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                  .addComponent(Medium_Size_Button)
                  .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                  .addComponent(Large_Size_Button)
                  .addGap(204, 204, 204))
               .addGroup(layout.createSequentialGroup()
                  .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                     .addComponent(Look_And_Feel_Scroll)
                     .addComponent(Theme_Info, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                  .addContainerGap())))
      );
      layout.setVerticalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
            .addContainerGap()
            .addComponent(Look_And_Feel_Scroll)
            .addGap(18, 18, 18)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
               .addComponent(Medium_Size_Button)
               .addComponent(Large_Size_Button)
               .addComponent(Small_Size_Button))
            .addGap(18, 18, 18)
            .addComponent(Theme_Info)
            .addContainerGap())
      );

      getAccessibleContext().setAccessibleName("Theme_Canvas");
      getAccessibleContext().setAccessibleDescription("");
   }// </editor-fold>//GEN-END:initComponents

   // Variables declaration - do not modify//GEN-BEGIN:variables
   final javax.swing.JRadioButton Large_Size_Button = new javax.swing.JRadioButton();
   final javax.swing.JList<String> Look_And_Feel_List = new javax.swing.JList<>();
   final javax.swing.JScrollPane Look_And_Feel_Scroll = new javax.swing.JScrollPane();
   final javax.swing.JRadioButton Medium_Size_Button = new javax.swing.JRadioButton();
   final javax.swing.ButtonGroup Size_Group = new javax.swing.ButtonGroup();
   final javax.swing.JRadioButton Small_Size_Button = new javax.swing.JRadioButton();
   final javax.swing.JLabel Theme_Info = new javax.swing.JLabel();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Theme_Canvas {\n" +
	 "super = " + super.toString () +
	 ",\nLarge_Size_Button = " + Large_Size_Button +
	 ",\nLook_And_Feel_List = " + Look_And_Feel_List +
	 ",\nLook_And_Feel_Scroll = " + Look_And_Feel_Scroll +
	 ",\nMedium_Size_Button = " + Medium_Size_Button +
	 ",\nSize_Group = " + Size_Group +
	 ",\nSmall_Size_Button = " + Small_Size_Button +
	 ",\nTheme_Info = " + Theme_Info +
	 "\n}";
   }
}
