/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.printer;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * @author martin
 */
@SuppressWarnings ({"FieldCanBeLocal", "deprecation"})
@java.beans.JavaBean (description = "printer canvas for calculators")
public class Canvas<
   Type extends IPrinter>
   extends javax.swing.JPanel
   implements net.sourceforge.uiq3.printer.ICanvas
{
   private static final long serialVersionUID = 5945721098895493710L;
   /**
    * <p>
    * class logger instance
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p> class logger tag. </p>
    */
   private static final String TAG;
   /**
    * Resource Bundle used for I18N
    */
   private static final java.util.ResourceBundle Calculator_Bundle; // NOI18N
   @Nullable
   private final net.sourceforge.uiq3.ui.IFrame owner;

   static
   {
      TAG = Canvas.class.getName ();
      Logger = java.util.logging.Logger.getLogger (TAG);
      //noinspection HardcodedFileSeparator
      Calculator_Bundle = java.util.ResourceBundle.getBundle ("net/sourceforge/uiq3/calculator/Bundle"); // NOI18N
   }

   /**
    * The printer engine used - needed for status related display.
    */
   @Nullable
   protected Type printer;

   /**
    * Creates new form Printer_Canvas
    */
   public Canvas ()
   {
      super ();
      Canvas.Logger.entering (Canvas.TAG, "Canvas");

      initComponents ();

      this.owner = null;
      this.printer = null;

      Canvas.Logger.exiting (Canvas.TAG, "Canvas");
   } // Printer_Canvas

   /**
    * Creates new form Printer_Canvas
    */
   public Canvas (final @NotNull net.sourceforge.uiq3.ui.IFrame owner)
   {
      super ();
      Canvas.Logger.entering (Canvas.TAG, "Canvas");

      initComponents ();

      this.owner = owner;
      this.printer = null;

      Canvas.Logger.exiting (Canvas.TAG, "Canvas");
   } // Canvas

   /**
    * Set the calculator engine used - needed for status related display.
    *
    * @param printer
    *    printer to display
    */
   public void setPrinter (@NotNull final Type printer)
   {
      this.printer = printer;
      this.printer.addPropertyChangeListener (this::update);
      this.update ();

      return;
   } // Set_Printer

   /**
    * check if the calculator has changed.
    *
    * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
    */
   @SuppressWarnings ("StringEquality")
   public void update (final java.beans.PropertyChangeEvent event)
   {
      try
      {
	 if (isVisible () && event.getPropertyName () == IPrinter.PRINTOUT_PROPERTY)
	 {
	    javax.swing.SwingUtilities.invokeLater (this::update);
	 }
      }
      catch (final java.lang.Exception Exception)
      {
	 Canvas.Logger.log (java.util.logging.Level.SEVERE, "LOG01630:", Exception);
      }
      return;
   } // update

   /**
    * Update Fields.
    */
   public void update ()
   {
      Canvas.Logger.entering (Canvas.TAG, "Update");

      //noinspection ConstantConditions
      if (this.printer != null)
      {
	 this.Printout_TextArea.setText (this.printer.getPrintout ());
	 this.Printout_TextArea.repaint ();
      }

      Canvas.Logger.exiting (Canvas.TAG, "Update");
      return;
   } // Update

   public void doPrintButton1 ()
   {
      Logger.entering (Canvas.TAG, "doPrintButton1");

      if (owner != null)
      {
	 owner.setSelectedComponent (this);
      }

      Logger.exiting (Canvas.TAG, "doPrintButton1");
   }

   public void doPrintButton2 ()
   {
      Logger.entering (Canvas.TAG, "doPrintButton2");

      if (owner != null)
      {
	 owner.setSelectedComponent (this);
      }

      Logger.exiting (Canvas.TAG, "doPrintButton2");
   }

   public void doPrintButton3 ()
   {
      Logger.entering (Canvas.TAG, "doPrintButton3");

      if (owner != null)
      {
	 owner.setSelectedComponent (this);
      }

      Logger.exiting (Canvas.TAG, "doPrintButton3");
   }

   public void doClearButton ()
   {
      Logger.entering (Canvas.TAG, "doClearButton");

      if (owner != null)
      {
	 owner.setSelectedComponent (this);
      }

      if (this.printer != null)
      {
	 this.printer.clearPrintout ();
      }

      Logger.exiting (Canvas.TAG, "doClearButton");
   }

   //@formatter:off
   @SuppressWarnings ({
      "AnonymousInnerClass",
      "ChainedMethodCall",
      "TooBroadScope",
      "DataFlowIssue",
      "LocalCanBeFinal",
      "DanglingJavadoc",
      "override",
      "MagicNumber"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {
      java.awt.GridBagConstraints gridBagConstraints;

      Printout_Menu.setName("Printout_Menu"); // NOI18N

      Copy_MenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/ic_menu_copy.png"))); // NOI18N
      Copy_MenuItem.setText(Calculator_Bundle.getString("Menu_Copy")); // NOI18N
      Copy_MenuItem.setName("Copy_MenuItem"); // NOI18N
      Copy_MenuItem.addActionListener(this::Copy_MenuItemActionPerformed);
      Printout_Menu.add(Copy_MenuItem);
      Copy_MenuItem.getAccessibleContext().setAccessibleDescription("");

      setDoubleBuffered(false);
      setName("Canvas"); // NOI18N
      addComponentListener(new java.awt.event.ComponentAdapter()
      {
         public void componentShown(java.awt.event.ComponentEvent evt)
         {
            formComponentShown(evt);
         }
      });
      setLayout(new java.awt.GridBagLayout());

      Printout_ScrollPane.setName("Printout_ScrollPane"); // NOI18N

      Printout_TextArea.setEditable(false);
      Printout_TextArea.setBackground(javax.swing.UIManager.getDefaults().getColor("EditorPane.background"));
      Printout_TextArea.setColumns(40);
      Printout_TextArea.setRows(5);
      Printout_TextArea.setComponentPopupMenu(Printout_Menu);
      Printout_TextArea.setName("Printout_TextArea"); // NOI18N
      Printout_ScrollPane.setViewportView(Printout_TextArea);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.ipadx = 519;
      gridBagConstraints.ipady = 84;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 4, 8);
      add(Printout_ScrollPane, gridBagConstraints);

      Print_Button_1.setText("jButton1");
      Print_Button_1.setName("Print_Button_1"); // NOI18N
      Print_Button_1.addActionListener(this::Print_Button_1ActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
      gridBagConstraints.insets = new java.awt.Insets(4, 8, 8, 4);
      add(Print_Button_1, gridBagConstraints);

      Print_Button_2.setText("jButton2");
      Print_Button_2.setName("Print_Button_2"); // NOI18N
      Print_Button_2.addActionListener(this::Print_Button_2ActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridy = 1;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 8, 4);
      add(Print_Button_2, gridBagConstraints);

      Print_Button_3.setText("jButton1");
      Print_Button_3.setName("Print_Button_3"); // NOI18N
      Print_Button_3.addActionListener(this::Print_Button_3ActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridy = 1;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 8, 4);
      add(Print_Button_3, gridBagConstraints);

      Feet_Button.setText(Calculator_Bundle.getString("Button_Feed")); // NOI18N
      Feet_Button.setName("Feet_Button"); // NOI18N
      Feet_Button.addActionListener(this::Feet_ButtonActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(4, 8, 8, 8);
      add(Feet_Button, gridBagConstraints);

      Clear_Button.setText(Calculator_Bundle.getString("Clear_Button")); // NOI18N
      Clear_Button.setName("Clear_Button"); // NOI18N
      Clear_Button.addActionListener(this::Clear_ButtonActionPerformed);
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 4;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
      gridBagConstraints.insets = new java.awt.Insets(8, 4, 8, 8);
      add(Clear_Button, gridBagConstraints);
   }// </editor-fold>//GEN-END:initComponents

   private void formComponentShown (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentShown
   {//GEN-HEADEREND:event_formComponentShown
      Logger.entering (Canvas.TAG, "formComponentShown");

      update ();

      Logger.exiting (Canvas.TAG, "formComponentShown");
   }//GEN-LAST:event_formComponentShown

   private void Print_Button_1ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Print_Button_1ActionPerformed
   {//GEN-HEADEREND:event_Print_Button_1ActionPerformed
      Logger.entering (Canvas.TAG, "Print_Button_1ActionPerformed");

      javax.swing.SwingUtilities.invokeLater (this::doPrintButton1);

      Logger.exiting (Canvas.TAG, "Print_Button_1ActionPerformed");
   }//GEN-LAST:event_Print_Button_1ActionPerformed

   private void Print_Button_2ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Print_Button_2ActionPerformed
   {//GEN-HEADEREND:event_Print_Button_2ActionPerformed
      Logger.entering (Canvas.TAG, "Print_Button_2ActionPerformed");

      javax.swing.SwingUtilities.invokeLater (this::doPrintButton2);

      Logger.exiting (Canvas.TAG, "Print_Button_2ActionPerformed");
   }//GEN-LAST:event_Print_Button_2ActionPerformed

   private void Print_Button_3ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Print_Button_3ActionPerformed
   {//GEN-HEADEREND:event_Print_Button_3ActionPerformed
      Logger.entering (Canvas.TAG, "Print_Button_3ActionPerformed");

      javax.swing.SwingUtilities.invokeLater (this::doPrintButton3);

      Logger.exiting (Canvas.TAG, "Print_Button_3ActionPerformed");
   }//GEN-LAST:event_Print_Button_3ActionPerformed

   private void Clear_ButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Clear_ButtonActionPerformed
   {//GEN-HEADEREND:event_Clear_ButtonActionPerformed
      Logger.entering (Canvas.TAG, "Clear_ButtonActionPerformed");

      javax.swing.SwingUtilities.invokeLater (this::doClearButton);

      Logger.exiting (Canvas.TAG, "Clear_ButtonActionPerformed");
   }//GEN-LAST:event_Clear_ButtonActionPerformed


   private void Feet_ButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Feet_ButtonActionPerformed
   {//GEN-HEADEREND:event_Feet_ButtonActionPerformed
      Logger.entering (Canvas.TAG, "Clear_ButtonActionPerformed");

      if (this.printer != null)
      {
	 javax.swing.SwingUtilities.invokeLater (this.printer::printLine);
      }

      Logger.exiting (Canvas.TAG, "Clear_ButtonActionPerformed");
   }//GEN-LAST:event_Feet_ButtonActionPerformed

   private void Copy_MenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Copy_MenuItemActionPerformed
   {//GEN-HEADEREND:event_Copy_MenuItemActionPerformed
      Logger.entering (Canvas.TAG, "Copy_MenuItemActionPerformed", evt);

      net.sourceforge.uiq3.OS.setClipboard (Printout_TextArea);
      
      Logger.exiting (Canvas.TAG, "Copy_MenuItemActionPerformed");
   }//GEN-LAST:event_Copy_MenuItemActionPerformed

   // Variables declaration - do not modify//GEN-BEGIN:variables
   protected final javax.swing.JButton Clear_Button = new javax.swing.JButton();
   protected final javax.swing.JMenuItem Copy_MenuItem = new javax.swing.JMenuItem();
   protected final javax.swing.JButton Feet_Button = new javax.swing.JButton();
   protected final javax.swing.JButton Print_Button_1 = new javax.swing.JButton();
   protected final javax.swing.JButton Print_Button_2 = new javax.swing.JButton();
   protected final javax.swing.JButton Print_Button_3 = new javax.swing.JButton();
   protected final javax.swing.JPopupMenu Printout_Menu = new javax.swing.JPopupMenu();
   protected final javax.swing.JScrollPane Printout_ScrollPane = new javax.swing.JScrollPane();
   protected final javax.swing.JTextArea Printout_TextArea = new javax.swing.JTextArea();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @TestOnly
   @Override @NotNull public String
   getPrintout ()
   {
      return Printout_TextArea.getText ();
   }

   @SuppressWarnings ("HardcodedLineSeparator")
   @TestOnly
   @Override @NotNull public String
   toString ()
   {
      return TAG +
	 "{\nsuper=" + super.toString () +
	 ",\nPrintout_ScrollPane=" + Printout_ScrollPane +
	 ",\nPrintout_TextArea=" + Printout_TextArea +
	 ",\nprinter=" + printer +
	 '}';
   }
} // Printer_Canvas

// vim: set textwidth=0 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=java fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
