/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005…2011 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: SE_Interface.java 7681 2021-09-02 14:15:29Z krischik $
 *   $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Desktop/src/main/java/net/sourceforge/uiq3/tape/SE_Interface.java $
 * ********************************************************** }}}1 ***********
 */

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
package net.sourceforge.uiq3.tape;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Cassette interface for JavaSE
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public final class SE_Interface
   extends
      Interface
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = SE_Interface.class.getName ();
      Logger = java.util.logging.Logger.getLogger (SE_Interface.TAG);
   } // static

   /**
    * File connection
    */
   @Nullable
   private java.io.File Connection;

   /**
    * File in connection
    */
   @Nullable
   private java.io.FileInputStream In_Connection;

   /**
    * File out connection
    */
   @Nullable
   private java.io.FileOutputStream Out_Connection;

   /**
    * Close files and all streams attached to them.
    *
    * @throws java.io.IOException
    *            could not close stream
    * @param Creator
    *           either 602 or 603 depending on system used
    */
   @SuppressWarnings ("HardcodedFileSeparator")
   public void Close (final String Creator)
      throws java.io.IOException
   {
      SE_Interface.Logger.entering (SE_Interface.TAG, "Close");

      if (this.Out_Connection != null)
      {
         this.Out_Connection.close ();
         this.Out_Connection = null;
      } // if
      if (this.In_Connection != null)
      {
         this.In_Connection.close ();

         this.In_Connection = null;
      } // if

      if (this.Connection != null)
      {
         if (Interface.MAC_OS_X)
         {
            final String absolutePath = this.Connection.getAbsolutePath ();
            final File_Type Type = Interface.Type_Of_File (absolutePath);
            final Runtime runtime = Runtime.getRuntime ();

            runtime.exec (
               new String [] {
                  "/usr/bin/SetFile",
                  "-a",
                  "e",
                  "-c",
                  Creator + "P",
                  "-t",
                  Type.Mac_OS_Type,
                  absolutePath });
            runtime.exec (
               new String [] {
                  "/usr/bin/xattr",
                  "-w",
                  "mime_type",
                  Type.Mime_Type (Creator),
                  absolutePath });
         } // if

         this.Connection = null;
      } // if

      SE_Interface.Logger.exiting (SE_Interface.TAG, "Close");
      return;
   } // Close

   /**
    * <p>
    * open stream for binarywriting.
    * </p>
    *
    * @param Home_Directory
    *           home directory to use when filename is not absolute
    * @param Filename
    *           file name
    * @return return data strea
    * @throws java.io.FileNotFoundException
    *            file was not found
    */
   @NotNull
   public java.io.DataOutputStream Create_Stream (@NotNull final String Home_Directory, @NotNull final String Filename)
      throws java.io.FileNotFoundException
   {
      SE_Interface.Logger.entering (
         SE_Interface.TAG,
         "Create",
         new Object [] {
            Home_Directory,
            Filename });

      this.Connection = new java.io.File (Home_Directory, Filename);
      this.Out_Connection = new java.io.FileOutputStream (this.Connection);
      // noinspection UnnecessaryLocalVariable
      final java.io.DataOutputStream Retval = new java.io.DataOutputStream (this.Out_Connection);

      SE_Interface.Logger.exiting (SE_Interface.TAG, "Create");
      return Retval;
   } // Create_Stream

   /**
    * <p>
    * open stream for binarywriting.
    * </p>
    *
    * @param Home_Directory
    *           home directory to use when filename is not absolute
    * @param Filename
    *           file name
    * @return return data strea
    * @throws java.io.FileNotFoundException
    *            file was not found
    */
   @NotNull
   public java.io.Writer Create_Writer (@NotNull final String Home_Directory, @NotNull final String Filename)
      throws java.io.FileNotFoundException,
         java.io.UnsupportedEncodingException
   {
      SE_Interface.Logger.entering (
         SE_Interface.TAG,
         "Create",
         new Object [] {
            Home_Directory,
            Filename });

      this.Connection = new java.io.File (Home_Directory, Filename);
      this.Out_Connection = new java.io.FileOutputStream (this.Connection);
      // noinspection UnnecessaryLocalVariable
      final java.io.Writer Retval = new java.io.OutputStreamWriter (
         this.Out_Connection,
         net.sourceforge.uiq3.tape.Interface.Simple_ASCII);

      SE_Interface.Logger.exiting (SE_Interface.TAG, "Create");
      return Retval;
   } // Create_Stream

   /**
    * Get file Size
    *
    * @return File size
    */
   public long File_Size ()
   {
      return this.Connection != null
         ? this.Connection.length ()
         : 0;
   } // File_Size

   /**
    * <p>
    * open stream for reading.
    * </p>
    *
    * @param Home_Directory
    *           home directory to use when filename is not absolute
    * @param Filename
    *           file name
    * @return return data strea
    * @throws java.io.FileNotFoundException
    *            file was not found
    */
   @NotNull
   public java.io.Reader Open_Reader (@NotNull final String Home_Directory, @NotNull final String Filename)
      throws java.io.FileNotFoundException,
         java.io.UnsupportedEncodingException
   {
      SE_Interface.Logger.entering (
         SE_Interface.TAG,
         "Open",
         new Object [] {
            Home_Directory,
            Filename });

      final java.io.File File = new java.io.File (Filename);
      this.Connection = File.isAbsolute ()
         ? File
         : new java.io.File (Home_Directory, Filename);
      this.In_Connection = new java.io.FileInputStream (this.Connection);
      // noinspection UnnecessaryLocalVariable
      final java.io.Reader Retval = new java.io.InputStreamReader (
         this.In_Connection,
         net.sourceforge.uiq3.tape.Interface.Simple_ASCII);

      SE_Interface.Logger.exiting (SE_Interface.TAG, "Open", Retval);
      return Retval;
   } // Open

   /**
    * <p>
    * open stream for reading.
    * </p>
    *
    * @param Home_Directory
    *           home directory to use when filename is not absolute
    * @param Filename
    *           file name
    * @return return data strea
    * @throws java.io.FileNotFoundException
    *            file was not found
    */
   @NotNull
   public java.io.DataInputStream Open_Stream (@NotNull final String Home_Directory, @NotNull final String Filename)
      throws java.io.FileNotFoundException
   {
      SE_Interface.Logger.entering (
         SE_Interface.TAG,
         "Open",
         new Object [] {
            Home_Directory,
            Filename });

      final java.io.File File = new java.io.File (Filename);
      this.Connection = File.isAbsolute ()
         ? File
         : new java.io.File (Home_Directory, Filename);
      this.In_Connection = new java.io.FileInputStream (this.Connection);
      // noinspection UnnecessaryLocalVariable
      final java.io.DataInputStream Retval = new java.io.DataInputStream (this.In_Connection);

      SE_Interface.Logger.exiting (SE_Interface.TAG, "Open", Retval);
      return Retval;
   } // Open

   @Override
   @org.jetbrains.annotations.TestOnly
   public String toString ()
   {
      return SE_Interface.TAG + "{super=" + super.toString () + ", Connection=" + this.Connection + ", In_Connection=" +
         this.In_Connection + ", Out_Connection=" + this.Out_Connection + '}';
   }
} // SE_Interface

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :
