/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

/**
 * Key mapping info between Java keys and keys of the virtual calculator.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
interface IKey
{
   companion object
   {
      /**
       * No key is pressed
       */
      const val No_Key: Short = -1
   }

   /**
    * GUI identification. Either the number in the keyboard array or the Android widget ID.
    */
   val keyNr: Int

   /**
    * Key Code used inside the Calculator
    *
    * @return key code or No_Key if no key pressed
    */
   val calculatorKeyCode: Short

   /**
    * Key Code used inside the Java
    *
    * @return key code or No_Key if no key pressed
    */
   val javaKeyCode: Int

   val buttonText: String

   /**
    * check if key with Java Key code has been hit.
    *
    * @param Java_Key_Code
    * Key code from the Java virtual machine
    * @return true when key was hit
    */
   fun isKeyHit(Java_Key_Code: Int): Boolean

} // Key

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
