/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005…2011  Martin Krischik
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: IService.java 7681 2021-09-02 14:15:29Z krischik $
 *   $HeadURL: svn+ssh://krischik@svn.code.sf
 * .net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/calculator/IService.java $
 * ********************************************************** }}}1 ***********
 */
package net.sourceforge.uiq3.calculator

typealias Text_Converter_Type = (textLine1: ShortArray, textLine2: ShortArray) -> Array<CharSequence>

/**
 * Background service when available (that would be Android)
 */
interface IService
{
   /**
    * Foreground text should only be generated then needed. Hence we sue call my name interface
    */
   interface Foreground_Text
   {
      /**
       *
       *
       * build the foreground text
       *
       *
       * @param textLine1
       * First Text row
       * @param textLine2
       * Second Text row (unused for FX-602P)
       * @return the actual text
       */
      fun build(textLine1: ShortArray, textLine2: ShortArray): Array<CharSequence?>?
   }

   /**
    * Set Canvas to display result read keyboard
    *
    * @param Calculator_Activity
    * Activity where calculator is displayed in
    */
   fun setCalculatorActivity(Calculator_Activity: net.sourceforge.uiq3.ui.ICalculatorCanvas?)

   /**
    * calculator is started in service
    *
    * @return true whe calculator is started
    */
   val isCalculatorStarted: Boolean

   /**
    * Create Calculator engine.
    *
    * @return New calculator engine
    * @throws java.io.IOException
    * I/O Error
    */
   @Throws(java.io.IOException::class)
   fun createCalculator(): ICalculator

   /**
    * Switch service to foreground and show a notification (which you have to since API 5)
    *
    * @param enable
    * true: show or remove foreground service notification, false: deactivate Foreground and remove notification
    * @param convertText
    * the foreground text as call by name
    * @param textLine1
    * First text row
    * @param textLine2
    * Second text row (unused for FX-602P)
    */
   fun foreground(
      enable: Boolean,
      convertText: Text_Converter_Type,
      textLine1: ShortArray,
      textLine2: ShortArray
   )

   /**
    * Restore calculator status
    */
   fun restoreStatus()

   /**
    * Save calculator status
    */
   fun saveStatus()
} // IService
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96:
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :

