/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.ui

import org.jetbrains.annotations.TestOnly

/**
 * Interface for a display canvas for the calculator
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
interface ICalculatorCanvas
{
   /**
    * Get calculator key code of the currently pressed key.
    *
    * @return calculator key code
    */
   var pressedKey: Short

   /**
    * Get calculator key code of the last key (de)pressed.
    *
    * @return calculator key code
    */
   var readKey: Short

   @get:TestOnly
   val isReadKeyAvailable: Boolean

   /**
    * Set text to be displayed (as java string, used by RPN-45 only).
    *
    * @param Text
    * to be displayed
    */
   @get:TestOnly
   var javaText: String

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    * indicators to be shown
    */
   @get:TestOnly
   var indicators: BooleanArray

   /**
    * Set calculator Thread so the canvas can wake up the calculator after key / touch screen clicks.
    *
    * @param CPU_Thread
    * CPU Thread.
    */
   fun bindCPUThread(CPU_Thread: Thread?)

   /**
    * display additional information on the error. this is device and setup dependent.
    *
    * @param Exception
    * Exception to display
    */
   fun displayException(Exception: Throwable)

   /**
    * set the calculators on of status.
    *
    * @param on
    * calculator is switched on
    */
   fun setOnOff(on: Boolean)

   /**
    * Set text to be displayed (calculator alpha codes). Null implementation for calculators without alpha display.
    *
    * @param Row
    * Display row to set
    * @param Text
    * to be displayed
    */
   fun setDisplayText(Row: Int, Text: ShortArray)

   /**
    * Fet text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   fun getDisplayText(Row: Int): ShortArray

   /**
    * Fet text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   fun getJavaText(Row: Int): String

   /**
    * Toggle Keyboard help on and off
    */
   fun toggleHelp()
} // ICalculatorCanvas
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :

