// @formatter:off
/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7717 $
 * $Date: 2021-10-30 18:54:21 +0200 (Sa, 30. Okt 2021) $
 * $Id: Utils.java 7717 2021-10-30 16:54:21Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/Utils.java $
 * ************************************************************ }}}1 **********
 */
// @formatter:on

package net.sourceforge.uiq3;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Utility functions - so much for a pure OO language.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @since 3.0.7
 */
@SuppressWarnings ({
   "UnusedDeclaration",
   "UnnecessaryLabelOnBreakStatement"})
public final class Utils
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;
   /**
    * <p>
    * Three minutes in nanoseconds
    * </p>
    */
   public static final long Minutes_3 = 180_000_000_000L;
   /**
    * <p>
    * Six minutes in nanoseconds
    * </p>
    */
   public static final long Minutes_6 = 360_000_000_000L;
   /**
    * <p>
    * One seconds in nanoseconds
    * </p>
    */
   public static final long Seconds_1 = 1_000_000_000L;

   /**
    * Read line from file
    *
    * @param Reader
    *    File to read from
    * @return Line read
    * @throws java.io.IOException
    *    can't read line
    * @since uiq 3.0.7
    */
   @Nullable
   @SuppressWarnings ("HardcodedLineSeparator")
   public static String Read_Line (@NotNull final java.io.InputStreamReader Reader)
      throws
      java.io.IOException
   {
      @Nullable final StringBuilder Retval;

      int Char = Reader.read ();

      if (Char == -1)
      {
	 Retval = null;
      }
      else
      {
	 Retval = new StringBuilder (1000);
	 while ((Char != -1) && (Char != '\n') && (Char != '\r'))
	 {
	    Retval.append ((char) Char);
	    Char = Reader.read ();
	 } // while
      } // if

      return Retval != null
	 ? Retval.toString ()
	 : null;
   } // Read_Line

   /**
    * @param Left
    *    Left value
    * @return Left or right - whichever is larger
    * @since uiq 3.0.7
    */
   public static int abs (final int Left)
   {
      final int Retval;

      if (Left < 0)
      {
	 Retval = -Left;
      }
      else
      {
	 Retval = Left;
      } // if

      return Retval;
   } // abs

   /**
    * Concatenate an element to an arrays - is Java primitive! Ada - of course - can do this out of the box
    *
    * @param Array
    *    Array to append to
    * @param Element
    *    element to append
    * @return Array_1 & Element
    */
   public static short @NotNull [] concat (final short @NotNull [] Array, final short Element)
   {
      final short[] Retval = new short[Array.length + 1];

      System.arraycopy (Array, 0, Retval, 0, Array.length);
      Retval[Array.length] = Element;

      return Retval;
   } // concat

   /**
    * Concatenate 2 arrays - is Java primitive! Ada - of course - can do this out of the box
    *
    * @param Array_1
    *    Array to append to
    * @param Array_2
    *    Array to append
    * @return Array_1 & Array_2
    * @since uiq 3.0.7
    */
   public static short @NotNull [] concat (final short @NotNull [] Array_1, final short @NotNull [] Array_2)
   {
      final short[] Retval = new short[Array_1.length + Array_2.length];

      System.arraycopy (Array_1, 0, Retval, 0, Array_1.length);
      System.arraycopy (Array_2, 0, Retval, Array_1.length, Array_2.length);

      return Retval;
   } // concat

   /**
    * Copy an int array to and short array.
    *
    * @param Destination
    *    short array to copy to
    * @param Source
    *    int array to copy from
    */
   public static void copy (final short @NotNull [] Destination, final int @NotNull [] Source)
   {
      Required.greaterEqual (Destination.length, Source.length, "Destination.length");

      for (int i = 0; i < Source.length; i = i + 1)
      {
	 Destination[i] = (short) Source[i];
      } // for

      return;
   } // Value

   /**
    * Copy an int array to and short array.
    *
    * @param Destination
    *    short array to copy to
    * @param Source
    *    int array to copy from
    */
   public static void copy (final short @NotNull [] Destination, final short @NotNull [] Source)
   {
      Required.greaterEqual (Destination.length, Source.length, "Destination.length");

      System.arraycopy (Source, 0, Destination, 0, Source.length);

      return;
   } // Value

   /**
    * compare the content of two arrays.
    *
    * @param Left
    *    Left array
    * @param Right
    *    right array
    * @return true if both arrays contain the same data
    * @since uiq 3.0.7
    */
   public static boolean equals (final boolean @NotNull [] Left, final boolean @NotNull [] Right)
   {
      boolean Retval = true;

      if (Left.length != Right.length)
      {
	 Retval = false;
      }
      else
      {
	 Search:
	 for (int i = 0; i < Left.length; i = i + 1)
	 {
	    if (Left[i] != Right[i])
	    {
	       Retval = false;
	       break Search;
	    } // if
	 } // for
      } // if
      return Retval;
   } // equals

   /**
    * compare the content of two arrays.
    *
    * @param Left
    *    Left array
    * @param Right
    *    right array
    * @return true if both arrays contain the same data
    * @since uiq 3.0.7
    */
   public static boolean equals (final short @NotNull [] Left, final short @NotNull [] Right)
   {
      boolean Retval = true;

      if (Left.length != Right.length)
      {
	 Retval = false;
      }
      else
      {
	 Search:
	 for (int i = 0; i < Left.length; i = i + 1)
	 {
	    if (Left[i] != Right[i])
	    {
	       Retval = false;
	       break Search;
	    } // if
	 } // for
      } // if
      return Retval;
   } // equals

   /**
    * @param Left
    *    Left value
    * @param Right
    *    Right Value
    * @return Left or right - whichever is larger
    * @since uiq 3.0.7
    */
   public static int max (final int Left, final int Right)
   {
      return Math.max (Left, Right);
   } // max

   /**
    * @param Left
    *    Left value
    * @param Middle
    *    Middle Value
    * @param Right
    *    Right Value
    * @return Left or right - whichever is larger
    * @since uiq 6.5.2
    */
   public static int max (final int Left, final int Middle, final int Right)
   {
      // I considered the ... notation here. but you would need an array, and with it
      // heap allocation. Not a good idea for the Android layouts as they make to many
      // recursive calls and backtracking.
      //
      return Utils.max (Left, Utils.max (Middle, Right));
   } // max

   /**
    * @param Value_1
    *    Left value
    * @param Value_2
    *    Middle Value
    * @param Value_3
    *    Right Value
    * @return Left or right - whichever is larger
    * @since uiq 6.5.2
    */
   public static int max (final int Value_1, final int Value_2, final int Value_3, final int Value_4, final int Value_5)
   {
      // I considered the ... notation and a loop here. but you would need an array, and with it
      // heap allocation. Not a good idea for the Android layouts as they make to many
      // recursive calls and backtracking.
      //
      return Utils.max (Value_1, Utils.max (Value_2, Utils.max (Value_3, Utils.max (Value_4, Value_5))));
   } //

   /**
    * @param Left
    *    Left value
    * @param Right
    *    Right Value
    * @return Left or right - whichever is larger
    * @since uiq 3.0.7
    */
   public static int min (final int Left, final int Right)
   {
      return Math.min (Left, Right);
   } // min

   private Utils ()
   {
   }

   static
   {
      TAG = Utils.class.getName ();
      Logger = java.util.logging.Logger.getLogger (Utils.TAG);
   } // static
} // Utils
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
