/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7681 $
 * $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 * $Id: Error.java 7681 2021-09-02 14:15:29Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/calculator/Error.java $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;

/**
 * "Error" is an abstract exception class representing all possible errors the calculator can
 * produce.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public abstract class Error
   extends
      Exception
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    */
   private static final long serialVersionUID = 1L;

   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = Error.class.getName ();
      Logger = java.util.logging.Logger.getLogger (Error.TAG);
   } // static

   /**
    * Superclass for Calculator Exceptions.
    *
    * @see java.lang.Exception#Exception()
    */
   protected Error ()
   {
      super ();
      // Error.Logger.entering(TAG, "Error");

      Error.Logger.log (java.util.logging.Level.SEVERE, "LOG00680:Throw Error : {0}", this.getClass ());

      // Error.Logger.exiting (TAG, "Error", this);
      return;
   } // Error

   /**
    * <p>
    * Exception with message of this error
    * </p>
    *
    * @param Message
    *           error message
    * @see java.lang.Exception#Exception(String)
    */
   protected Error (@NotNull final String Message)
   {
      super (Message);
      // Error.Logger.entering(TAG, "Error", Message);

      Error.Logger.log (java.util.logging.Level.SEVERE, "LOG00690:Throw Error : {0}", Message);

      // Error.Logger.exiting (TAG, "Error", this);
      return;
   } // Error

   /**
    * <p>
    * Exception with message and cause of this error
    * </p>
    *
    * @param Message
    *           error message
    * @param Cause
    *           Cause of this error
    * @see java.lang.Exception#Exception(String, Throwable)
    */
   protected Error (@NotNull final String Message, @NotNull final Throwable Cause)
   {
      super (Message, Cause);
      // Error.Logger.entering(TAG, "Error", new Object [] {Message, Cause});

      Error.Logger.log (
         java.util.logging.Level.SEVERE,
         "LOG00700:Throw Error : {0}",
         new Object [] {
            Message,
            Cause });

      // Error.Logger.exiting(TAG, "Error", this);
      return;
   } // Error

   /**
    * <p>
    * Exception with cause of this error
    * </p>
    *
    * @param Cause
    *           Cause of this error
    * @see java.lang.Exception#Exception(Throwable)
    */
   protected Error (@NotNull final Throwable Cause)
   {
      super (Cause);
      // Error.Logger.entering(TAG, "Error", Cause);

      Error.Logger.log (java.util.logging.Level.SEVERE, "LOG00710:Throw Error", Cause);

      // Error.Logger.exiting(TAG, "Error", this);
      return;
   } // Error
} // Error

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
