/*
 * ******************************************************** {{{1 ************
 *   Copyright © 2005 … 2018 "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 *  ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: IL_Register.java 7681 2021-09-02 14:15:29Z krischik $
 *  ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;

/**
 * <p>
 * </p>
 *
 * @author Martin Krischik «krischik@users.sourceforge.net»
 * @version 1.0
 * @since 1.0
 */
@SuppressWarnings ("UnnecessaryInterfaceModifier")
public interface IL_Register
   extends
      net.sourceforge.uiq3.calculator.IClear
{
   /**
    * Assign one L-Register to another one
    *
    * @param Right
    *           Value to assign
    */
   public void Assign (@NotNull final IL_Register Right);

   /**
    * Format in standard format.
    *
    * @return Formatted string
    */
   @NotNull
   public short [] Format ()
      throws Error;

   /**
    * Format in printer format
    *
    * @return Formatted string
    */
   @NotNull
   public short [] Format_Print ()
      throws Error;

   /**
    * Increment parenthesis level (+-×÷ ... )
    */
   void Inc_Parenthesis ();

   /**
    * Get parenthesis level (+-×÷ ... )
    *
    * @return parenthesis level (0 ... 3)
    */
   public int Parenthesis ();

   /**
    * Set parenthesis level (+-×÷ ... )
    *
    * @param Parenthesis
    *           level (0 ... 3)
    */
   public void Parenthesis (final int Parenthesis);

   /**
    * Set value of the register to NaN
    */
   void Set_NaN ();

   /**
    * Temp_Value of the register.
    *
    * @return current value
    */
   @NotNull
   public net.sourceforge.uiq3.math.Number Value ();

   /**
    * Set value of the register.
    *
    * @param Value
    *           Value to set.
    * @throws Error
    *            number is large or NaN not a value executable for the FX-602P Simulator
    */
   public void Value (@NotNull final net.sourceforge.uiq3.math.Number Value)
      throws Error;

} // IL_Register

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
