// @formatter:off
/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2014  "Martin Krischik" «krischik@users.sourceforge.net» (extensions for FX-602P)
 * Copyright © 2005  Kai G. Schwebke, upn.schwebke.com
* **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7681 $
 * $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 * $Id: BCDError.java 7681 2021-09-02 14:15:29Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/math/BCDError.java $
 * ************************************************************ }}}1 **********
 */
//  @formatter:on

package net.sourceforge.uiq3.math;

import org.jetbrains.annotations.NotNull;

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public final class BCDError
   extends
   Exception
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    *
    */
   private static final long serialVersionUID = 5L;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = BCDError.class.getName ();
      Logger = java.util.logging.Logger.getLogger (BCDError.TAG);
   } // static

   /**
    *
    */
   public BCDError ()
   {
      super ();

      return;
   } // BCDError

   /**
    * <p>
    * Exception with message of this error
    * </p>
    *
    * @param Message
    *    error message
    * @see java.lang.Exception#Exception(String)
    */
   public BCDError (@NotNull final String Message)
   {
      super (Message);
      // BCDError.Logger.entering(TAG, "BCDError", new Object [] {Message});

      BCDError.Logger.log (java.util.logging.Level.WARNING, "LOG00061: Throw {0}", Message);

      // BCDError.Logger.exiting(TAG, "BCDError", this);
      return;
   } // BCDError

   /**
    * <p>
    * Exception with message and cause of this BCDError
    * </p>
    *
    * @param Message
    *    error message
    * @param Cause
    *    Cause of this BCDError
    * @see java.lang.Exception#Exception(String, Throwable)
    */
   public BCDError (@NotNull final String Message, @NotNull final Throwable Cause)
   {
      super (Message, Cause);
      // BCDError.Logger.entering(TAG, "BCDError", new Object [] {Message, Cause});

      BCDError.Logger.log (
	 java.util.logging.Level.WARNING,
	 "LOG00060: Throw {0}",
	 new Object[] {
	    Message,
	    Cause});

      // BCDError.Logger.exiting(TAG, "BCDError", this);
      return;
   } // BCDError

   /**
    * <p>
    * Exception with cause of this BCDError
    * </p>
    *
    * @param Cause
    *    Cause of this BCDError
    * @see java.lang.Exception#Exception(Throwable)
    */
   public BCDError (@NotNull final Throwable Cause)
   {
      super (Cause);
      // BCDError.Logger.entering(TAG, "BCDError", Cause);

      BCDError.Logger.log (java.util.logging.Level.WARNING, "LOG00060: Throw", Cause);

      // BCDError.Logger.exiting(TAG, "BCDError", this);
      return;
   } // BCDError
} // BCDError

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
