// @formatter:off
/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2014  "Martin Krischik" «krischik@users.sourceforge.net» (extensions for FX-602P)
 * Copyright © 2005  Kai G. Schwebke, upn.schwebke.com
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7681 $
 * $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 * $Id: BCDFraction.java 7681 2021-09-02 14:15:29Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/math/BCDFraction.java $
 * ************************************************************ }}}1 **********
 */
//  @formatter:on

package net.sourceforge.uiq3.math;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

/**
 * Decimal_Fraction Arithmetic.
 * <p>
 * For the use in an FX-603P Simulator the precision is fixed at 14 decimal digits.
 * </p>
 */
@SuppressWarnings ({
   "ChainedMethodCall",
   "LawOfDemeter",
   "FeatureEnvy",
   "NonAsciiCharacters"})
public final class BCDFraction
   implements
   Number
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;
   /**
    * <p>
    * matches «2/3»
    * </p>
    */
   private static final java.util.regex.Pattern Values_2;
   /**
    * <p>
    * matches «1 2/3»
    * </p>
    */
   private static final java.util.regex.Pattern Values_3;
   /**
    * <p>
    * e approximation used for unit tests
    * </p>
    */
   @NotNull
   public static final BCDFraction Approx_e;
   /**
    * <p>
    * π approximation used for unit tests
    * </p>
    */
   @NotNull
   public static final BCDFraction Approx_π;
   /**
    * <p>
    * 0
    * </p>
    */
   @NotNull
   public static final BCDFraction Num_0;
   /**
    * <p>
    * 1
    * </p>
    */
   @NotNull
   public static final BCDFraction Num_1;
   /**
    * <p>
    * 180
    * </p>
    */
   @NotNull
   public static final BCDFraction Num_180;
   /**
    * <p>
    * 2
    * </p>
    */
   @NotNull
   public static final BCDFraction Num_2;
   /**
    * <p>
    * 200
    * </p>
    */
   @NotNull
   public static final BCDFraction Num_200;
   /**
    * <p>
    * The Denominator:
    * </p>
    * <samp> Whole + Numerator / Denominator</samp>
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   @NotNull
   private BCDInteger Denominator;
   /**
    * <p>
    * The Numerator:
    * </p>
    * <samp>Numerator / Denominator</samp>
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   @NotNull
   private BCDInteger Numerator;

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Value
    *    value to copy
    */
   public BCDFraction (@NotNull final BCDFraction Value)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", Value);

      this.Numerator = Value.Numerator;
      this.Denominator = Value.Denominator;

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Whole
    *    The whole part of the number
    */
   public BCDFraction (@NotNull final BCDInteger Whole)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", Whole);

      this.Numerator = Whole;
      this.Denominator = BCDInteger.Num_1;

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Numerator
    *    The numerator
    * @param Denominator
    *    The denominator
    */
   public BCDFraction (@NotNull final BCDInteger Numerator, @NotNull final BCDInteger Denominator)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", new Object [] {
      // Numerator, Denominator });

      this.Numerator = Numerator;
      this.Denominator = Denominator;

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Whole
    *    The whole part of the number
    */
   public BCDFraction (final long Whole)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", Whole);

      this.Numerator = new BCDInteger (Whole);
      this.Denominator = BCDInteger.Num_1;

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Whole
    *    The whole part of the number
    * @param Numerator
    *    The numerator
    * @param Denominator
    *    The denominator
    */
   public BCDFraction (final long Whole, final long Numerator, final long Denominator)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", new Object [] {
      // Whole, Numerator, Denominator });

      this.Numerator = new BCDInteger (Numerator + (Whole * Denominator));
      this.Denominator = new BCDInteger (Denominator);

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction
    * </p>
    *
    * @param Whole
    *    The whole part of the number
    * @param Numerator
    *    The numerator
    * @param Denominator
    *    The denominator
    */
   public BCDFraction (
      @NotNull final Number Whole,
      @NotNull final Number Numerator,
      @NotNull final BCDInteger Denominator)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", new Object [] {
      // Whole, Numerator, Denominator });

      this.Numerator = Numerator.$plus (Whole.$times (Denominator)).As_BCDInteger ();
      this.Denominator = Denominator;

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * <p>
    * create new fraction from a string
    * </p>
    *
    * @param Text
    *    A String of the type "999" or “999/999” or "999 999/999"
    */
   public BCDFraction (@NotNull final String Text)
   {
      super ();
      // BCDFraction.Logger.entering (BCDFraction.TAG, "BCDFraction", Text);

      final String Value;
      final boolean Negative;

      // Numbers formatted for the serial port might have preceding spaces
      if (Text.startsWith (" "))
      {
	 Value = Text.substring (1);
	 Negative = false;
      }
      else if (Text.startsWith ("-"))
      {
	 Value = Text.substring (1);
	 Negative = true;
      }
      else
      {
	 Value = Text;
	 Negative = false;
      }

      final java.util.regex.Matcher Matcher_3 = BCDFraction.Values_3.matcher (Value);

      if (Matcher_3.matches ())
      {
	 final Number Whole = new BCDInteger (Matcher_3.group (1));
	 final Number Numerator = new BCDInteger (Matcher_3.group (2));
	 final BCDInteger Denominator = new BCDInteger (Matcher_3.group (3));

	 this.Numerator = Numerator.$plus (Whole.$times (Denominator)).As_BCDInteger ();
	 this.Denominator = Denominator;
      }
      else
      {
	 final java.util.regex.Matcher Matcher_2 = BCDFraction.Values_2.matcher (Value);

	 if (Matcher_2.matches ())
	 {
	    this.Numerator = new BCDInteger (Matcher_2.group (1));
	    this.Denominator = new BCDInteger (Matcher_2.group (2));
	 }
	 else
	 {
	    this.Numerator = new BCDInteger (Value);
	    this.Denominator = BCDInteger.Num_1;
	 } // if
      } // if

      if (Negative)
      {
	 this.Numerator = this.Numerator.neg ().As_BCDInteger ();
      } // if

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "BCDFraction", this);
      return;
   } // BCDFraction

   /**
    * x ÷ y
    *
    * @param y
    *    divisor
    * @return x ÷ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $div (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFloat)
      {
	 if (y.Is_Integer ())
	 {
	    Retval = this.$div (y.As_BCDInteger ());
	 }
	 else
	 {
	    Retval = this.As_BCDFloat ().$div (y);
	 }
      }
      else
      {
	 final BCDFraction Left = y.As_BCDFraction ();
	 Number Numerator = this.Numerator.$times (Left.Denominator);
	 Number Denominator = this.Denominator.$times (Left.Numerator);

	 if (Denominator.Is_Negative ())
	 // Correct sign. Might be wrong because we multiply Denominator with
	 // Numerator
	 {
	    Numerator = Numerator.neg ();
	    Denominator = Denominator.neg ();
	 } // if

	 Retval = new BCDFraction (Numerator.As_BCDInteger (), Denominator.As_BCDInteger ());
      } // if

      Retval.Normalize ();

      return Retval;
   }

   /**
    * x - y
    *
    * @param y
    *    value to subtract
    * @return x - y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $minus (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFloat)
      {
	 if (y.Is_Integer ())
	 {
	    Retval = this.$minus (y.As_BCDInteger ());
	 }
	 else
	 {
	    Retval = this.As_BCDFloat ().$minus (y);
	 } // if
      }
      else
      {
	 final BCDFraction Left = y.As_BCDFraction ();
	 final Number Numerator = (this.Numerator.$times (Left.Denominator)).$minus (
	    Left.Numerator.$times (this.Denominator));
	 final Number Denominator = this.Denominator.$times (Left.Denominator);
	 Retval = new BCDFraction (Numerator.As_BCDInteger (), Denominator.As_BCDInteger ());
      } // if

      Retval.Normalize ();

      return Retval;
   }

   /**
    * x + y
    *
    * @param y
    *    value to add
    * @return x + y
    */
   @Contract (pure = true)
   @SuppressWarnings ("FeatureEnvy")
   @NotNull
   @Override
   public Number $plus (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFloat)
      {
	 if (y.Is_Integer ())
	 {
	    Retval = this.$plus (y.As_BCDInteger ());
	 }
	 else
	 {
	    Retval = this.As_BCDFloat ().$plus (y);
	 } // if
      }
      else
      {
	 final BCDFraction Left = y.As_BCDFraction ();
	 final Number Numerator = (this.Numerator.$times (Left.Denominator)).$plus (
	    Left.Numerator.$times (this.Denominator));
	 final Number Denominator = this.Denominator.$times (Left.Denominator);
	 Retval = new BCDFraction (Numerator.As_BCDInteger (), Denominator.As_BCDInteger ());
      } // if

      Retval.Normalize ();

      return Retval;
   }

   /**
    * x × y
    *
    * @param y
    *    Multiplier
    * @return x × y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $times (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFloat)
      {
	 if (y.Is_Integer ())
	 {
	    Retval = this.$times (y.As_BCDInteger ());
	 }
	 else
	 {
	    Retval = this.As_BCDFloat ().$times (y);
	 } // if
      }
      else
      {
	 final BCDFraction Left = y.As_BCDFraction ();
	 final Number Numerator = this.Numerator.$times (Left.Numerator);
	 final Number Denominator = this.Denominator.$times (Left.Denominator);

	 Retval = new BCDFraction (Numerator.As_BCDInteger (), Denominator.As_BCDInteger ());
      } // if

      Retval.Normalize ();

      return Retval;
   }

   /**
    * x<sup>y</sup>
    *
    * @param y
    *    power
    * @return x<sup>y</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $times$times (@NotNull final Number y)
      throws
      BCDError
   {
      return this.As_BCDFloat ().$times$times (y);
   } // pow

   /**
    * <p>
    * Either convert to a BCDFloat or return this.
    * </p>
    *
    * @return value as java int
    */
   @NotNull
   @Override
   public BCDFloat As_BCDFloat ()
   {
      return (this.Numerator.As_BCDFloat ().$div (this.Denominator.As_BCDFloat ())).As_BCDFloat ();
   }

   /**
    * <p>
    * Either convert to a or return this.
    * </p>
    *
    * @return value as java int
    */
   @SuppressWarnings ("ReturnOfThis")
   @Override
   @NotNull
   public BCDFraction As_BCDFraction ()
   {
      return this;
   }

   /**
    * <p>
    * Either convert to a BCDInteger or return this.
    * </p>
    *
    * @return value as java int
    */
   @NotNull
   @Override
   public BCDInteger As_BCDInteger ()
   {
      return this.a ();
   }

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDFloat from an existing one.
    * @see Object#clone()
    */
   @NotNull
   @Override
   public Number Clone ()
   {
      return new BCDFraction (this);
   } // Clone

   /**
    * <p>
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    * </p>
    *
    * @return Coefficient
    * @throws BCDError
    *    when it's not a normal number
    */
   @Override
   public long Coefficient_As_Long ()
      throws
      BCDError
   {
      return this.a ().Coefficient_As_Long ();
   } // Coefficient_As_Long

   /**
    * Compare values
    *
    * @param Value
    *    Value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   @Override
   public int Compare (@NotNull final Number Value)
   {
      // BCDFraction.Logger.entering (BCDFraction.TAG, "Compare", new Object [] {
      // this, Value });

      // noinspection UnnecessaryLocalVariable
      final int Retval = this.As_BCDFloat ().compareTo (Value);

      // BCDFraction.Logger.exiting (BCDFraction.TAG, "Compare", Retval);
      return Retval;
   } // Compare

   /**
    * number: sum(i = 0 ... Num_Digits-1; Digits[i]*10^i)
    */
   @Override
   public byte[] Digits ()
   {
      return this.a ().Digits ();
   }

   /**
    * current exponent - note that the exponent is based on the internal representation where the radix point is right
    * most. For example π is 314159265358979324×10<sup>-17</sup>.
    *
    * @return Current exponent.
    * @throws BCDError
    *    when the value is not finite
    */
   @Override
   public int Exponent ()
      throws
      BCDError
   {
      return 0;
   }

   /**
    * Round to a given Precision after the decimal point.
    *
    * @param Decimal
    *    decimal digits to keep
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Fix (final int Decimal, final int Exponent)
   {
      return new BCDFraction (this);
   }

   /**
    * Recompose a frac double from the given data.
    *
    * @return a double
    */
   @NotNull
   @Override
   public Number Frac ()
   {
      return this.As_BCDFloat ().Frac ();
   }

   /**
    * <p>
    * Get valuef or indirect addressing.
    * </p>
    *
    * @param Digit_Count
    *    Should be 1 … 2 depending if value is used to access memory or perform a goto.
    * @return int value for indirect addressing.
    */
   @Override
   public int Indirect_Value (final int Digit_Count)
   {
      return this.As_BCDInteger ().Indirect_Value (Digit_Count);
   }

   /**
    * <p>
    * integer part of the BCDFloat
    * </p>
    * </p>
    * Do not mix up with To_Integer - This version returns another BCDFloat!
    * </p>
    *
    * @return integer part of this
    */
   @NotNull
   @Override
   public Number Integer ()
   {
      return this.As_BCDFloat ().Integer ();
   }

   /**
    * current value is a normal number
    *
    * @return true when it is a normal number
    */
   @Override
   public boolean Is_Finite ()
   {
      return true;
   }

   /**
    * current value is Infinity (but not N/A).
    *
    * @return true when x is infinite
    */
   @Override
   public boolean Is_Infinite ()
   {
      return false;
   }

   /**
    * Current value is integer value.
    *
    * @return true when x is an integer number
    */
   @Override
   public boolean Is_Integer ()
   {
      return this.Denominator.equals (net.sourceforge.uiq3.math.BCDInteger.Num_1);
   }

   /**
    * current value is not a number
    *
    * @return when is not a number
    */
   @Override
   public boolean Is_NaN ()
   {
      return false;
   }

   /**
    * Current value us negative
    *
    * @return true when x <= -0
    */
   @Override
   public boolean Is_Negative ()
   {
      return this.Numerator.Is_Negative ();
   }

   /**
    * value is 0
    *
    * @return true when x is 0
    */
   @Override
   public boolean Is_Zero ()
   {
      return this.Numerator.Is_Zero ();
   }

   /**
    * <p>
    * normalize the value
    * </p>
    * <p>
    * depending on type: remove leading zeros and / or trailing zeros
    * </p>
    */
   @SuppressWarnings ("FeatureEnvy")
   @Override
   public void Normalize ()
   {
      final BCDInteger gcd = this.Numerator.gcd (this.Denominator);

      if (!gcd.equals (net.sourceforge.uiq3.math.BCDInteger.Num_1))
      {
	 this.Numerator = this.Numerator.$div (gcd).As_BCDInteger ();
	 this.Denominator = this.Denominator.$div (gcd).As_BCDInteger ();
      } // if

      return;
   } // Normalize

   /**
    * the current number of digits - not that when the number is not normalised there might be more 0's in the number
    * then strictly needed.
    *
    * @return number of digits
    * @throws BCDError
    */
   @Override
   public int Num_Digits ()
      throws
      BCDError
   {
      return this.a ().Num_Digits ();
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return x = cos (w) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_X (@NotNull final Number θ)
   {
      return this.As_BCDFloat ().P_To_X (θ);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return x = cos (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_X (@NotNull final Number θ, @NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().P_To_X (θ, Half_Circle);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_Y (@NotNull final Number θ)
   {
      return this.As_BCDFloat ().P_To_Y (θ);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_Y (@NotNull final Number θ, @NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().P_To_Y (θ, Half_Circle);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return r = √(x²+y²)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_R (@NotNull final Number y)
   {
      return this.As_BCDFloat ().R_To_R (y);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_θ (@NotNull final Number y)
   {
      return this.As_BCDFloat ().R_To_θ (y);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_θ (@NotNull final Number y, @NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().R_To_θ (y, Half_Circle);
   }

   /**
    * Round to effective Precision and Exponent.
    *
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Round ()
   {
      return new BCDFraction (this);
   }

   /**
    * Round to a given Precision and Exponent.
    *
    * @param Precision
    *    Total precison
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Round (final int Precision, final int Exponent)
   {
      return new BCDFraction (this);
   }

   /**
    * convert to native/debug string representation
    *
    * @return (Coefficient, Exponent)
    */
   @NotNull
   @Override
   public String To_Debug_String ()
   {
      return net.sourceforge.uiq3.math.BCDFraction.TAG + "{super=" + super.toString () + ", Numerator=" +
	 this.Numerator + ", Denominator=" + this.Denominator + '}';
   }

   /**
    * Convert to integer
    * </p>
    * Do not mix up with Integer - This version returns a Java int!
    * </p>
    *
    * @return value as java int
    */
   @Override
   public int To_Integer ()
   {
      return this.a ().To_Integer ();
   }

   /**
    * Convert to long integer
    * </p>
    * Do not mix up with Integer - This version returns a Java int!
    * </p>
    *
    * @return value as java int
    */
   @Override
   public long To_Long ()
   {
      return this.a ().To_Long ();
   }

   /**
    * <p>
    * Fraktur represented as «a b/c»
    * </p>
    *
    * @return a
    */
   @NotNull
   public BCDInteger a ()
   {
      final BCDInteger Retval;

      if (this.Numerator.Compare_Unsigned (this.Denominator) == Number.Compare_Greater)
      {
	 Retval = (this.Numerator.$div (this.Denominator)).As_BCDInteger ();
      }
      else if (this.Numerator.Compare_Unsigned (this.Denominator) == Number.Compare_Equal)
      {
	 Retval = BCDInteger.Num_1;
      }
      else
      {
	 Retval = BCDInteger.Num_0;
      } // if

      return Retval.abs ().As_BCDInteger ();
   } // a

   /**
    * |x|
    *
    * @return abs value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number abs ()
   {
      final BCDFraction Retval = new BCDFraction (this.As_BCDFraction ());

      Retval.Numerator = Retval.Numerator.abs ().As_BCDInteger ();

      return Retval;
   }

   /**
    * <p>
    * Logical and
    * </p>
    *
    * @return x ∧ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number and (@NotNull final Number y)
   {
      return this.As_BCDInteger ().and (y);
   }

   /**
    * arc cosine
    *
    * @return arc cosine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos ()
   {
      return this.As_BCDFloat ().arc_cos ();
   }

   /**
    * arc cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arccos (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().arc_cos (Half_Circle);
   }

   /**
    * area hyperbolic cosine: log<sub>e</sub> (x + √(x-1) × √(x+1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos_hyp ()
   {
      return this.As_BCDFloat ().arc_cos_hyp ();
   }

   /**
    * arc sine
    *
    * @return arc sine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin ()
   {
      return this.As_BCDFloat ().arc_sin ();
   }

   /**
    * arc sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arcsin (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().arc_sin (Half_Circle);
   }

   /**
    * area hyperbolic sine: log<sub>e</sub> (x + √(x² + 1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin_hyp ()
   {
      return this.As_BCDFloat ().arc_sin_hyp ();
   }

   /**
    * arc tangent
    *
    * @return arc tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan ()
   {
      return this.As_BCDFloat ().arc_tan ();
   }

   /**
    * arc tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arctan (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().arc_tan (Half_Circle);
   }

   /**
    * area hyperbolic tangent: log<sub>e</sub> ((1 + x) ÷ (1 - x)) / 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan_hyp ()
   {
      return this.As_BCDFloat ().arc_tan_hyp ();
   }

   /**
    * <p>
    * Fraktur represented as «a b/c»
    * </p>
    *
    * @return b
    */
   @NotNull
   public BCDInteger b ()
   {
      final BCDInteger Retval;

      if (this.Numerator.Compare_Unsigned (this.Denominator) == Number.Compare_Greater)
      {
	 // noinspection ResultOfMethodCallIgnored
	 this.Numerator.$div (this.Denominator);

	 Retval = this.Numerator.Last_Remainder ();
      }
      else if (this.Numerator.Compare_Unsigned (this.Denominator) == Number.Compare_Equal)
      {
	 Retval = BCDInteger.Num_0;
      }
      else
      {
	 Retval = this.Numerator;
      } // if

      assert Retval != null : "Last_Remainder is set after division";

      return Retval.abs ().As_BCDInteger ();
   } // b

   /**
    * <p>
    * Fraktur represented as «a b/c» or «d/c»
    * </p>
    *
    * @return c
    */
   @NotNull
   public BCDInteger c ()
   {
      return this.Denominator.abs ().As_BCDInteger ();
   } // c

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDInteger from an existing one.
    * @see java.lang.Object#clone()
    */
   @NotNull
   @Override
   public Object clone ()
      throws
      CloneNotSupportedException
   {
      return new BCDFraction (this);
   } // clone

   /**
    * @param rightValue
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @see Comparable#compareTo(Object)
    */
   @Override
   public int compareTo (@NotNull final Number rightValue)
   {
      if (!(rightValue instanceof BCDFraction))
      {
	 throw new java.lang.IllegalArgumentException ("Compare with other BCDFraction only.");
      }

      return this.Compare (rightValue);
   } // compareTo

   /**
    * cos (x)
    *
    * @return cos (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos ()
   {
      return this.As_BCDFloat ().cos ();
   } // cos

   /**
    * cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().cos ();
   } // cos

   /**
    * Hyperbolic cosine: (Exponent^x + Exponent^-x)/2
    *
    * @return Hyperbolic cosine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos_hyp ()
   {
      return this.As_BCDFloat ().cos_hyp ();
   } // cos_hyp

   /**
    * <p>
    * Fraktur represented as «d/c».
    * </p>
    *
    * @return d
    */
   @NotNull
   public BCDInteger d ()
   {
      return this.Numerator.abs ().As_BCDInteger ();
   } // d

   /**
    * @param rightValue
    *    right value
    * @return true when the values are equal
    */
   @Override
   public boolean equals (@NotNull final Object rightValue)
   {
      final boolean Retval;

      if (rightValue instanceof BCDFloat)
      {
	 final BCDFloat Right = ((BCDFloat) rightValue);
	 final BCDFloat Left = this.As_BCDFloat ();

	 try
	 {
	    Retval = Left.Compare (Right) == Number.Compare_Equal;
	 }
	 catch (@NotNull final BCDError exception)
	 {
	    BCDFraction.Logger.log (java.util.logging.Level.SEVERE, "LOG01050:", exception);
	    throw new java.lang.IllegalStateException ("LOG01050:", exception);
	 }
      }
      else
      {
	 final BCDFraction Right;
	 if (rightValue instanceof String)
	 {
	    Right = new BCDFraction ((String) rightValue);
	 }
	 else if (rightValue instanceof BCDInteger)
	 {
	    Right = new BCDFraction ((BCDInteger) rightValue);
	 }
	 else if (rightValue instanceof BCDFraction)
	 {
	    Right = ((BCDFraction) rightValue);
	 }
	 else
	 {
	    throw new java.lang.IllegalArgumentException ("Compare with other BCDInteger and String only.");
	 } // if
	 Retval = this.Compare (Right) == Number.Compare_Equal;
      } // if

      return Retval;
   } // equals

   /**
    * 10<sup>x</sup>
    *
    * @return 10<sup>x</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number exp_10 ()
      throws
      BCDError
   {
      return this.As_BCDFloat ().exp_10 ();
   }

   /**
    * e<sup>x</sup>
    *
    * @return e<sup>x</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number exp_e ()
   {
      return this.As_BCDFloat ().exp_e ();
   }

   @Override public int hashCode ()
   {
      return java.util.Objects.hash (Denominator, Numerator);
   }

   /**
    * log<sub>10</sub> (x)
    *
    * @return log<sub>10</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number log_10 ()
   {
      return this.As_BCDFloat ().log_10 ();
   } // log_10

   /**
    * log<sub>e</sub> (x)
    *
    * @return log<sub>e</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number log_e ()
   {
      return this.As_BCDFloat ().log_e ();
   } // log_e

   /**
    * @return negative value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number neg ()
   {
      final BCDFraction Retval = new BCDFraction (this.As_BCDFraction ());

      Retval.Numerator = Retval.Numerator.neg ().As_BCDInteger ();

      return Retval;
   }

   /**
    * <p>
    * Logical not
    * </p>
    *
    * @return ¬ x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number not ()
   {
      return this.As_BCDInteger ().not ();
   }

   /**
    * <p>
    * Logical or
    * </p>
    *
    * @return x ∨ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number or (@NotNull final Number y)
   {
      return this.As_BCDInteger ().or (y);
   }

   /**
    * <sup>r</sup>√x
    *
    * @param r
    *    root
    * @return e<sup>(log<sub>e</sub> x ÷ r)</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number root (@NotNull final Number r)
   {
      return this.As_BCDFloat ().root (r);
   }

   /**
    * sine
    *
    * @return sin (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin ()
   {
      return this.As_BCDFloat ().sin ();
   }

   /**
    * sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().sin (Half_Circle);
   }

   /**
    * hyperbolic sine: (e<sup>x</sup> - e<sup>-x</sup>) ÷ 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin_hyp ()
   {
      return this.As_BCDFloat ().sin_hyp ();
   }

   /**
    * x<sup>2</sup>
    *
    * @return x<sup>2</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number square ()
   {
      return this.As_BCDFloat ().square ();
   }

   /**
    * <sup>2</sup>√x
    *
    * @return <sup>2</sup>√x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number square_root ()
   {
      return this.As_BCDFloat ().square_root ();
   }

   /**
    * tan x
    *
    * @return tan (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number tan ()
   {
      return this.As_BCDFloat ().tan ();
   }

   /**
    * tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number tan (@NotNull final Number Half_Circle)
   {
      return this.As_BCDFloat ().tan (Half_Circle);
   }

   /**
    * Hyperbolic tangent: (e<sup>2x</sup> - 1) ÷ (e<sup>2x</sup> + 1)
    *
    * @return Hyperbolic tangent.
    */
   @NotNull
   @Override
   public Number tan_hyp ()
   {
      return this.As_BCDFloat ().tan_hyp ();
   }

   /**
    * <p>
    * display as debug string
    * </p>
    *
    * @see java.lang.Object#toString()
    */
   @SuppressWarnings ("HardcodedFileSeparator")
   @Override
   @NotNull
   public String toString ()
   {
      final StringBuilder Retval = new StringBuilder (100);

      if (this.Is_Negative ())
      // The math library supports -0
      {
	 Retval.append ('-');
      }

      if (this.Is_Zero ())
      {
	 Retval.append ('0');
      }
      else
      {
	 final BCDInteger a = this.a ();
	 final BCDInteger b = this.b ();
	 final BCDInteger c = this.c ();

	 if (!a.Is_Zero ())
	 {
	    Retval.append (a);

	    if (!b.Is_Zero ())
	    {
	       Retval.append (' ');
	    } // if
	 } // if

	 if (!b.Is_Zero ())
	 {
	    Retval.append (b.abs ());
	    Retval.append ('/');
	    Retval.append (c);
	 } // if
      } // if
      return Retval.toString ();
   } // toString

   /**
    * <p>
    * Logical xor
    * </p>
    *
    * @return x ⊻ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number xor (@NotNull final Number y)
   {
      return this.As_BCDInteger ().xor (y);
   }

   static
   {
      TAG = BCDFraction.class.getName ();
      Logger = java.util.logging.Logger.getLogger (BCDFraction.TAG);
      Approx_e = new BCDFraction (2L, 7L, 10L);
      Approx_π = new BCDFraction (3L, 1L, 7L);
      Num_0 = new BCDFraction (0L);
      Num_1 = new BCDFraction (1L);
      Num_180 = new BCDFraction (180L);
      Num_2 = new BCDFraction (2L);
      Num_200 = new BCDFraction (200L);
      Values_2 = java.util.regex.Pattern.compile ("(\\d*)/(\\d*)");
      // Numbers formatted for the serial port use two slashes instead of one space and
      // one slash
      Values_3 = java.util.regex.Pattern.compile ("(\\d*)[/ ](\\d*)/(\\d*)");
   } // static
} // BCDFraction

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
