/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005…2011 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: IPrinter.java 7681 2021-09-02 14:15:29Z krischik $
 *   $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/printer/IPrinter.java $
 * ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.printer;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

/**
 * <p>
 * Printer interface
 * </p>
 */
public interface IPrinter
{
   /**
    * <p>
    * Default size for the print buffer
    * </p>
    */
   static final int Default_Printout_Size = 8192;

   public static final String PRINTOUT_PROPERTY = "IPrinter::printout";

   public void addPropertyChangeListener (final java.beans.PropertyChangeListener listener);

   public void removePropertyChangeListener (final java.beans.PropertyChangeListener listener);

   /**
    * <p>
    * clear printout
    * </p>
    */
   public void clearPrintout ();

   /**
    * Print empty line
    */
   public void printLine ();

   /**
    * Print a single line to simulated printer
    *
    * @param Line
    *           line to print
    */
   public void printLine (short @NotNull  [] Line);

   /**
    * Print a single line to simulated printer
    *
    * @param Line
    *           Line to print
    */
   public void printLine (@NotNull String Line);

   /**
    * get printout for clipboard
    *
    * @return printout as java string
    */
   @java.beans.BeanProperty (
      bound = true,
      description = "The text to printed")
   @NotNull
   @TestOnly
   public String getPrintout ();

   /**
    * <p>
    * set the printout from saved data
    * </p>
    *
    * @param Printout
    *           new printout
    */
   @TestOnly
   public void setPrintout (@NotNull final String Printout);

   /**
    * get printout for clipboard
    *
    * @return the current length of the printout
    */
   public int getPrintoutLength ();

} // IPrinter

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
