/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7681 $
 * $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 * $Id: Interface.java 7681 2021-09-02 14:15:29Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/tape/Interface.java $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.tape;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * Cassette interface for JavaSE
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public class Interface
{
   /**
    * <p>
    * there are six file types
    * </p>
    */
   public enum File_Type
   {
      /**
       * All File
       */
      AF (
	 /* Description_Postfix => */"All file",
	 /* Mime_Type_Postfix   => */"p.all",
	 /* Mime_Type_Prefix    => */Interface.Application_Prefix,
	 /* Extension           => */"af",
	 /* Mac_OS_Type         => */"AFFX"),
      /**
       * All Text
       */
      AT (
	 /* Description_Postfix => */"All text",
	 /* Mime_Type_Postfix   => */"p.all",
	 /* Mime_Type_Prefix    => */Interface.Text_Prefix,
	 /* Extension           => */"at",
	 /* Mac_OS_Type         => */"ATFX"),
      /**
       * Data File
       */
      DF (
	 /* Description_Postfix => */"Data file",
	 /* Mime_Type_Postfix   => */"p.data",
	 /* Mime_Type_Prefix    => */Interface.Application_Prefix,
	 /* Extension           => */"df",
	 /* Mac_OS_Type         => */"DFFX"),
      /**
       * Data Text
       */
      DT (
	 /* Description_Postfix => */"Data text",
	 /* Mime_Type_Postfix   => */"p.data",
	 /* Mime_Type_Prefix    => */Interface.Text_Prefix,
	 /* Extension           => */"dt",
	 /* Mac_OS_Type         => */"DTFX"),
      /**
       * Program File
       */
      PF (
	 /* Description_Postfix => */"Program file",
	 /* Mime_Type_Postfix   => */"p.program",
	 /* Mime_Type_Prefix    => */Interface.Application_Prefix,
	 /* Extension           => */"pf",
	 /* Mac_OS_Type         => */"PFFX"),
      /**
       * Program Text
       */
      PT (
	 /* Description_Postfix => */"Program text",
	 /* Mime_Type_Postfix   => */"p.program",
	 /* Mime_Type_Prefix    => */Interface.Text_Prefix,
	 /* Extension           => */"pt",
	 /* Mac_OS_Type         => */"PTFX");
      /**
       *
       */
      public static final int AF_Header = 0x444E;
      /**
       *
       */
      public static final int AF_Password_Header = 0x404E;
      /**
       * <p>
       * Data file with Alpha register as content
       * </p>
       */
      public static final int DF_Alpha_Header = 0x4240;
      /**
       * <p>
       * Data file with X register as content
       * </p>
       */
      public static final int DF_Display_Header = 0x4140;
      /**
       * <p>
       * Data file with memory as content
       * </p>
       */
      public static final int DF_Memory_Header = 0x4040;
      /**
       *
       */
      public static final int PF_Header = 0x040E;
      /**
       *
       */
      public static final int PF_Password_Header = 0x400E;
      /**
       * <p>
       * Mime Type postfix (to be appended to "FX-602P" or "FX-603P"
       * </p>
       */
      @NotNull
      public final String Description_Postfix;
      /**
       * <p>
       * Extensions without dot
       * </p>
       */
      @NotNull
      public final String Extension;
      /**
       * <p>
       * Mac OS Type prefix
       * </p>
       */
      @NotNull
      public final String Mac_OS_Type;
      /**
       * <p>
       * Mime Type postfix. The middle consist of either "602" or "603".
       * </p>
       */
      @NotNull
      public final String Mime_Type_Postfix;
      /**
       * <p>
       * Mime Type postfix. The middle consist of either "602" or "603".
       * </p>
       */
      @NotNull
      public final String Mime_Type_Prefix;

      /**
       * @param Description_Postfix
       *    Mime Type postfix (to be appended to "FX-602P " or "FX-603P"
       * @param Mime_Type_Postfix
       *    Mime Type postfix. The middle consist of either "602" or "603".
       * @param Mime_Type_Prefix
       *    Mime Type prefix. The middle consist of either "602" or "603".
       * @param Extension
       *    Extensions without dot
       * @param Mac_OS_Type
       *    Mac OS X Finder data type
       */
      File_Type (
	 @NotNull final String Description_Postfix,
	 @NotNull final String Mime_Type_Postfix,
	 @NotNull final String Mime_Type_Prefix,
	 @NotNull final String Extension,
	 @NotNull final String Mac_OS_Type)
      {
	 this.Description_Postfix = Description_Postfix;
	 this.Mime_Type_Postfix = Mime_Type_Postfix;
	 this.Mime_Type_Prefix = Mime_Type_Prefix;
	 this.Extension = Extension;
	 this.Mac_OS_Type = Mac_OS_Type;
      } // File_Type

      public Boolean Is_Text ()
      {
	 return this.Mime_Type_Prefix.equals (Interface.Text_Prefix);
      }

      public String Mime_Type (final String Creator)
      {
	 return this.Mime_Type_Prefix + Creator + this.Mime_Type_Postfix;
      }

      @NotNull
      @Override
      @org.jetbrains.annotations.TestOnly
      public String toString ()
      {
	 return Interface.TAG + "{super=" + super.toString () + ", Description_Postfix=“" + this.Description_Postfix +
	    '”' + ", Extension=“" + this.Extension + '”' + ", Mac_OS_Type=“" + this.Mac_OS_Type + '”' +
	    ", Mime_Type_Postfix=“" + this.Mime_Type_Postfix + '”' + '}';
      }
   } // File_Type

   /**
    * <p>
    * class logger tag.
    * </p>
    */
   private static final String TAG = Interface.class.getName ();
   /**
    * <p>
    * Mime type prefix. The middle consist of either "602" or "603".
    * </p>
    */
   @SuppressWarnings ("HardcodedFileSeparator")
   public static final String Application_Prefix = "application/x-fx-";
   /**
    * <p>
    * File types for my calculator app. Must be an ArrayList — because that is what intent extras can take.
    * </p>
    * author: Martin Krischik" «krischik@users.sourceforge.net»
    */
   @SuppressWarnings ("CollectionDeclaredAsConcreteClass")
   @NotNull
   public static final java.util.ArrayList<String> Calculator_Files;
   /**
    * No CASIO style filename to embed into the header
    */
   public static final int[] Empty_File_Name = new int[] {
      /* 0 => */0xff,
      /* 1 => */0xff,
      /* 2 => */0xff,
      /* 3 => */0xff,
      /* 4 => */0xff,
      /* 5 => */0xff};
   /**
    * <p>
    * true when we run on mac os x
    * </p>
    */
   public static final boolean MAC_OS_X;
   @NonNls
   public static final String Simple_ASCII = "ISO-8859-1";
   /**
    * <p>
    * Mime type prefix. The middle consist of either "602" or "602".
    * </p>
    */
   @SuppressWarnings ("HardcodedFileSeparator")
   public static final String Text_Prefix = "text/x-fx-";

   /**
    * <p>
    * get type from database value.
    * </p>
    *
    * @param Filename
    *    a filename
    * @return the file type
    * @throws IllegalArgumentException
    *    the extension of the filename was not “pf”, “df” or “af”
    */
   @NotNull
   public static File_Type Type_Of_File (@NotNull final java.nio.file.Path Filename)
      throws
      IllegalArgumentException
   {
      return Type_Of_File (Filename.toFile ());
   } // Type_Of_File

   /**
    * <p>
    * get type from database value.
    * </p>
    *
    * @param Filename
    *    a filename
    * @return the file type
    * @throws IllegalArgumentException
    *    the extension of the filename was not “pf”, “df” or “af”
    */
   @NotNull
   public static File_Type Type_Of_File (@NotNull final java.io.File Filename)
      throws
      IllegalArgumentException
   {
      return Type_Of_File (Filename.getName ());
   } // Type_Of_File

   /**
    * <p>
    * get type from database value.
    * </p>
    *
    * @param Filename
    *    a filename
    * @return the file type
    * @throws IllegalArgumentException
    *    the extension of the filename was not “pf”, “df” or “af”
    */
   @NotNull
   public static File_Type Type_Of_File (@NotNull final String Filename)
      throws
      IllegalArgumentException
   {
      final File_Type Retval;

      Search_Filetype:
      {
	 for (final File_Type i : File_Type.values ())
	 {
	    if (Filename.endsWith ("." + i.Extension))
	    {
	       Retval = i;
	       break Search_Filetype;
	    } // if
	 } // for

	 throw new IllegalArgumentException ("No file type found for file «" + Filename + "»");
      } // Search_Filetype

      return Retval;
   } // value

   /**
    * <p>
    * check if file is a valid calculator file
    * </p>
    *
    * @param Filename
    *    a filename
    * @return true if calculator file
    */
   public static boolean Is_Calculator_File (@NotNull final java.nio.file.Path Filename)
   {
      return Is_Calculator_File (Filename.toFile ());
   } // Type_Of_File

   /**
    * <p>
    * check if file is a valid calculator file
    * </p>
    *
    * @param Filename
    *    a filename
    * @return true if calculator file
    */
   public static boolean Is_Calculator_File (@NotNull final java.io.File Filename)
   {
      return Is_Calculator_File (Filename.getName ());
   } // Type_Of_File

   /**
    * <p>
    * check if file is a valid calculator file
    * </p>
    *
    * @param Filename
    *    a filename
    * @return true if calculator file
    */
   public static boolean Is_Calculator_File (@NotNull final String Filename)
   {
      final boolean Retval;

      Search_Filetype:
      {
	 for (final File_Type i : File_Type.values ())
	 {
	    if (Filename.endsWith ("." + i.Extension))
	    {
	       Retval = true;
	       break Search_Filetype;
	    } // if
	 } // for

	 Retval = false;
      } // Search_Filetype

      return Retval;
   } // value

   static
   {
      final String Property = System.getProperty ("os.name");
      final String Property_Lower_Case = Property.toLowerCase ();
      MAC_OS_X = Property_Lower_Case.startsWith ("mac os x");

      Calculator_Files = new java.util.ArrayList<> ();
      for (final File_Type i : File_Type.values ())
      {
	 Interface.Calculator_Files.add ("." + i.Extension);
      } // for
   } // static
} // Interface

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
