/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7717 $
 * $Date: 2021-10-30 18:54:21 +0200 (Sa, 30. Okt 2021) $
 * $Id: Numeric_Utilities.scala 7717 2021-10-30 16:54:21Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/main/java/net.sourceforge.uiq3/test/Numeric_Utilities.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.test

import net.sourceforge.uiq3.math.BCDFloat
import net.sourceforge.uiq3.math.BCDFraction
import net.sourceforge.uiq3.math.BCDInteger
import net.sourceforge.uiq3.math.Number

/**
 * <p>logging tag</p>
 */
private val TAG = "net.sourceforge.uiq3.test.Numeric_Utilities"

/**
 * <p>logger</p>
 */
private val logger = java.util.logging.Logger.getLogger(TAG)

/**
 * convert String to BCDFloat
 *
 * @param Value string representing a float
 * @return a BCDFloat
 */
fun ℝ(Value: String): BCDFloat = BCDFloat(Value)

/**
 * convert long to BCD floating point
 *
 * @param Value long value
 * @return floating point
 */
fun ℝ(Value: Long): BCDFloat = BCDFloat(Value)

/**
 * convert long to BCD floating point
 *
 * @param Coefficient
 * Coefficient
 * @param Exponent
 * Exponent
 * @return floating point
 */
fun ℝ(Coefficient: Long, Exponent: Int): BCDFloat = BCDFloat(Coefficient, Exponent)

/**
 * convert String to BCDFraction
 *
 * @param Value string representing a float
 * @return a BCDFraction
 */
fun ℚ(Value: String): BCDFraction = BCDFraction(Value)

/**
 * convert Long to BCDFraction
 *
 * @param Value string representing a float
 * @return a BCDFraction
 */
fun ℚ(Value: Long): BCDFraction = BCDFraction(Value)

/**
 * convert String to BCDInteger
 *
 * @param Value string representing an integer
 * @return a BCDInteger
 */
fun ℤ(Value: String): BCDInteger = BCDInteger(Value)

/**
 * convert String to BCDInteger
 *
 * @param Value long value
 * @return a BCDInteger
 */
fun ℤ(Value: Long): BCDInteger = BCDInteger(Value)

/**
 * Asserted , trace. Just replace the == for a , at test which make problems.
 */
fun assert(Value: Number, Expected: Number)
{
   val success = Value.Round(10, 99) == Expected.Round(10, 99)

   if (!success)
   {
      logger.log(
	 java.util.logging.Level.INFO,
	 "LOG01511:" + "\nValue           = " + Value + "\nExpected        = " + Expected)
   } // if
   assert(success)
} // assert

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
