/*
 * ************************************************************ {{{1 **********
 * Copyright © 2020 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7953 $
 * $Date: 2022-01-19 07:59:28 +0100 (Mi, 19. Jan 2022) $
 * $Id: StringTokenizer_Test.kt 7953 2022-01-19 06:59:28Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/test/java/net.sourceforge.uiq3/StringTokenizer_Test.kt $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3

import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Info
import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

import java.util.logging.Level
import org.hamcrest.collection.IsCollectionWithSize.hasSize
import org.hamcrest.collection.IsIterableContainingInOrder.contains
import org.hamcrest.core.StringContains.containsString
import org.junit.jupiter.api.Assertions.assertThrows
import org.junit.jupiter.api.Test

/**
 * Tests new [StringTokenizer].
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class StringTokenizer_Test
{
   companion object
   {
      /**
       * <p>logging tag</p>
       */
      private val TAG = StringTokenizer_Test::class.qualifiedName

      /**
       * <p>logger</p>
       */
      private val logger = java.util.logging.Logger.getLogger(TAG)
      private fun Tokenize(Text: String) = StringTokenizer(Text)

      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.BeforeEach
   fun BeforeEach()
   {
      Feature("String Tokenizer")
      Info(
	 """As developer I want the string parse FX-603P code"""
      )
   }

   @Test
   fun test_01_Simple()
   {
      logger.entering(TAG, "test_01_Simple")

      val test = StringTokenizer("""AC HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(2))
      assertThat(test, contains("AC", "HLT"))

      logger.exiting(TAG, "test_01_Simple")
      return
   } // test_01_Simple

   @Test
   fun test_02_String()
   {
      logger.entering(TAG, "test_02_String")

      val test = StringTokenizer("""LBL0 "HLT" HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains("LBL0", "\"HLT\"", "HLT"))

      logger.exiting(TAG, "test_02_String")
      return
   } // test_02_String

   @Test
   fun test_03_String()
   {
      logger.entering(TAG, "test_03_String")

      val test = StringTokenizer(""" Min05 "ABC" MR05 """)

      logger.log(Level.INFO, "Object under Test = {0}", test)

      test.iterator()

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains("Min05", "\"ABC\"", "MR05"))

      logger.exiting(TAG, "test_03_String")
      return
   } // test_03_String

   @Test
   fun test_04_Middle_Space()
   {
      logger.entering(TAG, "test_04_Middle_Space")

      val test = StringTokenizer("""ab/c "ABC def" d/c""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains(equalTo("ab/c"), equalTo("\"ABC def\""), equalTo("d/c")))

      logger.exiting(TAG, "test_04_Middle_Space")
      return
   } // test_04_Middle_Space

   @Test
   fun test_05_Left_Space()
   {
      logger.entering(TAG, "test_05_Left_Space")

      val test = StringTokenizer("""LBL0 ";[AR00] [RTA] " DSZ GOTO0""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("LBL0", "\";[AR00] [RTA] \"", "DSZ", "GOTO0"))

      logger.exiting(TAG, "test_05_Left_Space")
      return
   } // test_05_Left_Space

   @Test
   fun test_06_Right_Space()
   {
      logger.entering(TAG, "test_06_Right_Space")

      val test = StringTokenizer("""MR21 SDMS " [AR20]" HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("MR21", "SDMS", "\" [AR20]\"", "HLT"))

      logger.exiting(TAG, "test_06_Right_Space")
      return
   } // test_06_Right_Space

   @Test
   fun test_07_All_Space()
   {
      logger.entering(TAG, "test_07_All_Space")

      val test = StringTokenizer("""AC DEG " Geburtstag : " PAUSE""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("AC", "DEG", "\" Geburtstag : \"", "PAUSE"))

      logger.exiting(TAG, "test_07_All_Space")
      return
   } // test_07_All_Space

   @Test
   fun test_08_Empty_String()
   {
      logger.entering(TAG, "test_08_Empty_String")

      val test = StringTokenizer("AC 100 Min00 \"\"")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("AC", "100", "Min00", "\"\""))

      logger.exiting(TAG, "test_08_Empty_String")
      return
   } // test_08_Empty_String

   @Test
   fun test_09_Only_String()
   {
      logger.entering(TAG, "test_09_Only_String")

      val test = StringTokenizer("\"Test\"")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(1))
      assertThat(test, contains("\"Test\""))

      logger.exiting(TAG, "test_09_Only_String")
      return
   } // test_09_Only_String

   @Test
   fun test_10_Quote_In_String()
   {
      logger.entering(TAG, "test_10_Quote_In_String")

      val test: Exception = assertThrows(java.text.ParseException::class.java) {
	 StringTokenizer("Test\"Test")
      }

      assertThat(test, Is(notNullValue()))
      assertThat(test.message, containsString("Unexpected quotes"))

      logger.exiting(TAG, "test_10_Quote_In_String")
      return
   } // test_09_Only_String

   @Test
   fun test_11_Double_Simple()
   {
      logger.entering(TAG, "test_11_Double_Simple")

      val test = StringTokenizer("""AC  HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(2))
      assertThat(test, contains("AC", "HLT"))

      logger.exiting(TAG, "test_11_Double_Simple")
      return
   } // test_11_Double_Simple

   @Test
   fun test_12_Double_String()
   {
      logger.entering(TAG, "test_12_Double_String")

      val test = StringTokenizer("""LBL0  "HLT"  HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains("LBL0", "\"HLT\"", "HLT"))

      logger.exiting(TAG, "test_12_Double_String")
      return
   } // test_12_Double_String

   @Test
   fun test_13_Double_String()
   {
      logger.entering(TAG, "test_13_Double_String")

      val test = StringTokenizer(""" Min05  "ABC"  MR05  """)

      logger.log(Level.INFO, "Object under Test = {0}", test)

      test.iterator()

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains("Min05", "\"ABC\"", "MR05"))

      logger.exiting(TAG, "test_13_Double_String")
      return
   } // test_13_Double_String

   @Test
   fun test_14_Double_Middle_Space()
   {
      logger.entering(TAG, "test_14_Double_Middle_Space")

      val test = StringTokenizer("""ab/c "ABC  def" d/c""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(3))
      assertThat(test, contains(equalTo("ab/c"), equalTo("\"ABC  def\""), equalTo("d/c")))

      logger.exiting(TAG, "test_14_Double_Middle_Space")
      return
   } // test_14_Double_Middle_Space

   @Test
   fun test_15_Double_Left_Space()
   {
      logger.entering(TAG, "test_15_Double_Left_Space")

      val test = StringTokenizer("""LBL0 ";[AR00]  [RTA]  " DSZ GOTO0""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("LBL0", "\";[AR00]  [RTA]  \"", "DSZ", "GOTO0"))

      logger.exiting(TAG, "test_15_Double_Left_Space")
      return
   } // test_15_Double_Left_Space

   @Test
   fun test_16_Double_Right_Space()
   {
      logger.entering(TAG, "test_16_Double_Right_Space")

      val test = StringTokenizer("""MR21 SDMS "  [AR20]" HLT""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("MR21", "SDMS", "\"  [AR20]\"", "HLT"))

      logger.exiting(TAG, "test_16_Double_Right_Space")
      return
   } // test_16_Double_Right_Space

   @Test
   fun test_17_Double_All_Space()
   {
      logger.entering(TAG, "test_17_Double_All_Space")

      val test = StringTokenizer("""AC DEG "  Geburtstag  :  " PAUSE""")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("AC", "DEG", "\"  Geburtstag  :  \"", "PAUSE"))

      logger.exiting(TAG, "test_17_Double_All_Space")
      return
   } // test_17_Double_All_Space

   @Test
   fun test_18_Double_Empty_String()
   {
      logger.entering(TAG, "test_18_Double_Empty_String")

      val test = StringTokenizer("AC 100 Min00 \"  \"")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(4))
      assertThat(test, contains("AC", "100", "Min00", "\"  \""))

      logger.exiting(TAG, "test_18_Double_Empty_String")
      return
   } // test_18_Double_Empty_String

   @Test
   fun test_19_Double_Only_String()
   {
      logger.entering(TAG, "test_19_Double_Only_String")

      val test = StringTokenizer("\"  Test  \"")

      logger.log(Level.INFO, "Object under Test = {0}", test)

      assertThat(test, Is(notNullValue()))
      assertThat(test, hasSize(1))
      assertThat(test, contains("\"  Test  \""))

      logger.exiting(TAG, "test_19_Double_Only_String")
      return
   } // test_19_Double_Only_String

   @Test
   fun test_20_Unbalanced_Quotes()
   {
      logger.entering(TAG, "test_10_Quote_In_String")

      val test: Exception = assertThrows(java.text.ParseException::class.java) {
	 StringTokenizer("Test  \"  Test")
      }

      assertThat(test, Is(notNullValue()))
      assertThat(test.message, containsString("are unbalanced"))

      logger.exiting(TAG, "test_10_Quote_In_String")
      return
   } // test_09_Only_String


   @org.junit.jupiter.api.Test
   fun `Test ““““`()
   {
      assertThat(Tokenize("""""").size, equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test ““+⁄- ab⁄c ENG DMS LOG LN““`()
   {
      assertThat(Tokenize("""+/- ab/c ENG DMS LOG LN""").size, equalTo(6))
   }

   @org.junit.jupiter.api.Test
   fun `Test ““+⁄-  ab⁄c  ENG  DMS  LOG  LN““`()
   {
      assertThat(Tokenize("""+/-  ab/c  ENG  DMS  LOG  LN""").size, equalTo(6))
   }

   @org.junit.jupiter.api.Test
   fun `Test ““+⁄- ab⁄c ENG “Test“ DMS LOG LN““`()
   {
      assertThat(Tokenize("""+/- ab/c ENG "Test" DMS LOG LN""").size, equalTo(7))
   }

   @org.junit.jupiter.api.Test
   fun `Test “““Test“ +⁄- ab⁄c ENG DMS LOG LN““`()
   {
      assertThat(Tokenize(""""Test" +/- ab/c ENG DMS LOG LN""").get(0), equalTo("\"Test\""))
   }

   @org.junit.jupiter.api.Test
   fun `Test ““+⁄- ab⁄c ENG “Test Test“ DMS LOG LN““`()
   {
      assertThat(Tokenize("""+/- ab/c ENG "Test Test" DMS LOG LN""").size, equalTo(7))
   }

   @org.junit.jupiter.api.Test
   fun `Test “““Test Test“ +⁄- ab⁄c ENG DMS LOG LN““`()
   {
      assertThat(Tokenize(""""Test Test" +/- ab/c ENG DMS LOG LN""").get(0), equalTo("\"Test Test\""))
   }

   @org.junit.jupiter.api.Test
   fun `Test ““EXP FIX4 “PI =#“““`()
   {
      val Tokens = Tokenize("""EXP FIX4 "[PI ]=#"""")

      assertThat(Tokens.size, equalTo(3))
      assertThat(Tokens.get(0), equalTo("EXP"))
      assertThat(Tokens.get(1), equalTo("FIX4"))
      assertThat(Tokens.get(2), equalTo("\"[PI ]=#\""))
   }
} // StringTokenizer_Test

//************************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab textwidth=120:
// vim: set textwidth=0 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
