/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 8155 $
 * $Date: 2022-05-06 13:03:09 +0200 (Fr, 06. Mai 2022) $
 * $Id: BCDFloat_Base_Test.scala 8155 2022-05-06 11:03:09Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Base_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Base_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test important constants are not changed`()
   {
      assertThat(Num_0, equalTo(ℝ("0")))
   }

   // Adding //////////////////////////////////////////////// {{{1 ///////////
   //
   // Remember that string has an + operator as well.
   @org.junit.jupiter.api.Test
   fun `Test 0․5 + 0․25`()
   {
      assertThat((ℝ("0.5") + ℝ("0.25")), equalTo(ℝ("0.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 + 0․75`()
   {
      assertThat((ℝ("0.5") + ℝ("0.75")), equalTo(ℝ("1.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 + 0․25`()
   {
      assertThat((ℝ("-0.5") + ℝ("0.25")), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 + 0․75`()
   {
      assertThat((ℝ("-0.5") + ℝ("0.75")), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 + -0․25`()
   {
      assertThat(ℝ("0.5") + ℝ("-0.25"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 + -0․75`()
   {
      assertThat((ℝ("0.5") + ℝ("-0.75")), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 + -0․25`()
   {
      assertThat((ℝ("-0.5") + ℝ("-0.25")), equalTo(ℝ("-0.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 + -0․75`()
   {
      assertThat(ℝ("-0.5") + ℝ("-0.75"), equalTo(ℝ("-1.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 + 1․25`()
   {
      assertThat(ℝ("1.5") + ℝ("1.25"), equalTo(ℝ("2.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 + 1․75`()
   {
      assertThat(ℝ("1.5") + ℝ("1.75"), equalTo(ℝ("3.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 + 1․25`()
   {
      assertThat(ℝ("-1.5") + ℝ("1.25"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 + 1․75`()
   {
      assertThat(ℝ("-1.5") + ℝ("1.75"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 + -1․25`()
   {
      assertThat(ℝ("1.5") + ℝ("-1.25"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 + -1․75`()
   {
      assertThat(ℝ("1.5") + ℝ("-1.75"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 + -1․25`()
   {
      assertThat(ℝ("-1.5") + ℝ("-1.25"), equalTo(ℝ("-2.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 + -1․75`()
   {
      assertThat(ℝ("-1.5") + ℝ("-1.75"), equalTo(ℝ("-3.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 + 1E-40`()
   {
      assertThat(Num_0 + ℝ("1E-40"), equalTo(ℝ("1E-40")))
   }

   // Subtracting /////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test 0․5 - 0․25`()
   {
      assertThat(ℝ("0.5") - ℝ("0.25"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 - 0․75`()
   {
      assertThat(ℝ("0.5") - ℝ("0.75"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 - 0․25`()
   {
      assertThat(ℝ("-0.5") - ℝ("0.25"), equalTo(ℝ("-0.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 - 0․75`()
   {
      assertThat(ℝ("-0.5") - ℝ("0.75"), equalTo(ℝ("-1.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 - -0․25`()
   {
      assertThat(ℝ("0.5") - ℝ("-0.25"), equalTo(ℝ("0.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 - -0․75`()
   {
      assertThat(ℝ("0.5") - ℝ("-0.75"), equalTo(ℝ("1.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 - -0․25`()
   {
      assertThat(ℝ("-0.5") - ℝ("-0.25"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 - -0․75`()
   {
      assertThat(ℝ("-0.5") - ℝ("-0.75"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 - 1․25`()
   {
      assertThat(ℝ("1.5") - ℝ("1.25"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 - 1․75`()
   {
      assertThat(ℝ("1.5") - ℝ("1.75"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 - 1․25`()
   {
      assertThat(ℝ("-1.5") - ℝ("1.25"), equalTo(ℝ("-2.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 - 1․75`()
   {
      assertThat(ℝ("-1.5") - ℝ("1.75"), equalTo(ℝ("-3.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 - -1․25`()
   {
      assertThat(ℝ("1.5") - ℝ("-1.25"), equalTo(ℝ("2.75")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 - -1․75`()
   {
      assertThat(ℝ("1.5") - ℝ("-1.75"), equalTo(ℝ("3.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 - -1․25`()
   {
      assertThat(ℝ("-1.5") - ℝ("-1.25"), equalTo(ℝ("-0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 - -1․75`()
   {
      assertThat(ℝ("-1.5") - ℝ("-1.75"), equalTo(ℝ("0.25")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 - 1E-40`()
   {
      assertThat(Num_0 - ℝ("1E-40"), equalTo("-1E-40"))
   }

   // Multiply ////////////////////////////////////////////// {{{1 ///////////
   //
   // Remember that string has an * operator as well.
   @org.junit.jupiter.api.Test
   fun `Test 0․5 ℝ 0․25`()
   {
      assertThat(ℝ("0.5") * ℝ("0.25"), equalTo(ℝ("0.125")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 × 0․75`()
   {
      assertThat(ℝ("0.5") * ℝ("0.75"), equalTo(ℝ("0.375")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 × 0․25`()
   {
      assertThat(ℝ("-0.5") * ℝ("0.25"), equalTo(ℝ("-0.125")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 × 0․75`()
   {
      assertThat(ℝ("-0.5") * ℝ("0.75"), equalTo(ℝ("-0.375")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 × -0․25`()
   {
      assertThat(ℝ("0.5") * ℝ("-0.25"), equalTo(ℝ("-0.125")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 × -0․75`()
   {
      assertThat(ℝ("0.5") * ℝ("-0.75"), equalTo(ℝ("-0.375")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 × -0․25`()
   {
      assertThat(ℝ("-0.5") * ℝ("-0.25"), equalTo(ℝ("0.125")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 × -0․75`()
   {
      assertThat(ℝ("-0.5") * ℝ("-0.75"), equalTo(ℝ("0.375")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 × 1․25`()
   {
      assertThat(ℝ("1.5") * ℝ("1.25"), equalTo(ℝ("1.875")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 × 1․75`()
   {
      assertThat(ℝ("1.5") * ℝ("1.75"), equalTo(ℝ("2.625")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 × 1․25`()
   {
      assertThat(ℝ("-1.5") * ℝ("1.25"), equalTo(ℝ("-1.875")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 × 1․75`()
   {
      assertThat(ℝ("-1.5") * ℝ("1.75"), equalTo(ℝ("-2.625")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 × -1․25`()
   {
      assertThat(ℝ("1.5") * ℝ("-1.25"), equalTo(ℝ("-1.875")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 × -1․75`()
   {
      assertThat(ℝ("1.5") * ℝ("-1.75"), equalTo(ℝ("-2.625")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 × -1․25`()
   {
      assertThat(ℝ("-1.5") * ℝ("-1.25"), equalTo(ℝ("1.875")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 × -1․75`()
   {
      assertThat(ℝ("-1.5") * ℝ("-1.75"), equalTo(ℝ("2.625")))
   }

   // Division ////////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test 0․5 ÷ 0․25`()
   {
      assertThat(ℝ("0.5") / ℝ("0.25"), equalTo(ℝ("2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 ÷ 0․75`()
   {
      assertThat(ℝ("0.5") / ℝ("0.75"), equalTo(ℚ("2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 ÷ 0․25`()
   {
      assertThat(ℝ("-0.5") / ℝ("0.25"), equalTo(ℝ("-2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 ÷ 0․75`()
   {
      assertThat(ℝ("-0.5") / ℝ("0.75"), equalTo(ℚ("-2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 ÷ -0․25`()
   {
      assertThat(ℝ("0.5") / ℝ("-0.25"), equalTo(ℝ("-2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․5 ÷ -0․75`()
   {
      assertThat(ℝ("0.5") / ℝ("-0.75"), equalTo(ℚ("-2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 ÷ -0․25`()
   {
      assertThat(ℝ("-0.5") / ℝ("-0.25"), equalTo(ℝ("2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -0․5 ÷ -0․75`()
   {
      assertThat(ℝ("-0.5") / ℝ("-0.75"), equalTo(ℚ("2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 ÷ 1․25`()
   {
      assertThat(ℝ("1.5") / ℝ("1.25"), equalTo(ℝ("1.2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 ÷ 1․75`()
   {
      assertThat(ℝ("1.5") / ℝ("1.75"), equalTo(ℚ("6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 ÷ 1․25`()
   {
      assertThat(ℝ("-1.5") / ℝ("1.25"), equalTo(ℝ("-1.2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 ÷ 1․75`()
   {
      assertThat(ℝ("-1.5") / ℝ("1.75"), equalTo(ℚ("-6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 ÷ -1․25`()
   {
      assertThat(ℝ("1.5") / ℝ("-1.25"), equalTo(ℝ("-1.2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․5 ÷ -1․75`()
   {
      assertThat(ℝ("1.5") / ℝ("-1.75"), equalTo(ℚ("-6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 ÷ -1․25`()
   {
      assertThat(ℝ("-1.5") / ℝ("-1.25"), equalTo(ℝ("1.2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1․5 ÷ -1․75`()
   {
      assertThat(ℝ("-1.5") / ℝ("-1.75"), equalTo(ℚ("6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2 + 10`()
   {
      assertThat(ℝ("2") / ℝ("10"), equalTo(ℝ("0.2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0․005 + -0․001`()
   {
      assertThat(ℝ("0.005") / ℝ("-0.001"), equalTo(ℝ("-5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 3`()
   {
      assertThat(ℝ("1") / ℝ("3"), equalTo(ℝ("0.333333333333333")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 3 × 3`()
   {
      assertThat(ℝ("1") / ℝ("3") * ℝ("3"), equalTo(ℝ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 3 × 3 - 1`()
   {
      assertThat(ℝ("1") / ℝ("3") * ℝ("3") - ℝ("1"), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 54`()
   {
      assertThat(ℝ("1") / ℝ("54"), equalTo(ℝ("0.0185185185185185")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 54 × 54`()
   {
      assertThat(ℝ("1") / ℝ("54") * ℝ("54"), equalTo(ℝ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 54 × 54 - 1`()
   {
      assertThat(ℝ("1") / ℝ("54") * ℝ("54") - ℝ("1"), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 55`()
   {
      assertThat(ℝ("1") / ℝ("55"), equalTo(ℝ("0.018181818181818")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 55 × 55`()
   {
      assertThat(ℝ("1") / ℝ("55") * ℝ("55"), equalTo(ℝ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 55 × 55 - 1`()
   {
      assertThat(ℝ("1") / ℝ("55") * ℝ("55") - ℝ("1"), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 56`()
   {
      assertThat(ℝ("1") / ℝ("56"), equalTo(ℝ("0.017857142857142")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 56 × 56`()
   {
      assertThat(ℝ("1") / ℝ("56") * ℝ("56"), equalTo(ℝ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 ÷ 56 × 56 - 1`()
   {
      assertThat(ℝ("1") / ℝ("56") * ℝ("56") - ℝ("1"), equalTo(BCDFloat.Num_0))
   }

   // Unary ///////////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test -(7․1)`()
   {
      assertThat(ℝ("7.1").neg(), equalTo(ℝ("-7.1")))
   }

   // Logical /////////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test 7 ∧ 11`()
   {
      assertThat(ℝ("7.1").and(ℝ("11.1")), equalTo(ℤ("3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 7 ∨ 11`()
   {
      assertThat(ℝ("7.1").or(ℝ("11.1")), equalTo(ℤ("15")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 7 ⊕ 11`()
   {
      assertThat(ℝ("7.1").xor(ℝ("11.1")), equalTo(ℤ("12")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ¬7`()
   {
      assertThat(ℝ("7.1").not(), equalTo(ℤ("-8")))
   }
   ////////////////////////////////////////////////////////// 1}}} ///////////
} // BCDFloat_Base_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb
