/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Compare_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Compare_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.math.BCDFloat.Num_e
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Compare_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test π Compare e`()
   {
      assertThat(Num_π.Compare(Num_e), equalTo(Number.Compare_Greater))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1234 Compare 123`()
   {
      assertThat(ℝ("1234").Compare(ℝ("123")), equalTo(Number.Compare_Greater))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․234 Compare 1․23`()
   {
      assertThat(ℝ("1.234").Compare(ℝ("1.23")), equalTo(Number.Compare_Greater))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․2340 Compare 123000`()
   {
      assertThat(ℝ("1.2340").Compare(ℝ("1.23000")), equalTo(Number.Compare_Greater))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1×10¹⁰ Compare 1×10¹¹`()
   {
      assertThat(ℝ("1e10").Compare(ℝ("1e11")), equalTo(Number.Compare_Less))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Compare 123`()
   {
      assertThat(ℝ("123").Compare(ℝ("123")), equalTo(Number.Compare_Equal))
   }

   @org.junit.jupiter.api.Test
   fun `Test 19 Compare 3e1`()
   {
      assertThat(ℝ("19").Compare(ℝ("3e1")), equalTo(Number.Compare_Less))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2e1 Compare 3e1`()
   {
      assertThat(ℝ("2e1").Compare(ℝ("3e1")), equalTo(Number.Compare_Less))
   }

   @org.junit.jupiter.api.Test
   fun `Test 21 Compare 3e1`()
   {
      assertThat(ℝ("21").Compare(ℝ("3e1")), equalTo(Number.Compare_Less))
   }
   @Test
   fun `Test string 0`()
   {
      assertThat(ℝ("0").Is_Zero(), equalTo(true))
   }

   @Test
   fun `Test long 0`()
   {
      assertThat(ℝ(0L).Is_Zero(), equalTo(true))
   }

   @Test
   fun `Test 0`()
   {
      assertThat(BCDFraction.Num_0.Is_Zero(), equalTo(true))
   }

   @Test
   fun `Test 1 1⁄4 is not zero`()
   {
      assertThat(ℝ("1.25").Is_Zero(), equalTo(false))
   }

   @Test
   fun `Test 1 is a number`()
   {
      assertThat(BCDFraction.Num_1.Is_NaN(), equalTo(false))
   }

   @Test
   fun `Test 2 is a finite number`()
   {
      assertThat(BCDFraction.Num_2.Is_Finite(), equalTo(true))
   }

   @Test
   fun `Test 200 is a finite number`()
   {
      assertThat(BCDFraction.Num_200.Is_Infinite(), equalTo(false))
   }

   @Test
   fun `Test -1⁄2 is negative`()
   {
      assertThat(ℝ("-1.5").Is_Negative(), equalTo(true))
   }

   @Test
   fun `Test 1 is integer`()
   {
      assertThat(ℝ("1").Is_Integer(), equalTo(true))
   }

   @Test
   fun `Test -1⁄2 is not an integer`()
   {
      assertThat(ℝ("-1.5").Is_Integer(), equalTo(false))
   }

   @Test
   fun `Test 1 is not negative`()
   {
      assertThat(ℝ("1").Is_Negative(), equalTo(false))
   }
} // BCDFloat_Function_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
