/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Function_Test․scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn․code․sf․net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net․sourceforge․uiq3/math/BCDFloat_Function_Test․scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test
import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.math.BCDFloat.Num_e
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ


/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Function_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test π + e`()
   {
      assertThat((Num_π + BCDFloat.Num_e), equalTo(ℝ("5.8598744820488385")))
   }

   @org.junit.jupiter.api.Test
   fun `Test π - ⌊π⌋`()
   {
      assertThat((Num_π - Num_π.Integer()), equalTo(ℝ("0.14159265358979324")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │-3․123│`()
   {
      assertThat(ℝ("-3.123").abs(), equalTo(ℝ("3.123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │3․123│`()
   {
      assertThat(ℝ("3.123").abs(), equalTo(ℝ("3.123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(-3․123)`()
   {
      assertThat(ℝ("-3.123").neg(), equalTo(ℝ("3.123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(3․123)`()
   {
      assertThat(ℝ("3.123").neg(), equalTo(ℝ("-3.123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -e - ⌊-e⌋`()
   {
      assertThat((Num_e.neg() - Num_e.neg().Integer()), equalTo(ℝ("-0.71828182845904526")))
   }

   @org.junit.jupiter.api.Test
   fun `Test log₂4096`()
   {
      assertThat(ℝ("4096").log_10() / ℝ("2").log_10(), equalTo(ℝ("11.9999999999999998")))
   }

   @org.junit.jupiter.api.Test
   fun `Test round (log₂4096, 12, 99)`()
   {
      assertThat(ℝ("4096").log_10() / ℝ("2").log_10().Round(12, 99), equalTo(ℝ("12")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 2`()
   {
      assertThat(Num_0.Indirect_Value(2), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 2`()
   {
      assertThat(ℝ("1").Indirect_Value(2), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 2`()
   {
      assertThat(ℝ("10").Indirect_Value(2), equalTo(10))
   }

   @org.junit.jupiter.api.Test
   fun `Test 13 Indirect_Value 2`()
   {
      assertThat(ℝ("12").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 2`()
   {
      assertThat(ℝ("120").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 2`()
   {
      assertThat(ℝ("123").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 1`()
   {
      assertThat(Num_0.Indirect_Value(1), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 1`()
   {
      assertThat(ℝ("1").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 1`()
   {
      assertThat(ℝ("10").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 12 Indirect_Value 1`()
   {
      assertThat(ℝ("12").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 1`()
   {
      assertThat(ℝ("120").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 1`()
   {
      assertThat(ℝ("123").Indirect_Value(1), equalTo(1))
   }
} // BCDFloat_Function_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
