/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Hyperbolic_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Hyperbolic_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_e
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ
import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Hyperbolic_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test important constants are not changed`()
   {
      assertThat(Num_0, equalTo(ℝ("0")))
   }

   @org.junit.jupiter.api.Test
   fun `Test acosh 9`()
   {
      assertThat(ℝ("9").arc_cos_hyp(), equalTo(ℝ("2.88727095035761552")))
   }

   @org.junit.jupiter.api.Test
   fun `Test asinh 9`()
   {
      assertThat(ℝ("9").arc_sin_hyp(), equalTo(ℝ("2.89344398588586512")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atanh 9`()
   {
      assertThat(ℝ("0.9").arc_tan_hyp(), equalTo(ℝ("1.4722194895832042")))
   }

   @org.junit.jupiter.api.Test
   fun `Test cosh 9`()
   {
      assertThat(ℝ("9").cos_hyp(), equalTo(ℝ("4051.54202549259424")))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic hyp 1․8`()
   {
      assertThat(
	 ℝ("1.8").sin_hyp().cos_hyp().tan_hyp().arc_tan_hyp().arc_cos_hyp().arc_sin_hyp(),
	 equalTo(ℝ("1.79999999999945557")))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic hyp 1․8 - 1․8`()
   {
      assertThat(
	 ℝ("1.8").sin_hyp().cos_hyp().tan_hyp().arc_tan_hyp().arc_cos_hyp().arc_sin_hyp() - ℝ("1.8"),
	 equalTo(ℝ("-5.4443E-13")))
   }

   @org.junit.jupiter.api.Test
   fun `Test sinh 9`()
   {
      assertThat(ℝ("9").sin_hyp(), equalTo(ℝ("4051.54190208279015")))
   }

   @org.junit.jupiter.api.Test
   fun `Test tanh 9`()
   {
      assertThat(ℝ("9").tan_hyp(), equalTo(ℝ("0.999999969540040974")))
   }
} // BCDFloat_Function_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
