/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Log_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Log_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_1
import net.sourceforge.uiq3.math.BCDFloat.Num_e
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ
import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test


/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Log_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test important constants are not changed`()
   {
      assertThat(Num_0, equalTo(ℝ("0")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10⁵`()
   {
      assertThat(ℝ("10").pow(ℝ("5")), equalTo(ℝ("100000")))
   }

   @org.junit.jupiter.api.Test
   fun `Test e⁵`()
   {
      assertThat(Num_e.pow(ℝ("5")), equalTo(ℝ("148.413159102576610")))
   }

   @org.junit.jupiter.api.Test
   fun `Test log₁₀ 5`()
   {
      assertThat(ℝ("5").log_10(), equalTo(ℝ("0.698970004336018816")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10^(log₁₀ 5`()
   {
      assertThat(ℝ("5").log_10().exp_10(), equalTo(ℝ("5.00000000000000008")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10⁰`()
   {
      assertThat(ℝ("0").exp_10(), equalTo(Num_1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10⁻⁰`()
   {
      assertThat(ℝ("0").neg().exp_10(), equalTo(Num_1))
   }

   @org.junit.jupiter.api.Test
   fun `Test ln 5`()
   {
      assertThat(ℝ("5").log_e(), equalTo(ℝ("1.60943791243410039")))
   }

   @org.junit.jupiter.api.Test
   fun `Test e^(ln 5)`()
   {
      assertThat(ℝ("5").log_e().exp_e(), equalTo(ℝ("5.00000000000000008")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ln 10`()
   {
      assertThat(ℝ("10").log_e(), equalTo(ℝ("2.30258509299404567")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0^0`()
   {
      assertThat(ℝ("0").pow(ℝ("0")).Is_NaN(), equalTo(true))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0^1`()
   {
      assertThat(ℝ("0").pow(ℝ("1")), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123^0`()
   {
      assertThat(ℝ("123").pow(ℝ("0")), equalTo(Num_1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123^1`()
   {
      assertThat(ℝ("123").pow(ℝ("1")), equalTo(ℝ("123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test π^e`()
   {
      assertThat(Num_π.pow(Num_e), equalTo(ℝ("22.4591577183610443")))
   }

   @org.junit.jupiter.api.Test
   fun `Test e¹⁰⁰`()
   {
      assertThat(Num_e.pow(ℝ("100")), equalTo(ℝ("2.68811714181613789E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test e¹⁰⁰․⁵`()
   {
      assertThat(Num_e.pow(ℝ("100.5")), equalTo(ℝ("4.43195590984589813E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test e¹⁰¹`()
   {
      assertThat(Num_e.pow(ℝ("101")), equalTo(ℝ("7.30705997936807398E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test π¹⁰⁰`()
   {
      assertThat(Num_π.pow(ℝ("100")), equalTo(ℝ("5.18784831431961342E49")))
   }

   @org.junit.jupiter.api.Test
   fun `Test π¹⁰⁰․⁵`()
   {
      assertThat(Num_π.pow(ℝ("100.5")), equalTo(ℝ("9.19522172262946087E49")))
   }

   @org.junit.jupiter.api.Test
   fun `Test π¹⁰¹`()
   {
      assertThat(Num_π.pow(ℝ("101")), equalTo(ℝ("1.62981061522046901E50")))
   }
} // BCDFloat_Function_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
