/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_P_R_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_P_R_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_1
import net.sourceforge.uiq3.math.BCDFloat.Num_180
import net.sourceforge.uiq3.math.BCDFloat.Num_200
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.IsEqual.equalTo
import org.junit.jupiter.api.Test


/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_P_R_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   // Base functions //////////////////////////////////////// {{{1 ///////////
   // Positive values
   @Test
   fun `Test important constants are not changed`()
   {
      assertThat(Num_0, equalTo(ℝ("0")))
   }

   @Test
   fun `Test 3 P→X 1`()
   {
      assertThat(ℝ("3").P_To_X(Num_1), equalTo(ℝ("1.62090691760441912")))
   }

   @Test
   fun `Test 3 P→Y 1`()
   {
      assertThat(ℝ("3").P_To_Y(Num_1), equalTo(ℝ("2.52441295442368952")))
   }

   @Test
   fun `Test 3 R→R 4`()
   {
      assertThat(ℝ("3").R_To_R(ℝ("4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test 3 R→θ 4`()
   {
      assertThat(ℝ("3").R_To_θ(ℝ("4")), equalTo(ℝ("0.927295218001612233")))
   }

   // negative x
   @Test
   fun `Test -3 P→X 1`()
   {
      assertThat(ℝ("-3").P_To_X(Num_1), equalTo(ℝ("-1.62090691760441912")))
   }

   @Test
   fun `Test -3 P→Y 1`()
   {
      assertThat(ℝ("-3").P_To_Y(Num_1), equalTo(ℝ("-2.52441295442368952")))
   }

   @Test
   fun `Test -3 R→R 4`()
   {
      assertThat(ℝ("-3").R_To_R(ℝ("4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test -3 R→θ 4`()
   {
      assertThat(ℝ("-3").R_To_θ(ℝ("4")), equalTo(ℝ("2.21429743558818101")))
   }

   // negative y
   @Test
   fun `Test 3 P→X -1`()
   {
      assertThat(ℝ("3").P_To_X(ℝ("-1")), equalTo(ℝ("1.62090691760441915")))
   }

   @Test
   fun `Test 3 P→Y -1`()
   {
      assertThat(ℝ("3").P_To_Y(ℝ("-1")), equalTo(ℝ("-2.52441295442368952")))
   }

   @Test
   fun `Test 3 R→R -4`()
   {
      assertThat(ℝ("3").R_To_R(ℝ("-4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test 3 R→θ -4`()
   {
      assertThat(ℝ("3").R_To_θ(ℝ("-4")), equalTo(ℝ("-0.927295218001612233")))
   }

   // negative x and y
   @Test
   fun `Test -3 P→X -1`()
   {
      assertThat(ℝ("-3").P_To_X(ℝ("-1")), equalTo(ℝ("-1.62090691760441915")))
   }

   @Test
   fun `Test -3 P→Y -1`()
   {
      assertThat(ℝ("-3").P_To_Y(ℝ("-1")), equalTo(ℝ("2.52441295442368952")))
   }

   @Test
   fun `Test -3 R→R -4`()
   {
      assertThat(ℝ("-3").R_To_R(ℝ("-4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test -3 R→θ -4`()
   {
      assertThat(ℝ("-3").R_To_θ(ℝ("-4")), equalTo(ℝ("-2.21429743558818101")))
   }

   // For degree //////////////////////////////////////////// {{{1 ///////////
   // Positive values for degree
   @Test
   fun `Test 3 P→X₁₈₀ 60`()
   {
      assertThat(ℝ("3").P_To_X(ℝ("60"), Num_180), equalTo(ℝ("1.5")))
   }

   @Test
   fun `Test 3 P→Y₁₈₀ 60`()
   {
      assertThat(ℝ("3").P_To_Y(ℝ("60"), Num_180), equalTo(ℝ("2.59807621135331594")))
   }

   @Test
   fun `Test 3 R→R₁₈₀ 4`()
   {
      assertThat(ℝ("3").R_To_R(ℝ("4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test 3 R→θ₁₈₀ 4`()
   {
      assertThat(ℝ("3").R_To_θ(ℝ("4"), Num_180), equalTo(ℝ("53.1301023541559786")))
   }

   // negative x for degree
   @Test
   fun `Test -3 P→X₁₈₀ 60`()
   {
      assertThat(ℝ("-3").P_To_X(ℝ("60"), Num_180), equalTo(ℝ("-1.5")))
   }

   @Test
   fun `Test -3 P→Y₁₈₀ 60`()
   {
      assertThat(ℝ("-3").P_To_Y(ℝ("60"), Num_180), equalTo(ℝ("-2.59807621135331594")))
   }

   @Test
   fun `Test -3 R→R₁₈₀ 4`()
   {
      assertThat(ℝ("-3").R_To_R(ℝ("4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test -3 R→θ₁₈₀ 4`()
   {
      assertThat(ℝ("-3").R_To_θ(ℝ("4"), Num_180), equalTo(ℝ("126.869897645844021")))
   }

   // negative y for degree
   @Test
   fun `Test 3 P→X₁₈₀ -60`()
   {
      assertThat(ℝ("3").P_To_X(ℝ("-60"), Num_180), equalTo(ℝ("1.5")))
   }

   @Test
   fun `Test 3 P→Y₁₈₀ -60`()
   {
      assertThat(ℝ("3").P_To_Y(ℝ("-60"), Num_180), equalTo(ℝ("-2.59807621135331594")))
   }

   @Test
   fun `Test 3 R→R₁₈₀ -4`()
   {
      assertThat(ℝ("3").R_To_R(ℝ("-4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test 3 R→θ₁₈₀ -4`()
   {
      assertThat(ℝ("3").R_To_θ(ℝ("-4"), Num_180), equalTo(ℝ("-53.1301023541559788")))
   }

   // negative x and y for degree
   @Test
   fun `Test -3 P→X₁₈₀ -60`()
   {
      assertThat(ℝ("-3").P_To_X(ℝ("-60"), Num_180), equalTo(ℝ("-1.5")))
   }

   @Test
   fun `Test -3 P→Y₁₈₀ -60`()
   {
      assertThat(ℝ("-3").P_To_Y(ℝ("-60"), Num_180), equalTo(ℝ("2.59807621135331594")))
   }

   @Test
   fun `Test -3 R→R₁₈₀ -4`()
   {
      assertThat(ℝ("-3").R_To_R(ℝ("-4")), equalTo(ℝ("5")))
   }

   @Test
   fun `Test -3 R→θ₁₈₀ -4`()
   {
      assertThat(ℝ("-3").R_To_θ(ℝ("-4"), Num_180), equalTo(ℝ("-126.869897645844021")))
   }

   // Both null ///////////////////////////////////////////// {{{1 ///////////
   @Test
   fun `Test 0 P→X 0`()
   {
      assertThat(Num_0.P_To_X(Num_0), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 0 P→Y 0`()
   {
      assertThat(Num_0.P_To_Y(Num_0), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 0 R→R 0`()
   {
      assertThat(Num_0.R_To_R(Num_0), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 3 R→θ 0`()
   {
      assertThat(Num_0.R_To_θ(Num_0).Is_NaN(), equalTo(true))
   }

   // 0 x and gradients ///////////////////////////////////// {{{1 ///////////
   // 0 x and positive y values for gradiants
   @Test
   fun `Test 0 P→X₂₀₀ 50`()
   {
      assertThat(ℝ("0").P_To_X(ℝ("60"), Num_200), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 0 P→Y₂₀₀ 50`()
   {
      assertThat(ℝ("0").P_To_Y(ℝ("60"), Num_200), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 0 R→R₂₀₀ 4`()
   {
      assertThat(ℝ("0").R_To_R(ℝ("4")), equalTo(ℝ("4")))
   }

   @Test
   fun `Test 0 R→θ₂₀₀ 4`()
   {
      assertThat(ℝ("0").R_To_θ(ℝ("4"), Num_200), equalTo(ℝ("100")))
   }

   //  0 x and negative y for degree
   @Test
   fun `Test 0 P→X₂₀₀ -50`()
   {
      assertThat(ℝ("0").P_To_X(ℝ("-60"), Num_200), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 0 P→Y₂₀₀ -50`()
   {
      assertThat(ℝ("0").P_To_Y(ℝ("-60"), Num_200), equalTo(ℝ("-0")))
   }

   @Test
   fun `Test 0 R→R₂₀₀ -4`()
   {
      assertThat(ℝ("0").R_To_R(ℝ("-4")), equalTo(ℝ("4")))
   }

   @Test
   fun `Test 0 R→θ₂₀₀ -4`()
   {
      assertThat(ℝ("0").R_To_θ(ℝ("-4"), Num_200), equalTo(ℝ("-100")))
   }

   // 0 y and radiant /////////////////////////////////////// {{{1 ///////////
   // Positive x and 0 y values for radiant
   @Test
   fun `Test 3 P→X (0, π)`()
   {
      assertThat(ℝ("3").P_To_X(ℝ("0"), Num_π), equalTo(ℝ("3")))
   }

   @Test
   fun `Test 3 P→Y (0, π)`()
   {
      assertThat(ℝ("3").P_To_Y(ℝ("0"), Num_π), equalTo(ℝ("0")))
   }

   @Test
   fun `Test 3 R→R (0, π)`()
   {
      assertThat(ℝ("3").R_To_R(ℝ("0")), equalTo(ℝ("3")))
   }

   @Test
   fun `Test 3 R→θ (0, π)`()
   {
      assertThat(ℝ("3").R_To_θ(ℝ("0"), Num_π), equalTo(ℝ("0")))
   }

   // negative x for degree
   @Test
   fun `Test -3 P→X (0, π)`()
   {
      assertThat(ℝ("-3").P_To_X(ℝ("0"), Num_π), equalTo(ℝ("-3")))
   }

   @Test
   fun `Test -3 P→Y (0, π)`()
   {
      assertThat(ℝ("-3").P_To_Y(ℝ("0"), Num_π).Is_NaN(), equalTo(true))
   }

   @Test
   fun `Test -3 R→R (0, π)`()
   {
      assertThat(ℝ("-3").R_To_R(ℝ("0")), equalTo(ℝ("3")))
   }

   @Test
   fun `Test -3 R→θ (0, π)`()
   {
      assertThat(ℝ("-3").R_To_θ(ℝ("0"), Num_π), equalTo(Num_π))
   }

   // Test from customers
   @Test
   fun `Test -0․001 R→R 0․005`()
   {
      assertThat(ℝ("-0.001").R_To_R(ℝ("0.005")), equalTo(ℝ("0.00509901951359278485")))
   }

   @Test
   fun `Test -0․001 R→θ 0․005`()
   {
      assertThat(ℝ("-0.001").R_To_θ(ℝ("0.005")), equalTo(ℝ("1.76819188664477738")))
   }

   @Test
   fun `Test -0․001 P→θ₁₈₀ 0․005`()
   {
      assertThat(ℝ("-0.001").R_To_θ(ℝ("0.005"), Num_180), equalTo(ℝ("101.309932474020213")))
   }
   ////////////////////////////////////////////////////////// 1}}} ///////////
} // BCDFloat_Root_Test
// vim: set nowrap tabstop=8 shiftwidth=0 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=marker spell spelllang=en_gb :
