/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 8155 $
 * $Date: 2022-05-06 13:03:09 +0200 (Fr, 06. Mai 2022) $
 * $Id: BCDFloat_Rounding_Test.scala 8155 2022-05-06 11:03:09Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge
 * .uiq3/math/BCDFloat_Rounding_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_3600
import net.sourceforge.uiq3.math.BCDFloat.Num_e
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Rounding_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   private val Large_1 = Num_π * Num_e * Num_3600

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: frac")
   inner class `Test_frac`
   {
      init
      {
	 Feature("frac")
      }
      @org.junit.jupiter.api.Test
      fun `Test frac〈π〉`()
      {
	 assertThat(Num_π.Frac(), equalTo(ℝ("0.14159265358979324")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac〈-π〉`()
      {
	 assertThat(Num_π.neg().Frac(), equalTo(ℝ("-0.14159265358979324")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac〈e〉`()
      {
	 assertThat(Num_e.Frac(), equalTo(ℝ("0.71828182845904526")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac〈-e〉`()
      {
	 assertThat(Num_e.neg().Frac(), equalTo(ℝ("-0.71828182845904526")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac〈l1〉`()
      {
	 assertThat(Large_1.Frac(), equalTo(ℝ("0.0432016248417")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac〈-l1〉`()
      {
	 assertThat(Large_1.neg().Frac(), equalTo(ℝ("-0.0432016248417")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac 〈0․7〉`()
      {
	 assertThat(ℝ("0.7").Frac(), equalTo(ℝ("0.7")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac 〈-0․7〉`()
      {
	 assertThat(ℝ("-0.7").Frac(), equalTo(ℝ("-0.7")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac 〈0․2〉`()
      {
	 assertThat(ℝ("0.2").Frac(), equalTo(ℝ("0.2")))
      }

      @org.junit.jupiter.api.Test
      fun `Test frac 〈-0․2〉`()
      {
	 assertThat(ℝ("-0.2").Frac(), equalTo(ℝ("-0.2")))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: int")
   inner class `Test_int`
   {
      init
      {
	 Feature("int")
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊π⌋`()
      {
	 assertThat(Num_π.Integer(), equalTo(ℝ("3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊-π⌋`()
      {
	 assertThat(Num_π.neg().Integer(), equalTo(ℝ("-3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊e⌋`()
      {
	 assertThat(Num_e.Integer(), equalTo(ℝ("2")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊-e⌋`()
      {
	 assertThat(Num_e.neg().Integer(), equalTo(ℝ("-2")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊l1⌋`()
      {
	 assertThat(Large_1.Integer(), equalTo(ℝ("30743")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊-l1⌋`()
      {
	 assertThat(Large_1.neg().Integer(), equalTo(ℝ("-30743")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊0․7⌋`()
      {
	 assertThat(ℝ("0.7").Integer(), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊-0․7⌋`()
      {
	 assertThat(ℝ("-0.7").Integer(), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊0․2⌋`()
      {
	 assertThat(ℝ("0.2").Integer(), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.api.Test
      fun `Test ⌊-0․2⌋`()
      {
	 assertThat(ℝ("-0.2").Integer(), equalTo(ℝ("0")))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: round 10 digits")
   inner class `Test_round_10_digits`
   {
      init {

      Feature ("round 10 digits")
      }
      val Digits = 10;

      @org.junit.jupiter.api.Test
      fun `Test round 〈π, 10`()
      {
	 assertThat(Num_π.Round(Digits, 99), equalTo(ℝ("3.141592654")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-π, 10`()
      {
	 assertThat(Num_π.neg().Round(Digits, 99), equalTo(ℝ("-3.141592654")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.000000002  , -0.000000002 ",
	    "-0.000000007  , -0.000000007 ",
	    "-0.00000002   , -0.00000002  ",
	    "-0.00000007   , -0.00000007  ",
	    "-0.0000007    , -0.0000007   ",
	    "-0.000002     , -0.000002    ",
	    "-0.000007     , -0.000007    ",
	    "-0.00007      , -0.00007     ",
	    "-0.0002       , -0.0002      ",
	    "-0.0007       , -0.0007      ",
	    "-0.007        , -0.007       ",
	    "-0.02         , -0.02        ",
	    "-0.07         , -0.07        ",
	    "-0.7          , -0.7         ",
	    "0.000000002   , 0.000000002  ",
	    "0.000000007   , 0.000000007  ",
	    "0.00000002    , 0.00000002   ",
	    "0.00000007    , 0.00000007   ",
	    "0.0000007     , 0.0000007    ",
	    "0.000002      , 0.000002     ",
	    "0.000007      , 0.000007     ",
	    "0.00007       , 0.00007      ",
	    "0.0002        , 0.0002       ",
	    "0.0007        , 0.0007       ",
	    "0.007         , 0.007        ",
	    "0.02          , 0.02         ",
	    "0.07          , 0.07         ",
	    "0.7           , 0.7          ",
	    "-0.002        , -0.002       ",
	    "-0.007        , -0.007       ",
	    "-0.01035      , -0.01035     ",
	    "-0.02         , -0.02        ",
	    "-0.07         , -0.07        ",
	    "-0.2          , -0.2         ",
	    "-0.7          , -0.7         ",
	    "-1.000000002  , -1.000000002 ",
	    "-1.000000007  , -1.000000007 ",
	    "-1.00000002   , -1.00000002  ",
	    "-1.00000007   , -1.00000007  ",
	    "-1.0000002    , -1.0000002   ",
	    "-1.0000007    , -1.0000007   ",
	    "-1.000002     , -1.000002    ",
	    "-1.000007     , -1.000007    ",
	    "-1.00002      , -1.00002     ",
	    "-1.00007      , -1.00007     ",
	    "-1.0002       , -1.0002      ",
	    "-1.0007       , -1.0007      ",
	    "-1.002        , -1.002       ",
	    "-1.007        , -1.007       ",
	    "-1.02         , -1.02        ",
	    "-1.07         , -1.07        ",
	    "-1.2          , -1.2         ",
	    "-1.7          , -1.7         ",
	    "0.002         , 0.002        ",
	    "0.007         , 0.007        ",
	    "0.01035       , 0.01035      ",
	    "0.02          , 0.02         ",
	    "0.07          , 0.07         ",
	    "0.2           , 0.2          ",
	    "0.7           , 0.7          ",
	    "1.000000002   , 1.000000002  ",
	    "1.000000007   , 1.000000007  ",
	    "1.00000002    , 1.00000002   ",
	    "1.00000007    , 1.00000007   ",
	    "1.0000002     , 1.0000002    ",
	    "1.0000007     , 1.0000007    ",
	    "1.000002      , 1.000002     ",
	    "1.000007      , 1.000007     ",
	    "1.00002       , 1.00002      ",
	    "1.00007       , 1.00007      ",
	    "1.0002        , 1.0002       ",
	    "1.0007        , 1.0007       ",
	    "1.002         , 1.002        ",
	    "1.007         , 1.007        ",
	    "1.02          , 1.02         ",
	    "1.07          , 1.07         ",
	    "1.2           , 1.2          ",
	    "1.7           , 1.7          ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test round`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Round(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: round 9 digits")
   inner class `Test_round_9_digits`
   {
      init {
         
      Feature ("round 9 digits")

      }
      val Digits = 9;

      @org.junit.jupiter.api.Test
      fun `Test round 〈e, 9〉`()
      {
	 assertThat(Num_e.Round(Digits, 99), equalTo(ℝ("2.71828183")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-e, 9〉`()
      {
	 assertThat(Num_e.neg().Round(Digits, 99), equalTo(ℝ("-2.71828183")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈l1, 9〉`()
      {
	 assertThat(Large_1.Round(Digits, 99), equalTo(ℝ("30743.0432")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-l1, 9〉`()
      {
	 assertThat(Large_1.neg().Round(Digits, 99), equalTo(ℝ("-30743.0432")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.000000002  , -0.000000002 ",
	    "-0.000000007  , -0.000000007 ",
	    "-0.00000002   , -0.00000002  ",
	    "-0.00000007   , -0.00000007  ",
	    "-0.0000007    , -0.0000007   ",
	    "-0.000002     , -0.000002    ",
	    "-0.000007     , -0.000007    ",
	    "-0.00007      , -0.00007     ",
	    "-0.0002       , -0.0002      ",
	    "-0.0007       , -0.0007      ",
	    "-0.002        , -0.002       ",
	    "-0.007        , -0.007       ",
	    "-0.01035      , -0.01035     ",
	    "-0.02         , -0.02        ",
	    "-0.07         , -0.07        ",
	    "-0.2          , -0.2         ",
	    "-0.7          , -0.7         ",
	    "-1.000000002  , -1           ",
	    "-1.000000007  , -1.00000001  ",
	    "-1.00000002   , -1.00000002  ",
	    "-1.00000007   , -1.00000007  ",
	    "-1.0000002    , -1.0000002   ",
	    "-1.0000007    , -1.0000007   ",
	    "-1.000002     , -1.000002    ",
	    "-1.000007     , -1.000007    ",
	    "-1.00002      , -1.00002     ",
	    "-1.00007      , -1.00007     ",
	    "-1.0002       , -1.0002      ",
	    "-1.0007       , -1.0007      ",
	    "-1.002        , -1.002       ",
	    "-1.007        , -1.007       ",
	    "-1.02         , -1.02        ",
	    "-1.07         , -1.07        ",
	    "-1.2          , -1.2         ",
	    "-1.7          , -1.7         ",
	    "0.000000002   , 0.000000002  ",
	    "0.000000007   , 0.000000007  ",
	    "0.00000002    , 0.00000002   ",
	    "0.00000007    , 0.00000007   ",
	    "0.0000007     , 0.0000007    ",
	    "0.000002      , 0.000002     ",
	    "0.000007      , 0.000007     ",
	    "0.00007       , 0.00007      ",
	    "0.0002        , 0.0002       ",
	    "0.0007        , 0.0007       ",
	    "0.002         , 0.002        ",
	    "0.007         , 0.007        ",
	    "0.01035       , 0.01035      ",
	    "0.02          , 0.02         ",
	    "0.07          , 0.07         ",
	    "0.2           , 0.2          ",
	    "0.7           , 0.7          ",
	    "1.000000002   , 1            ",
	    "1.000000007   , 1.00000001   ",
	    "1.00000002    , 1.00000002   ",
	    "1.00000007    , 1.00000007   ",
	    "1.0000002     , 1.0000002    ",
	    "1.0000007     , 1.0000007    ",
	    "1.000002      , 1.000002     ",
	    "1.000007      , 1.000007     ",
	    "1.00002       , 1.00002      ",
	    "1.00007       , 1.00007      ",
	    "1.0002        , 1.0002       ",
	    "1.0007        , 1.0007       ",
	    "1.002         , 1.002        ",
	    "1.007         , 1.007        ",
	    "1.02          , 1.02         ",
	    "1.07          , 1.07         ",
	    "1.2           , 1.2          ",
	    "1.7           , 1.7          ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test round`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Round(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: round 5 digits")
   inner class `Test_round_5_digits`
   {
      init {

      Feature ("round 5 digits")
      }
      val Digits = 5;

      @org.junit.jupiter.api.Test
      fun `Test round 〈π, 5〉`()
      {
	 assertThat(Num_π.Round(Digits, 99), equalTo(ℝ("3.1416")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-π, 5〉`()
      {
	 assertThat(Num_π.neg().Round(Digits, 99), equalTo(ℝ("-3.1416")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈e, 5〉`()
      {
	 assertThat(Num_e.Round(Digits, 99), equalTo(ℝ("2.7183")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-e, 5〉`()
      {
	 assertThat(Num_e.neg().Round(Digits, 99), equalTo(ℝ("-2.7183")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈l1, 5〉`()
      {
	 assertThat(Large_1.Round(Digits, 99), equalTo(ℝ("30743")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-l1, 5〉`()
      {
	 assertThat(Large_1.neg().Round(Digits, 99), equalTo(ℝ("-30743")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected     ",
	    "-0.000000007  , -0.000000007 ",
	    "-0.00000002   , -0.00000002  ",
	    "-0.00000007   , -0.00000007  ",
	    "-0.0000007    , -0.0000007   ",
	    "-0.000002     , -0.000002    ",
	    "-0.000007     , -0.000007    ",
	    "-0.00007      , -0.00007     ",
	    "-0.0002       , -0.0002      ",
	    "-0.0007       , -0.0007      ",
	    "-0.002        , -0.002       ",
	    "-0.007        , -0.007       ",
	    "-0.01035      , -0.01035     ",
	    "-0.02         , -0.02        ",
	    "-0.07         , -0.07        ",
	    "-0.2          , -0.2         ",
	    "-0.7          , -0.7         ",
	    "-1.000000002  , -1           ",
	    "-1.000000007  , -1           ",
	    "-1.00000002   , -1           ",
	    "-1.00000007   , -1           ",
	    "-1.0000002    , -1           ",
	    "-1.0000007    , -1           ",
	    "-1.000002     , -1           ",
	    "-1.000007     , -1           ",
	    "-1.00002      , -1           ",
	    "-1.00007      , -1.0001      ",
	    "-1.0002       , -1.0002      ",
	    "-1.0007       , -1.0007      ",
	    "-1.002        , -1.002       ",
	    "-1.007        , -1.007       ",
	    "-1.02         , -1.02        ",
	    "-1.07         , -1.07        ",
	    "-1.2          , -1.2         ",
	    "-1.7          , -1.7         ",
	    "0.000000007   , 0.000000007  ",
	    "0.00000002    , 0.00000002   ",
	    "0.00000007    , 0.00000007   ",
	    "0.0000007     , 0.0000007    ",
	    "0.000002      , 0.000002     ",
	    "0.000007      , 0.000007     ",
	    "0.00007       , 0.00007      ",
	    "0.0002        , 0.0002       ",
	    "0.0007        , 0.0007       ",
	    "0.002         , 0.002        ",
	    "0.007         , 0.007        ",
	    "0.01035       , 0.01035      ",
	    "0.02          , 0.02         ",
	    "0.07          , 0.07         ",
	    "0.2           , 0.2          ",
	    "0.7           , 0.7          ",
	    "1.000000002   , 1            ",
	    "1.000000007   , 1            ",
	    "1.00000002    , 1            ",
	    "1.00000007    , 1            ",
	    "1.0000002     , 1            ",
	    "1.0000007     , 1            ",
	    "1.000002      , 1            ",
	    "1.000007      , 1            ",
	    "1.00002       , 1            ",
	    "1.00007       , 1.0001       ",
	    "1.0002        , 1.0002       ",
	    "1.0007        , 1.0007       ",
	    "1.002         , 1.002        ",
	    "1.007         , 1.007        ",
	    "1.02          , 1.02         ",
	    "1.07          , 1.07         ",
	    "1.2           , 1.2          ",
	    "1.7           , 1.7          ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test round`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Round(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: round 1 digit")
   inner class `Test_round_1_digit`
   {
      init {

      Feature ("round 1 digit")
      }
      val Digits = 1;

      @org.junit.jupiter.api.Test
      fun `Test round 〈π, 1〉`()
      {
	 assertThat(Num_π.Round(Digits, 99), equalTo(ℝ("3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-π, 1〉`()
      {
	 assertThat(Num_π.neg().Round(Digits, 99), equalTo(ℝ("-3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈e, 1〉`()
      {
	 assertThat(Num_e.Round(Digits, 99), equalTo(ℝ("3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-e, 1〉`()
      {
	 assertThat(Num_e.neg().Round(Digits, 99), equalTo(ℝ("-3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈l1, 1〉`()
      {
	 assertThat(Large_1.Round(Digits, 99), equalTo(ℝ("30000")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈-l1, 1〉`()
      {
	 assertThat(Large_1.neg().Round(Digits, 99), equalTo(ℝ("-30000")))
      }

      @org.junit.jupiter.api.Test
      fun `Test round 〈null-value, 1〉`()
      {
	 assertThat(BCDFloat().Round(Digits, 99), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.002        , -0.002",
	    "-0.007        , -0.007",
	    "-0.01035      , -0.01",
	    "-0.02         , -0.02",
	    "-0.07         , -0.07",
	    "-0.2          , -0.2",
	    "-0.7          , -0.7",
	    "-1.000000002  , -1",
	    "-1.0000002    , -1",
	    "-1.00002      , -1",
	    "-1.002        , -1",
	    "-1.2          , -1",
	    "0.002         , 0.002",
	    "0.007         , 0.007",
	    "0.01035       , 0.01",
	    "0.02          , 0.02",
	    "0.07          , 0.07",
	    "0.2           , 0.2",
	    "0.7           , 0.7",
	    "1.000000002   , 1",
	    "1.0000002     , 1",
	    "1.00002       , 1",
	    "1.002         , 1",
	    "1.2           , 1",
	    "-1.000000007  , -1",
	    "-1.00000002   , -1",
	    "-1.00000007   , -1",
	    "-1.0000007    , -1",
	    "-1.000002     , -1",
	    "-1.000007     , -1",
	    "-1.00007      , -1",
	    "-1.0002       , -1",
	    "-1.0007       , -1",
	    "-1.007        , -1",
	    "-1.02         , -1",
	    "-1.07         , -1",
	    "-1.7          , -2",
	    "1.000000007   , 1",
	    "1.00000002    , 1",
	    "1.00000007    , 1",
	    "1.0000007     , 1",
	    "1.000002      , 1",
	    "1.000007      , 1",
	    "1.00007       , 1",
	    "1.0002        , 1",
	    "1.0007        , 1",
	    "1.007         , 1",
	    "1.02          , 1",
	    "1.07          , 1",
	    "1.7           , 2",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test round`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Round(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: fix 5 digits")
   inner class `Test_fix_5_digits`
   {
      init {

      Feature ("fix 5 digits")
      }
      val Digits = 5;

      @org.junit.jupiter.api.Test
      fun `Test fix 〈π, 5〉`()
      {
	 assertThat(Num_π.Fix(Digits, 99), equalTo(ℝ("3.14159")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-π, 5〉`()
      {
	 assertThat(Num_π.neg().Fix(Digits, 99), equalTo(ℝ("-3.14159")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈e, 5〉`()
      {
	 assertThat(Num_e.Fix(Digits, 99), equalTo(ℝ("2.71828")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-e, 5〉`()
      {
	 assertThat(Num_e.neg().Fix(Digits, 99), equalTo(ℝ("-2.71828")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈l1, 5〉`()
      {
	 assertThat(Large_1.Fix(Digits, 99), equalTo(ℝ("30743.04320")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-l1, 5〉`()
      {
	 assertThat(Large_1.neg().Fix(Digits, 99), equalTo(ℝ("-30743.04320")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈null-value, 0〉`()
      {
	 assertThat(BCDFloat().Fix(Digits, 99), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected  ",
	    "-0.000000007  , 0         ",
	    "-0.00000002   , 0         ",
	    "-0.00000007   , 0         ",
	    "-0.0000007    , 0         ",
	    "-0.000002     , 0         ",
	    "-0.000007     , -0.00001  ",
	    "-0.00007      , -0.00007  ",
	    "-0.0002       , -0.0002   ",
	    "-0.0007       , -0.0007   ",
	    "-0.002        , -0.002    ",
	    "-0.007        , -0.007    ",
	    "-0.007        , -0.007    ",
	    "-0.01035      , -0.01035  ",
	    "-0.02         , -0.02     ",
	    "-0.02         , -0.02     ",
	    "-0.07         , -0.07     ",
	    "-0.07         , -0.07     ",
	    "-0.2          , -0.2      ",
	    "-0.7          , -0.7      ",
	    "-0.7          , -0.7      ",
	    "-1.000000002  , -1        ",
	    "-1.000000007  , -1        ",
	    "-1.00000002   , -1        ",
	    "-1.00000007   , -1        ",
	    "-1.0000002    , -1        ",
	    "-1.0000007    , -1        ",
	    "-1.000002     , -1        ",
	    "-1.000007     , -1.00001  ",
	    "-1.00002      , -1.00002  ",
	    "-1.00007      , -1.00007  ",
	    "-1.0002       , -1.0002   ",
	    "-1.0007       , -1.0007   ",
	    "-1.002        , -1.002    ",
	    "-1.007        , -1.007    ",
	    "-1.02         , -1.02     ",
	    "-1.07         , -1.07     ",
	    "-1.2          , -1.2      ",
	    "-1.7          , -1.7      ",
	    "0.000000007   , 0         ",
	    "0.00000002    , 0         ",
	    "0.00000007    , 0         ",
	    "0.0000007     , 0         ",
	    "0.000002      , 0         ",
	    "0.000007      , 0.00001   ",
	    "0.00007       , 0.00007   ",
	    "0.0002        , 0.0002    ",
	    "0.0007        , 0.0007    ",
	    "0.002         , 0.002     ",
	    "0.007         , 0.007     ",
	    "0.007         , 0.007     ",
	    "0.01035       , 0.01035   ",
	    "0.02          , 0.02      ",
	    "0.02          , 0.02      ",
	    "0.07          , 0.07      ",
	    "0.07          , 0.07      ",
	    "0.2           , 0.2       ",
	    "0.7           , 0.7       ",
	    "0.7           , 0.7       ",
	    "1.000000002   , 1         ",
	    "1.000000007   , 1         ",
	    "1.00000002    , 1         ",
	    "1.00000007    , 1         ",
	    "1.0000002     , 1         ",
	    "1.0000007     , 1         ",
	    "1.000002      , 1         ",
	    "1.000007      , 1.00001   ",
	    "1.00002       , 1.00002   ",
	    "1.00007       , 1.00007   ",
	    "1.0002        , 1.0002    ",
	    "1.0007        , 1.0007    ",
	    "1.007         , 1.007     ",
	    "1.01035       , 1.01035   ",
	    "1.02          , 1.02      ",
	    "1.07          , 1.07      ",
	    "1.2           , 1.2       ",
	    "1.7           , 1.7       ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test fix`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Fix(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: fix 2 digits")
   inner class `Test_fix_2_digits`
   {
      init {

      Feature ("fix 2 digits")
      }
      val Digits = 2;

      @org.junit.jupiter.api.Test
      fun `Test fix 〈π, 1〉`()
      {
	 assertThat(Num_π.Fix(Digits, 99), equalTo(ℝ("3.14")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-π, 1〉`()
      {
	 assertThat(Num_π.neg().Fix(Digits, 99), equalTo(ℝ("-3.14")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈e, 1〉`()
      {
	 assertThat(Num_e.Fix(Digits, 99), equalTo(ℝ("2.72")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-e, 1〉`()
      {
	 assertThat(Num_e.neg().Fix(Digits, 99), equalTo(ℝ("-2.72")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈l1, 1〉`()
      {
	 assertThat(Large_1.Fix(Digits, 99), equalTo(ℝ("30743.04")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-l1, 1〉`()
      {
	 assertThat(Large_1.neg().Fix(Digits, 99), equalTo(ℝ("-30743.04")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈null-value, 0〉`()
      {
	 assertThat(BCDFloat().Fix(Digits, 99), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.000000002  , 0",
	    "-0.000000007  , 0",
	    "-0.00000002   , 0",
	    "-0.00000007   , 0",
	    "-0.0000002    , 0",
	    "-0.0000007    , 0",
	    "-0.000002     , 0",
	    "-0.000007     , 0",
	    "-0.00002      , 0",
	    "-0.00007      , 0",
	    "-0.0002       , 0",
	    "-0.0007       , 0",
	    "-0.002        , 0",
	    "-0.007        , -0.01   ",
	    "-0.02         , -0.02   ",
	    "-0.07         , -0.07",
	    "-0.2          , -0.2",
	    "-0.7          , -0.7",
	    "-1.000000002  , -1",
	    "-1.000000007  , -1",
	    "-1.00000002   , -1",
	    "-1.00000007   , -1",
	    "-1.0000002    , -1",
	    "-1.0000007    , -1",
	    "-1.000002     , -1",
	    "-1.000007     , -1",
	    "-1.00002      , -1",
	    "-1.00007      , -1",
	    "-1.0002       , -1",
	    "-1.0007       , -1",
	    "-1.002        , -1",
	    "-1.007        , -1.01",
	    "-1.02         , -1.02",
	    "-1.07         , -1.07",
	    "-1.2          , -1.2",
	    "-1.7          , -1.7",
	    "0.000000002   , 0",
	    "0.000000007   , 0",
	    "0.00000002    , 0",
	    "0.00000007    , 0",
	    "0.0000002     , 0",
	    "0.0000007     , 0",
	    "0.000002      , 0",
	    "0.000007      , 0",
	    "0.00002       , 0",
	    "0.00007       , 0",
	    "0.0002        , 0",
	    "0.0007        , 0",
	    "0.002         , 0",
	    "0.007         , 0.01",
	    "0.02          , 0.02",
	    "0.07          , 0.07",
	    "0.2           , 0.2",
	    "0.7           , 0.7",
	    "1.000000002   , 1",
	    "1.000000007   , 1",
	    "1.00000002    , 1",
	    "1.00000007    , 1",
	    "1.0000002     , 1",
	    "1.0000007     , 1",
	    "1.000002      , 1",
	    "1.000007      , 1",
	    "1.00002       , 1",
	    "1.00007       , 1",
	    "1.0002        , 1",
	    "1.0007        , 1",
	    "1.002         , 1",
	    "1.007         , 1.01   ",
	    "1.02          , 1.02   ",
	    "1.07          , 1.07   ",
	    "1.2           , 1.2    ",
	    "1.7           , 1.7    ",
	    "-0.00103      , 0      ",
	    "-0.001035     , 0      ",
	    "-0.0010352    , 0      ",
	    "-0.00113      , 0      ",
	    "-0.001135     , 0      ",
	    "-0.0011352    , 0      ",
	    "-0.0103       , -0.01  ",
	    "-0.01035      , -0.01  ",
	    "-0.010352     , -0.01  ",
	    "0.00103       , 0      ",
	    "0.001035      , 0      ",
	    "0.0010352     , 0      ",
	    "0.00113       , 0      ",
	    "0.001135      , 0      ",
	    "0.0011352     , 0      ",
	    "0.0103        , 0.01   ",
	    "0.01035       , 0.01   ",
	    "0.010352      , 0.01   ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test fix`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Fix(Digits, 99), equalTo(ℝ(expected)))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: fix 1 digits")
   inner class `Test_fix_1_digits`
   {
      init {

      Feature ("fix 1 digits")
      }
      val Digits = 1;

      @org.junit.jupiter.api.Test
      fun `Test fix 〈π, 1〉`()
      {
	 assertThat(Num_π.Fix(Digits, 99), equalTo(ℝ("3.1")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-π, 1〉`()
      {
	 assertThat(Num_π.neg().Fix(Digits, 99), equalTo(ℝ("-3.1")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈e, 1〉`()
      {
	 assertThat(Num_e.Fix(Digits, 99), equalTo(ℝ("2.7")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-e, 1〉`()
      {
	 assertThat(Num_e.neg().Fix(Digits, 99), equalTo(ℝ("-2.7")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈l1, 1〉`()
      {
	 assertThat(Large_1.Fix(Digits, 99), equalTo(ℝ("30743.0")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-l1, 1〉`()
      {
	 assertThat(Large_1.neg().Fix(Digits, 99), equalTo(ℝ("-30743.0")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈null-value, 0〉`()
      {
	 assertThat(BCDFloat().Fix(Digits, 99), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.000000002  , 0",
	    "-0.000000007  , 0",
	    "-0.00000002   , 0",
	    "-0.00000007   , 0",
	    "-0.0000002    , 0",
	    "-0.0000007    , 0",
	    "-0.000002     , 0",
	    "-0.000007     , 0",
	    "-0.00002      , 0",
	    "-0.00007      , 0",
	    "-0.0002       , 0",
	    "-0.0007       , 0",
	    "-0.002        , 0",
	    "-0.002        , 0",
	    "-0.007        , 0",
	    "-0.007        , 0",
	    "-0.02         , 0",
	    "-0.02         , 0",
	    "-0.07         , -0.1",
	    "-0.2          , -0.2",
	    "-0.7          , -0.7",
	    "-1.000000002  , -1",
	    "-1.000000007  , -1",
	    "-1.00000002   , -1",
	    "-1.00000007   , -1",
	    "-1.0000002    , -1",
	    "-1.0000007    , -1",
	    "-1.000002     , -1",
	    "-1.000007     , -1",
	    "-1.00002      , -1",
	    "-1.00007      , -1",
	    "-1.0002       , -1",
	    "-1.0007       , -1",
	    "-1.002        , -1",
	    "-1.007        , -1",
	    "-1.02         , -1",
	    "-1.07         , -1.1",
	    "-1.2          , -1.2",
	    "-1.7          , -1.7",
	    "0.000000002   , 0",
	    "0.000000007   , 0",
	    "0.00000002    , 0",
	    "0.00000007    , 0",
	    "0.0000002     , 0",
	    "0.0000007     , 0",
	    "0.000002      , 0",
	    "0.000007      , 0",
	    "0.00002       , 0",
	    "0.00007       , 0",
	    "0.0002        , 0",
	    "0.0007        , 0",
	    "0.002         , 0",
	    "0.007         , 0",
	    "0.02          , 0",
	    "0.07          , 0.1",
	    "0.2           , 0.2",
	    "0.7           , 0.7",
	    "1.000000002   , 1",
	    "1.000000007   , 1",
	    "1.00000002    , 1",
	    "1.00000007    , 1",
	    "1.0000002     , 1",
	    "1.0000007     , 1",
	    "1.000002      , 1",
	    "1.000007      , 1",
	    "1.00002       , 1",
	    "1.00007       , 1",
	    "1.0002        , 1",
	    "1.0007        , 1",
	    "1.002         , 1",
	    "1.007         , 1",
	    "1.02          , 1",
	    "1.07          , 1.1",
	    "1.2           , 1.2",
	    "1.7           , 1.7",
	    "-0.00103      , 0          ",
	    "-0.001032     , 0          ",
	    "-0.001035     , 0          ",
	    "-0.0010352    , 0          ",
	    "-0.0010355    , 0          ",
	    "-0.0103       , 0          ",
	    "-0.01032      , 0          ",
	    "-0.01035      , 0          ",
	    "-0.010352     , 0          ",
	    "-0.010355     , 0          ",
	    "-0.0113       , 0          ",
	    "-0.01132      , 0          ",
	    "-0.01135      , 0          ",
	    "-0.011352     , 0          ",
	    "-0.011355     , 0          ",
	    "0.00103       , 0          ",
	    "0.001032      , 0          ",
	    "0.001035      , 0          ",
	    "0.0010352     , 0          ",
	    "0.0010355     , 0          ",
	    "0.0103        , 0          ",
	    "0.01032       , 0          ",
	    "0.01035       , 0          ",
	    "0.010352      , 0          ",
	    "0.010355      , 0          ",
	    "0.0113        , 0          ",
	    "0.01132       , 0          ",
	    "0.01135       , 0          ",
	    "0.011352      , 0          ",
	    "0.011355      , 0          ",
	    "-0.0503       , -0.1       ",
	    "-0.05032      , -0.1       ",
	    "-0.05035      , -0.1       ",
	    "-0.050352     , -0.1       ",
	    "-0.050355     , -0.1       ",
	    "-0.0513       , -0.1       ",
	    "-0.05132      , -0.1       ",
	    "-0.05135      , -0.1       ",
	    "-0.051352     , -0.1       ",
	    "-0.051355     , -0.1       ",
	    "0.0503        , 0.1        ",
	    "0.05032       , 0.1        ",
	    "0.05035       , 0.1        ",
	    "0.050352      , 0.1        ",
	    "0.050355      , 0.1        ",
	    "0.0513        , 0.1        ",
	    "0.05132       , 0.1        ",
	    "0.05135       , 0.1        ",
	    "0.051352      , 0.1        ",
	    "0.051355      , 0.1        ",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test fix`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Fix(Digits, 99), equalTo(ℝ(expected)))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 0_01135`()
      {
	 assertThat(ℝ("0.01135").Fix(Digits, 99), equalTo(ℝ("0")))
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: fix 0 digits")
   inner class `Test_fix_0_digits`
   {
      init {

      Feature ("fix 0 digits")
      }
      val Digits = 0;

      @org.junit.jupiter.api.Test
      fun `Test fix 〈π, 0〉`()
      {
	 assertThat(Num_π.Fix(Digits, 99), equalTo(ℝ("3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-π, 0〉`()
      {
	 assertThat(Num_π.neg().Fix(Digits, 99), equalTo(ℝ("-3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈e, 0〉`()
      {
	 assertThat(Num_e.Fix(Digits, 99), equalTo(ℝ("3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-e, 0〉`()
      {
	 assertThat(Num_e.neg().Fix(Digits, 99), equalTo(ℝ("-3")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈l1, 0〉`()
      {
	 assertThat(Large_1.Fix(Digits, 99), equalTo(ℝ("30743")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈-l1, 0〉`()
      {
	 assertThat(Large_1.neg().Fix(Digits, 99), equalTo(ℝ("-30743")))
      }

      @org.junit.jupiter.api.Test
      fun `Test fix 〈null-value, 0〉`()
      {
	 assertThat(BCDFloat().Fix(Digits, 99), equalTo(ℝ("0")))
      }

      @org.junit.jupiter.params.ParameterizedTest
      @org.junit.jupiter.params.provider.CsvSource(
	 value = [
	    "input         , expected",
	    "-0.000000002  , 0",
	    "-0.000000007  , 0",
	    "-0.00000002   , 0",
	    "-0.00000007   , 0",
	    "-0.0000002    , 0",
	    "-0.0000007    , 0",
	    "-0.000002     , 0",
	    "-0.000007     , 0",
	    "-0.00002      , 0",
	    "-0.00007      , 0",
	    "-0.0002       , 0",
	    "-0.0007       , 0",
	    "-0.002        , 0",
	    "-0.007        , 0",
	    "-0.01035      , 0",
	    "-0.02         , 0",
	    "-0.07         , 0",
	    "-0.2          , 0",
	    "-0.7          , -1",
	    "-1.000000002  , -1",
	    "-1.000000007  , -1",
	    "-1.00000002   , -1",
	    "-1.00000007   , -1",
	    "-1.0000002    , -1",
	    "-1.0000007    , -1",
	    "-1.000002     , -1",
	    "-1.000007     , -1",
	    "-1.00002      , -1",
	    "-1.00007      , -1",
	    "-1.0002       , -1",
	    "-1.0007       , -1",
	    "-1.002        , -1",
	    "-1.007        , -1",
	    "-1.02         , -1",
	    "-1.07         , -1",
	    "-1.2          , -1",
	    "-1.7          , -2",
	    "0.000000002   , 0",
	    "0.000000007   , 0",
	    "0.00000002    , 0",
	    "0.00000007    , 0",
	    "0.0000002     , 0",
	    "0.0000007     , 0",
	    "0.000002      , 0",
	    "0.000007      , 0",
	    "0.00002       , 0",
	    "0.00007       , 0",
	    "0.0002        , 0",
	    "0.0007        , 0",
	    "0.002         , 0",
	    "0.007         , 0",
	    "0.01035       , 0",
	    "0.02          , 0",
	    "0.07          , 0",
	    "0.2           , 0",
	    "0.7           , 1",
	    "1.000000002   , 1",
	    "1.000000007   , 1",
	    "1.00000002    , 1",
	    "1.00000007    , 1",
	    "1.0000002     , 1",
	    "1.0000007     , 1",
	    "1.000002      , 1",
	    "1.000007      , 1",
	    "1.00002       , 1",
	    "1.00007       , 1",
	    "1.0002        , 1",
	    "1.0007        , 1",
	    "1.002         , 1",
	    "1.007         , 1",
	    "1.01035       , 1",
	    "1.02          , 1",
	    "1.07          , 1",
	    "1.2           , 1",
	    "1.7           , 2",
	 ],
	 useHeadersInDisplayName = true,
	 ignoreLeadingAndTrailingWhitespace = true
      )
      fun `Test fix`(input: String, expected: String)
      {
	 assertThat(ℝ(input).Fix(Digits, 99), equalTo(ℝ(expected)))
      }
   }
} // BCDFloat_Rounding_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
