/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.test.And
import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Given;
import net.sourceforge.uiq3.test.Info;
import net.sourceforge.uiq3.test.Scenario
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.When;
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.`object`.HasToString.hasToString

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }


   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: Parse from Java String")
   inner class Test_Parse_from_Java_String
   {
      init
      {
	 Feature("Parse from Java String")
	 Info("The Decmposed Class can be parsed from a java String user input.")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: parse “2․0e-3”")
      fun `test_parse_“2․0e-3”`()
      {
	 Scenario("parse “2․0e-3”")
	 Given("a java string representing “2.0e-3”")
	 val Text = "2.0e-3"

	 When("a BCDFloat is contructed from the java string.")
	 val Value = BCDFloat(Text)
	 Value.Normalize()

	 Then("“0.002” is returned.")

	 assertThat(Value, hasToString("0.002"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: parse “2․0E-3”")
      fun `test_parse_“2․0E-3”`()
      {
	 Scenario("parse “2․0E-3”")
	 Given("a java string representing “2.0E-3”")
	 val Text = "2.0E-3"

	 When("a BCDFloat is contructed from the java string.")
	 val Value =
	    BCDFloat(Text)
	 Value.Normalize()

	 Then("“0.002” is returned.")

	 assertThat(Value, hasToString("0.002"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Split 2°53“ for inv DRG")
      fun `test_Split_2°53“_for_inv_DRG`()
      {
	 Scenario("Split 2°53“ for inv DRG")
	 Given("A BCD of 2°53“")
	 val Test = ℝ("2") + (ℝ("53") / ℝ("60"))

	 Then("Test value is 2.883̅")
	 assertThat(Test, hasToString(("2.88333333333333333")))

	 And("Minutes = Test × 60")
	 val Minutes = Test.abs().Frac() * BCDFloat.Num_60
	 Then("Minutes are 52.9̅")
	 assertThat(Minutes.toString(), hasToString("52.9999999999999998"))

	 And("Seconds = ∤Minutes∤ * 60")
	 val Seconds = Minutes.Frac() * BCDFloat.Num_60
	 Then("Seconds are 59.9̅")
	 assertThat(Seconds, hasToString("59.999999999999988"))

	 And("Integer_Degree  = ∣Test∣ * 60")
	 val Integer_Degree = Test.Integer()
	 Then("Integer_Degree is 2")
	 assertThat(Integer_Degree, hasToString("2"))

	 And("Integer_Minutes  = ∣Minutes∣ * 60")
	 val Integer_Minutes = Minutes.Integer()
	 Then("∣Minutes∣ is 52 (as a round down happened)")
	 assertThat(Integer_Minutes, hasToString("52"))

	 And("Fix_Seconds  = Seconds rounded to 2 decimal places")
	 val Fix_Seconds = Seconds.As_BCDFloat().Fix(
	    /* Precision -> */ 2,
	    /* Exponent  -> */ 100
	 )
	 Then("Rounded Seconds value is 60 (as a round up happened)")
	 assertThat(Fix_Seconds, hasToString(("6E1")))

	 And("Rounding errors have been corrected")
	 val Result = if ((Fix_Seconds.Compare(ℝ("60"))) >= Number.Compare_Equal)
	 {
	    Triple(Integer_Degree, Integer_Minutes + ℝ("1"), Fix_Seconds - ℝ("60"))
	 }
	 else
	 {
	    Triple(Integer_Degree, Integer_Minutes, Fix_Seconds)
	 } // if
	 Then("Result is (2,53,0)")

	 assertThat(Result, equalTo(Triple(ℝ("2"), ℝ("53"), Num_0)))
      } // Scenario
   } // Feature
} // BCDFloat_Feature_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
