/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFloat_Trigonometric_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFloat_Trigonometric_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFloat.Num_0
import net.sourceforge.uiq3.math.BCDFloat.Num_180
import net.sourceforge.uiq3.math.BCDFloat.Num_200
import net.sourceforge.uiq3.math.BCDFloat.Num_π
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 * Test trigonometric functions
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFloat_Trigonometric_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test important constants are not changed`()
   {
      assertThat(Num_0, equalTo(ℝ("0")))
   }

   @org.junit.jupiter.api.Test
   fun `Test acos 0․9`()
   {
      assertThat(ℝ("0.9").arc_cos(), equalTo(ℝ("0.45102681179626243")))
   }

   @org.junit.jupiter.api.Test
   fun `Test asin 0․9`()
   {
      assertThat(ℝ("0.9").arc_sin(), equalTo(ℝ("1.11976951499863419")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan 0․9`()
   {
      assertThat(ℝ("0.9").arc_tan(), equalTo(ℝ("0.732815101786506588")))
   }

   @org.junit.jupiter.api.Test
   fun `Test cos 9`()
   {
      assertThat(ℝ("9").cos(), equalTo(ℝ("-0.911130261884676979")))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic 9`()
   {
      assertThat(
	 ℝ("9").sin(Num_180).cos(Num_180).tan(Num_180).arc_tan(Num_180).arc_cos(Num_180).arc_sin(Num_180),
	 equalTo("9.00000000000354832"))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic 9 - 9`()
   {
      assertThat(
	 ℝ("9").sin(Num_180).cos(Num_180).tan(Num_180).arc_tan(Num_180).arc_cos(Num_180).arc_sin(Num_180) - ℝ("9"),
	 equalTo("3.54832E-12"))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin 9`()
   {
      assertThat(ℝ("9").sin(), equalTo(ℝ("0.412118485241756571")))
   }

   @org.junit.jupiter.api.Test
   fun `Test tan 9`()
   {
      assertThat(ℝ("9").tan(), equalTo(ℝ("-0.452315659441809847")))
   }

   @org.junit.jupiter.api.Test
   fun `Test cos (1․5 π)`()
   {
      assertThat((ℝ("1.5") * Num_π).cos(), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin₁₈₀ (360)`()
   {
      assertThat(ℝ("360").sin(Num_180), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin₂₀₀ (400)`()
   {
      assertThat(ℝ("400").sin(Num_200), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan -5`()
   {
      assertThat(ℝ("-5").arc_tan(), equalTo(ℝ("-1.37340076694501586")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan 5`()
   {
      assertThat(ℝ("5").arc_tan(), equalTo(ℝ("1.37340076694501586")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan₁₈₀ -5`()
   {
      assertThat(ℝ("-5").arc_tan(Num_180), equalTo(ℝ("-78.6900675259797869")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan₁₈₀ 5`()
   {
      assertThat(ℝ("5").arc_tan(Num_180), equalTo(ℝ("78.6900675259797869")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan -0․2`()
   {
      assertThat(ℝ("-0.2").arc_tan(), equalTo(ℝ("-0.197395559849880758")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan 0․2`()
   {
      assertThat(ℝ("0.2").arc_tan(), equalTo(ℝ("0.197395559849880758")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan₁₈₀ -0․2`()
   {
      assertThat(ℝ("-0.2").arc_tan(Num_180), equalTo(ℝ("-11.3099324740202131")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan₁₈₀ 0․2`()
   {
      assertThat(ℝ("0.2").arc_tan(Num_180), equalTo(ℝ("11.3099324740202131")))
   }

   @org.junit.jupiter.api.Test
   fun `Test asin₁₈₀ 0`()
   {
      assertThat(ℝ("0").arc_sin(Num_180), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test asin₁₈₀ 1`()
   {
      assertThat(ℝ("1").arc_sin(Num_180), equalTo(ℝ("90")))
   }

   @org.junit.jupiter.api.Test
   fun `Test asin₁₈₀ -1`()
   {
      assertThat(ℝ("-1").arc_sin(Num_180), equalTo(ℝ("-90")))
   }

   @org.junit.jupiter.api.Test
   fun `Test acos₁₈₀ 0`()
   {
      assertThat(ℝ("0").arc_cos(Num_180), equalTo(ℝ("90")))
   }

   @org.junit.jupiter.api.Test
   fun `Test acos₁₈₀ 1`()
   {
      assertThat(ℝ("1").arc_cos(Num_180), equalTo(ℝ("0")))
   }

   @org.junit.jupiter.api.Test
   fun `Test acos₁₈₀ -1`()
   {
      assertThat(ℝ("-1").arc_cos(Num_180), equalTo(ℝ("180")))
   }
} // BCDFloat_Trigonometric_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
