/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005 … 2012 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7698 $
 *   $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 *   $Id: BCDFraction_Base_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 *   $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_Base_Test.scala $
 * ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.core.IsInstanceOf
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.IsInstanceOf.instanceOf

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_Base_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   // Adding //////////////////////////////////////////////// {{{1 ///////////
   //
   // Remember that string has an + operator as well.

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + 1⁄4`()
   {
      assertThat(ℚ("1/2") + ℚ("1/4"), equalTo(ℚ("3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + 3⁄4`()
   {
      assertThat(ℚ("1/2") + ℚ("3/4"), equalTo(ℚ("1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 + 1⁄4`()
   {
      assertThat(ℚ("-1/2") + ℚ("1/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 + 3⁄4`()
   {
      assertThat(ℚ("-1/2") + ℚ("3/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + -1⁄4`()
   {
      assertThat(ℚ("1/2") + ℚ("-1/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + -3⁄4`()
   {
      assertThat(ℚ("1/2") + ℚ("-3/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 + -1⁄4`()
   {
      assertThat(ℚ("-1/2") + ℚ("-1/4"), equalTo(ℚ("-3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 + -3⁄4`()
   {
      assertThat(ℚ("-1/2") + ℚ("-3/4"), equalTo(ℚ("-1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 + 1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") + ℚ("1 1/4"), equalTo(ℚ("2 3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 + 1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") + ℚ("1 3/4"), equalTo(ℚ("3 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 + 1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") + ℚ("1 1/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 + 1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") + ℚ("1 3/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 + -1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") + ℚ("-1 1/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 + -1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") + ℚ("-1 3/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 + -1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") + ℚ("-1 1/4"), equalTo(ℚ("-2 3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 + -1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") + ℚ("-1 3/4"), equalTo(ℚ("-3 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․0 + 1⁄4`()
   {
      val Result = ℝ("1") + ℚ("1/4")

      assertThat(Result, IsInstanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + 1․0`()
   {
      val Result = ℚ("1/2") + ℝ("1")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1 1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 + 1⁄4`()
   {
      val Result = ℤ("1") + ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 + 1`()
   {
      val Result = ℚ("1/2") + ℤ("1")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1 1/2")))
   }

   // Subtracting /////////////////////////////////////////// {{{1 ///////////

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - 1⁄4`()
   {
      assertThat(ℚ("1/2") - ℚ("1/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - 3⁄4`()
   {
      assertThat(ℚ("1/2") - ℚ("3/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 - 1⁄4`()
   {
      assertThat(ℚ("-1/2") - ℚ("1/4"), equalTo(ℚ("-3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 - 3⁄4`()
   {
      assertThat(ℚ("-1/2") - ℚ("3/4"), equalTo(ℚ("-1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - -1⁄4`()
   {
      assertThat(ℚ("1/2") - ℚ("-1/4"), equalTo(ℚ("3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - -3⁄4`()
   {
      assertThat(ℚ("1/2") - ℚ("-3/4"), equalTo(ℚ("1 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 - -1⁄4`()
   {
      assertThat(ℚ("-1/2") - ℚ("-1/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 - -3⁄4`()
   {
      assertThat(ℚ("-1/2") - ℚ("-3/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 - 1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") - ℚ("1 1/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 - 1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") - ℚ("1 3/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 - 1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") - ℚ("1 1/4"), equalTo(ℚ("-2 3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 - 1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") - ℚ("1 3/4"), equalTo(ℚ("-3 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 - -1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") - ℚ("-1 1/4"), equalTo(ℚ("2 3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 - -1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") - ℚ("-1 3/4"), equalTo(ℚ("3 1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 - -1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") - ℚ("-1 1/4"), equalTo(ℚ("-1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 - -1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") - ℚ("-1 3/4"), equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1․0 - 1⁄4`()
   {
      val Result = ℝ("1") - ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - 1․0`()
   {
      val Result = ℚ("1/2") - ℝ("1")

      assertThat(Result.javaClass, equalTo(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("-1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 - 1⁄4`()
   {
      val Result = ℤ("1") - ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("3/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 - 1`()
   {
      val Result = ℚ("1/2") - ℤ("1")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("-1/2")))
   }

   // Multiply ////////////////////////////////////////////// {{{1 ///////////
   //
   // Remember that string has an * operator as well.

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × 1⁄4`()
   {
      assertThat(ℚ("1/2") * ℚ("1/4"), equalTo(ℚ("1/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × 3⁄4`()
   {
      assertThat(ℚ("1/2") * ℚ("3/4"), equalTo(ℚ("3/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 × 1⁄4`()
   {
      assertThat(ℚ("-1/2") * ℚ("1/4"), equalTo(ℚ("-1/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 × 3⁄4`()
   {
      assertThat(ℚ("-1/2") * ℚ("3/4"), equalTo(ℚ("-3/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × -1⁄4`()
   {
      assertThat(ℚ("1/2") * ℚ("-1/4"), equalTo(ℚ("-1/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × -3⁄4`()
   {
      assertThat(ℚ("1/2") * ℚ("-3/4"), equalTo(ℚ("-3/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 × -1⁄4`()
   {
      assertThat(ℚ("-1/2") * ℚ("-1/4"), equalTo(ℚ("1/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 × -3⁄4`()
   {
      assertThat(ℚ("-1/2") * ℚ("-3/4"), equalTo(ℚ("3/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 × 1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") * ℚ("1 1/4"), equalTo(ℚ("1 7/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 × 1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") * ℚ("1 3/4"), equalTo(ℚ("2 5/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 × 1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") * ℚ("1 1/4"), equalTo(ℚ("-1 7/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 × 1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") * ℚ("1 3/4"), equalTo(ℚ("-2 5/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 × -1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") * ℚ("-1 1/4"), equalTo(ℚ("-1 7/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 × -1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") * ℚ("-1 3/4"), equalTo(ℚ("-2 5/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 × -1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") * ℚ("-1 1/4"), equalTo(ℚ("1 7/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 × -1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") * ℚ("-1 3/4"), equalTo(ℚ("2 5/8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2․0 × 1⁄4`()
   {
      val Result = ℝ("2") * ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × 2․0`()
   {
      val Result = ℚ("1/2") * ℝ("2")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2 × 1⁄4`()
   {
      val Result = ℤ("2") * ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 × 2`()
   {
      val Result = ℚ("1/2") * ℤ("2")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1")))
   }

   // Division ////////////////////////////////////////////// {{{1 ///////////

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ 1⁄4`()
   {
      assertThat(ℚ("1/2") / ℚ("1/4"), equalTo(ℚ("2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ 3⁄4`()
   {
      assertThat(ℚ("1/2") / ℚ("3/4"), equalTo(ℚ("2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 ÷ 1⁄4`()
   {
      assertThat(ℚ("-1/2") / ℚ("1/4"), equalTo(ℚ("-2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 ÷ 3⁄4`()
   {
      assertThat(ℚ("-1/2") / ℚ("3/4"), equalTo(ℚ("-2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ -1⁄4`()
   {
      assertThat(ℚ("1/2") / ℚ("-1/4"), equalTo(ℚ("-2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ -3⁄4`()
   {
      assertThat(ℚ("1/2") / ℚ("-3/4"), equalTo(ℚ("-2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 ÷ -1⁄4`()
   {
      assertThat(ℚ("-1/2") / ℚ("-1/4"), equalTo(ℚ("2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1⁄2 ÷ -3⁄4`()
   {
      assertThat(ℚ("-1/2") / ℚ("-3/4"), equalTo(ℚ("2/3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 ÷ 1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") / ℚ("1 1/4"), equalTo(ℚ("1 1/5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 ÷ 1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") / ℚ("1 3/4"), equalTo(ℚ("6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 ÷ 1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") / ℚ("1 1/4"), equalTo(ℚ("-1 1/5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 ÷ 1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") / ℚ("1 3/4"), equalTo(ℚ("-6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 ÷ -1 1⁄4`()
   {
      assertThat(ℚ("1 1/2") / ℚ("-1 1/4"), equalTo(ℚ("-1 1/5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 1⁄2 ÷ -1 3⁄4`()
   {
      assertThat(ℚ("1 1/2") / ℚ("-1 3/4"), equalTo(ℚ("-6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 ÷ -1 1⁄4`()
   {
      assertThat(ℚ("-1 1/2") / ℚ("-1 1/4"), equalTo(ℚ("1 1/5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1 1⁄2 ÷ -1 3⁄4`()
   {
      assertThat(ℚ("-1 1/2") / ℚ("-1 3/4"), equalTo(ℚ("6/7")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2․0 ÷ 1⁄4`()
   {
      val Result = ℝ("2") / ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ 2․0`()
   {
      val Result = ℚ("1/2") / ℝ("2")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1/4")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 2 ÷ 1⁄4`()
   {
      val Result = ℤ("2") / ℚ("1/4")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("8")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1⁄2 ÷ 2`()
   {
      val Result = ℚ("1/2") / ℤ("2")

      assertThat(Result, instanceOf(BCDFraction::class.java))
      assertThat(Result, equalTo(ℚ("1/4")))
   }

   // Unary ///////////////////////////////////////////////// {{{1 ///////////

   @org.junit.jupiter.api.Test
   fun `Test -(7)`()
   {
      assertThat(ℚ("7").neg(), equalTo(ℤ("-7")))
   }

   // Logical /////////////////////////////////////////////// {{{1 ///////////

   @org.junit.jupiter.api.Test
   fun `Test 7 ∧ 11`()
   {
      assertThat(ℚ("7").and(ℚ("11")), equalTo(ℤ("3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 7 ∨ 11`()
   {
      assertThat(ℚ("7").or(ℚ("11")), equalTo(ℤ("15")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 7 ⊕ 11`()
   {
      assertThat(ℚ("7").xor(ℚ("11")), equalTo(ℤ("12")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ¬7`()
   {
      assertThat(ℚ("7").not(), equalTo(ℤ("-8")))
   }

   ////////////////////////////////////////////////////////// 1}}} ///////////
} // BCDFraction_Base_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=marker spell spelllang=en_gb :
