/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFraction_Function_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_Function_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.test.ℚ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_Function_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   // neg /////////////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test -(1⁄2)`()
   {
      assertThat(ℚ("1/2").neg(), equalTo(ℚ("-1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(-1⁄2)`()
   {
      assertThat(ℚ("-1/2").neg(), equalTo(ℚ("1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(1 1⁄2)`()
   {
      assertThat(ℚ("1 1/2").neg(), equalTo(ℚ("-1 1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(-1 1⁄2)`()
   {
      assertThat(ℚ("-1 1/2").neg(), equalTo(ℚ("1 1/2")))
   }

   // abs /////////////////////////////////////////////////// {{{1 ///////////
   @org.junit.jupiter.api.Test
   fun `Test │1⁄2│`()
   {
      assertThat(ℚ("1/2").abs(), equalTo(ℚ("1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │-1⁄2│`()
   {
      assertThat(ℚ("-1/2").abs(), equalTo(ℚ("1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │(1 1⁄2│`()
   {
      assertThat(ℚ("1 1/2").abs(), equalTo(ℚ("1 1/2")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │-1 1⁄2│`()
   {
      assertThat(ℚ("-1 1/2").abs(), equalTo(ℚ("1 1/2")))
   }

   ////////////////////////////////////////////////////////// 1}}} ///////////
   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 2`()
   {
      assertThat(ℚ("0").Indirect_Value(2), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 2`()
   {
      assertThat(ℚ("1").Indirect_Value(2), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 2`()
   {
      assertThat(ℚ("10").Indirect_Value(2), equalTo(10))
   }

   @org.junit.jupiter.api.Test
   fun `Test 13 Indirect_Value 2`()
   {
      assertThat(ℚ("12").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 2`()
   {
      assertThat(ℚ("120").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 2`()
   {
      assertThat(ℚ("123").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 1`()
   {
      assertThat(ℚ("0").Indirect_Value(2), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 1`()
   {
      assertThat(ℚ("1").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 1`()
   {
      assertThat(ℚ("10").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 12 Indirect_Value 1`()
   {
      assertThat(ℚ("12").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 1`()
   {
      assertThat(ℚ("120").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 1`()
   {
      assertThat(ℚ("123").Indirect_Value(1), equalTo(1))
   }
} // BCDFraction_Base_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
