/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFraction_Log_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_Log_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFraction.Approx_e
import net.sourceforge.uiq3.math.BCDFraction.Approx_π
import net.sourceforge.uiq3.math.BCDFraction.Num_0
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_Log_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test 10⁵`()
   {
      assertThat(ℚ("10").pow(ℚ("5")), equalTo(ℝ("100000")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (2 7⁄10)⁵`()
   {
      assertThat(Approx_e.pow(ℚ("5")), equalTo(ℝ("143.48907")))
   }

   @org.junit.jupiter.api.Test
   fun `Test log₁₀ 5`()
   {
      assertThat(ℚ("5").log_10(), equalTo(ℝ("0.698970004336018816")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10^(log₁₀ 5`()
   {
      assertThat(ℚ("5").log_10().exp_10(), equalTo(ℝ("5.00000000000000008")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ln 5`()
   {
      assertThat(ℚ("5").log_e(), equalTo(ℝ("1.60943791243410039")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (2 7⁄10)^(ln 5)`()
   {
      assertThat(ℚ("5").log_e().exp_e(), equalTo(ℝ("5.00000000000000008")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ln 10`()
   {
      assertThat(ℚ("10").log_e(), equalTo(ℝ("2.30258509299404567")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0^0`()
   {
      assertThat(ℚ("0").pow(ℚ("0")).Is_NaN(), equalTo(true))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0^1`()
   {
      assertThat(ℚ("0").pow(ℚ("1")), equalTo(Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123^0`()
   {
      assertThat(ℚ("123").pow(ℚ("0")), equalTo(ℝ("1")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123^1`()
   {
      assertThat(ℚ("123").pow(ℚ("1")), equalTo(ℝ("123")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (3 1⁄7)^(2 7⁄10)`()
   {
      assertThat(Approx_π.pow(Approx_e), equalTo(ℝ("22.0179322060195289")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (2 7⁄10)¹⁰⁰`()
   {
      assertThat(Approx_e.pow(ℚ("100")), equalTo(ℝ("1.36891479058588378E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (2 7⁄10)^(100 1⁄2)`()
   {
      assertThat(Approx_e.pow(ℚ("100 1/2")), equalTo(ℝ("2.24935653031904946E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (2 7⁄10)¹⁰¹`()
   {
      assertThat(Approx_e.pow(ℚ("101")), equalTo(ℝ("3.69606993458188621E43")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (3 1⁄7)¹⁰⁰`()
   {
      assertThat(Approx_π.pow(ℚ("100")), equalTo(ℝ("5.40087443891032686E49")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (3 1⁄7)^(100 1⁄2)`()
   {
      assertThat(Approx_π.pow(ℚ("100 1/2")), equalTo(ℝ("9.57472702712159412E49")))
   }

   @org.junit.jupiter.api.Test
   fun `Test (3 1⁄7)¹⁰¹`()
   {
      assertThat(Approx_π.pow(ℚ("101")), equalTo(ℝ("1.69741768080038844E50")))
   }
} // BCDFraction_Function_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
