/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFraction_P_R_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_P_R_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDFraction.Num_1
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_P_R_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   // Positive values
   @org.junit.jupiter.api.Test
   fun `Test 3 P→X 1`()
   {
      assertThat(ℚ("3").P_To_X(Num_1), equalTo(ℝ("1.62090691760441912")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 P→Y 1`()
   {
      assertThat(ℚ("3").P_To_Y(Num_1), equalTo(ℝ("2.52441295442368952")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 R→R 4`()
   {
      assertThat(ℚ("3").R_To_R(ℚ("4")), equalTo(ℝ("5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 R→θ 4`()
   {
      assertThat(ℚ("3").R_To_θ(ℚ("4")), equalTo(ℝ("0.927295218001612233")))
   }

   // negative x
   @org.junit.jupiter.api.Test
   fun `Test -3 P→X 1`()
   {
      assertThat(ℚ("-3").P_To_X(Num_1), equalTo(ℝ("-1.62090691760441912")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 P→Y 1`()
   {
      assertThat(ℚ("-3").P_To_Y(Num_1), equalTo(ℝ("-2.52441295442368952")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 R→R 4`()
   {
      assertThat(ℚ("-3").R_To_R(ℚ("4")), equalTo(ℝ("5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 R→θ 4`()
   {
      assertThat(ℚ("-3").R_To_θ(ℚ("4")), equalTo(ℝ("2.21429743558818101")))
   }

   // negative y
   @org.junit.jupiter.api.Test
   fun `Test 3 P→X -1`()
   {
      assertThat(ℚ("3").P_To_X(ℚ("-1")), equalTo(ℝ("1.62090691760441915")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 P→Y -1`()
   {
      assertThat(ℚ("3").P_To_Y(ℚ("-1")), equalTo(ℝ("-2.52441295442368952")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 R→R -4`()
   {
      assertThat(ℚ("3").R_To_R(ℚ("-4")), equalTo(ℝ("5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3 R→θ -4`()
   {
      assertThat(ℚ("3").R_To_θ(ℚ("-4")), equalTo(ℝ("-0.927295218001612233")))
   }

   // negative x and y
   @org.junit.jupiter.api.Test
   fun `Test -3 P→X -1`()
   {
      assertThat(ℚ("-3").P_To_X(ℚ("-1")), equalTo(ℝ("-1.62090691760441915")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 P→Y -1`()
   {
      assertThat(ℚ("-3").P_To_Y(ℚ("-1")), equalTo(ℝ("2.52441295442368952")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 R→R -4`()
   {
      assertThat(ℚ("-3").R_To_R(ℚ("-4")), equalTo(ℝ("5")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -3 R→θ -4`()
   {
      assertThat(ℚ("-3").R_To_θ(ℚ("-4")), equalTo(ℝ("-2.21429743558818101")))
   }
} // BCDFraction_Root_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
