/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFraction_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Scenario
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.When;
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import net.sourceforge.uiq3.test.ℤ
import net.sourceforge.uiq3.test.ℝ
import org.hamcrest.`object`.HasToString.hasToString

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: Create standard constructor")
   inner class Test_Create_standard_constructor
   {
      init
      {
	 Feature("Create standard constructor")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: only whole")
      fun test_only_whole()
      {
	 Scenario("only whole")
	 When("a BCDFraction is constructed from one numbers.")
	 val Value = BCDFraction( /* Whole   -> */ ℤ(2L))
	 Then("“2/1” is returned.")
	 assertThat(Value, hasToString("2"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: without whole")
      fun test_without_whole()
      {
	 Scenario("without whole")
	 When("a BCDFraction is constructed from two numbers.")
	 val Value = BCDFraction(
	    /* Numerator   -> */ ℤ(1L),
	    /* Denominator => */ ℤ(2L)
	 )
	 Then("“1/2” is returned.")
	 assertThat(Value, hasToString("1/2"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: whole")
      fun test_whole()
      {
	 Scenario("whole")
	 When("a BCDFraction is constructed from three numbers.")
	 val Value = BCDFraction(
	    /* Whole       -> */ ℤ(1L),
	    /* Numerator   => */ ℤ(2L),
	    /* Denominator => */ ℤ(3L)
	 )
	 Then("“1 1/2” is returned.")
	 assertThat(Value, hasToString("1 2/3"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: 0")
      fun test_0()
      {
	 Scenario("0")
	 When("a BCDFraction is constructed 0")
	 val Value = BCDFraction(/* Whole       -> */ ℤ(0L))
	 Then("“0” is returned.")
	 assertThat(Value, hasToString("0"))
      } // Scenario
   } // Feature

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: Create string constructor")
   inner class Test_Create_string_constructor
   {
      init
      {
	 Feature("Create string constructor")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: only whole 1")
      fun test_only_whole_1()
      {
	 Scenario("only whole 1")
	 When("a BCDFraction is constructed from one numbers.")
	 val Value = BCDFraction("1")
	 Then("“0 1/1” is returned.")
	 assertThat(Value, hasToString("1"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: only whole 2")
      fun test_only_whole_2()
      {
	 Scenario("only whole 2")
	 When("a BCDFraction is constructed from one numbers.")
	 val Value = BCDFraction("2")
	 Then("“2 0/1” is returned.")
	 assertThat(Value, hasToString("2"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: without whole")
      fun test_without_whole()
      {
	 Scenario("without whole")
	 When("a BCDFraction is constructed from two numbers.")
	 val Value = BCDFraction("1/2")
	 Then("“1/2” is returned.")
	 assertThat(Value, hasToString("1/2"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: whole")
      fun test_whole()
      {
	 Scenario("whole")
	 When("a BCDFraction is constructed from three numbers.")
	 val Value = BCDFraction("1 2/3")
	 Then("“1 1/2” is returned.")
	 assertThat(Value, hasToString("1 2/3"))
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: 0")
      fun test_0()
      {
	 Scenario("0")
	 When("a BCDFraction is constructed 0")
	 val Value = BCDFraction("0")
	 Then("“0” is returned.")
	 assertThat(Value, hasToString("0"))
      } // Scenario
   } // Feature
} // BCDFraction_Feature_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
