/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDFraction_Trigonometric_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDFraction_Trigonometric_Test.scala $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math

import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsNull.notNullValue
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test

import net.sourceforge.uiq3.math.BCDFraction.Num_0
import net.sourceforge.uiq3.math.BCDFraction.Num_1
import net.sourceforge.uiq3.math.BCDFraction.Approx_π
import net.sourceforge.uiq3.math.BCDFraction.Num_180
import net.sourceforge.uiq3.math.BCDFraction.Num_200
import net.sourceforge.uiq3.test.ℚ
import net.sourceforge.uiq3.test.ℝ
import net.sourceforge.uiq3.test.ℤ

/**
 * Test trigonometric functions
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDFraction_Trigonometric_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test acos 9⁄10`()
   {
      assertThat(ℚ("9/10").arc_cos(), equalTo(ℝ("0.45102681179626243")))
   }

   @org.junit.jupiter.api.Test
   fun `Test asin 9⁄10`()
   {
      assertThat(ℚ("9/10").arc_sin(), equalTo(ℝ("1.11976951499863419")))
   }

   @org.junit.jupiter.api.Test
   fun `Test atan 0․9`()
   {
      assertThat(ℚ("9/10").arc_tan(), equalTo(ℝ("0.732815101786506588")))
   }

   @org.junit.jupiter.api.Test
   fun `Test cos 9`()
   {
      assertThat(ℚ("9").cos(), equalTo(ℝ("-0.911130261884676979")))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic 9`()
   {
      assertThat(
	 ℚ("9").sin(Num_180).cos(Num_180).tan(Num_180).arc_tan(Num_180).arc_cos(Num_180).arc_sin(Num_180),
	 equalTo(ℝ("9.00000000000354832")))
   }

   @org.junit.jupiter.api.Test
   fun `Test Forensic 9 - 9`()
   {
      assertThat(
	 ℚ("9").sin(Num_180).cos(Num_180).tan(Num_180).arc_tan(Num_180).arc_cos(Num_180).arc_sin(Num_180) - ℚ("9"),
	 equalTo(ℝ("3.54832E-12")))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin 9`()
   {
      assertThat(ℚ("9").sin(), equalTo(ℝ("0.412118485241756571")))
   }

   @org.junit.jupiter.api.Test
   fun `Test tan 9`()
   {
      assertThat(ℚ("9").tan(), equalTo(ℝ("-0.452315659441809847")))
   }

   @org.junit.jupiter.api.Test
   fun `Test cos (1․5 π)`()
   {
      assertThat((ℚ("1 1/2") * Approx_π).cos(), equalTo(ℝ("0.00189673276374314833")))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin (360)`()
   {
      assertThat(ℚ("360").sin(Num_180), equalTo(BCDFloat.Num_0))
   }

   @org.junit.jupiter.api.Test
   fun `Test sin (400)`()
   {
      assertThat(ℚ("400").sin(Num_200), equalTo(BCDFloat.Num_0))
   }
} // BCDFraction_Trigonometric_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
