/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7698 $
 * $Date: 2021-09-23 20:30:07 +0200 (Do, 23. Sep 2021) $
 * $Id: BCDInteger_Function_Test.scala 7698 2021-09-23 18:30:07Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Scala/Calculator/Test/src/test/java/net.sourceforge.uiq3/math/BCDInteger_Function_Test.scala $
 * ************************************************************ }}}1 **********
 */


package net.sourceforge.uiq3.math

import net.sourceforge.uiq3.math.BCDInteger.Num_1
import net.sourceforge.uiq3.test.ℤ
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.MatcherAssert.assertThat


@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class BCDInteger_Function_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @org.junit.jupiter.api.Test
   fun `Test │-3│`()
   {
      assertThat(ℤ("-3").abs(), equalTo(ℤ("3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test │3│`()
   {
      assertThat(ℤ("3").abs(), equalTo(ℤ("3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(-3)`()
   {
      assertThat(ℤ("-3").neg(), equalTo(ℤ("3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test -(3)`()
   {
      assertThat(ℤ("3").neg(), equalTo(ℤ("-3")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 3²`()
   {
      assertThat(ℤ("3").square(), equalTo(ℤ("9")))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1234 to int`()
   {
      assertThat(ℤ("1234").To_Integer(), equalTo(1234))
   }

   @org.junit.jupiter.api.Test
   fun `Test -1234 to int`()
   {
      assertThat(ℤ("-1234").To_Integer(), equalTo(-1234))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123456789012345678 to long`()
   {
      assertThat(ℤ("123456789012345678").To_Long(), equalTo(123456789012345678L))
   }

   @org.junit.jupiter.api.Test
   fun `Test -123456789012345678 to long`()
   {
      assertThat(ℤ("-123456789012345678").To_Long(), equalTo(-123456789012345678L))
   }

   @org.junit.jupiter.api.Test
   fun `Test ggv (1071) = 10290`()
   {
      assertThat(ℤ("1071").gcd(ℤ("1029")), equalTo(ℤ("21")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ggv (1029) = 107)`()
   {
      assertThat(ℤ("1029").gcd(ℤ("1071")), equalTo(ℤ("21")))
   }

   @org.junit.jupiter.api.Test
   fun `Test ggv (3) = 7`()
   {
      assertThat(ℤ("3").gcd(ℤ("7")), equalTo(Num_1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 2`()
   {
      assertThat(ℤ("0").Indirect_Value(2), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 2`()
   {
      assertThat(ℤ("1").Indirect_Value(2), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 2`()
   {
      assertThat(ℤ("10").Indirect_Value(2), equalTo(10))
   }

   @org.junit.jupiter.api.Test
   fun `Test 13 Indirect_Value 2`()
   {
      assertThat(ℤ("12").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 2`()
   {
      assertThat(ℤ("120").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 2`()
   {
      assertThat(ℤ("123").Indirect_Value(2), equalTo(12))
   }

   @org.junit.jupiter.api.Test
   fun `Test 0 Indirect_Value 1`()
   {
      assertThat(ℤ("0").Indirect_Value(1), equalTo(0))
   }

   @org.junit.jupiter.api.Test
   fun `Test 1 Indirect_Value 1`()
   {
      assertThat(ℤ("1").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 10 Indirect_Value 1`()
   {
      assertThat(ℤ("10").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 12 Indirect_Value 1`()
   {
      assertThat(ℤ("12").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 120 Indirect_Value 1`()
   {
      assertThat(ℤ("120").Indirect_Value(1), equalTo(1))
   }

   @org.junit.jupiter.api.Test
   fun `Test 123 Indirect_Value 1`()
   {
      assertThat(ℤ("123").Indirect_Value(1), equalTo(1))
   }
} // BCDInteger_Function_Test
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
