/*
 ************************************************************ {{{1 **********
 *   Copyright © 2010 Martin Krischik
 ****************************************************************************
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or ENDIFFTNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ************************************************************* }}}1 ********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.Asserted
import net.sourceforge.uiq3.calculator.IProperties

/**
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@java.beans.JavaBean(description = "Calculator Button with two functions")
class Calculator_Button_2 :
   net.sourceforge.uiq3.calculator.Button_3<Calculator_Canvas?, Key?>
{
   companion object
   {
      /**
       * class logger instance
       */
      private val Logger: java.util.logging.Logger

      /**
       * class logger tag.
       */
      private val TAG: String = Calculator_Button_2::class.java.name

      fun convertToKey(
         canvas: net.sourceforge.uiq3.calculator.Canvas,
         keyNr: Int
      ): Key
      {
         val seKey = canvas.getKey(keyNr)

         assert(seKey is Key)

         return seKey as Key
      }

      init
      {
         Logger = java.util.logging.Logger.getLogger(TAG)
      }
   }

   /**
    * Creates new form Calculator_Button_2
    */
   constructor() : super()
   {
      Logger.entering(TAG, "Calculator_Button_2")

      this.initComponents()

      Logger.exiting(TAG, "Calculator_Button_2")
   }

   /**
    * Creates new form Calculator_Button_2
    */
   constructor(
      canvas: Calculator_Canvas,
      keyNr: Int
   ) : super(canvas, convertToKey(canvas, keyNr), keyNr)
   {
      Logger.entering(TAG, "Calculator_Button_2", arrayOf<Any>(canvas, keyNr))

      initComponents()

      this.name = keyNr.toString()

      val data = this.data!!

      Asserted.notNull(data, "data")

      data.canvas = canvas

      run {
         val accentColor = Properties.Accent_Color
         this.Standard_Button.text = data.buttonText
         this.Standard_Button.background = data.buttonBackground
         this.Standard_Button.foreground = data.buttonForeground
         this.Standard_Button.mnemonic = data.javaKeyCode
         this.Standard_Button.putClientProperty(
            com.formdev.flatlaf.FlatClientProperties.OUTLINE,
            arrayOf(
               accentColor.value,
               data.buttonBorder
            )
         )
      }
      run {
         this.Shift_Text.text = data.goldText
      }

      Properties.addPropertyChangeListener { event ->
         val propertyName = event.propertyName

         if (propertyName == IProperties.DIALOG_FONT_ID ||
            propertyName == IProperties.DIALOG_SMALL_FONT_ID ||
            propertyName == IProperties.BUTTON_SIZE_ID
         )
         {
            initComponents()
         }
      }

      Logger.exiting(TAG, "Calculator_Button_2")
   } // Calculator_Button_2

   private fun initComponents()
   {
      Logger.entering(TAG, "initComponents")

      this.isOpaque = false

      run {
         val buttonStyle = Properties.Button_Style
         val buttonMinimumSize = Properties.Button_Size

         this.Standard_Button.minimumSize = buttonMinimumSize
         this.Standard_Button.preferredSize = buttonMinimumSize
         this.Standard_Button.font = Properties.Dialog_Large_Font
         this.Standard_Button.putClientProperty(
            com.formdev.flatlaf.FlatClientProperties.BUTTON_TYPE,
            buttonStyle.value
         )
      }
      run {
         val goldTextColor = Properties.Gold_Text_Color
         this.Shift_Text.font = Properties.Dialog_Small_Font
         this.Shift_Text.foreground = goldTextColor.value
      }
      run {
         this.Alpha_Text.text = ""

         this.remove(Alpha_Text)
         this.doLayout()
      }

      Logger.exiting(TAG, "initComponents")
   } // initComponents
}

// vim: set textwidth=120 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=kotlin fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set spell spelllang=en_gb :
