/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.calculator.IProperties

/**
 * @author martin
 */
@java.beans.JavaBean(description = "printer canvas for the RPN-45 emulator")
class PrinterCanvas :
   net.sourceforge.uiq3.printer.Canvas<Printer>
{
   companion object
   {
      private const val serialVersionUID = 5945721098895493711L

      /**
       * class logger instance
       */
      private var Logger: java.util.logging.Logger

      /**
       *  class logger tag.
       */
      private var TAG: String

      /**
       * Resource Bundle used for I18N
       */
      private var HP45_Bundle: java.util.ResourceBundle

      init
      {
	 this.TAG = PrinterCanvas::class.java.getName()
	 this.Logger = java.util.logging.Logger.getLogger(this.TAG);
	 this.HP45_Bundle = Properties.applicationBundle
      }
   }

   constructor(owner: net.sourceforge.uiq3.ui.IFrame) : super(owner)

   constructor () : super()

   var calculator: Desktop_Calculator? = null
      set(calculator)
      {
	 field = calculator

	 if (calculator != null)
	 {
	    setPrinter(calculator.printer)
	 }
      }

   init
   {
      PrinterCanvas.Logger.entering(PrinterCanvas.TAG, "init")

      this.Printout_TextArea.columns = 16
      this.Printout_TextArea.rows = 5
      this.Printout_TextArea.font = Properties.Mono_Large_Font
      this.Print_Button_1.text = PrinterCanvas.Companion.HP45_Bundle.getString("Button_Print_Display")
      this.Print_Button_2.text = PrinterCanvas.Companion.HP45_Bundle.getString("Button_Print_Register")
      this.Print_Button_3.text = PrinterCanvas.Companion.HP45_Bundle.getString("Button_Print_Memory")
      this.doLayout()

      Properties.addPropertyChangeListener { evt ->
	 if (evt.propertyName == IProperties.MONO_LARGE_FONT_ID)
	 {
	    this.Printout_TextArea.font = Properties.Mono_Large_Font
	 }
      }

      PrinterCanvas.Logger.exiting(PrinterCanvas.TAG, "init")
   } // Printer_Canvas

   override fun doPrintButton1()
   {
      PrinterCanvas.Logger.entering(PrinterCanvas.TAG, "doPrintButton1")

      super.doPrintButton1();
      this.printer?.printLine(this.calculator?.displayText ?: "")

      PrinterCanvas.Logger.exiting(PrinterCanvas.TAG, "doPrintButton1")
   }

   override fun doPrintButton2()
   {
      PrinterCanvas.Logger.entering(PrinterCanvas.TAG, "doPrintButton1")

      super.doPrintButton2();
      
      for (i in 0..6)
      {
	 this.printer?.printLine(this.calculator?.registerToString(i) ?: "")
      }

      PrinterCanvas.Logger.exiting(PrinterCanvas.TAG, "doPrintButton1")
   }

   override fun doPrintButton3()
   {
      PrinterCanvas.Logger.entering(PrinterCanvas.TAG, "doPrintButton1")

      super.doPrintButton3();

      for (i in 0..9)
      {
	 this.printer?.printLine(this.calculator?.memoryToString(i) ?: "")
      }

      PrinterCanvas.Logger.exiting(PrinterCanvas.TAG, "doPrintButton1")
   }

   override fun toString(): String
   {
      return "PrinterCanvas(super=${super.toString()};calculator=${this.calculator})"
   }

}
