/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005 … 2012 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ********************************************************** }}}1 ***********
 */
package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.calculator.IProperties

/**
 * Application properties singleton.
 *
 * @author Martin Krischik
 */
object Properties :
   net.sourceforge.uiq3.calculator.Properties()
{
   object Defaults
   {
      const val Look_And_Feel_Name: String = " net.sourceforge.uiq3.calculator.hp45"

      private val Display_Base_Font = java.awt.Font(
	 /* name  = */ "LCD Display: 14 Segment",
	 /* style = */ java.awt.Font.PLAIN,
	 /* size  = */ IProperties.Defaults.Font_Base_Size.toInt()
      )

      const val Accent_Color = 0xff00bfa5
      const val Background_Dark_Color = 0xff46454a
      const val Background_Light_Color = 0xfff5f3f9
      const val Button_Background_Color = 0xff46454a
      const val Button_Type_0_Dark_Color = 0xff393933
      const val Button_Type_0_Light_Color = 0xff555f5f
      const val Button_Type_0_Text_Color = 0xffe6edd6
      const val Button_Type_1_Dark_Color = 0xffc89433
      const val Button_Type_1_Light_Color = 0xfff6de96
      const val Button_Type_1_Text_Color = 0xfffae7c9
      const val Button_Type_2_Dark_Color = 0xff1f1f1e
      const val Button_Type_2_Light_Color = 0xff5c5e61
      const val Button_Type_2_Text_Color = 0xffd7dbbb
      const val Button_Type_3_Dark_Color = 0xff7f7968
      const val Button_Type_3_Light_Color = 0xffbdbdb3
      const val Button_Type_3_Text_Color = 0xff040600
      const val Button_Type_4_Dark_Color = 0xffb8b297
      const val Button_Type_4_Light_Color = 0xffe4e2d4
      const val Button_Type_4_Text_Color = 0xff2d311f
      const val Button_Type_5_Dark_Color = 0xff7f7968
      const val Button_Type_5_Light_Color = 0xffbdbdb3
      const val Button_Type_5_Text_Color = 0xff040600
      const val Display_Background_Color = 0xff572a33
      const val Display_Text_Color = 0xffff767f
      const val Gold_Text_Color = 0xffccc288
      const val Indicator_Selected_Color = 0xfff6de96
      const val Indicator_Unselected_Color = 0xffbdbdb3
      const val Primary_Color = 0xffe91e63
      const val Primary_Dark_Color = 0xffc2185b
      const val Printer_Background_Color = 0xff46454a
      const val Printer_Button_Dark_Color = 0xff7f7968
      const val Printer_Button_Light_Color = 0xffbdbdb3
      const val Printer_Button_Text_Color = 0xff040600
      const val Printout_Light_Color = 0xfff3f3f3
      const val Printout_Text_Color = 0xff0c0c0c

      val Display_Font = Display_Base_Font.deriveFont(IProperties.Defaults.Font_Base_Size * Display_Scale)
   } // Defaults

   private const val Display_Scale = 3.0f

   /**
    * class logger instance
    */
   private val Logger: java.util.logging.Logger

   /**
    * class logger tag.
    */
   private val TAG: String

   /**
    * User configurable properties.
    */
   const val HP45_PROPERTIES = "rpn-45.properties"

   /**
    * Internationalization properties.
    */
   const val HP45_Resources = "Bundle"

   init
   {
      this.TAG = Properties::class.java.name
      this.Logger = java.util.logging.Logger.getLogger(this.TAG)
   }

   override val Properties_Filename
      get() = java.io.File(
	 IProperties.Defaults.Home_Directory.value,
	 this.HP45_PROPERTIES
      )

   /**
    *  Applications resource bundle
    */
   val Resources = lazy { java.util.ResourceBundle.getBundle(this.HP45_Resources) }

   /**
    *  Application resource bundle
    */
   override val applicationBundle: java.util.ResourceBundle
      get() = Resources.value

   override val Accent_Color = lazy { this.readColor("Accent.color", Defaults.Accent_Color) }
   override val Background_Dark_Color = lazy { this.readColor("Background_Dark.color", Defaults.Background_Dark_Color) }
   override val Background_Light_Color = lazy {
      this.readColor("Background_Light.color", Defaults.Background_Light_Color)
   }
   val Button_Background_Color = lazy { this.readColor("Button_Background.color", Defaults.Button_Background_Color) }
   val Button_Type_0_Dark_Color = lazy { this.readColor("Button_Type_0_Dark.color", Defaults.Button_Type_0_Dark_Color) }
   val Button_Type_0_Light_Color = lazy {
      this.readColor("Button_Type_0_Light.color", Defaults.Button_Type_0_Light_Color)
   }
   val Button_Type_0_Text_Color = lazy { this.readColor("Button_Type_0_Text.color", Defaults.Button_Type_0_Text_Color) }
   val Button_Type_1_Dark_Color = lazy { this.readColor("Button_Type_1_Dark.color", Defaults.Button_Type_1_Dark_Color) }
   val Button_Type_1_Light_Color = lazy {
      this.readColor("Button_Type_1_Light.color", Defaults.Button_Type_1_Light_Color)
   }
   val Button_Type_1_Text_Color = lazy { this.readColor("Button_Type_1_Text.color", Defaults.Button_Type_1_Text_Color) }
   val Button_Type_2_Dark_Color = lazy { this.readColor("Button_Type_2_Dark.color", Defaults.Button_Type_2_Dark_Color) }
   val Button_Type_2_Light_Color = lazy {
      this.readColor("Button_Type_2_Light.color", Defaults.Button_Type_2_Light_Color)
   }
   val Button_Type_2_Text_Color = lazy { this.readColor("Button_Type_2_Text.color", Defaults.Button_Type_2_Text_Color) }
   val Button_Type_3_Dark_Color = lazy { this.readColor("Button_Type_3_Dark.color", Defaults.Button_Type_3_Dark_Color) }
   val Button_Type_3_Light_Color = lazy {
      this.readColor("Button_Type_3_Light.color", Defaults.Button_Type_3_Light_Color)
   }
   val Button_Type_3_Text_Color = lazy { this.readColor("Button_Type_3_Text.color", Defaults.Button_Type_3_Text_Color) }
   val Button_Type_4_Dark_Color = lazy { this.readColor("Button_Type_4_Dark.color", Defaults.Button_Type_4_Dark_Color) }
   val Button_Type_4_Light_Color = lazy {
      this.readColor("Button_Type_4_Light.color", Defaults.Button_Type_4_Light_Color)
   }
   val Button_Type_4_Text_Color = lazy { this.readColor("Button_Type_4_Text.color", Defaults.Button_Type_4_Text_Color) }
   val Button_Type_5_Dark_Color = lazy { this.readColor("Button_Type_5_Dark.color", Defaults.Button_Type_5_Dark_Color) }
   val Button_Type_5_Light_Color = lazy {
      this.readColor("Button_Type_5_Light.color", Defaults.Button_Type_5_Light_Color)
   }
   val Button_Type_5_Text_Color = lazy { this.readColor("Button_Type_5_Text.color", Defaults.Button_Type_5_Text_Color) }
   val Display_Background_Color = lazy { this.readColor("Display_Background.color", Defaults.Display_Background_Color) }
   val Display_Text_Color = lazy { this.readColor("Display_Text.color", Defaults.Display_Text_Color) }
   val Indicator_Selected_Color = lazy { this.readColor("Indicator_Selected.color", Defaults.Indicator_Selected_Color) }
   val Indicator_Unselected_Color = lazy {
      this.readColor("Indicator_Unselected.color", Defaults.Indicator_Unselected_Color)
   }
   val Gold_Text_Color = lazy { this.readColor("Gold_Text.color", Defaults.Gold_Text_Color) }
   val Primary_Color = lazy { this.readColor("Primary.color", Defaults.Primary_Color) }
   val Primary_Dark_Color = lazy { this.readColor("Primary_Dark.color", Defaults.Primary_Dark_Color) }
   val Printer_Background_Color = lazy { this.readColor("Printer_Background.color", Defaults.Printer_Background_Color) }
   val Printer_Button_Dark_Color =
      lazy { this.readColor("Printer_Button_Dark.color", Defaults.Printer_Button_Dark_Color) }
   val Printer_Button_Light_Color = lazy {
      this.readColor("Printer_Button_Light.color", Defaults.Printer_Button_Light_Color)
   }
   val Printer_Button_Text_Color = lazy {
      this.readColor("Printer_Button_Text.color", Defaults.Printer_Button_Text_Color)
   }
   val Printout_Light_Color = lazy { this.readColor("Printout_Light.color", Defaults.Printout_Light_Color) }
   val Printout_Text_Color = lazy { this.readColor("Printout_Text.color", Defaults.Printout_Text_Color) }

   /**
    * Get Look and Feel
    */
   @get:java.beans.BeanProperty(bound = true, description = "Application look and feel.")
   override var Look_And_Feel_Name
      get() = IProperties.readString(
	 this.self.value,
	 IProperties.LOOK_AND_FEEL_NAME_ID,
	 Defaults.Look_And_Feel_Name
      )
      set(value)
      {
	 val oldValue = this.Look_And_Feel_Name
	 this.self.value.setProperty(IProperties.LOOK_AND_FEEL_NAME_ID, value)
	 this.propertyChangeSupport.firePropertyChange(IProperties.LOOK_AND_FEEL_NAME_ID, oldValue, value);
      }

   /**
    * Used for number display
    */
   const val DISPLAY_FONT_ID = java.awt.Font.MONOSPACED + "_Display.font"

   @JvmStatic
   var Display_Font
      get() = this.readFont(DISPLAY_FONT_ID, Defaults.Display_Font)
      set(value)
      {
	 val oldValue = this.Display_Font
	 this.self.value.setProperty(DISPLAY_FONT_ID, net.sourceforge.uiq3.calculator.IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(DISPLAY_FONT_ID, oldValue, value);
      }

   /**
    * Look and feel was changed
    */
   override fun lookAndFeelChanged(size: net.sourceforge.uiq3.calculator.IProperties.Look_And_Feel_Size_Type)
   {
      this.Logger.entering(this.TAG, "Look_And_Feel_Changed", size)

      super.lookAndFeelChanged(size)
      Display_Font = size.deriveFont(Defaults.Display_Font)

      this.Logger.entering(this.TAG, "Look_And_Feel_Changed")
      return;
   }

   fun readResource(Key: String) = this.Resources.value.getString(Key)

   /**
    * Restore calculator status
    *
    * @param calculatorEngine
    * calculator
    */
   override fun restoreCalculatorStatus(calculatorEngine: net.sourceforge.uiq3.calculator.ICalculator)
   {
      this.Logger.entering(this.TAG, "Restore_Calculator_Status", calculatorEngine)

      if (calculatorEngine is Calculator)
      {
	 try
	 {
	    val P = this.self.value
	 }
	 catch (Exception: Exception)
	 {
	    this.Logger.log(java.util.logging.Level.SEVERE, "LOG00630:" + this.Exception_Info, Exception)
	 } // try
      } // if

      this.Logger.exiting(this.TAG, "Restore_Calculator_Status")
      return
   } // Restore_Window_Status

   /**
    * Save calculator status
    *
    * @param calculatorEngine
    * Calculator
    */
   override fun saveCalculatorStatus(calculatorEngine: net.sourceforge.uiq3.calculator.ICalculator)
   {
      if (calculatorEngine is Calculator)
      {
	 val P = this.self.value
      } // if
      return
   } // Save_Calculator_Status

   /**
    * Save Property file
    */
   override fun saveProperties()
   {
      this.Logger.entering(this.TAG, "saveProperties")
      try
      {
	 val P = this.self.value

	 java.io.FileOutputStream(this.Properties_Filename).use { Writer ->
	    P.store(
	       /* out      = */ Writer,
	       /* comments = */"RPN-45 Simulator status."
	    )
	 }
      }
      catch (Exception: java.io.IOException)
      {
	 this.Logger.log(java.util.logging.Level.SEVERE, "LOG00660:" + this.Exception_Info, Exception)
      } // try

      this.Logger.exiting(this.TAG, "saveProperties")
      return
   } // saveProperties
} // Properties

// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab :
// vim: set textwidth=0 filetype=kotlin foldmethod=syntax nospell :
