/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * @author martin
 */
@SuppressWarnings ({"TooBroadScope", "FieldCanBeLocal"})
public final class Calculator_Canvas
   extends
   net.sourceforge.uiq3.calculator.Canvas
{
   private static final double Maximum_Size_Scale = 1.5;
   private static final double SCALE_HEIGHT = 14.2;
   private static final double SCALE_WIDTH = 5.2;
   private static final long serialVersionUID = 2510695920609251004L;
   /**
    * <p>
    * class logger instance
    * </p>
    */
   @NotNull
   private static final java.util.logging.Logger logger;
   /**
    * <p> class logger tag. </p>
    */
   @NotNull
   private static final String TAG;

   static
   {
      TAG = Calculator_Canvas.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
   }

   @Nullable
   private final net.sourceforge.uiq3.ui.IFrame owner;
   /**
    * Calculator to be displayed
    */
   @Nullable
   private Desktop_Calculator Engine;

   private static java.awt.Font
   getMonoFont ()
   {
      return Properties.getDisplay_Font ();
   }

   private java.awt.Color getDisplayBackgroundColor ()
   {
      final kotlin.Lazy<java.awt.Color> Retval = Properties.INSTANCE.getDisplay_Background_Color ();

      return Retval.getValue ();
   }

   private java.awt.Color
   getDisplayTextColor ()
   {
      final kotlin.Lazy<java.awt.Color> Retval = Properties.INSTANCE.getDisplay_Text_Color ();

      return Retval.getValue ();
   }

   private java.awt.Color getBackgroundDarkColor ()
   {
      final kotlin.Lazy<java.awt.Color> Retval = Properties.INSTANCE.getBackground_Dark_Color ();

      return Retval.getValue ();
   }

   /**
    * Creates new form Calculator_Canvas
    */
   public Calculator_Canvas ()
   {
      super ();
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "Calculator_Canvas");

      this.owner = null;
      this.initPre ();
      this.initComponents ();
      this.initPost ();

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "Calculator_Canvas");
   } // Calculator_Canvas

   /**
    * Creates new form Calculator_Canvas
    */
   public Calculator_Canvas (final @Nullable net.sourceforge.uiq3.ui.IFrame owner)
   {
      super ();
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "Calculator_Canvas");

      this.owner = owner;

      this.initPre ();
      this.initComponents ();
      this.initPost ();

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "Calculator_Canvas");
   } // Calculator_Canvas

   private void
   initPre ()
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "initPre");
      final kotlin.Lazy<Key[]> keyboard = net.sourceforge.uiq3.hp45.KeyboardKt.getKeyboard ();

      this.keyboard = keyboard.getValue ();

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "initPre");
      return;
   }

   private void
   initPost ()
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "initPost");

      this.calculateMinimumSize ();

      Properties.INSTANCE.addPropertyChangeListener (this::propertyChange);
      Button_0.addPropertyChangeListener (this::propertyChange);

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "initPost");
      return;
   } // initPost

   private void
   calculateMinimumSize ()
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "calculateMinimumSize");

      // Scroll canvas only works properly when preferred size equals minimum size
      //
      final java.awt.Dimension minimumSize = Button_0.getMinimumSize ();
      final java.awt.Dimension border = net.sourceforge.uiq3.calculator.Canvas.Companion.getBorder ();
      final java.awt.Dimension scaledMinimumSize = net.sourceforge.uiq3.calculator.Utils.resize (
	 minimumSize, SCALE_WIDTH, SCALE_HEIGHT, border);
      final java.awt.Dimension scaledMaximumSize = net.sourceforge.uiq3.calculator.Utils.resize (
	 scaledMinimumSize, Maximum_Size_Scale, Maximum_Size_Scale);

      this.setMinimumSize (scaledMinimumSize);
      this.setPreferredSize (scaledMinimumSize);
      this.setMaximumSize (scaledMaximumSize);

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "calculateMinimumSize");
      return;
   }

   private void
   propertyChange (final java.beans.PropertyChangeEvent evt)
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "propertyChange", evt);

      final String name = evt.getPropertyName ();
      final Object sender = evt.getSource ();

      if (java.util.Objects.equals (sender, this.Button_0))
      {
	 if (java.util.Objects.equals (name, "minimumSize"))
	 {
	    calculateMinimumSize ();
	 } // if
      }
      else if (java.util.Objects.equals (sender, Properties.INSTANCE))
      {
	 if (java.util.Objects.equals (name, Properties.DISPLAY_FONT_ID))
	 {
	    final java.awt.Font displayFont = Properties.getDisplay_Font ();

	    Display_Label.setFont (displayFont);
	 } // if
      } // if

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "propertyChange");
   }

   /**
    * Set the calculator engine used - needed for status related display.
    *
    * @param Engine
    *    calculator engine
    */
   public void
   setCalculator (final Desktop_Calculator Engine)
   {
      this.Engine = Engine;

      return;
   } // Set_Calculator

   /**
    * <p> display additional information on the error. this is device and setup dependent. </p>
    *
    * @param Exception
    *    Exception to display
    */
   @Override public void
   displayException (@NotNull final Throwable Exception)
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "displayException", new Object[] {Exception});

      if (owner != null)
      {
	 owner.displayException (Exception);
      }

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "displayException");
      return;
   }

   /**
    * <p> Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.) </p>
    *
    * @param Indicators
    *    indicators to be shown
    */
   @Override public void
   setIndicators (final boolean @NotNull [] Indicators)
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "Set_Indicators", Indicators);

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "Set_Indicators");
      return;
   }

   @SuppressWarnings ("ZeroLengthArrayAllocation")
   @TestOnly
   @Override public boolean @NotNull []
   getIndicators ()
   {
      return new boolean[0];
   }

   /**
    * <p> Set text to be displayed (calculator alpha codes). Null implementation for calculators
    * without alpha display. </p>
    *
    * @param Row
    *    Display row to set
    * @param Text
    *    Text to display
    */
   @Override public void
   setDisplayText (final int Row, final short @NotNull [] Text)
   {
      throw new UnsupportedOperationException (Constants.Not_Needed);
   }

   /**
    * Set text to be displayed.
    *
    * @param Text
    *    to be displayed
    */
   @Override public void
   setJavaText (@NotNull final String Text)
   {
      logger.entering (TAG, "Set_Text", Text);

      Display_Label.setText (Text);
      super.setJavaText (Text);

      logger.exiting (TAG, "Set_Text");
      return;
   } // Set_Text

   /**
    * Get text currently displayed (as one java string).
    *
    * @param Row
    *    Display row to set
    * @return text currently displayed
    */
   @TestOnly
   @Override
   public short @NotNull [] getDisplayText (final int Row)
   {
      throw new UnsupportedOperationException (Constants.Not_Needed);
   }

   /**
    * Get text currently displayed (as one java string).
    */
   @NotNull
   @TestOnly
   @Override
   public String getJavaText (final int Row)
   {
      return Display_Label.getText ();
   }

   /**
    * Get text currently displayed (as one java string).
    */
   @NotNull
   @TestOnly
   @Override
   public String getJavaText ()
   {
      return Display_Label.getText ();
   }

   /**
    * <p> set the calculators on of status. </p>
    *
    * @param on
    *    calculator is switched on
    */
   @Override
   public void setOnOff (final boolean on)
   {
      Calculator_Canvas.logger.entering (Calculator_Canvas.TAG, "Set_On_Off", on);

      Calculator_Canvas.logger.exiting (Calculator_Canvas.TAG, "Set_On_Off");
      return;
   }

   //@formatter:off
   @SuppressWarnings({"MagicNumber", "DanglingJavadoc"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {
      java.awt.GridBagConstraints gridBagConstraints;

      Display_Panel = new javax.swing.JPanel();
      Display_Label = new javax.swing.JLabel();
      Top_Button_Panel = new javax.swing.JPanel();
      Button_1x = new Calculator_Button_2(this, 0);
      Button_ln = new Calculator_Button_2(this, 1);
      Button_ex = new Calculator_Button_2(this, 2);
      Button_FIX = new Calculator_Button_2(this, 3);
      Button_GOLD = new Calculator_Button_2(this, 4);
      Button_x2 = new Calculator_Button_2(this, 5);
      Button_toP = new Calculator_Button_2(this, 6);
      Button_sin = new Calculator_Button_2(this, 7);
      Button_cos = new Calculator_Button_2(this, 8);
      Button_tan = new Calculator_Button_2(this, 9);
      Button_xy = new Calculator_Button_2(this, 10);
      Button_Rdown = new Calculator_Button_2(this, 11);
      Button_STO = new Calculator_Button_2(this, 12);
      Button_RCL = new Calculator_Button_2(this, 13);
      Button_percent = new Calculator_Button_2(this, 14);
      Button_ENTER = new Calculator_Button_2(this, 15);
      Button_CHS = new Calculator_Button_2(this, 16);
      Button_EEX = new Calculator_Button_2(this, 17);
      Button_CLx = new Calculator_Button_2(this, 18);
      Bottom_Button_Panel = new javax.swing.JPanel();
      Button_Minus = new Calculator_Button_2(this, 19);
      Button_7 = new Calculator_Button_2(this, 20);
      Button_8 = new Calculator_Button_2(this, 21);
      Button_9 = new Calculator_Button_2(this, 22);
      Button_Plus = new Calculator_Button_2(this, 23);
      Button_4 = new Calculator_Button_2(this, 24);
      Button_5 = new Calculator_Button_2(this, 25);
      Button_6 = new Calculator_Button_2(this, 26);
      Button_Mult = new Calculator_Button_2(this, 27);
      Button_1 = new Calculator_Button_2(this, 28);
      Button_2 = new Calculator_Button_2(this, 29);
      Button_3 = new Calculator_Button_2(this, 30);
      Button_Div = new Calculator_Button_2(this, 31);
      Button_0 = new Calculator_Button_2(this, 32);
      Button_Dot = new Calculator_Button_2(this, 33);
      Button_Σplus = new Calculator_Button_2(this, 34);

      setBackground(getBackgroundDarkColor());
      setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
      setLayout(new java.awt.GridBagLayout());

      Display_Panel.setBackground(getDisplayBackgroundColor());
      Display_Panel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
      Display_Panel.setForeground(getDisplayTextColor());
      Display_Panel.setAutoscrolls(true);
      Display_Panel.setName("Display_Panel"); // NOI18N
      Display_Panel.setLayout(new java.awt.GridLayout(1, 1));

      Display_Label.setBackground(getDisplayBackgroundColor());
      Display_Label.setFont(getMonoFont());
      Display_Label.setForeground(getDisplayTextColor());
      Display_Label.setText("-1.234567890 -99");
      Display_Label.setBorder(javax.swing.BorderFactory.createEmptyBorder(8, 8, 8, 8));
      Display_Label.setName("Display_Label"); // NOI18N
      Display_Panel.add(Display_Label);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
      gridBagConstraints.ipadx = 8;
      gridBagConstraints.ipady = 8;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(6, 6, 0, 6);
      add(Display_Panel, gridBagConstraints);

      Top_Button_Panel.setBackground(getBackgroundDarkColor());
      Top_Button_Panel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
      Top_Button_Panel.setName("Top_Button_Panel"); // NOI18N
      Top_Button_Panel.setLayout(new java.awt.GridBagLayout());

      Button_1x.setName("Button_1x"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_1x, gridBagConstraints);

      Button_ln.setName("Button_ln"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_ln, gridBagConstraints);

      Button_ex.setName("Button_ex"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_ex, gridBagConstraints);

      Button_FIX.setName("Button_FIX"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_FIX, gridBagConstraints);

      Button_GOLD.setName("Button_GOLD"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 4;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_GOLD, gridBagConstraints);

      Button_x2.setName("Button_x2"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_x2, gridBagConstraints);

      Button_toP.setName("Button_toP"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_toP, gridBagConstraints);

      Button_sin.setName("Button_sin"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_sin, gridBagConstraints);

      Button_cos.setName("Button_cos"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_cos, gridBagConstraints);

      Button_tan.setName("Button_tan"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 4;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_tan, gridBagConstraints);

      Button_xy.setName("Button_xy"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_xy, gridBagConstraints);

      Button_Rdown.setName("Button_Rdown"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_Rdown, gridBagConstraints);

      Button_STO.setName("Button_STO"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_STO, gridBagConstraints);

      Button_RCL.setName("Button_RCL"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_RCL, gridBagConstraints);

      Button_percent.setName("Button_percent"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 4;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_percent, gridBagConstraints);

      Button_ENTER.setName("Button_ENTER"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 4;
      gridBagConstraints.gridwidth = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 2.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_ENTER, gridBagConstraints);

      Button_CHS.setName("Button_CHS"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_CHS, gridBagConstraints);

      Button_EEX.setName("Button_EEX"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_EEX, gridBagConstraints);

      Button_CLx.setName("Button_CLx"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 4;
      gridBagConstraints.gridy = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Top_Button_Panel.add(Button_CLx, gridBagConstraints);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.ipadx = 8;
      gridBagConstraints.ipady = 8;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 0, 8);
      add(Top_Button_Panel, gridBagConstraints);

      Bottom_Button_Panel.setBackground(getBackgroundDarkColor());
      Bottom_Button_Panel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
      Bottom_Button_Panel.setName("Bottom_Button_Panel"); // NOI18N
      Bottom_Button_Panel.setLayout(new java.awt.GridBagLayout());

      Button_Minus.setName("Button_Minus"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 0.7;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Minus, gridBagConstraints);

      Button_7.setName("Button_7"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_7, gridBagConstraints);

      Button_8.setName("Button_8"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_8, gridBagConstraints);

      Button_9.setName("Button_9"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_9, gridBagConstraints);

      Button_Plus.setName("Button_Plus"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 0.7;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Plus, gridBagConstraints);

      Button_4.setName("Button_4"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_4, gridBagConstraints);

      Button_5.setName("Button_5"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_5, gridBagConstraints);

      Button_6.setName("Button_6"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_6, gridBagConstraints);

      Button_Mult.setName("Button_Mult"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 0.7;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Mult, gridBagConstraints);

      Button_1.setName("Button_1"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_1, gridBagConstraints);

      Button_2.setName("Button_2"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_2, gridBagConstraints);

      Button_3.setName("Button_3"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_3, gridBagConstraints);

      Button_Div.setName("Button_Div"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 0.7;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Div, gridBagConstraints);

      Button_0.setName("Button_0"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 1;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_0, gridBagConstraints);

      Button_Dot.setName("Button_Dot"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 2;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Dot, gridBagConstraints);

      Button_Σplus.setName("Button_Σplus"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 3;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.1;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
      Bottom_Button_Panel.add(Button_Σplus, gridBagConstraints);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.ipadx = 8;
      gridBagConstraints.ipady = 8;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 1.0;
      gridBagConstraints.insets = new java.awt.Insets(0, 8, 8, 8);
      add(Bottom_Button_Panel, gridBagConstraints);
   }// </editor-fold>//GEN-END:initComponents

   // Variables declaration - do not modify//GEN-BEGIN:variables
   javax.swing.JPanel Bottom_Button_Panel;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_0;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_1;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_1x;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_2;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_3;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_4;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_5;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_6;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_7;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_8;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_9;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_CHS;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_CLx;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Div;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Dot;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_EEX;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_ENTER;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_FIX;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_GOLD;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Minus;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Mult;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Plus;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_RCL;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Rdown;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_STO;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_cos;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_ex;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_ln;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_percent;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_sin;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_tan;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_toP;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_x2;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_xy;
   net.sourceforge.uiq3.hp45.Calculator_Button_2 Button_Σplus;
   javax.swing.JLabel Display_Label;
   javax.swing.JPanel Display_Panel;
   javax.swing.JPanel Top_Button_Panel;
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @TestOnly 
   @Override public @NotNull String
   toString ()
   {
      return "Calculator_Canvas{" +
	 "Engine=" + Engine +
	 ", Bottom_Button_Panel=" + Bottom_Button_Panel +
	 ", Button_0=" + Button_0 +
	 ", Button_1=" + Button_1 +
	 ", Button_1x=" + Button_1x +
	 ", Button_2=" + Button_2 +
	 ", Button_3=" + Button_3 +
	 ", Button_4=" + Button_4 +
	 ", Button_5=" + Button_5 +
	 ", Button_6=" + Button_6 +
	 ", Button_7=" + Button_7 +
	 ", Button_8=" + Button_8 +
	 ", Button_9=" + Button_9 +
	 ", Button_CHS=" + Button_CHS +
	 ", Button_CLx=" + Button_CLx +
	 ", Button_Div=" + Button_Div +
	 ", Button_Dot=" + Button_Dot +
	 ", Button_EEX=" + Button_EEX +
	 ", Button_ENTER=" + Button_ENTER +
	 ", Button_FIX=" + Button_FIX +
	 ", Button_GOLD=" + Button_GOLD +
	 ", Button_Minus=" + Button_Minus +
	 ", Button_Mult=" + Button_Mult +
	 ", Button_Plus=" + Button_Plus +
	 ", Button_RCL=" + Button_RCL +
	 ", Button_Rdown=" + Button_Rdown +
	 ", Button_STO=" + Button_STO +
	 ", Button_cos=" + Button_cos +
	 ", Button_ex=" + Button_ex +
	 ", Button_ln=" + Button_ln +
	 ", Button_percent=" + Button_percent +
	 ", Button_sin=" + Button_sin +
	 ", Button_tan=" + Button_tan +
	 ", Button_toP=" + Button_toP +
	 ", Button_x2=" + Button_x2 +
	 ", Button_xy=" + Button_xy +
	 ", Button_Σplus=" + Button_Σplus +
	 ", Display_Label=" + Display_Label +
	 ", Display_Panel=" + Display_Panel +
	 ", Top_Button_Panel=" + Top_Button_Panel +
	 '}';
   }
} // Calculator_Canvas

// vim: set textwidth=120 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=java fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
