/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

import net.sourceforge.uiq3.Asserted;

/**
 * @author martin
 */
public final class Calculator_Frame
   extends net.sourceforge.uiq3.calculator.Frame<Calculator_Canvas>
   implements
   java.awt.datatransfer.ClipboardOwner,
   java.io.Serializable
{
   /**
    *
    */
   private static final long serialVersionUID = -5289493567741081115L;
   /**
    * class logger instance
    */
   @NotNull
   private static final java.util.logging.Logger logger;
   /**
    * class logger tag.
    */
   @NotNull
   private static final String TAG;
   /**
    * Resource Bundle used for I18N
    */
   private static final java.util.ResourceBundle HP45_Bundle;
   private static final java.util.ResourceBundle Calculator_Bundle; // NOI18
   @Nullable
   private final net.sourceforge.uiq3.calculator.IMain owner;

   static
   {
      final kotlin.Lazy<java.util.ResourceBundle> resources = Properties.INSTANCE.getResources ();

      TAG = Calculator_Frame.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
      //noinspection HardcodedFileSeparator
      Calculator_Bundle = java.util.ResourceBundle.getBundle ("net/sourceforge/uiq3/calculator/Bundle"); // NOI18N
      HP45_Bundle = resources.getValue ();
   }

   /**
    * Calculator to be displayed
    */
   private Desktop_Calculator desktopCalculator;
   /**
    * About Dialog
    */
   @Nullable
   private net.sourceforge.uiq3.calculator.About_Dialog aboutDialog;
   /**
    * Preferences Dialog
    */
   @Nullable
   private Preferences_Dialog preferencesDialog;
   /**
    * Help Dialog
    */
   @Nullable
   private net.sourceforge.uiq3.calculator.Help_Dialog helpDialog;

   /**
    * Creates new form Calculator_Frame
    */
   public Calculator_Frame ()
   {
      super ();
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "Calculator_Frame");

      this.owner = null;
      this.initPre ();
      this.initComponents ();
      this.initPost ();

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "Calculator_Frame");
      return;
   }

   /**
    * Creates new form Calculator_Frame
    *
    * @param owner
    *    Instance of main class
    */
   public Calculator_Frame (@NotNull final net.sourceforge.uiq3.calculator.IMain owner)
   {
      super ();
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "Calculator_Frame", owner);

      this.owner = owner;
      this.initPre ();
      this.initComponents ();
      this.initPost ();

      final kotlin.Lazy<Boolean> macOS = com.krischik.ui.MacApplication.getMacOS ();
      if (macOS.getValue ())
      {
//         this.Menu_File.remove (1);
//         this.Menu_File.remove (this.Menu_File_Quit);
//         this.Menu_File.remove (this.Menu_File_1);
//         this.Menu_File.remove (this.Menu_File_Preferences);
         this.Menu_File.remove (this.Menu_Help_2);
         this.Menu_Help.remove (this.Menu_Help_About);
      } // if

      this.Split_Pane.setDividerLocation (this.Calculator_Canvas.getPreferredSize ().width);

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "Calculator_Frame");
      return;
   } // Calculator_Frame

   private void
   initPre ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "initPre");

      this.aboutDialog = null;
      this.helpDialog = null;
      this.preferencesDialog = null;

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "initPre");
      return;
   } // initPre

   @SuppressWarnings ("HardcodedFileSeparator")
   private void
   initPost ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "initPost");

      this.calculateMinimumSize ();
      this.Calculator_Canvas.addPropertyChangeListener (this::propertyChange);
      Main.Companion.addPropertyChangeListener (this::propertyChange);

      net.sourceforge.uiq3.OS.setWindowIcon (this, "ic_launcher_hp45.png");
      com.krischik.ui.MacApplication.setApplicationIcon ("/net/sourceforge/uiq3/hp45/ic_launcher_hp45.png");

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "initPost");
      return;
   }

   @SuppressWarnings ("FeatureEnvy")
   private void
   calculateMinimumSize ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "calculateMinimumSize");

      final java.awt.Dimension calculatorMinimumSize = Calculator_Canvas.getMinimumSize ();
      final java.awt.Dimension calculatorPreferredSize = Calculator_Canvas.getPreferredSize ();
      final java.awt.Dimension calculatorMaximumSize = Calculator_Canvas.getMaximumSize ();
      final java.awt.Dimension panelMinimumSize = Info_TabbedPane.getMinimumSize ();
      final java.awt.Dimension panelPreferredSize = Info_TabbedPane.getPreferredSize ();
      final java.awt.Dimension panelMaximumSize = Info_TabbedPane.getMaximumSize ();

      this.setMinimumSize (net.sourceforge.uiq3.calculator.Utils.addWidth (
         calculatorMinimumSize,
         panelMinimumSize,
         getBorder ()));
      this.setPreferredSize (net.sourceforge.uiq3.calculator.Utils.addWidth (
         calculatorPreferredSize,
         panelPreferredSize,
         getBorder ()));
      this.setMaximumSize (net.sourceforge.uiq3.calculator.Utils.addWidth (
         calculatorMaximumSize,
         panelMaximumSize,
         getBorder ()));

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "calculateMinimumSize");
      return;
   }

   private void propertyChange (final java.beans.PropertyChangeEvent evt)
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "propertyChange", evt);

      final String name = evt.getPropertyName ();
      final Object sender = evt.getSource ();

      if (java.util.Objects.equals (sender, Main.Companion))
      {
         if (name == Main.LOOK_AND_FEEL_CHANGED)
         {
            javax.swing.SwingUtilities.updateComponentTreeUI (this);
            this.pack ();
         }
      }
      else if (java.util.Objects.equals (sender, this.Calculator_Canvas))
      {
         if (name.equals ("maximumSize") || name.equals ("minimumSize"))
         {
            calculateMinimumSize ();
         } // if
      }
      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "propertyChange");
   }

   /**
    * Save Window Status
    */
   public void
   saveStatus ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "saveStatus");

      Properties.INSTANCE.saveWindowStatus (this);
      Properties.INSTANCE.saveSplitStatus (this.Split_Pane);

      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "saveStatus");
      return;
   } // saveStatus

   /**
    * Save Window Status
    */
   public void
   restoreStatus ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "restoreStatus");

      Properties.INSTANCE.restoreWindowStatus (this);
      Properties.INSTANCE.restoreSplitStatus (this.Split_Pane);

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "restoreStatus");
      return;
   } // restoreStatus

   /**
    * <p> Display about box. </p>
    */
   @Override public void
   displayAboutBox ()
   {
      javax.swing.SwingUtilities.invokeLater (
         () ->
         {
            if (this.aboutDialog == null)
            {
               final Class<? extends Calculator_Frame> selfClass = getClass ();
               final java.net.URL iconResource = selfClass.getResource ("ic_launcher_hp45.png");

               Asserted.notNull (iconResource, "iconResource");

               this.aboutDialog = new net.sourceforge.uiq3.calculator.About_Dialog (
                  /* parent       = */ this,
                  /* properties   = */ Properties.INSTANCE,
                  /* title        = */ HP45_Bundle.getString ("About_Title"),
                  /* programName  = */ HP45_Bundle.getString ("About_Label_Title"),
                  /* icon         = */ new javax.swing.ImageIcon (iconResource),
                  /* Homepage     = */ Constants.Homepage,
                  /* Manuals      = */ Constants.Manuals,
                  /* Sourceforge  = */ Constants.Bug_Report,
                  /* YouTube      = */ Constants.YouTube_Videos);
            } // if

            this.aboutDialog.Initialize ();
            this.aboutDialog.setIconImages (this.getIconImages ());
            this.aboutDialog.setVisible (true);
         });

      return;
   } // Display_About_Box

   /**
    * <p> Display about box. </p>
    */
   @Override
   public void displayHelpIndex ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "displayHelpIndex");

      javax.swing.SwingUtilities.invokeLater (
         () ->
         {
            if (this.helpDialog == null)
            {
               this.helpDialog = new net.sourceforge.uiq3.calculator.Help_Dialog (
                  /* parent       = */ this,
                  /* properties   = */ Properties.INSTANCE);
            } // if

            this.helpDialog.Initialize ();
            this.helpDialog.setIconImages (this.getIconImages ());
            this.helpDialog.setVisible (true);
         });

      return;
   } // displayHelpIndex

   /**
    * <p> display preferences </p>
    */
   @Override public void
   displayPreferencesBox ()
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "displayPreferencesBox");

      javax.swing.SwingUtilities.invokeLater (
         () ->
         {
            if (this.preferencesDialog == null)
            {
               this.preferencesDialog = new Preferences_Dialog (
                  /* parent = */this,
                  /* modal  = */true);
            } // if

            this.preferencesDialog.Initialize (this.desktopCalculator);
            this.preferencesDialog.setIconImages (this.getIconImages ());
            this.preferencesDialog.setVisible (true);
         });

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "displayPreferencesBox");
      return;
   } // displayPreferencesBox
   /**
    * Get Calculator Canvas - interface needed by calculator engine.
    *
    * @return calculator canvas
    */
   @Override public Calculator_Canvas
   getCalculatorCanvas ()
   {
      return this.Calculator_Canvas;
   } // Get_Canvas

   /**
    * Set the calculator engine used - needed for status related display.
    *
    * @param Engine
    *    calculator engine
    */
   public void
   setCalculator (final Desktop_Calculator Engine)
   {
      this.desktopCalculator = Engine;
      this.Calculator_Canvas.setCalculator (Engine);
      this.Printer_Canvas.setCalculator (Engine);
      this.desktopCalculator.setForm (this);
      this.Register_Panel.setCalculator (Engine);

      return;
   } // Set_Calculator

   /**
    * Notifies this object that it is no longer the clipboard owner. This method will be called when another application
    * or another object within this application asserts ownership of the clipboard.
    *
    * @param clipboard
    *    the clipboard that is no longer owned
    * @param contents
    *    the contents which this owner had placed on the clipboard
    * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(java.awt.datatransfer.Clipboard,
    * java.awt.datatransfer.Transferable)
    */
   @Contract (pure = true)
   @Override public void
   lostOwnership (
      final java.awt.datatransfer.Clipboard clipboard,
      final java.awt.datatransfer.Transferable contents)
   {
      //do nothing
      return;
   } // lostOwnership

   @Override public void
   bindCPUThread (final @Nullable Thread CPU_Thread)
   {
      this.Calculator_Canvas.bindCPUThread (CPU_Thread);
   }

   /**
    * <p> display additional information on the error. this is device and setup dependent. </p>
    *
    * @param exception
    *    exception to display
    */
   @Override public void
   displayException (final @NotNull Throwable exception)
   {
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "displayException", new Object[] {exception});

      Error_Canvas.displayException (exception);
      Info_TabbedPane.setSelectedComponent (Error_Canvas);

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "displayException");
      return;
   }

   @Override public short
   getPressedKey ()
   {
      return this.Calculator_Canvas.getPressedKey ();
   }

   @Override public short
   getReadKey ()
   {
      return this.Calculator_Canvas.getReadKey ();
   }

   @Override public boolean
   isReadKeyAvailable ()
   {
      return this.Calculator_Canvas.isReadKeyAvailable ();
   }

   @Override public void
   setIndicators (final boolean @NotNull [] Indicators)
   {
      this.Calculator_Canvas.setIndicators (Indicators);
   }

   @TestOnly @Override public boolean @NotNull []
   getIndicators ()
   {
      return this.Calculator_Canvas.getIndicators ();
   }

   @Override public void
   setOnOff (final boolean on)
   {
      this.Calculator_Canvas.setOnOff (on);
   }

   @Override public void
   setDisplayText (final int Row, final short @NotNull [] Text)
   {
      this.Calculator_Canvas.setDisplayText (Row, Text);
   }

   @Override public void
   setJavaText (final @NotNull String Text)
   {
      this.Calculator_Canvas.setJavaText (Text);
   }

   /**
    * Fet text currently displayed (as one java string).
    *
    * @param Row
    *    Display row to set
    * @return text currently displayed
    */
   @TestOnly @Override public short @NotNull []
   getDisplayText (final int Row)
   {
      return this.Calculator_Canvas.getDisplayText (Row);
   }

   @TestOnly @Override @NotNull public String
   getJavaText (final int Row)
   {
      return this.Calculator_Canvas.getJavaText (Row);
   }

   /**
    * Fet text currently displayed (as one java string).
    */
   @TestOnly @Override @NotNull public String
   getJavaText ()
   {
      return Calculator_Canvas.getJavaText ();
   }

   @Override public void
   toggleHelp ()
   {
      this.Calculator_Canvas.toggleHelp ();
   }

   /**
    * Sets the selected component for this tabbedpane.  This will automatically set the <code>selectedIndex</code> to
    * the index corresponding to the specified component.
    *
    * @param c
    *    the selected {@code Component} for this {@code TabbedPane}
    * @throws IllegalArgumentException
    *    if component not found in tabbed pane
    */
   @java.beans.BeanProperty (
      bound = false,
      preferred = true,
      description = "The tabbedpane's selected component.")
   @Override public void
   setSelectedComponent (final java.awt.Component c)
   {
      Info_TabbedPane.setSelectedComponent (c);
   }

   //@formatter:off
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {

      setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
      setTitle(HP45_Bundle.getString("Calculator_Title")); // NOI18N
      setName("Calculator_Frame"); // NOI18N
      addWindowListener(new java.awt.event.WindowAdapter()
      {
         public void windowClosing(java.awt.event.WindowEvent evt)
         {
            formWindowClosing(evt);
         }
      });

      Split_Pane.setDividerLocation(544);
      Split_Pane.setResizeWeight(0.25);
      Split_Pane.setToolTipText("");
      Split_Pane.setName("Split_Pane"); // NOI18N

      Calculator_ScrollPane.setName("Calculator_ScrollPane"); // NOI18N

      Calculator_Canvas.setName("Calculator_Canvas"); // NOI18N
      Calculator_ScrollPane.setViewportView(Calculator_Canvas);

      Split_Pane.setLeftComponent(Calculator_ScrollPane);

      Info_TabbedPane.setName("Info_TabbedPane"); // NOI18N

      Printer_Canvas.setName("Printer_Canvas"); // NOI18N
      Info_TabbedPane.addTab(Calculator_Bundle.getString("Printer"), new javax.swing.ImageIcon(getClass().getResource("/net/sourceforge/uiq3/hp45/ic_launcher_printer.png")), Printer_Canvas); // NOI18N
      Info_TabbedPane.addTab(HP45_Bundle.getString("Register_Tab"), Register_Panel); // NOI18N

      Error_Canvas.setName("Error_Canvas"); // NOI18N
      Info_TabbedPane.addTab(Calculator_Bundle.getString("Errors_Tab"), Error_Canvas); // NOI18N

      Split_Pane.setRightComponent(Info_TabbedPane);

      getContentPane().add(Split_Pane, java.awt.BorderLayout.CENTER);

      Menu_Bar.setName("Menu_Bar"); // NOI18N

      Menu_File.setText(HP45_Bundle.getString("Menu_File")); // NOI18N
      Menu_File.setName("Menu_File"); // NOI18N

      Menu_File_Preferences.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_COMMA, java.awt.event.InputEvent.META_DOWN_MASK));
      Menu_File_Preferences.setText(HP45_Bundle.getString("Menu_File_Preferences")); // NOI18N
      Menu_File_Preferences.setActionCommand("Menu_File_Preferences");
      Menu_File_Preferences.setName("Menu_File_Preferences"); // NOI18N
      Menu_File_Preferences.addActionListener(this::Menu_File_PreferencesActionPerformed);
      Menu_File.add(Menu_File_Preferences);

      Menu_File_1.setName("Menu_File_1"); // NOI18N
      Menu_File.add(Menu_File_1);

      Menu_File_Quit.setText(HP45_Bundle.getString("Menu_File_Quit")); // NOI18N
      Menu_File_Quit.setActionCommand("Menu_File_Quit");
      Menu_File_Quit.setName("Menu_File_Quit"); // NOI18N
      Menu_File_Quit.addActionListener(this::Menu_File_QuitActionPerformed);
      Menu_File.add(Menu_File_Quit);

      Menu_Bar.add(Menu_File);

      Menu_Edit.setText(HP45_Bundle.getString("Menu_Edit")); // NOI18N
      Menu_Edit.setName("Menu_Edit"); // NOI18N

      Menu_Edit_Copy.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.META_DOWN_MASK));
      Menu_Edit_Copy.setText(HP45_Bundle.getString("Menu_Edit_Copy")); // NOI18N
      Menu_Edit_Copy.setActionCommand("Menu_Edit_Copy");
      Menu_Edit_Copy.setName("Menu_Edit_Copy"); // NOI18N
      Menu_Edit.add(Menu_Edit_Copy);

      Menu_Bar.add(Menu_Edit);

      Menu_Help.setText(HP45_Bundle.getString("Menu_Help")); // NOI18N
      Menu_Help.setName("Menu_Help"); // NOI18N

      Menu_Help_Index.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
      Menu_Help_Index.setText(HP45_Bundle.getString("Menu_Help_Index")); // NOI18N
      Menu_Help_Index.setActionCommand("Menu_Help_Index");
      Menu_Help_Index.setName("Menu_Help_Index"); // NOI18N
      Menu_Help_Index.addActionListener(this::Menu_Help_IndexActionPerformed);
      Menu_Help.add(Menu_Help_Index);

      Menu_Help_1.setName("Menu_Help_1"); // NOI18N
      Menu_Help.add(Menu_Help_1);

      Menu_Help_Homepage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/sourceforge/uiq3/calculator/globe.png"))); // NOI18N
      Menu_Help_Homepage.setText(HP45_Bundle.getString("Menu_Help_Homepage")); // NOI18N
      Menu_Help_Homepage.setActionCommand("Menu_Help_Homepage");
      Menu_Help_Homepage.setName("Menu_Help_Homepage"); // NOI18N
      Menu_Help_Homepage.addActionListener(this::Menu_Help_HomepageActionPerformed);
      Menu_Help.add(Menu_Help_Homepage);

      Menu_Help_Manuals.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/sourceforge/uiq3/calculator/manual.png"))); // NOI18N
      Menu_Help_Manuals.setText(HP45_Bundle.getString("Menu_Help_Manuals")); // NOI18N
      Menu_Help_Manuals.setActionCommand("Menu_Help_Manuals");
      Menu_Help_Manuals.setName("Menu_Help_Manuals"); // NOI18N
      Menu_Help_Manuals.addActionListener(this::Menu_Help_ManualsActionPerformed);
      Menu_Help.add(Menu_Help_Manuals);

      Menu_Help_YouTube.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/sourceforge/uiq3/calculator/youtube.png"))); // NOI18N
      Menu_Help_YouTube.setText(HP45_Bundle.getString("Menu_Help_YouTube")); // NOI18N
      Menu_Help_YouTube.setActionCommand("Menu_Help_YouTube");
      Menu_Help_YouTube.setName("Menu_Help_YouTube"); // NOI18N
      Menu_Help_YouTube.addActionListener(this::Menu_Help_YouTubeActionPerformed);
      Menu_Help.add(Menu_Help_YouTube);

      Menu_Help_SourceForge.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/sourceforge/uiq3/calculator/sourceforge.png"))); // NOI18N
      Menu_Help_SourceForge.setText(HP45_Bundle.getString("Menu_Help_SourceForge")); // NOI18N
      Menu_Help_SourceForge.setActionCommand("Menu_Help_SourceForge");
      Menu_Help_SourceForge.setName("Menu_Help_SourceForge"); // NOI18N
      Menu_Help_SourceForge.addActionListener(this::Menu_Help_SourceForgeActionPerformed);
      Menu_Help.add(Menu_Help_SourceForge);

      Menu_Help_2.setName("Menu_Help_2"); // NOI18N
      Menu_Help.add(Menu_Help_2);

      Menu_Help_About.setText(HP45_Bundle.getString("Menu_About")); // NOI18N
      Menu_Help_About.setActionCommand("Menu_Help_About");
      Menu_Help_About.setName("Menu_Help_About"); // NOI18N
      Menu_Help_About.addActionListener(this::Menu_Help_AboutActionPerformed);
      Menu_Help.add(Menu_Help_About);

      Menu_Bar.add(Menu_Help);

      setJMenuBar(Menu_Bar);

      pack();
   }// </editor-fold>//GEN-END:initComponents

   private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
   {//GEN-HEADEREND:event_formWindowClosing
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "formWindowClosing", evt);

      if (this.owner != null)
      {
         this.owner.quit ();
      }

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "formWindowClosing");
      return;
   }//GEN-LAST:event_formWindowClosing

   private void Menu_File_PreferencesActionPerformed(java.awt.event.ActionEvent evt)
   {
      logger.entering (TAG, "Menu_File_PreferencesActionPerformed", evt);

      this.displayPreferencesBox();

      logger.exiting (TAG, "Menu_File_PreferencesActionPerformed");
      return;
   }

   private void Menu_Help_AboutActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_AboutActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_AboutActionPerformed
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "Menu_Help_AboutActionPerformed");

      this.displayAboutBox ();

      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "Menu_Help_AboutActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_AboutActionPerformed


   private void Menu_Help_HomepageActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_HomepageActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_HomepageActionPerformed
      logger.entering (TAG, "Menu_Help_SourceForgeActionPerformed", evt);

      net.sourceforge.uiq3.OS.openPage (net.sourceforge.uiq3.hp45.Constants.Homepage);

      logger.exiting (TAG, "Menu_Help_SourceForgeActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_HomepageActionPerformed

   private void Menu_Help_SourceForgeActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_SourceForgeActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_SourceForgeActionPerformed
      logger.entering (TAG, "Menu_Help_SourceForgeActionPerformed", evt);

      net.sourceforge.uiq3.OS.openPage (net.sourceforge.uiq3.hp45.Constants.SourceForge_Project);

      logger.exiting (TAG, "Menu_Help_SourceForgeActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_SourceForgeActionPerformed

   private void Menu_Help_YouTubeActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_YouTubeActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_YouTubeActionPerformed
      logger.entering (TAG, "Menu_Help_SourceForgeActionPerformed", evt);

      net.sourceforge.uiq3.OS.openPage (net.sourceforge.uiq3.hp45.Constants.YouTube_Videos);

      logger.exiting (TAG, "Menu_Help_SourceForgeActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_YouTubeActionPerformed

   private void Menu_Help_ManualsActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_ManualsActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_ManualsActionPerformed
      logger.entering (TAG, "Menu_Help_ManualsActionPerformed", evt);

      net.sourceforge.uiq3.OS.openPage (net.sourceforge.uiq3.hp45.Constants.Manuals);

      logger.exiting (TAG, "Menu_Help_ManualsActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_ManualsActionPerformed

   private void Menu_File_QuitActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_File_QuitActionPerformed
   {//GEN-HEADEREND:event_Menu_File_QuitActionPerformed
      logger.entering (TAG, "Menu_File_QuitActionPerformed", evt);

      if (this.owner != null)
      {
         this.owner.quit ();
      }
      else
      {
         // FALLBACK: quit will also close save the status and dispose the window.

         this.saveStatus ();
         this.dispose ();
      }

      logger.exiting (TAG, "Menu_File_QuitActionPerformed");
      return;
   }//GEN-LAST:event_Menu_File_QuitActionPerformed

   private void Menu_Help_IndexActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_Menu_Help_IndexActionPerformed
   {//GEN-HEADEREND:event_Menu_Help_IndexActionPerformed
      Calculator_Frame.logger.entering (Calculator_Frame.TAG, "Menu_Help_IndexActionPerformed", evt);

      this.displayHelpIndex ();

      Calculator_Frame.logger.exiting (Calculator_Frame.TAG, "Menu_Help_IndexActionPerformed");
      return;
   }//GEN-LAST:event_Menu_Help_IndexActionPerformed

   // Variables declaration - do not modify//GEN-BEGIN:variables
   final net.sourceforge.uiq3.hp45.Calculator_Canvas Calculator_Canvas = new net.sourceforge.uiq3.hp45.Calculator_Canvas (this);
   final javax.swing.JScrollPane Calculator_ScrollPane = new javax.swing.JScrollPane();
   final net.sourceforge.uiq3.calculator.Error_Canvas Error_Canvas = new net.sourceforge.uiq3.calculator.Error_Canvas();
   final javax.swing.JTabbedPane Info_TabbedPane = new javax.swing.JTabbedPane();
   final javax.swing.JMenuBar Menu_Bar = new javax.swing.JMenuBar();
   final javax.swing.JMenu Menu_Edit = new javax.swing.JMenu();
   final javax.swing.JMenuItem Menu_Edit_Copy = new javax.swing.JMenuItem();
   final javax.swing.JMenu Menu_File = new javax.swing.JMenu();
   final javax.swing.JPopupMenu.Separator Menu_File_1 = new javax.swing.JPopupMenu.Separator();
   final javax.swing.JMenuItem Menu_File_Preferences = new javax.swing.JMenuItem();
   final javax.swing.JMenuItem Menu_File_Quit = new javax.swing.JMenuItem();
   final javax.swing.JMenu Menu_Help = new javax.swing.JMenu();
   final javax.swing.JPopupMenu.Separator Menu_Help_1 = new javax.swing.JPopupMenu.Separator();
   final javax.swing.JPopupMenu.Separator Menu_Help_2 = new javax.swing.JPopupMenu.Separator();
   final javax.swing.JMenuItem Menu_Help_About = new javax.swing.JMenuItem();
   private final javax.swing.JMenuItem Menu_Help_Homepage = new javax.swing.JMenuItem();
   final javax.swing.JMenuItem Menu_Help_Index = new javax.swing.JMenuItem();
   private final javax.swing.JMenuItem Menu_Help_Manuals = new javax.swing.JMenuItem();
   final javax.swing.JMenuItem Menu_Help_SourceForge = new javax.swing.JMenuItem();
   final javax.swing.JMenuItem Menu_Help_YouTube = new javax.swing.JMenuItem();
   final net.sourceforge.uiq3.hp45.PrinterCanvas Printer_Canvas = new net.sourceforge.uiq3.hp45.PrinterCanvas(this);
   final net.sourceforge.uiq3.hp45.Register_Canvas Register_Panel = new net.sourceforge.uiq3.hp45.Register_Canvas();
   final javax.swing.JSplitPane Split_Pane = new javax.swing.JSplitPane();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @Override public String toString ()
   {
      return "Calculator_Frame{" +
         "Engine=" + desktopCalculator +
         ", aboutDialog=" + aboutDialog +
         ", preferencesDialog=" + preferencesDialog +
         ", helpDialog=" + helpDialog +
         ", Calculator_Canvas=" + Calculator_Canvas +
         ", Calculator_ScrollPane=" + Calculator_ScrollPane +
         ", Error_Canvas=" + Error_Canvas +
         ", Info_TabbedPane=" + Info_TabbedPane +
         ", Menu_Bar=" + Menu_Bar +
         ", Menu_Edit=" + Menu_Edit +
         ", Menu_Edit_Copy=" + Menu_Edit_Copy +
         ", Menu_File=" + Menu_File +
         ", Menu_File_1=" + Menu_File_1 +
         ", Menu_File_Preferences=" + Menu_File_Preferences +
         ", Menu_File_Quit=" + Menu_File_Quit +
         ", Menu_Help=" + Menu_Help +
         ", Menu_Help_1=" + Menu_Help_1 +
         ", Menu_Help_2=" + Menu_Help_2 +
         ", Menu_Help_About=" + Menu_Help_About +
         ", Menu_Help_Homepage=" + Menu_Help_Homepage +
         ", Menu_Help_Index=" + Menu_Help_Index +
         ", Menu_Help_Manuals=" + Menu_Help_Manuals +
         ", Menu_Help_SourceForge=" + Menu_Help_SourceForge +
         ", Menu_Help_YouTube=" + Menu_Help_YouTube +
         ", Printer_Canvas=" + Printer_Canvas +
         ", Split_Pane=" + Split_Pane +
         '}';
   }
}
