/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * Preferences_Dialog.java
 * <p>
 * Created on 08.08.2010, 10:17:30
 *
 * @author martin
 */
public final class Preferences_Dialog
   extends javax.swing.JDialog
{

   /**
    */
   private static final long serialVersionUID = 2813324100586206113L;

   /**
    * <p> class logger instance </p>
    */
   private static final java.util.logging.Logger logger;

   /**
    * <p> class logger tag. </p>
    */
   private static final String TAG;

   /**
    * Resource Bundle used for I18N
    */
   private static final java.util.ResourceBundle HP45_Bundle;

   static
   {
      final kotlin.Lazy<java.util.ResourceBundle> resources = Properties.INSTANCE.getResources ();

      TAG = Preferences_Dialog.class.getName ();
      logger = java.util.logging.Logger.getLogger (Preferences_Dialog.class.getName ());
      HP45_Bundle = resources.getValue ();
   }

   /**
    * Calculator whose setting we handle
    */
   @Nullable
   private Desktop_Calculator Engine = null;

   /**
    * Creates new form Preferences_Dialog
    *
    * @param parent
    * @param modal
    */
   public Preferences_Dialog (
      final java.awt.Frame parent,
      final boolean modal)
   {
      super (parent, modal);
      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "Preferences_Dialog", new Object[] {parent, modal});

      this.initComponents ();

      logger.exiting (TAG, "Preferences_Dialog");
      return;
   }

   /**
    * Creates new form Help
    * @param parent parent window
    * @param properties open modal
    */
   public Preferences_Dialog (
      @NotNull final java.awt.Frame parent,
      @NotNull final net.sourceforge.uiq3.calculator.IProperties properties)
   {
      super (parent, false);
      logger.entering (TAG, "Preferences_Dialog", new Object[] {parent, properties});

      this.initComponents ();

      logger.exiting (TAG, "Preferences_Dialog");
      return;
   } // Preferences_Dialog

   /**
    * Initialse Values
    *
    * @param Engine Calculator whose setting we handle
    */
   public void Initialize (final Desktop_Calculator Engine)
   {
      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "Initialize", Engine);

      this.Engine = Engine;
      this.Initialize ();

      Preferences_Dialog.logger.exiting (Preferences_Dialog.TAG, "Initialize");
      return;
   } // Initialize

   /**
    * Initialize Values
    */
   public void Initialize ()
   {
      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "Initialize");

      final Properties properties = Properties.INSTANCE;

      this.FA6_Folder_Text.setText (properties.getHome_Directory ());
      this.Theme_Canvas.setLookAndFeelSize (properties.getLook_And_Feel_Size ());
      this.Theme_Canvas.setLookAndFeelName (properties.getLook_And_Feel_Name ());
      properties.restoreWindowStatus (this);

      Preferences_Dialog.logger.exiting (Preferences_Dialog.TAG, "Initialize");
      return;
   } // Initialize

   //@formatter:off
   @SuppressWarnings({"DanglingJavadoc", "LocalCanBeFinal", "ChainedMethodCall"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT
    * modify this code. The content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {

      FA6_Folder_Label = new javax.swing.JLabel();
      FA6_Folder_Text = new javax.swing.JTextField();
      Theme_Canvas = new net.sourceforge.uiq3.calculator.Theme_Canvas(Properties.INSTANCE);
      OK_Button = new javax.swing.JButton();
      Cancel_Button = new javax.swing.JButton();

      setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
      setTitle(HP45_Bundle.getString("Preferences_Title")); // NOI18N
      setName("Preferences_Dialog"); // NOI18N

      FA6_Folder_Label.setText(HP45_Bundle.getString("Preferences_Output_Folder")); // NOI18N
      FA6_Folder_Label.setName("FA6_Folder_Label"); // NOI18N

      FA6_Folder_Text.setName("FA6_Folder_Text"); // NOI18N

      OK_Button.setText(HP45_Bundle.getString("Preferences_OK")); // NOI18N
      OK_Button.setName("OK_Button"); // NOI18N
      OK_Button.addActionListener(this::OK_ButtonActionPerformed);

      Cancel_Button.setText(HP45_Bundle.getString("Preferences_Cancel")); // NOI18N
      Cancel_Button.setName("Cancel_Button"); // NOI18N
      Cancel_Button.addActionListener(this::Cancel_ButtonActionPerformed);

      javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
      getContentPane().setLayout(layout);
      layout.setHorizontalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
            .addContainerGap()
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
               .addComponent(FA6_Folder_Text)
               .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                  .addGap(0, 0, Short.MAX_VALUE)
                  .addComponent(Cancel_Button)
                  .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                  .addComponent(OK_Button))
               .addComponent(FA6_Folder_Label, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
               .addComponent(Theme_Canvas, javax.swing.GroupLayout.PREFERRED_SIZE, 379, Short.MAX_VALUE))
            .addContainerGap())
      );
      layout.setVerticalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
            .addContainerGap()
            .addComponent(FA6_Folder_Label)
            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
            .addComponent(FA6_Folder_Text, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
            .addGap(18, 18, 18)
            .addComponent(Theme_Canvas, javax.swing.GroupLayout.DEFAULT_SIZE, 180, Short.MAX_VALUE)
            .addGap(18, 18, 18)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
               .addComponent(Cancel_Button)
               .addComponent(OK_Button))
            .addContainerGap())
      );

      pack();
   }// </editor-fold>//GEN-END:initComponents

   private void OK_ButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_OK_ButtonActionPerformed
   {//GEN-HEADEREND:event_OK_ButtonActionPerformed
      //@formatter:on
      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "OK_ButtonActionPerformed", evt);

      final Properties properties = Properties.INSTANCE;

      final String Text = this.FA6_Folder_Text.getText ();

      if (this.Engine != null)
      {
         properties.saveCalculatorStatus (this.Engine);
      }

      properties.saveWindowStatus (this);
      properties.setLook_And_Feel_Size (this.Theme_Canvas.getLookAndFeelSize ());
      properties.setLook_And_Feel_Name (this.Theme_Canvas.getLookAndFeelName ());
      properties.saveProperties ();

      this.setVisible (false);

      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "OK_ButtonActionPerformed");
      return;
      //@formatter:off
   }//GEN-LAST:event_OK_ButtonActionPerformed

   private void Cancel_ButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_Cancel_ButtonActionPerformed
      Preferences_Dialog.logger.entering (Preferences_Dialog.TAG, "Cancel_ButtonActionPerformed", evt);

      Properties.INSTANCE.saveWindowStatus (this);
      this.setVisible (false);

      Preferences_Dialog.logger.exiting (Preferences_Dialog.TAG, "Cancel_ButtonActionPerformed");
      return;
   }//GEN-LAST:event_Cancel_ButtonActionPerformed

   // Variables declaration - do not modify//GEN-BEGIN:variables
   private javax.swing.JButton Cancel_Button;
   private javax.swing.JLabel FA6_Folder_Label;
   private javax.swing.JTextField FA6_Folder_Text;
   private javax.swing.JButton OK_Button;
   private net.sourceforge.uiq3.calculator.Theme_Canvas Theme_Canvas;
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @SuppressWarnings ("HardcodedLineSeparator")
   @Override @TestOnly @NotNull public String
   toString ()
   {
      return "Preferences_Dialog {\n" +
	 "super = " + super.toString () +
	 ",\nCancel_Button = " + Cancel_Button +
	 ",\nFA6_Folder_Label = " + FA6_Folder_Label +
	 ",\nFA6_Folder_Text = " + FA6_Folder_Text +
	 ",\nOK_Button = " + OK_Button +
	 ",\nTheme_Canvas = " + Theme_Canvas +
	 "\n}";
   }
} // Preferences_Dialog
