/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * Display calculator register data.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@java.beans.JavaBean (description = "Panel to display calculator register data.")
public class Register_Canvas
   extends javax.swing.JPanel
{
   /**
    *
    */
   private static final java.util.ResourceBundle HP45_Bundle;

   // Code for dispatching events from components to event handlers.
   /**
    * <p> class logger tag. </p>
    */
   private static final String TAG;
   /**
    * <p> class logger instance </p>
    */
   private static final java.util.logging.Logger logger;
   private static final long serialVersionUID = -3538192545324370508L;

   static
   {
      TAG = net.sourceforge.uiq3.hp45.Register_Canvas.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
      HP45_Bundle = Properties.INSTANCE.getApplicationBundle ();
   }

   /**
    * Calculator to display informations for.
    */
   @Nullable private Desktop_Calculator Engine;

   /**
    * Creates new form Status_Canvas
    */
   public Register_Canvas ()
   {
      super ();

      Engine = null;
      initComponents ();
      initPost ();
   }

   /**
    * <p> I had used lazy loading before but frequently there were load interrupted errors with
    * images only loaded partially. </p>
    */
   private void initPost ()
   {
      Register_Canvas.logger.entering (Register_Canvas.TAG, "initPost");

      setTextFonts ();

      Properties.INSTANCE.addPropertyChangeListener (evt -> {
	 if (evt.getPropertyName () == net.sourceforge.uiq3.calculator.IProperties.MONO_FONT_ID)
	 {
	    setTextFonts ();
	 }
      });

      Register_Canvas.logger.entering (Register_Canvas.TAG, "initPost");
      return;
   } // initPost

   private void setTextFonts ()
   {
      Register_Canvas.logger.entering (Register_Canvas.TAG, "setTextFonts");

      final java.awt.Font monoFont = Properties.INSTANCE.getMono_Font ();

      CPU_Register_List.setFont (monoFont);
      RAM_Register_List.setFont (monoFont);

      Register_Canvas.logger.entering (Register_Canvas.TAG, "setTextFonts");
      return;
   }  // setTextFonts

   /**
    * Set the calculator engine used - needed for status related display.
    *
    * @param Engine
    *    calculator to be displayed
    */
   public void setCalculator (@NotNull final Desktop_Calculator Engine)
   {
      this.Engine = Engine;
      this.Engine.addPropertyChangeListener (this::update);
      this.update ();

      return;
   } // Set_Calculator

   /**
    * check if the calculator has changed.
    *
    * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
    */
   @SuppressWarnings ("StringEquality")
   public void update (final java.beans.PropertyChangeEvent event)
   {
      try
      {
	 if (isVisible () && event.getPropertyName () == Calculator.DISPLAY_TEXT_CHANGED)
	 {
	    javax.swing.SwingUtilities.invokeLater (this::update);
	 }
      }
      catch (final java.lang.Exception Exception)
      {
	 Register_Canvas.logger.log (java.util.logging.Level.SEVERE, "LOG01630:", Exception);
      }
      return;
   } // update

   /**
    * Update Fields.
    */
   private void update ()
   {
      if (this.Engine != null)
      {
	 final javax.swing.DefaultListModel<String> CPU_Data = new javax.swing.DefaultListModel<> ();
	 
	 for (int i = 0; i <= 6; i++)
	 {
	    CPU_Data.addElement (Engine.registerToString (i));
	 }

	 this.CPU_Register_List.setModel (CPU_Data);

	 final javax.swing.DefaultListModel<String> RAM_Data = new javax.swing.DefaultListModel<> ();

	 for (int i = 0; i < 10; i++)
	 {
	    RAM_Data.addElement (Engine.memoryToString (i));
	 }

	 this.RAM_Register_List.setModel (RAM_Data);
      } // if
      return;
   } // Update

   //@formatter:off
   @SuppressWarnings({"override", "DanglingJavadoc", "LocalCanBeFinal", "AnonymousInnerClass", "TooBroadScope", "MagicNumber"})
   /**
    * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
    * content of this method is always regenerated by the Form Editor.
    */
   // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
   private void initComponents()
   {
      java.awt.GridBagConstraints gridBagConstraints;

      setName("Register_Canvas"); // NOI18N
      addComponentListener(new java.awt.event.ComponentAdapter()
      {
         public void componentShown(java.awt.event.ComponentEvent evt)
         {
            formComponentShown(evt);
         }
      });
      setLayout(new java.awt.GridBagLayout());

      CPU_Register_Label.setText(HP45_Bundle.getString("CPU-Register")); // NOI18N
      CPU_Register_Label.setName("CPU_Register_Label"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 0;
      gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
      add(CPU_Register_Label, gridBagConstraints);

      CPU_Register_ScrollPane.setName("CPU_Register_ScrollPane"); // NOI18N

      CPU_Register_List.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
      CPU_Register_List.setName("CPU_Register_List"); // NOI18N
      CPU_Register_ScrollPane.setViewportView(CPU_Register_List);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 1;
      gridBagConstraints.gridwidth = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 5.0;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
      add(CPU_Register_ScrollPane, gridBagConstraints);

      RAM_Register_Label.setText(HP45_Bundle.getString("RAM-Register")); // NOI18N
      RAM_Register_Label.setName("RAM_Register_Label"); // NOI18N
      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 2;
      gridBagConstraints.gridwidth = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
      gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
      add(RAM_Register_Label, gridBagConstraints);

      RAM_Register_ScrollPane.setName("RAM_Register_ScrollPane"); // NOI18N

      RAM_Register_List.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
      RAM_Register_List.setName("RAM_Register_List"); // NOI18N
      RAM_Register_ScrollPane.setViewportView(RAM_Register_List);

      gridBagConstraints = new java.awt.GridBagConstraints();
      gridBagConstraints.gridx = 0;
      gridBagConstraints.gridy = 3;
      gridBagConstraints.gridwidth = 4;
      gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
      gridBagConstraints.weightx = 1.0;
      gridBagConstraints.weighty = 12.0;
      gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
      add(RAM_Register_ScrollPane, gridBagConstraints);
   }// </editor-fold>//GEN-END:initComponents

   private void formComponentShown (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentShown
   {//GEN-HEADEREND:event_formComponentShown
      logger.entering (TAG, "formComponentShown");

      update ();

      logger.exiting (TAG, "formComponentShown");
   }//GEN-LAST:event_formComponentShown

   // Variables declaration - do not modify//GEN-BEGIN:variables
   private final javax.swing.JLabel CPU_Register_Label = new javax.swing.JLabel();
   private final javax.swing.JList<String> CPU_Register_List = new javax.swing.JList<>();
   private final javax.swing.JScrollPane CPU_Register_ScrollPane = new javax.swing.JScrollPane();
   private final javax.swing.JLabel RAM_Register_Label = new javax.swing.JLabel();
   private final javax.swing.JList<String> RAM_Register_List = new javax.swing.JList<>();
   private final javax.swing.JScrollPane RAM_Register_ScrollPane = new javax.swing.JScrollPane();
   // End of variables declaration//GEN-END:variables
   //@formatter:on

   @TestOnly
   @Override public String
   toString ()
   {
      return TAG +
	 "{super=" + super.toString () +
	 ", CPU_Register_Label=" + CPU_Register_Label +
	 ", CPU_Register_List=" + CPU_Register_List +
	 ", CPU_Register_ScrollPane=" + CPU_Register_ScrollPane +
	 ", RAM_Register_Label=" + RAM_Register_Label +
	 ", RAM_Register_Register_List=" + RAM_Register_List +
	 ", RAM_Register_ScrollPane=" + RAM_Register_ScrollPane +
	 ", Engine=" + Engine +
	 '}';
   }
} // Register_Canvas

// vim: set textwidth=0 nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set filetype=java fileencoding=utf-8 fileformat=unix foldmethod=syntax :
// vim: set nospell spelllang=en :
