/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.And
import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Given
import net.sourceforge.uiq3.test.Info
import net.sourceforge.uiq3.test.Scenario
import net.sourceforge.uiq3.test.Second
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.When
import org.hamcrest.CoreMatchers.notNullValue
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.core.Is.`is` as Is

/**
 * <p>test the calculator engine</p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class Desktop_Calculator_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   @Suppress("PLATFORM_CLASS_MAPPED_TO_KOTLIN")
   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: Initializing")
   inner class Test_Initializing
   {
      init
      {
	 Feature("Initializing")
	 Info("As a user I want to initialize And start the calculator")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Startup without running")
      fun test_Startup_without_running()
      {
	 Scenario("Startup without running")
	 Scenario("Startup without running")
	 Given("A calculator instance")

	 val Test = Desktop_Calculator()

	 When("operation codes are initialized")

	 Test.Init_Opcodes()

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then no error is reported")

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)

      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Startup running")
      fun test_Startup_running()
      {
	 Scenario("Startup running")
	 Given("A calculator instance")

	 val Test = Desktop_Calculator()

	 When("calculator is initialized")

	 Test.Init_Opcodes()

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then the calculator can be run")

	 val Calculator_Thread = Thread(Test, "Calculator Thread")

	 Calculator_Thread.start()

	 And("All Register are zero")

	 // the X Register is stores in hardware register c
	 assertThat("a", Calculator.registerToString(Test.a), Is(equalTo(" 00.000000000 000")))
	 assertThat("b", Calculator.registerToString(Test.b), Is(equalTo(" 00.000000000 000")))
	 assertThat("c / X", Calculator.memoryToString(Test.c), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("d / Y", Calculator.memoryToString(Test.d), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("e / Z", Calculator.memoryToString(Test.e), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("f / T", Calculator.memoryToString(Test.f), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("m", Calculator.registerToString(Test.m), Is(equalTo(" 00.000000000 000")))

	 And("calculator can be stopped stopped")

	 synchronized(Test)
	 {
	    (Test as Object).wait(Second(0.5f))
	 }
	 Test.stop()
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: toString")
      fun test_toString()
      {
	 Scenario("toString")
	 Scenario("toString")
	 Given("an Calculator instance created And started")

	 val Test = Desktop_Calculator()

	 Then("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)

	 And("then the calculator can be stopped")

	 Test.stop()
      } // Scenario
   } // Feature
} // Desktop_Calculator_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
