/*
 * ************************************************************ {{{1 **********
 * Copyright © 2020 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7684 $
 * $Date: 2021-09-13 20:49:22 +0200 (Mo, 13. Sep 2021) $
 * $Id: Calculator_Frame_Test.kt 7684 2021-09-13 18:49:22Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/HP45/Desktop/src/test/java/net.sourceforge.uiq3/ui/Calculator_Frame_Test.kt $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.And
import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Given
import net.sourceforge.uiq3.test.Info
import net.sourceforge.uiq3.test.Press_Buttons
import net.sourceforge.uiq3.test.Scenario
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.Wait_For_Display
import net.sourceforge.uiq3.test.Wait_For_Printout
import net.sourceforge.uiq3.test.Wait_For_Program_Finished
import net.sourceforge.uiq3.test.When
import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsEqual
import org.hamcrest.core.StringContains.containsString
import org.hamcrest.CoreMatchers.notNullValue
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Assertions.fail

private const val Spit_Offset = 2

/**
 * Tests new [Main_Test].
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class Main_Test
{
   companion object
   {
      init
      {
         net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   val objectUnderTest = lazy {
      val retval = Main()
      retval.startApp()

      return@lazy retval
   }

// private lateinit var Look_And_Feel_Size: IProperties.Look_And_Feel_Size_Type
// private lateinit var Look_And_Feel_Name: String
//
// @org.junit.jupiter.api.BeforeAll
// fun Before_All()
// {
//    this.Look_And_Feel_Size = Properties.Look_And_Feel_Size
//    this.Look_And_Feel_Name = Properties.Look_And_Feel_Name
// }
//
// @org.junit.jupiter.api.AfterAll
// fun After_All()
// {
//    Properties.Look_And_Feel_Size = this.Look_And_Feel_Size
//    Properties.Look_And_Feel_Name = this.Look_And_Feel_Name
// }
//
// @org.junit.jupiter.api.BeforeEach
// fun Before_Each()
// {
//    Properties.Look_And_Feel_Size = IProperties.Defaults.Look_And_Feel_Size
//    Properties.Look_And_Feel_Name = IProperties.Defaults.Look_And_Feel_Name
// }

   @org.junit.jupiter.api.BeforeAll
   fun Before_All()
   {
      Feature(this.javaClass.simpleName)
      Info("""The application should start and be able to execute a few simple smoke tests.""")
      
      Main.initializeLAF()

      val main = objectUnderTest.value

      assertThat(main, Is(notNullValue()))

      val calculator = main.calculator
      
      assertThat(calculator, Is(notNullValue()))
   }

   @org.junit.jupiter.api.AfterAll
   fun After_All()
   {
      val main = objectUnderTest.value
      val frame = main.frame ?: fail("should´t be null after initialisation")

      main.pauseApp()
      frame.dispose()
   }

   @org.junit.jupiter.api.BeforeEach
   fun Before_Each()
   {
      val main = objectUnderTest.value
      val frame = main.frame ?: fail("should´t be null after initialisation")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      Press_Buttons(
         listOf(
            canvas.Button_CLx,
            canvas.Button_FIX,
            canvas.Button_2
         )
      )

      frame.setLocation(64, 64)
      frame.size = canvas.preferredSize
      frame.Split_Pane.dividerLocation = canvas.preferredSize.width + Spit_Offset
      return
   } // clear

   @org.junit.jupiter.api.Test
   fun frame_Test()
   {
      Scenario("accessing frame")
      Given("a running application.")
      val main = objectUnderTest.value

      Then("main should have a valid frame.")
      assertThat("running app has a frame", main.frame, Is(notNullValue()))
   }

   @org.junit.jupiter.api.Test
   fun canvas_Test()
   {
      Scenario("accessing canvas")
      Given("a running application.")
      val main = objectUnderTest.value

      Then("main should have a valid canvas.")
      val canvas = main.canvas

      assertThat("running app has a canvas", canvas, Is(notNullValue()))
   }

   @org.junit.jupiter.api.Test
   fun calculator_Test()
   {
      Scenario("accessing calculator")
      Given("a running application.")
      val main = objectUnderTest.value

      Then("main should have a valid calculator.")
      val calculator = main.calculator

      assertThat("running app has a calculator", calculator, Is(notNullValue()))
   }

   @org.junit.jupiter.api.Test
   fun display_Test()
   {
      Scenario("display result")
      Given("a running application.")
      val main = objectUnderTest.value

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      And("A display label")
      val display = canvas.Display_Label

      assertThat("display present", display, Is(notNullValue()))

      Then("display shows 0.")
      Wait_For_Display(canvas, 0, " 0.00          ")
   }

   @org.junit.jupiter.api.Test
   fun press_CLX_Test()
   {
      Scenario("press CLX")
      Given("a valid canvas and display.")
      val main = objectUnderTest.value

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      When("[CLX] was pressed")
      Press_Buttons(
         listOf(
            canvas.Button_CLx
         )
      )

      Then("display shows 0.")
      Wait_For_Display(canvas, Index = 0, Expected = " 0.00          ")
   }

   @org.junit.jupiter.api.Test
   fun Simple_Math_Test()
   {
      Scenario("Simple Math")
      Given("a valid canvas and display.")
      val main = objectUnderTest.value

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      When("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
      Press_Buttons(
         listOf(
            canvas.Button_CLx,
            canvas.Button_1,
            canvas.Button_ENTER,
            canvas.Button_2,
            canvas.Button_ENTER,
            canvas.Button_3,
            canvas.Button_Mult,
            canvas.Button_Plus
         )
      )

      Then("display shows 7.00.")
      Wait_For_Display(canvas, Index = 0, Expected = " 7.00          ")
   }

   @org.junit.jupiter.api.Test
   fun Miniumum_Size_Test()
   {
      Scenario("Minimum size")
      Given("a running application.")
      val main = objectUnderTest.value

      And("a valid frame.")
      val frame = main.frame ?: fail("should´t be null after initialisation")

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      When("frame is size is minimum")
      frame.size = frame.minimumSize
      frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset

      Press_Buttons(
         listOf(
            canvas.Button_1,
            canvas.Button_CHS,
            canvas.Button_EEX,
            canvas.Button_1,
            canvas.Button_CHS,
            canvas.Button_ENTER,
            canvas.Button_GOLD,
            canvas.Button_FIX,
            canvas.Button_9
         )
      )
      And("SCI 9 display is used")

      Then("fame is readable.")
      Wait_For_Display(canvas, 0, "-1.000000000 -01")
   }

   @org.junit.jupiter.api.Test
   fun Prefered_Size_Test()
   {
      Scenario("Preferred size")
      Given("a valid frame.")
      val main = objectUnderTest.value

      And("a valid frame.")
      val frame = main.frame ?: fail("should´t be null after initialisation")

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      When("frame is size is minimum")
      frame.size = frame.preferredSize
      frame.Split_Pane.dividerLocation = canvas.preferredSize.width + Spit_Offset

      Then("fame is readable.")
      Wait_For_Display(canvas, 0, " 0.00          ")
   }

   @org.junit.jupiter.api.Test
   fun MaximumSize_Size_Test()
   {
      Scenario("maximum size")
      Given("a valid frame.")
      val main = objectUnderTest.value

      And("a valid frame.")
      val frame = main.frame ?: fail("should´t be null after initialisation")

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      When("frame is size is minimum")
      frame.size = frame.maximumSize
      frame.Split_Pane.dividerLocation = canvas.maximumSize.width

      Then("fame is readable.")
      Wait_For_Display(canvas, 0, " 0.00          ")
   }

// @org.junit.jupiter.api.Test
// fun Small_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame ?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas ?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Size = IProperties.Look_And_Feel_Size_Type.Small
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       )
//    )
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }
//
// @org.junit.jupiter.api.Test
// fun Medium_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame ?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas ?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Size = IProperties.Look_And_Feel_Size_Type.Medium
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       )
//    )
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }
//
// @org.junit.jupiter.api.Test
// fun Large_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame ?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas ?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Size = IProperties.Look_And_Feel_Size_Type.Large
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       )
//    )
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }

// @org.junit.jupiter.api.Test
// fun Light_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Name = "com.formdev.flatlaf.FlatLightLaf"
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       ))
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }
//
// @org.junit.jupiter.api.Test
// fun Dark_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Name = "com.formdev.flatlaf.FlatDarkLaf"
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       ))
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }
//
// @org.junit.jupiter.api.Test
// fun Darcula_Style_Test()
// {
//    Given("a running application.")
//    val main = objectUnderTest.value
//
//    And("a valid frame.")
//    val frame = main.frame?: fail("should´t be null after initialisation")
//
//    And("a valid canvas.")
//    val canvas = main.canvas?: fail("should´t be null after initialisation")
//
//    When("The small style is activated.")
//    Properties.Look_And_Feel_Name = "com.formdev.flatlaf.FlatDarculaLaf"
//
//    And("frame is size is minimum")
//    frame.size = frame.minimumSize
//    frame.Split_Pane.dividerLocation = canvas.minimumSize.width + Spit_Offset;
//
//    And("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
//    Press_Buttons(
//       listOf(
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_EEX,
//          canvas.Button_1,
//          canvas.Button_CHS,
//          canvas.Button_ENTER,
//          canvas.Button_GOLD,
//          canvas.Button_FIX,
//          canvas.Button_9
//       ))
//    And("SCI 9 display is used")
//
//    Then("fame is readable.")
//    Wait_For_Display(canvas, 0, "-1.000000000 -01")
// }

   @org.junit.jupiter.api.Test
   fun Exception_Test()
   {
      Scenario("exceptions")
      Given("a valid frame.")
      val main = objectUnderTest.value

      And("a valid frame.")
      val frame = main.frame ?: fail("should´t be null after initialisation")

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      And("a valid error canvas.")
      val error = frame.Error_Canvas

      When("Exception should be displayed")
      canvas.displayException(Exception("Test Exception"))

      Then("Exception message is displayed is readable.")
      assertThat(error.messageText, IsEqual.equalTo(/* operand = */ "Test Exception"))
      assertThat(error.classText, IsEqual.equalTo(/* operand = */ "java.lang.Exception"))
      assertThat(error.stackText, containsString(/* substring = */ "Exception_Test"))
   }

   @org.junit.jupiter.api.Test
   fun Printer_Test()
   {
      Scenario("using the printer")
      Given("a valid canvas and display.")
      val main = objectUnderTest.value

      And("a valid frame.")
      val frame = main.frame ?: fail("should´t be null after initialisation")

      And("a valid canvas.")
      val canvas = main.canvas ?: fail("should´t be null after initialisation")

      And("a valid calculator.")
      val calculator = main.calculator ?: fail("should´t be null after initialisation")

      And("a valid printer canvas.")
      val printer = frame.Printer_Canvas ?: fail("should´t be null after initialisation")

      assertThat("running app has a canvas", canvas, Is(notNullValue()))

      When("[CLX] [1] [ENTER] [2] [ENTER] [3] [*] [+] was pressed")
      Press_Buttons(
         listOf(
            canvas.Button_CLx,
            canvas.Button_1,
            canvas.Button_ENTER,
            canvas.Button_2,
            canvas.Button_ENTER,
            canvas.Button_3,
            canvas.Button_Mult,
            canvas.Button_Plus
         )
      )

      And("display shows 7.00.")
      Wait_For_Display(canvas, Index = 0, Expected = " 7.00          ")

      And("Print presses")

      printer.doClearButton()
      printer.doPrintButton1()
      Wait_For_Program_Finished(calculator)

      Then("Printout displays")

      Wait_For_Printout(printer, Expected = " 7.00          ")
   }
}
//************************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab textwidth=120:
// vim: set textwidth=0 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
