/*
 * ************************************************************ {{{1 **********
 * Copyright © 2020 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.calculator.IProperties
import net.sourceforge.uiq3.calculator.IProperties.Companion.longToColor
import net.sourceforge.uiq3.test.Feature
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.io.FileMatchers.anExistingFile
import org.junit.jupiter.api.Test

@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class Properties_Test
{
   companion object
   {
      init
      {
	 net.sourceforge.uiq3.test.Init_Logger()
      }
   }

   private lateinit var Look_And_Feel_Size : IProperties.Look_And_Feel_Size_Type
   private lateinit var Look_And_Feel_Name : String

   @org.junit.jupiter.api.BeforeAll
   fun Before_All()
   {
      Feature(this.javaClass.simpleName)
      
      this.Look_And_Feel_Size = Properties.Look_And_Feel_Size
      this.Look_And_Feel_Name = Properties.Look_And_Feel_Name
   }

   @org.junit.jupiter.api.AfterAll
   fun After_All ()
   {
      Properties.Look_And_Feel_Size = this.Look_And_Feel_Size
      Properties.Look_And_Feel_Name = this.Look_And_Feel_Name
   }

   @org.junit.jupiter.api.BeforeEach
   fun Before_Each()
   {
      Properties.Look_And_Feel_Size = IProperties.Defaults.Look_And_Feel_Size
      Properties.Look_And_Feel_Name = IProperties.Defaults.Look_And_Feel_Name
   }

   @Test
   fun test_Accent_Color()
   {
      assertThat(
	 /* reason = */ "Accent_Color should be default",
	 /* actual = */ Properties.Accent_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Accent_Color)))
   }

   @Test
   fun test_Background_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Background_Dark_Color should be default",
	 /* actual = */ Properties.Background_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Background_Dark_Color)))
   }

   @Test
   fun test_Background_Light_Color()
   {
      assertThat(
	 /* reason = */ "Background_Light_Color should be default",
	 /* actual = */ Properties.Background_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Background_Light_Color)))
   }

   @Test
   fun test_Button_Background_Color()
   {
      assertThat(
	 /* reason = */ "Button_Background_Color should be default",
	 /* actual = */ Properties.Button_Background_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Background_Color)))
   }

   @Test
   fun test_Button_Type_0_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_0_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_0_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_0_Dark_Color)))
   }

   @Test
   fun test_Button_Type_0_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_0_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_0_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_0_Light_Color)))
   }

   @Test
   fun test_Button_Type_0_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_0_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_0_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_0_Text_Color)))
   }

   @Test
   fun test_Button_Type_1_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_1_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_1_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_1_Dark_Color)))
   }

   @Test
   fun test_Button_Type_1_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_1_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_1_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_1_Light_Color)))
   }

   @Test
   fun test_Button_Type_1_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_1_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_1_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_1_Text_Color)))
   }

   @Test
   fun test_Button_Type_2_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_2_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_2_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_2_Dark_Color)))
   }

   @Test
   fun test_Button_Type_2_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_2_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_2_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_2_Light_Color)))
   }

   @Test
   fun test_Button_Type_2_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_2_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_2_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_2_Text_Color)))
   }

   @Test
   fun test_Button_Type_3_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_3_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_3_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_3_Dark_Color)))
   }

   @Test
   fun test_Button_Type_3_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_3_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_3_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_3_Light_Color)))
   }

   @Test
   fun test_Button_Type_3_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_3_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_3_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_3_Text_Color)))
   }

   @Test
   fun test_Button_Type_4_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_4_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_4_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_4_Dark_Color)))
   }

   @Test
   fun test_Button_Type_4_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_4_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_4_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_4_Light_Color)))
   }

   @Test
   fun test_Button_Type_4_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_4_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_4_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_4_Text_Color)))
   }

   @Test
   fun test_Button_Type_5_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_5_Dark_Color should be default",
	 /* actual = */ Properties.Button_Type_5_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_5_Dark_Color)))
   }

   @Test
   fun test_Button_Type_5_Light_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_5_Light_Color should be default",
	 /* actual = */ Properties.Button_Type_5_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_5_Light_Color)))
   }

   @Test
   fun test_Button_Type_5_Text_Color()
   {
      assertThat(
	 /* reason = */ "Button_Type_5_Text_Color should be default",
	 /* actual = */ Properties.Button_Type_5_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Button_Type_5_Text_Color)))
   }

   @Test
   fun test_Button_Minimum_Size()
   {
      assertThat(
	 /* reason = */ "Button minimum size should be default",
	 /* actual = */ Properties.Button_Size,
	 /* matcher = */ equalTo(IProperties.Defaults.Button_Size))
   }

   @Test
   fun test_Display_Background_Color()
   {
      assertThat(
	 /* reason = */ "Display_Background_Color should be default",
	 /* actual = */ Properties.Display_Background_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Display_Background_Color)))
   }

   @Test
   fun test_Display_Text_Color()
   {
      assertThat(
	 /* reason = */ "Display_Text_Color should be default",
	 /* actual = */ Properties.Display_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Display_Text_Color)))
   }

   @Test
   fun test_Indicator_Selected_Color_Color()
   {
      assertThat(
	 /* reason = */ "Indicator_Selected_Color_Color should be default",
	 /* actual = */ Properties.Indicator_Selected_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Indicator_Selected_Color)))
   }

   @Test
   fun test_Indicator_Unselected_Color_Color()
   {
      assertThat(
	 /* reason = */ "Indicator_Unselected_Color_Color should be default",
	 /* actual = */ Properties.Indicator_Unselected_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Indicator_Unselected_Color)))
   }

   @Test
   fun test_Gold_Text_Color()
   {
      assertThat(
	 /* reason = */ "Gold_Text_Color should be default",
	 /* actual = */ Properties.Gold_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Gold_Text_Color)))
   }

   @Test
   fun test_Primary_Color()
   {
      assertThat(
	 /* reason = */ "Primary_Color should be default",
	 /* actual = */ Properties.Primary_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Primary_Color)))
   }

   @Test
   fun test_Primary_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Primary_Dark_Color should be default",
	 /* actual = */ Properties.Primary_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Primary_Dark_Color)))
   }

   @Test
   fun test_Printer_Background_Color()
   {
      assertThat(
	 /* reason = */ "Printer_Background_Color should be default",
	 /* actual = */ Properties.Printer_Background_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printer_Background_Color)))
   }

   @Test
   fun test_Printer_Button_Dark_Color()
   {
      assertThat(
	 /* reason = */ "Printer_Button_Dark_Color should be default",
	 /* actual = */ Properties.Printer_Button_Dark_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printer_Button_Dark_Color)))
   }

   @Test
   fun test_Printer_Button_Light_Color()
   {
      assertThat(
	 /* reason = */ "Printer_Button_Light_Color should be default",
	 /* actual = */ Properties.Printer_Button_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printer_Button_Light_Color)))
   }

   @Test
   fun test_Printer_Button_Text_Color()
   {
      assertThat(
	 /* reason = */ "Printer_Button_Text_Color should be default",
	 /* actual = */ Properties.Printer_Button_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printer_Button_Text_Color)))
   }

   @Test
   fun test_Printout_Light_Color()
   {
      assertThat(
	 /* reason = */ "Printout_Light_Color should be default",
	 /* actual = */ Properties.Printout_Light_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printout_Light_Color)))
   }

   @Test
   fun test_Printout_Text_Color()
   {
      assertThat(
	 /* reason = */ "Printout_Text_Color should be default",
	 /* actual = */ Properties.Printout_Text_Color.value,
	 /* matcher = */ equalTo(longToColor(Properties.Defaults.Printout_Text_Color)))
   }

   @Test
   fun test_Display_Font()
   {
      assertThat(
	 /* reason = */ "Display_Font should be default",
	 /* actual = */ Properties.Display_Font,
	 /* matcher = */ equalTo(Properties.Defaults.Display_Font))
   }

   @Test
   fun test_Mono_Font_Font()
   {
      assertThat(
	 /* reason = */ "Mono_Font should be default",
	 /* actual = */ Properties.Mono_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Mono_Font))
   }

   @Test
   fun test_Mono_Small_Font_Font()
   {
      assertThat(
	 /* reason = */ "Mono_Font should be default",
	 /* actual = */ Properties.Mono_Small_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Mono_Small_Font))
   }

   @Test
   fun test_Mono_Large_Font()
   {
      assertThat(
	 /* reason = */ "Mono_Font should be default",
	 /* actual = */ Properties.Mono_Large_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Mono_Large_Font))
   }

   @Test
   fun test_Dialog_Font()
   {
      assertThat(
	 /* reason = */ "Normal_Font should be default",
	 /* actual = */ Properties.Dialog_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Dialog_Font))
   }

   @Test
   fun test_Dialog_Small_Font()
   {
      assertThat(
	 /* reason = */ "Small_Font should be default",
	 /* actual = */ Properties.Dialog_Small_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Dialog_Small_Font))
   }

   @Test
   fun test_Dialog_Large_Font()
   {
      assertThat(
	 /* reason = */ "Large_Font should be default",
	 /* actual = */ Properties.Dialog_Large_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Dialog_Large_Font))
   }

   @Test
   fun test_Text_Font()
   {
      assertThat(
	 /* reason = */ "Normal_Font should be default",
	 /* actual = */ Properties.Text_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Text_Font))
   }

   @Test
   fun test_Text_Small_Font()
   {
      assertThat(
	 /* reason = */ "Small_Font should be default",
	 /* actual = */ Properties.Text_Small_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Text_Small_Font))
   }

   @Test
   fun test_Text_Large_Font()
   {
      assertThat(
	 /* reason = */ "Large_Font should be default",
	 /* actual = */ Properties.Text_Large_Font,
	 /* matcher = */ equalTo(IProperties.Defaults.Text_Large_Font))
   }

   @Test
   fun test_Look_And_Feel()
   {
      assertThat(
	 /* reason = */ "Large_Font should be default",
	 /* actual = */ Properties.Look_And_Feel_Name,
	 /* matcher = */ equalTo(net.sourceforge.uiq3.calculator.IProperties.Defaults.Look_And_Feel_Name))
   }

   @Test
   @org.junit.jupiter.api.AfterAll
   fun test_Save_Properties()
   {
      Properties.saveProperties()

      assertThat("Property file was created.", Properties.Properties_Filename, anExistingFile())
   }

}
//************************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab textwidth=120:
// vim: set textwidth=0 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
