/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * @author martin
 */
@SuppressWarnings ({"WeakerAccess", "HardcodedFileSeparator"})
public final class Constants
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   /**
    * <p>
    * default font size to use
    * </p>
    */
   public static final int Default_Font_Size;
   /**
    * <p>
    * Application runs in demo mode.
    * </p>
    */
   public static final boolean DEMO;

   @NotNull
   public static final java.net.URI SourceForge_Project;
   /**
    * <p>
    * Google+ website.
    * </p>
    */
   @NotNull
   public static final java.net.URI Bug_Report;
   /**
    * <p>
    * Videos on Google+ website.
    * </p>
    */
   @NotNull
   public static final java.net.URI YouTube_Videos;
   /**
    * <p>
    * History page on the calculator home page.
    * </p>
    */
   @NotNull
   public static final java.net.URI History;
   /**
    * <p>
    * Calculator Privacy_Policy page.
    * </p>
    */
   @NotNull
   public static final java.net.URI Privacy_Policy;
   /**
    * <p>
    * Calculator Home page.
    * </p>
    */
   @NotNull
   public static final java.net.URI Homepage;
   /**
    * <p>
    * Original Documents.
    * </p>
    */
   @NotNull
   public static final java.net.URI Manuals;

   /**
    * <p>market:// link to the app</p>
    */
   @NotNull
   @NonNls
   public static final String Market;

   /**
    * <p>https.// link to the app</p>
    */
   @NotNull
   @NonNls
   public static final String Shop;

   @NotNull
   @NonNls
   public static final String Not_Needed;

   static
   {
      TAG = Constants.class.getName ();
      Logger = java.util.logging.Logger.getLogger (TAG);

      try
      {
	 Bug_Report = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "sourceforge.net",
	    /* path     => */ "/p/uiq3/bugs/search/",
	    /* query    => */ "q=labels:\"FX-603P\"",
	    /* fragment => */ null);
	 Manuals = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "uiq3.sourceforge.io",
	    /* path     => */ "/rpn-45/manuals/",
	    /* query    => */ null,
	    /* fragment => */ null);
         History = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "uiq3.sourceforge.io",
	    /* path     => */ "/rpn-45/history/",
	    /* query    => */ null,
	    /* fragment => */ null);
         Homepage = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "uiq3.sourceforge.io",
	    /* path     => */ "/rpn-45/",
	    /* query    => */ null,
	    /* fragment => */ null);
         SourceForge_Project = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "sourceforge.net",
	    /* path     => */ "/p/uiq3/discussion/HP45/",
	    /* query    => */ null,
	    /* fragment => */ null);
         YouTube_Videos = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "www.youtube.com",
	    /* path     => */ "/c/HP45EmulatorApp",
	    /* query    => */ null,
	    /* fragment => */ null);
         Privacy_Policy = new java.net.URI (
	    /* scheme   => */ "https",
	    /* host     => */ "uiq3.sourceforge.io",
	    /* path     => */ "/privacy/",
	    /* query    => */ null,
	    /* fragment => */ null);
	 Market = "market://details?id=net.sourceforge.uiq3.hp45&hl=pt_PT";
	 Shop = "https://play.google.com/store/apps/details?id=net.sourceforge.uiq3.hp45";
      }
      catch (final java.net.URISyntaxException Exception)
      {
	 Constants.Logger.log (java.util.logging.Level.SEVERE, "LOG00791:", Exception);
	 throw new RuntimeException ("Faulty URL given.", Exception);
      } // try

      Default_Font_Size = 15;
      DEMO = false;
      Not_Needed = "The HP-45 doesn't need this operation to be implemented.";
   }

   private Constants ()
   {
      // Utility class
   }
} // Constants

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
