/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024 "Martin Krischik" «krischik@users.sourceforge.net»
 * Copyright 2000          David G. Hicks    (Java port)
 * Copyright 1995          Eric L. Smith     (Original C/X Windows)
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
package net.sourceforge.uiq3.hp45;

/**
 */
final class op_arith
   implements
      net.sourceforge.uiq3.calculator.IOperation
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = op_arith.class.getName ();
      Logger = java.util.logging.Logger.getLogger (op_arith.TAG);
   } // static

   /**
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   private final Calculator Owner;

   /**
    * @param calculator
    */
   op_arith (final Calculator calculator)
   {
      this.Owner = calculator;
   }

   /**
    * @param Opcode
    */
   @Override
   public void Execute (final short Opcode)
   {
      int op, field;
      final byte zero = 0;
      int first = 0, last = 0;
      byte temp;
      int i;
      final byte t[] = new byte [Calculator.Word_Size];

      op = Opcode >> 5;
      field = (Opcode >> 2) & 7;

      switch (field)
      {
         case 0: /* p  */

         {
            first = this.Owner.p;
            last = this.Owner.p;
            if (this.Owner.p >= Calculator.Word_Size)
            {
               // #ifdef DEBUG
               // # System.out.println ("Warning! p > WSIZE at "
               // # + Owner.Previous_Program_Counter);
               // #endif
               last = 0; /* don't do anything */
            }
            break;
         }
         case 1: /* m  */

         {
            first = 3;
            last = 12;
            break;
         }
         case 2: /* x  */

         {
            first = 0;
            last = 2;
            break;
         }
         case 3: /* w  */

         {
            first = 0;
            last = 13;
            break;
         }
         case 4: /* wp */

         {
            first = 0;
            last = this.Owner.p; /* break; bug in orig??? */
            if (this.Owner.p > 13)
            {
               // #ifdef DEBUG
               // # System.out.println ("Warning! p >= WSIZE at "
               // # + Owner.Previous_Program_Counter);
               // #endif
               last = 13;
            }
            break;
         }
         case 5: /* ms */

         {
            first = 3;
            last = 13;
            break;
         }
         case 6: /* xs */

         {
            first = 2;
            last = 2;
            break;
         }
         case 7: /* s  */

         {
            first = 13;
            last = 13;
            break;
         }
      }

      switch (op)
      {
         case 0x00: /* if b[f] = 0 */

         {
            this.Owner.carry = 0; // dgh added - needed?
            for (i = first; i <= last; i = i + 1)
            {
               this.Owner.carry |= (this.Owner.b [i] != 0)
                  ? 1
                  : 0;
            }
            break;
         }
         case 0x01: /* 0 -> b[f] */

         {
            for (i = first; i <= last; i = i + 1)
            {
               this.Owner.b [i] = 0;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x02: /* if a >= Coefficient[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i = i + 1)
            {
               t [i] = this.Owner.do_sub (this.Owner.a [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x03: /* if Coefficient[f] >= 1 */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i = i + 1)
            {
               this.Owner.carry &= (this.Owner.c [i] == 0)
                  ? 1
                  : 0;
            }
            break;
         }
         case 0x04: /* b -> Coefficient[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.b [i];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x05: /* 0 - Coefficient -> Coefficient[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_sub (zero, this.Owner.c [i]);
            }
            break;
         }
         case 0x06: /* 0 -> Coefficient[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = 0;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x07: /* 0 - Coefficient - 1 -> Coefficient[f] */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_sub (zero, this.Owner.c [i]);
            }
            break;
         }
         case 0x08: /* shift left a[f] */

         {
            for (i = last; i >= first; i--)
            {
               this.Owner.a [i] = (i == first)
                  ? 0
                  : this.Owner.a [i - 1];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x09: /* a -> b[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.b [i] = this.Owner.a [i];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x0a: /* a - Coefficient -> Coefficient[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_sub (this.Owner.a [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x0b: /* Coefficient - 1 -> Coefficient[f] */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_sub (this.Owner.c [i], zero);
            }
            break;
         }
         case 0x0c: /* Coefficient -> a[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.c [i];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x0d: /* if Coefficient[f] = 0 */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.carry |= (this.Owner.c [i] != 0)
                  ? 1
                  : 0;
            }
            break;
         }
         case 0x0e: /* a + Coefficient -> Coefficient[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_add (this.Owner.a [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x0f: /* Coefficient + 1 -> Coefficient[f] */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_add (this.Owner.c [i], zero);
            }
            break;
         }
         case 0x10: /* if a >= b[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               t [i] = this.Owner.do_sub (this.Owner.a [i], this.Owner.b [i]);
            }
            break;
         }
         case 0x11: /* b exchange Coefficient[f] */

         {
            for (i = first; i <= last; i++)
            {
               temp = this.Owner.b [i];
               this.Owner.b [i] = this.Owner.c [i];
               this.Owner.c [i] = temp;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x12: /* shift right Coefficient[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = (i == last)
                  ? 0
                  : this.Owner.c [i + 1];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x13: /* if a[f] >= 1 */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.carry &= (this.Owner.a [i] == 0)
                  ? 1
                  : 0;
            }
            break;
         }
         case 0x14: /* shift right b[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.b [i] = (i == last)
                  ? 0
                  : this.Owner.b [i + 1];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x15: /* Coefficient + Coefficient -> Coefficient[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.c [i] = this.Owner.do_add (this.Owner.c [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x16: /* shift right a[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = (i == last)
                  ? 0
                  : this.Owner.a [i + 1];
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x17: /* 0 -> a[f] */

         {
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = 0;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x18: /* a - b -> a[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_sub (this.Owner.a [i], this.Owner.b [i]);
            }
            break;
         }
         case 0x19: /* a exchange b[f] */

         {
            for (i = first; i <= last; i++)
            {
               temp = this.Owner.a [i];
               this.Owner.a [i] = this.Owner.b [i];
               this.Owner.b [i] = temp;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x1a: /* a - Coefficient -> a[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_sub (this.Owner.a [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x1b: /* a - 1 -> a[f] */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_sub (this.Owner.a [i], zero);
            }
            break;
         }
         case 0x1c: /* a + b -> a[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_add (this.Owner.a [i], this.Owner.b [i]);
            }
            break;
         }
         case 0x1d: /* a exchange Coefficient[f] */

         {
            for (i = first; i <= last; i++)
            {
               temp = this.Owner.a [i];
               this.Owner.a [i] = this.Owner.c [i];
               this.Owner.c [i] = temp;
            }
            this.Owner.carry = 0;
            break;
         }
         case 0x1e: /* a + Coefficient -> a[f] */

         {
            this.Owner.carry = 0;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_add (this.Owner.a [i], this.Owner.c [i]);
            }
            break;
         }
         case 0x1f: /* a + 1 -> a[f] */

         {
            this.Owner.carry = 1;
            for (i = first; i <= last; i++)
            {
               this.Owner.a [i] = this.Owner.do_add (this.Owner.a [i], zero);
            }
            break;
         }
      }
      return;
   } // Execute
}
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
