/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024 "Martin Krischik" «krischik@users.sourceforge.net»
 * Copyright 2000          David G. Hicks    (Java port)
 * Copyright 1995          Eric L. Smith     (Original C/X Windows)
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
package net.sourceforge.uiq3.hp45;

/**
 */
final class op_load_constant
   implements
      net.sourceforge.uiq3.calculator.IOperation
{
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;

   static
   {
      TAG = op_load_constant.class.getName ();
      Logger = java.util.logging.Logger.getLogger (op_load_constant.TAG);
   } // static

   /**
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   private final Calculator Owner;

   /**
    * @param calculator
    */
   op_load_constant (final Calculator calculator)
   {
      this.Owner = calculator;
   }

   /**
    * @param Opcode
    *           Opcode to be executed
    */
   @Override
   public void Execute (final short Opcode)
   {
      if (this.Owner.p >= Calculator.Word_Size)
      {
         // I didn't consider the hP-55 case because Eric's comments
         // indicated that it didn't work that well anyway
         //
         // if 0 HP-45 depends on load constant with p > 13 not affecting C
         // printf ("load constant w/ p >= WSIZE at %05o\n", Previous_Program_Counter)
         // ;
         // endif
      }
      else if ((Opcode >> 6) > 9)
      {
         // #ifdef DEBUG
         // # System.out.println ("load constant > 9");
         // #endif
      }
      else
      {
         this.Owner.c [this.Owner.p] = (byte) (Opcode >> 6);
      } // if
      this.Owner.p = (this.Owner.p - 1) & 0xf;
      return;
   } // Execute
}
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
